package org.act.soap.authen;
import org.act.soap.soapsec.securitytoken.SecurityToken;
import org.act.soap.soapsec.securitytoken.X509CertToken;
import org.act.soap.soapsec.SOAPSecurity;
import org.act.soap.soapsec.Signature;
import org.act.soap.soapsec.Constants;
import org.act.soap.server.secmanager.CredManager;
import org.act.soap.server.secmanager.UserManager;
import org.act.soap.server.config.ConfigManager;
import java.security.cert.X509Certificate;
import java.util.Set;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;
import org.act.soap.util.i18n.Message;
import java.security.Key;

public class X509CertLoginModule implements LoginModule {
  
  private static Log log = LogFactory.getLog(X509CertLoginModule.class.getName());
  
  public X509CertLoginModule() {
  }
  
  
  public boolean login (LoginContext loginContext) throws LoginException,
      Exception {
            UserManager usermanager = ConfigManager.getInstance().getUserManager();
            SOAPSecurity soapsec = loginContext.getSOAPSecurity();
      if( soapsec == null ) return false;
            SecurityToken[] tokens = soapsec.getSecurityTokens();
      if( tokens != null ) {
          for( int i=0; i<tokens.length; i++ ) {
              if( tokens[i] instanceof X509CertToken ) {
                  X509CertToken certToken = (X509CertToken)tokens[i];
                  X509Certificate cert = certToken.getX509v3Cert();
                                    CredManager credManager = ConfigManager.getInstance().getCredManager();
                  if( credManager.validateCert( cert ) == false ) {
                     System.out.println(">>> ֤Ч֤ʧ");
                                          continue;
                  }
                                    if( verifySignature( soapsec.getSignatures(), cert ) == false ) {
                     System.out.println ( ">>> ֤֤ǩʧ" );
                                            continue;
                  }
                                    String userDN = cert.getSubjectDN().getName();
                  int userID = usermanager.getIDbyDN( userDN );
                  if( userID == 0 ) {
                      System.out.println ( ">>> ûڡ" + " DN = " + userDN );
                                            continue;
                  }
                                    LoginPrincipal userPrincipal = new LoginPrincipal(String.valueOf(userID));
                  userPrincipal.setAuthType( "X509CERTIFICATE" );
                  Set principals = loginContext.getSubject().getPrincipals();
                  principals.add( userPrincipal );
                  System.out.println ( "************************ ֤ģ ***********************" );
                  System.out.println ( "   ֤֤ɹûDN = " + userDN );
                  System.out.println ( "                 ûID = " + userID );
                  System.out.println ( "*********************************************************" );
                  return true;
              }
          }
      }
      System.out.println ( Message.getMessage("InvalidX509"));
      return false;
  }
  
  public boolean logout(LoginContext loginContext) throws LoginException {
      return false;
  }
  
  private boolean verifySignature( Signature[] sigs, X509Certificate cert ) throws LoginException {
      if( sigs != null ) {
          for( int i=0; i<sigs.length; i++ ) {
              if( sigs[i] == null ) continue;
              
                                          Key pubKey = cert.getPublicKey();
              boolean vad = false;
              try{
                  vad = sigs[i].verify( pubKey );
              }catch( Exception e ){
                                }
              if( vad ){
                                    if( sigs[i].isSigningPart( Constants.PARTINFO_BODY ) ) {
                                            return true;
                  }
              }
          }
      }
      return false;
  }
}
