package org.act.soap.client;
import java.lang.reflect.*;
import java.util.*;
import javax.xml.namespace.QName;
import javax.xml.rpc.*;
import org.act.soap.util.i18n.Message;

public class ClientProxy implements InvocationHandler{
    
    private Call call = null;
    
    private HashMap properties = new HashMap( 4 );

	
	protected ClientProxy() {
	}
    
    public ClientProxy( Call _call ){
        this.call = _call;
    }


    
    public final Object invoke( Object proxy, Method method, Object[] params )
        throws Throwable{
        Object ret = null;
                if( "_setProperty".equals( method.getName() ) ){
            if( null == params
                || 2 != params.length
                || !(params[0] instanceof String) ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidProp" ) );
            }
            this.setProperty( (String)params[0], params[1] );
            return null;
        }else if( "_getProperty".equals( method.getName() ) ){
            if( null == params
                || 1 != params.length
                || !(params[0] instanceof String) ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidProp" ) );
            }
            return this.getProperty( (String)params[0] );
        }else if( "_getPropertyNames".equals( method.getName() ) ){
            if( null != params && 0 != params.length ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidProp" ) );
            }
            return this.getPropertyNames();
        }
                call.recycle();
                Iterator iter = this.properties.keySet().iterator();
        String propertyName = null;
        int size = this.properties.size();
        for( int i = 0; i < size; i++ ){
            propertyName = (String)iter.next();
            if( propertyName.equals( Stub.ENDPOINT_ADDRESS_PROPERTY ) ){
                call.setTargetEndpointAddress( (String)this.properties
                                               .get( propertyName ) );
            }else{
                call.setProperty( propertyName, this.properties.get( propertyName ) );
            }
        }
        call.setOperationName( new QName( call.getService().getDefinition()
                                          .getTargetNamespace(), method.getName()) );
                Class returnType = method.getReturnType();
        call.returnType.setJavaType( returnType );
                ret = call.invoke( params );
        return ret;
    }


    
   private final void setProperty( String name, Object value ){
              if( null == name ){
           throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropName" ) );
       }else if( null == value ){
           throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValue" ) );
       }else if( name.equals( Stub.USERNAME_PROPERTY ) ) {
                      if( !( value instanceof String ) ) {
               throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.String.class.getName(), value.getClass().getName() ) );
           }
           this.properties.put( Stub.USERNAME_PROPERTY, value );
       }else if( name.equals( Stub.PASSWORD_PROPERTY ) ) {
                      if( !( value instanceof String ) ) {
               throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.String.class.getName(), value.getClass().getName() ) );
           }
           this.properties.put( Stub.PASSWORD_PROPERTY, value );
       }else if( name.equals( Stub.ENDPOINT_ADDRESS_PROPERTY ) ) {
                      if( !( value instanceof String ) ) {
               throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.String.class.getName(), value.getClass().getName() ) );
           }
           this.properties.put( Stub.ENDPOINT_ADDRESS_PROPERTY, value );
       }else if( name.equals( Stub.SESSION_MAINTAIN_PROPERTY ) ) {
                      if( !( value instanceof Boolean ) ) {
               throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.Boolean.class.getName(), value.getClass().getName() ) );
           }
           this.properties.put( Stub.SESSION_MAINTAIN_PROPERTY, value );
       }else{
                      throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_unsupportedProp" ) );
       }
   }
   
   private final Object getProperty( String name ){
       if( null == name ){
           throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropName" ) );
       }else if( ! this.properties.containsKey( name ) ) {
           throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_unsupportedProperty", name ) );
       }else{
           Object ret = this.properties.get( name );
           return ret;
       }
   }
   
   private final Iterator getPropertyNames(){
       return this.properties.keySet().iterator();
   }

}
