package org.act.soap.client;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.soap.SOAPMessage;
import org.act.soap.util.i18n.Message;

public class MessageContext implements
    javax.xml.rpc.handler.soap.SOAPMessageContext {
    
    private SOAPMessage mRequestMessage = null;
    
    private SOAPMessage mResponseMessage = null;
    
    private URL mServiceEndpointAdd = null;
    
    private HashMap mPropertyTable = null;
    
    private TypeMapping mTypeMap = null;
    
    private boolean mResponsing = false;

    
    protected MessageContext() {
    }


    
    public final Object getProperty( String name ) {
        if ( null != name && null != mPropertyTable ) {
            return mPropertyTable.get(name);
        }
        return null;
    }
    
    public final void setProperty( String name, Object value ) {
        if ( null == name || null == value ) {
            throw new java.lang.IllegalArgumentException( Message.getMessage( "ERR_ClientProxy_invalidProp" ) );
        }
        if ( null == mPropertyTable ) {
            mPropertyTable = new HashMap( 10 );
        }
        mPropertyTable.put( name, value );
    }
    
    public final void removeProperty( String propName ){
        if( null != this.getProperty( propName ) ) {
            this.mPropertyTable.remove( propName );
        }
    }
    
    public final boolean containsProperty( String propName ){
    	return (this.getProperty( propName ) != null);
    }
    
    public final Iterator getPropertyNames(){
    	return (this.mPropertyTable == null) ? null : this.mPropertyTable
            .keySet().iterator();
    }
    
    public final SOAPMessage getMessage(){
    	return this.mResponsing ? this.mResponseMessage : this.mRequestMessage;
    }
    
    public final void setMessage( SOAPMessage message ){
        if( this.mResponsing ) {
            this.mResponseMessage = message;
        } else {
            this.mRequestMessage = message;
        }
    }
    
    public final String[] getRoles(){
        throw new java.lang.UnsupportedOperationException(
            Message.getMessage( "ERR_SF_MethodNotSupported" ) );
    }


    
    public void clear() {
        this.mRequestMessage = null;
        this.mResponseMessage = null;
        this.mPropertyTable = null;
        this.mResponsing = false;
        this.mTypeMap = null;
        this.mServiceEndpointAdd = null;
    }
    
    public final void setResponsing( boolean value ){
        this.mResponsing = value;
    }
    
    public final boolean isResponsing(){
        return this.mResponsing;
    }
    
    public final SOAPMessage getRequestMessage() {
        return this.mRequestMessage;
    }
    
    public final void setRequestMessage( SOAPMessage reqMsg ) {
        mRequestMessage = reqMsg;
    }
    
    public final SOAPMessage getResponseMessage() {
        return this.mResponseMessage;
    }
    
    public final void setResponseMessage( SOAPMessage respMsg ) {
        mResponseMessage = respMsg;
    }
    
    public final void setServiceAdd( URL value ){
        this.mServiceEndpointAdd = value;
    }
    
    public final URL getServiceAdd(){
        return this.mServiceEndpointAdd;
    }
    
    public final void setTypeMapping( TypeMapping value ){
        this.mTypeMap = value;
    }
    
    public final TypeMapping getTypeMapping(){
        return this.mTypeMap;
    }

}
