package org.act.soap.client;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.Proxy;
import java.net.URL;
import java.rmi.Remote;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.Port;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.rpc.encoding.TypeMappingRegistry;
import javax.xml.rpc.handler.HandlerRegistry;
import org.act.soap.encoding.TypeMappingRegistryImpl;
import org.act.soap.util.i18n.Message;
import org.xml.sax.InputSource;

public class Service implements javax.xml.rpc.Service {
    
    private QName serviceName = null;
    
    private URL wsdlLocation = null;
    
    private javax.wsdl.Service bindingService = null;
    
    private TypeMappingRegistry typeMapReg = new TypeMappingRegistryImpl();
    
    private HandlerRegistry handlReg = new org.act.soap.client.handler.
        HandlerRegistry();
    
    private Definition def = null;
    
    
    protected Service () {
    }
    
    protected Service(QName servicename) throws ServiceException {
        if ( null == servicename ) {
            throw new ServiceException( Message.
                                       getMessage("ERR_Service_noServiceName"));
        }
        this.serviceName = servicename;
    }
    
    protected Service( URL wsdlDocLocation, QName servicename )
        throws ServiceException{
        if( null == wsdlDocLocation ) {
            throw new ServiceException( Message.getMessage( "ERR_Service_noWSDL" ) );
        }
        if( null == servicename ) {
            throw new ServiceException(  Message.getMessage( "ERR_Service_noServiceName" ) );
        }
        this.serviceName = servicename;
        this.wsdlLocation = wsdlDocLocation;
                InputStream is = null;
        try {
            is = wsdlLocation.openStream();
                        WSDLReader reader = WSDLFactory.newInstance().newWSDLReader();
            this.def = reader.readWSDL(null, new InputSource(is));
                        this.bindingService = this.def.getService( serviceName );
        }catch( WSDLException e ){
            throw new ServiceException( e );
        }catch( IOException e ){
            throw new ServiceException( e );
        }finally{
			            if( null != is ){
                try {
                    is.close();
                }
                catch (IOException ex) {
                    throw new ServiceException(ex);
                }
            }
        }
    }
    
    
    
    public final javax.xml.rpc.Call createCall () throws ServiceException {
        return new Call( this );
    }
    
    public final javax.xml.rpc.Call createCall (QName portName) throws
        ServiceException {
                Call call = (org.act.soap.client.Call) createCall();
                if( portName != null ){
            call.setPortName(portName);
        }
        return call;
    }
    
    public final javax.xml.rpc.Call createCall ( QName portName,
                                          QName operationName ) throws
        ServiceException {
                Call call = (org.act.soap.client.Call) createCall();
        if( portName != null ){
            call.setPortName( portName );
        }
        if( operationName != null ){
            call.setOperationName( operationName );
        }
        return call;
    }
    
    public final javax.xml.rpc.Call createCall ( QName portName,
                                          String operationName ) throws
        ServiceException {
        if ( null == portName || null == operationName ) {
            throw new ServiceException( Message.getMessage( "ERR_Service_nullPortOrOpeName" ) );
        }
                String targetNS = "";
        if ( null != this.def ) {
            targetNS = this.def.getTargetNamespace();
        }
        QName opeName = new QName( targetNS, operationName );
        return createCall( portName, opeName );
    }
    
    public final Remote getPort (QName portName, Class serviceEndpointInterface)
        throws ServiceException {
		Remote ret = null;
        if ( null == portName ) {
            throw new ServiceException( Message.
                                       getMessage("ERR_Service_nullPortName"));
        }
        if ( null == serviceEndpointInterface ) {
            throw new ServiceException( Message.
                                       getMessage(
                "ERR_Service_noServiceInterface"));
        }
                if ( null == bindingService ) {
            throw new ServiceException( Message.
                                       getMessage(
                "ERR_Service_noBindingService"));
        }
        else if ( null == bindingService.getPort( portName.getLocalPart() ) ) {
            throw new ServiceException( Message.
                                       getMessage("ERR_Service_noPort"));
        }
                String serviceEndpointAdd = null;
        Port tmpPort = this.bindingService.getPort( portName.getLocalPart() );
        List portList = tmpPort.getExtensibilityElements();
        if ( null == portList ) {
            throw new ServiceException( Message.getMessage( "ERR_Call_noPortListErr" ) );
        }
        int size = portList.size();
        for ( int i = 0; i < size; i++ ) {
            Object tmpObject = portList.get(i);
            if (tmpObject instanceof SOAPAddress) {
                serviceEndpointAdd = ((SOAPAddress)tmpObject).getLocationURI();
                break;
            }
        }
        ret = getStub( portName, serviceEndpointInterface );
        if (null == ret) {
            ret = getDynamicProxy( serviceEndpointAdd, portName,
                                  serviceEndpointInterface );
        }
        else {
            ( (Stub) ret)._setProperty( Stub.ENDPOINT_ADDRESS_PROPERTY,
                                       serviceEndpointAdd );
        }
        return ret;
    }
    
    public final Remote getPort (Class serviceEndpointInterface) throws
        ServiceException {
        if ( null == serviceEndpointInterface ) {
            throw new ServiceException( Message
                                        .getMessage( "ERR_Service_noServiceInterface") );
        }
        Remote ret = null;
                QName portName = null;
        if ( null != bindingService ) {
            Map ports = bindingService.getPorts();
            if ( 0 != ports.size() ) {
                                String className = null;
                                    int index = -1;
                index = className.lastIndexOf( "." );
                if ( -1 != index ) {
                    className = className.substring( index + 1,
                                                     className.length() );
                }
                                    Port port = (Port) ports.get( className );
                if ( null != port ) {
                    portName = new QName( "", port.getName() );
                }
                else {
                                        portName = new QName( "", ( (Port) ports.keySet()
                                                .iterator().next()).getName() );
                }
            }
        }else {
            throw new ServiceException(  Message.getMessage( "ERR_Service_noBindingService" ) );
        }
        ret = getPort( portName, serviceEndpointInterface );
        return ret;
    }
    
    public final javax.xml.rpc.Call[] getCalls( QName portName )
        throws ServiceException {
        if( null == portName ) {
            throw new ServiceException(  Message.getMessage( "ERR_Service_nullPortName" ) );
        }
        Port port = this.bindingService.getPort( portName.getLocalPart() );
        if ( null == port ) {
            throw new ServiceException(  Message.getMessage( "ERR_Service_noWSDLInfo" ) );
        }
        Binding binding = port.getBinding();
        if( null == binding ) {
            throw new ServiceException(  Message.getMessage( "ERR_Service_noWSDLInfo" ) );
        }
        List operationList = binding.getBindingOperations();
        int size = operationList.size();
        javax.xml.rpc.Call[] calls = new javax.xml.rpc.Call[ size ];
        List tmpVector = new Vector( size );
        for (int i = 0; i < size; i++) {
            Operation oper = (Operation)operationList.get( i );
            javax.xml.rpc.Call call = createCall( portName, oper.getName() );
            tmpVector.add( call );
        }
        tmpVector.toArray( calls );
        return calls;
    }
    
    public final QName getServiceName () {
        return this.serviceName;
    }
    
    public final Iterator getPorts () throws ServiceException {
        if (null != this.bindingService) {
            throw new ServiceException( Message.
                                       getMessage(
                "ERR_Service_noBindingService"));
        }
        Iterator portNames = this.bindingService.getPorts().keySet().iterator();
        if (null == portNames) {return null;
        }
        List ret = new ArrayList( this.bindingService.getPorts().size() );
        for ( ; portNames.hasNext(); ) {
            ret.add(new QName( (String) portNames.next()));
        }
        return ret.iterator();
    }
    
    public final URL getWSDLDocumentLocation () {
        return this.wsdlLocation;
    }
    
    public final TypeMappingRegistry getTypeMappingRegistry () {
        return this.typeMapReg;
    }
    
    public final HandlerRegistry getHandlerRegistry () {
        return this.handlReg;
    }
    
    
    
    public final Definition getDefinition () {
        return this.def;
    }
    
    public void setTypeMapping (String encodingStyleURI, TypeMapping value)
        throws ServiceException {
        if (null == encodingStyleURI) {
            throw new ServiceException( Message.
                                       getMessage("ERR_Service_nullEncStyle"));
        }
        if (null == value) {
            throw new ServiceException( Message.
                                       getMessage("ERR_Service_nullTypeMapping"));
        }
        this.typeMapReg.register(encodingStyleURI, value);
    }
    
    public final javax.wsdl.Service getBindingService () {
        return this.bindingService;
    }
    
    
    
    protected final Remote getDynamicProxy (String serviceEndpointAdd,
                                            QName portName,
                                            Class serviceEndpointInterface)
        throws ServiceException {
        Remote ret = null;
                if (! (serviceEndpointInterface.isInterface())) {
            throw new ServiceException( Message.
                                       getMessage("ERR_Service_invalidClassSet",
                                                  serviceEndpointInterface.
                                                  getName()));
        }
        if (! (Remote.class.isAssignableFrom(serviceEndpointInterface))) {
            throw new ServiceException( Message.
                                       getMessage(
                "ERR_Service_invalidClassExtention",
                serviceEndpointInterface.getName()));
        }
                Call call = null;
        if (null == portName) {
            call = (org.act.soap.client.Call) createCall();
        }
        else {
            call = (org.act.soap.client.Call) createCall(portName);
        }
        if (null != serviceEndpointAdd) {
            call.setTargetEndpointAddress(serviceEndpointAdd);
        }
        ClientProxy handler = new ClientProxy(call);
                ClassLoader loader = Thread.currentThread().getContextClassLoader();
        ret = (Remote) Proxy.newProxyInstance( loader,
                                              new Class[] { javax.xml.rpc.Stub.class,
                                              serviceEndpointInterface },
                                              handler);
        return ret;
    }
    
    protected final Remote getStub (QName portName,
                                    Class serviceEndpointInterface) {
		try {
        	String interfaceName = serviceEndpointInterface.getName();
        	Class stubClass = null;
                                                    int index = -1;
            index = interfaceName.lastIndexOf( "." );
            if ( -1 != index ) {
                interfaceName = interfaceName.substring( index + 1, interfaceName.length() );
            }
                            stubClass = Class.forName( interfaceName + '_' + "Stub" );
            if ( serviceEndpointInterface.isAssignableFrom( stubClass ) ) {
                Class[] formalArgs = { javax.xml.rpc.Service.class };
                Object[] actualArgs = { this };
                Constructor ctor = null;
                ctor = stubClass.getConstructor( formalArgs );
                if ( null == ctor ) {
                    return null;
                }
                Stub stub = null;
                stub = (Stub) ctor.newInstance( actualArgs );
                stub.setPortName( portName );
                return (Remote) stub;
            } else {
                return null;
            }
        }
        catch ( Exception e ) {
            return null;
        }
    }
    
}
