package org.act.soap.client;
import java.util.HashMap;
import java.util.Iterator;
import javax.xml.namespace.QName;
import javax.xml.rpc.JAXRPCException;
import javax.xml.rpc.ServiceException;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPHeaderElement;
import org.act.soap.util.i18n.Message;

public abstract class Stub implements javax.xml.rpc.Stub{
    
    protected javax.xml.rpc.Service service = null;
    
    protected QName portName = null;
    
    protected HashMap properties = new HashMap( 4 );
    
    protected Call call = null;
    
    protected boolean isResponse = false;

    
    public final void _setProperty( String name, Object value ){
                if( null == name ){
            throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropName" ) );
        }else if( null == value ){
            throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValue" ) );
        }else if( name.equals( Stub.USERNAME_PROPERTY ) ) {
                        if( !( value instanceof String ) ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.String.class.getName(), value.getClass().getName() ) );
            }
            this.properties.put( Stub.USERNAME_PROPERTY, value );
        }else if( name.equals( Stub.PASSWORD_PROPERTY ) ) {
                        if( !( value instanceof String ) ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.String.class.getName(), value.getClass().getName() ) );
            }
            this.properties.put( Stub.PASSWORD_PROPERTY, value );
        }else if( name.equals( Stub.ENDPOINT_ADDRESS_PROPERTY ) ) {
                        if( !( value instanceof String ) ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.String.class.getName(), value.getClass().getName() ) );
            }
            this.properties.put( Stub.ENDPOINT_ADDRESS_PROPERTY, value );
        }else if( name.equals( Stub.SESSION_MAINTAIN_PROPERTY ) ) {
                        if( !( value instanceof Boolean ) ) {
                throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropValueType", java.lang.Boolean.class.getName(), value.getClass().getName() ) );
            }
            this.properties.put( Stub.SESSION_MAINTAIN_PROPERTY, value );
        }else{
                        throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_unsupportedProp" ) );
        }
    }
    
    public final Object _getProperty( String name ){
        if( null == name ){
            throw new JAXRPCException( Message.getMessage( "ERR_ClientProxy_invalidPropName" ) );
        }
        return this.properties.get( name );
        
    }
    
    public final Iterator _getPropertyNames(){
        return this.properties.keySet().iterator();
    }


    
    public SOAPHeaderElement[] getResponseHeaders() {
        if( null == call ) {
            return new SOAPHeaderElement[0];
        }else{
			SOAPHeaderElement[] headerEles = call.getResponseHeaders();
        	return headerEles;
        }
    }
    
    public void addRequestHeader( SOAPHeaderElement header )
        throws ServiceException {
        if( null == header ) {
            throw new JAXRPCException( Message.getMessage( "ERR_Stub_nullPoint" , "Header" ) );
        }
        if( null == call ) {
            call = createCall();
        }
                if( this.isResponse ){
            recycle();
            this.isResponse = false;
        }
        call.addRequestHeader( header );
    }
    
    public AttachmentPart[] getResponseAttachments() {
		AttachmentPart[] attachs = null;
        if( null == call ) {
            return new AttachmentPart[0];
        }
        attachs = call.getResponseAttachments();
        return attachs;
    }
    
    public void addRequestAttachmentPart( AttachmentPart attchmentPart )
        throws ServiceException {
        if( null == attchmentPart ) {
            throw new JAXRPCException( Message.getMessage( "ERR_Stub_nullPoint" , "Attachmentpart" ) );
        }
        if( null == call ) {
            call = createCall();
        }
                if( this.isResponse ){
            recycle();
            this.isResponse = false;
        }
        call.addRequestAttachmentPart( attchmentPart );
    }
    
    public final void setPortName( QName name ){
        this.portName = name;
    }
    
    public final QName getPortName(){
        return this.portName;
    }
	
	public final void recycle(){
		if( null == call ) {
			throw new JAXRPCException( Message.getMessage( "ERR_Stub_invalidMethodCall" ) );
		}
		call.recycle();
	}
    
    protected final org.act.soap.client.Call createCall()
        throws javax.xml.rpc.ServiceException{
                if( this.call != null ){
                        if( this.isResponse ){
                this.recycle();
                this.isResponse = false;
            }
                        return this.call;
        }
        		org.act.soap.client.Call _call = null;
        if( null == this.service ){
            javax.xml.rpc.ServiceFactory factory =
                ServiceFactory.newInstance();
            this.service = factory.createService( new QName( "" ) );
        }
        try{
            _call = (org.act.soap.client.Call) service.createCall();
        }catch( java.lang.ClassCastException e ){
            throw new ServiceException( e );
        }
                if( null != _getProperty( Stub.ENDPOINT_ADDRESS_PROPERTY ) ) {
            _call.setTargetEndpointAddress(
                _getProperty( Stub.ENDPOINT_ADDRESS_PROPERTY ).toString() );
        }
        if( null != portName ) {
            _call.setPortTypeName( portName );
        }
        if( 0 != properties.size() ){
            Iterator ite = properties.keySet().iterator();
            int size = properties.size();
            for( int i = 0; i < size; i++ ){
                String name = (String)ite.next();
                if( !name.equals( Stub.ENDPOINT_ADDRESS_PROPERTY ) ){
                    Object value = properties.get( name );
                    _call.setProperty( name, value );
                }
            }
        }
                this.call = _call;
        return _call;
    }

}
