package org.act.soap.client.handler;
import java.security.*;
import java.security.cert.*;
import javax.security.auth.Subject;
import javax.xml.rpc.handler.HandlerInfo;
import javax.xml.rpc.handler.soap.SOAPMessageContext;
import javax.xml.soap.SOAPMessage;
import org.act.soap.authen.LoginPrincipal;
import org.act.soap.client.MessageContext;
import org.act.soap.soapsec.SOAPSecurity;
import org.act.soap.soapsec.Encryption;
import org.act.soap.soapsec.Signature;
import org.act.soap.soapsec.securitytoken.X509CertToken;
import org.act.soap.server.SOAPEngine;
import org.act.soap.soapsec.securitytoken.keyprocessor.SecurityTokenKeyProcessor;
import org.act.soap.soapsec.securitytoken.keyprocessor.DefaultSecurityKeyProcessor;
import org.act.soap.server.secmanager.CredManager;
import org.act.soap.server.secmanager.UserManager;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;
import org.act.soap.util.SOAPUtils;
import org.act.soap.util.logging.Log;
import org.act.soap.soapsec.securitytoken.SecurityToken;
import java.io.FileInputStream;
import java.util.Iterator;
import javax.xml.soap.AttachmentPart;
import java.io.InputStream;
import javax.xml.soap.SOAPFault;

public class SOAPSecHandler extends BasicHandler{
    
    private boolean signRequest = false;
    
    private boolean encRequest = false;
    
    private boolean signFirst = false;
    
    private SOAPSecurity soapsec = null;
    
    private Key privateKey = null;
    private static Log log = LogFactory.getLog( SOAPSecHandler.class.getName() );
    
    public SOAPSecHandler(){}
    
    public void init( HandlerInfo hi )throws Exception{
                Boolean symbolSign = (Boolean)hi.getHandlerConfig()
            .get( "SignRequest" );
        if( null != symbolSign )
            this.signRequest = symbolSign.booleanValue();
        Boolean symbolEnc  = (Boolean)hi.getHandlerConfig()
            .get( "EncryptRequest" );
        if( null != symbolEnc )
            this.encRequest = symbolEnc.booleanValue();
        Boolean symbolSignFirst  = (Boolean)hi.getHandlerConfig()
            .get( "SignFirst" );
        if( null != symbolSignFirst )
            this.signFirst = symbolSignFirst.booleanValue();
        Key configKey = (Key)hi.getHandlerConfig()
            .get( "PrivateKey" );
        if( null != configKey )
            this.privateKey = configKey;
                this.soapsec = (SOAPSecurity)hi.getHandlerConfig().get(
            SOAPSecurity.class.getName() );
        if( true == this.encRequest && true == this.signRequest ){
                        if( true == this.signFirst ){
                this.addSignature();
                this.addEncryption();
            }else{
                this.addEncryption();
                this.addSignature();
            }
            this.encRequest = false;
            this.signRequest = false;
        }
        if( true == this.encRequest ){
                        this.addEncryption();
            this.encRequest = false;
        }
        if( true == this.signRequest ){
                        this.addSignature();
            this.signRequest = false;
        }
    }
    
    public void cleanup(){
    }
    
    public boolean handleRequest( MessageContext msgContext ) throws Exception
    {
        if( soapsec != null ) {
                        SOAPMessage soapMsg = msgContext.getRequestMessage();
                                    Iterator attas = soapMsg.getAttachments();
            soapMsg.removeAllAttachments();
                        String soapMessage = SOAPUtils.soapMessageToString( soapMsg );
            String soapSecMessage = soapsec.getSoapSecMessage( soapMessage );
            SOAPMessage soapSecMsg = SOAPUtils.stringToSOAPMessage( soapSecMessage );
                                    AttachmentPart ap = null;
            while( attas.hasNext() ){
                ap = (AttachmentPart)attas.next();
                soapSecMsg.addAttachmentPart( ap );
            }
                                    msgContext.setRequestMessage( soapSecMsg );
            log.info("-----------------------------------SoapSec Process Result-------------------------------");
            log.info(new String( SOAPUtils.soapMessageToString(soapSecMsg) ) );
            log.info("--------------------------------------------------------------------------------------------");
        }
        return true;
    }
    
    public boolean handleResponse( MessageContext msgContext ) throws Exception
    {
                SOAPMessage soapSecMsg = msgContext.getResponseMessage();
                        Iterator attas = soapSecMsg.getAttachments();
        soapSecMsg.removeAllAttachments();
                String soapSecMessage = SOAPUtils.soapMessageToString( soapSecMsg );
        if( null == soapsec ){
            soapsec = new SOAPSecurity();
        }
        
        SecurityTokenKeyProcessor stkp = new DefaultSecurityKeyProcessor();
        soapsec.setSecurityTokenKeyProcessor( stkp );
                        soapsec.setDecKey( this.privateKey );
        String soapMessage = soapsec.getSoapMessage( soapSecMessage );
        SOAPMessage soapMsg = SOAPUtils.stringToSOAPMessage( soapMessage );
                        AttachmentPart ap = null;
        while( attas.hasNext() ){
            ap = (AttachmentPart)attas.next();
            soapMsg.addAttachmentPart( ap );
        }
                        msgContext.setResponseMessage( soapMsg );
                msgContext.setProperty( SOAPSecurity.class.getName(),soapsec );
        System.out.println("-----------------------------------SoapSec Process Result-------------------------------");
        System.out.println( new String( SOAPUtils.soapMessageToString(soapMsg) ) );
        System.out.println("----------------------------------------------------------------------------------------");
        return true;
    }
    
    public boolean handleFault( MessageContext msgContext ) throws Exception{
        SOAPFault soapFault = msgContext.getResponseMessage().getSOAPBody().getFault();
        System.out.println( soapFault.getFaultCode() + soapFault.getFaultString());
        return true;
    }
    
    private void addEncryption() throws Exception {
        Encryption enc = new Encryption();
                String keystore_location = "./conf/ws.keystore";
        char[] password = "WSkeystore".toCharArray();
        KeyStore keystore = KeyStore.getInstance( "JKS" );
        FileInputStream in = new FileInputStream( keystore_location );
        keystore.load( in, password );
        in.close();
        Key encKeyKey = keystore.getCertificate( "aspserver1" ).getPublicKey();
        enc.setEncKeyKey( encKeyKey );
        this.soapsec.addEncryption( enc );
    }
    
    private void addSignature() throws Exception {
        Signature sign = new Signature();
        SecurityToken secToken = new SecurityToken();
        secToken.setSignKey( this.privateKey );
        secToken.setSignKeyName( "mykey" );
        sign.setSecurityToken( secToken );
        sign.setSignatureMethod( "http://www.w3.org/2000/09/xmldsig#rsa-sha1" );
                String keystore_location = "./conf/ws.keystore";
        char[] password = "WSkeystore".toCharArray();
        KeyStore keystore = KeyStore.getInstance( "JKS" );
        FileInputStream in = new FileInputStream( keystore_location );
        keystore.load( in, password );
        in.close();
        Key verifyKey = keystore.getCertificate( "aspserver1" ).getPublicKey();
        secToken.setVerifyKey( verifyKey );
        this.soapsec.addSignature( sign );
    }
}
