package org.act.soap.encoding;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.xml.namespace.QName;
import javax.xml.rpc.JAXRPCException;
import javax.xml.rpc.encoding.DeserializerFactory;
import javax.xml.rpc.encoding.SerializerFactory;
import javax.xml.rpc.encoding.TypeMapping;
import org.act.soap.encoding.ser.ArrayDeserializerFactory;
import org.act.soap.encoding.ser.ArraySerializerFactory;
import org.act.soap.encoding.ser.Base64DeserializerFactory;
import org.act.soap.encoding.ser.Base64SerializerFactory;
import org.act.soap.encoding.ser.CalendarDeserializerFactory;
import org.act.soap.encoding.ser.CalendarSerializerFactory;
import org.act.soap.encoding.ser.DataHandlerDeserializerFactory;
import org.act.soap.encoding.ser.DataHandlerSerializerFactory;
import org.act.soap.encoding.ser.DateDeserializerFactory;
import org.act.soap.encoding.ser.DateSerializerFactory;
import org.act.soap.encoding.ser.HexDeserializerFactory;
import org.act.soap.encoding.ser.HexSerializerFactory;
import org.act.soap.encoding.ser.MapDeserializerFactory;
import org.act.soap.encoding.ser.MapSerializerFactory;
import org.act.soap.encoding.ser.QNameDeserializerFactory;
import org.act.soap.encoding.ser.QNameSerializerFactory;
import org.act.soap.encoding.ser.SimpleDeserializerFactory;
import org.act.soap.encoding.ser.SimpleSerializerFactory;
import org.act.soap.encoding.ser.TimeDeserializerFactory;
import org.act.soap.encoding.ser.TimeSerializerFactory;
import org.act.soap.encoding.ser.VectorDeserializerFactory;
import org.act.soap.encoding.ser.VectorSerializerFactory;
import org.act.soap.util.TMConstants;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;

public class TypeMappingImpl
    implements TypeMapping {
    
    private static Log log = LogFactory.getLog(TypeMappingImpl.class.getName());
    
    private class Pair {
        
        private Class javaType = null;
        
        private QName xmlType = null;
        
        private Pair(Class javaType, QName xmlType) {
            this.javaType = javaType;
            this.xmlType = xmlType;
        }
        
        public boolean equals(Object o) {
            if (null == o) {
                return false;
            }
            if (! (o instanceof Pair)) {
                return false;
            }
            Pair p = (Pair) o;
            if ( (p.xmlType == this.xmlType) &&
                (p.javaType == this.javaType)) {
                return true;
            }
            return p.xmlType.equals(this.xmlType) &&
                p.javaType.equals(this.javaType);
        }
        
        public int hashCode() {
            int hashcode = 0;
            if (null != javaType) {
                hashcode ^= javaType.hashCode();
            }
            if (null != xmlType) {
                hashcode ^= xmlType.hashCode();
            }
            return hashcode;
        }
    }
    
    private HashMap xmlType2Pair = null;
    
    private HashMap javaType2Pair = null;
    
    private HashMap pair2SF = null;
    
    private HashMap pair2DF = null;
    
    private List encodingStyleURIs = null;
    
    public TypeMappingImpl() {
                initParameter();
                registerDefaultTypes();
    }
    
    
    public String[] getSupportedEncodings() {
        String[] stringArray = new String[encodingStyleURIs.size()];
        return (String[]) encodingStyleURIs.toArray(stringArray);
    }
    
    public void setSupportedEncodings(String[] namespaceURIs) {
                encodingStyleURIs.clear();
                for (int i = 0; i < namespaceURIs.length; i++) {
            if (!encodingStyleURIs.contains(namespaceURIs[i])) {
                encodingStyleURIs.add(namespaceURIs[i]);
            }
        }
    }
    
    public boolean isRegistered(Class javaType, QName xmlType) {
                if ( (null == javaType) || (null == xmlType)) {
            return false;
        }
                if (javaType.isArray()) {
            return true;
        }
        Pair pair = new Pair(javaType, xmlType);
        if (pair2SF.keySet().contains(pair)) {
            return true;
        }
        else {
            return pair2DF.keySet().contains(pair);
        }
    }
    
    public void register(Class javaType, QName xmlType,
                         SerializerFactory sf,
                         DeserializerFactory dsf) {
                if ( (null == sf) && (null == dsf)) {
            throw new JAXRPCException(Message.getMessage("ERR_TM_badSerFac"));
        }
                internalRegister(javaType, xmlType, sf, dsf);
    }
    
    public SerializerFactory getSerializer(Class javaType, QName xmlType) {
                if ( (null == javaType) || (null == xmlType)) {
            return null;
        }
                Pair pair = new Pair(javaType, xmlType);
                SerializerFactory sf = (SerializerFactory) pair2SF.get(pair);
                        if (null == sf) {
                        if (javaType.isArray()) {
                pair = (Pair) xmlType2Pair.get(TMConstants.SOAP_ARRAY);
                sf = (SerializerFactory) pair2SF.get(pair);
            }
        }
                return sf;
    }
    
    public DeserializerFactory getDeserializer(Class javaType, QName xmlType) {
                if ( (null == javaType) || (null == xmlType)) {
            return null;
        }
                Pair pair = new Pair(javaType, xmlType);
                DeserializerFactory df = (DeserializerFactory) pair2DF.get(pair);
                        if (null == df) {
                        if (javaType.isArray()) {
                pair = (Pair) xmlType2Pair.get(TMConstants.SOAP_ARRAY);
                df = (DeserializerFactory) pair2DF.get(pair);
            }
        }
                return df;
    }
    
    public void removeSerializer(Class javaType, QName xmlType) {
                if ( (null == javaType) || (null == xmlType)) {
            throw new JAXRPCException(
                Message.getMessage(null == javaType ?
                                   "ERR_TM_badJavaType" : "ERR_TM_badXmlType"));
        }
        Pair pair = new Pair(javaType, xmlType);
                pair2SF.remove(pair);
        javaType2Pair.remove(javaType);
    }
    
    public void removeDeserializer(Class javaType, QName xmlType) {
                if ( (null == javaType) || (null == xmlType)) {
            throw new JAXRPCException(
                Message.getMessage(null == javaType ?
                                   "ERR_TM_badJavaType" : "ERR_TM_badXmlType"));
        }
        Pair pair = new Pair(javaType, xmlType);
                pair2DF.remove(pair);
        xmlType2Pair.remove(xmlType);
    }
    
    
    
    public Class getJavaType(QName xmlType) {
                if (null == xmlType) {
            return null;
        }
                Class javaType = null;
        Pair pair = (Pair) xmlType2Pair.get(xmlType);
        if (null != pair) {
            javaType = pair.javaType;
        }
        return javaType;
    }
    
    public QName getXmlType(Class javaType) {
                if (null == javaType) {
            return null;
        }
                QName xmlType = null;
        Pair pair = (Pair) javaType2Pair.get(javaType);
        if (null != pair) {
            xmlType = pair.xmlType;
        }
                if ( (null == xmlType) && (javaType.isArray())) {
                        pair = (Pair) javaType2Pair.get(Object[].class);
            if (null != pair) {
                xmlType = pair.xmlType;
            }
            else {
                xmlType = TMConstants.SOAP_ARRAY;
            }
        }
                return xmlType;
    }
    
    
    
    private void internalRegister(Class javaType, QName xmlType,
                                  SerializerFactory sf,
                                  DeserializerFactory
                                  dsf) {
                if ( (null == javaType) || (null == xmlType)) {
            throw new JAXRPCException(
                Message.getMessage(null == javaType ?
                                   "ERR_TM_badJavaType" : "ERR_TM_badXmlType"));
        }
                Pair pair = new Pair(javaType, xmlType);
                if ( (null != dsf) || (null == xmlType2Pair.get(xmlType))) {
            xmlType2Pair.put(xmlType, pair);
        }
        if ( (null != sf) || (null == javaType2Pair.get(javaType))) {
            javaType2Pair.put(javaType, pair);
        }
                if (null != sf) {
            pair2SF.put(pair, sf);
        }
                if (null != dsf) {
            pair2DF.put(pair, dsf);
        }
    }
    
    private void initParameter() {
        xmlType2Pair = new HashMap();
        javaType2Pair = new HashMap();
        pair2SF = new HashMap();
        pair2DF = new HashMap();
        encodingStyleURIs = new ArrayList();
                encodingStyleURIs.add(TMConstants.DEFAULT_ENCODINGSTYLE);
    }
    
    private void registerDefaultTypes() {
                registerSOAPTypes();
                        myRegister(TMConstants.XSD_HEXBIN, byte[].class,
                   new HexSerializerFactory(
            byte[].class, TMConstants.XSD_HEXBIN),
                   new HexDeserializerFactory(
            byte[].class, TMConstants.XSD_HEXBIN));
        myRegister(TMConstants.XSD_BASE64, byte[].class,
                   new Base64SerializerFactory(byte[].class,
                                               TMConstants.XSD_BASE64),
                   new Base64DeserializerFactory(byte[].class,
                                                 TMConstants.XSD_BASE64));
        myRegisterSimple(TMConstants.XSD_STRING, java.lang.Character.class);
                myRegisterSimple(TMConstants.XSD_YEARMONTH, java.lang.String.class);
        myRegisterSimple(TMConstants.XSD_YEAR, java.lang.String.class);
        myRegisterSimple(TMConstants.XSD_MONTH, java.lang.String.class);
        myRegisterSimple(TMConstants.XSD_DAY, java.lang.String.class);
        myRegisterSimple(TMConstants.XSD_MONTHDAY, java.lang.String.class);
        myRegisterSimple(TMConstants.XSD_YEARMONTH, java.lang.String.class);
        myRegisterSimple(TMConstants.XSD_YEARMONTH, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_UNSIGNEDLONG, java.math.BigInteger.class);
                myRegisterSimple(TMConstants.XSD_POSITIVEINTEGER,
                         java.math.BigInteger.class);
                myRegisterSimple(TMConstants.XSD_NONPOSITIVEINTEGER,
                         java.math.BigInteger.class);
                myRegisterSimple(TMConstants.XSD_NEGATIVEINTEGER,
                         java.math.BigInteger.class);
                myRegisterSimple(TMConstants.XSD_NONNEGATIVEINTEGER,
                         java.math.BigInteger.class);
                myRegisterSimple(TMConstants.XSD_UNSIGNEDBYTE, java.lang.Short.class);
                myRegisterSimple(TMConstants.XSD_UNSIGNEDSHORT, java.lang.Integer.class);
                myRegisterSimple(TMConstants.XSD_UNSIGNEDINT, java.lang.Long.class);
                myRegister(TMConstants.XSD_QNAME,
                   javax.xml.namespace.QName.class,
                   new QNameSerializerFactory(javax.xml.namespace.QName.class,
                                              TMConstants.XSD_QNAME),
                   new QNameDeserializerFactory(javax.xml.namespace.QName.class,
                                                TMConstants.XSD_QNAME)
                   );
        
        myRegister(TMConstants.XSD_ANYTYPE, java.lang.Object.class, null, null);
                myRegister(TMConstants.XSD_TIME, java.util.Calendar.class,
                   new TimeSerializerFactory(java.util.Calendar.class,
                                             TMConstants.XSD_TIME),
                   new TimeDeserializerFactory(java.util.Calendar.class,
                                               TMConstants.XSD_TIME)
                   );
        myRegister(TMConstants.XSD_DATE, java.util.Calendar.class,
                   new DateSerializerFactory(java.util.Calendar.class,
                                             TMConstants.XSD_DATE),
                   new DateDeserializerFactory(java.util.Calendar.class,
                                               TMConstants.XSD_DATE)
                   );
        myRegister(TMConstants.XSD_DATETIME, java.util.Date.class,
                   new CalendarSerializerFactory(java.util.Date.class,
                                                 TMConstants.XSD_DATETIME),
                   new CalendarDeserializerFactory(java.util.Date.class,
            TMConstants.XSD_DATETIME)
                   );
        myRegister(TMConstants.XSD_DATETIME, java.util.GregorianCalendar.class,
                   new CalendarSerializerFactory(java.util.GregorianCalendar.class,
                                                 TMConstants.XSD_DATETIME),
                   new CalendarDeserializerFactory(java.util.GregorianCalendar.class,
            TMConstants.XSD_DATETIME)
                   );
        myRegister(TMConstants.XSD_DATETIME, java.util.Calendar.class,
                   new CalendarSerializerFactory(java.util.Calendar.class,
                                                 TMConstants.XSD_DATETIME),
                   new CalendarDeserializerFactory(java.util.Calendar.class,
            TMConstants.XSD_DATETIME)
                   );
                myRegisterSimple(TMConstants.XSD_ANYURI, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_NORMALIZEDSTRING, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_TOKEN, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_NAME, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_NCNAME, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_LANGUAGE, java.lang.String.class);
                myRegisterSimple(TMConstants.XML_LANG, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_NMTOKEN, java.lang.String.class);
                myRegisterSimple(TMConstants.XSD_DURATION, java.lang.String.class);
        myRegister(TMConstants.QNAME_TYPE_LIST, java.util.List.class,
                   new ArraySerializerFactory(java.util.List.class,
                                              TMConstants.SOAP_ARRAY),
                   new ArrayDeserializerFactory(java.util.List.class,
                                                TMConstants.SOAP_ARRAY)
                   );
        myRegister(TMConstants.QNAME_TYPE_LIST, java.util.ArrayList.class,
                   new ArraySerializerFactory(java.util.ArrayList.class,
                                              TMConstants.SOAP_ARRAY),
                   new ArrayDeserializerFactory(java.util.ArrayList.class,
                                                TMConstants.SOAP_ARRAY)
                   );
        myRegister(TMConstants.QNAME_TYPE_COLLECTION, java.util.Collection.class,
                   new ArraySerializerFactory(java.util.Collection.class,
                                              TMConstants.SOAP_ARRAY),
                   new ArrayDeserializerFactory(java.util.Collection.class,
                                                TMConstants.SOAP_ARRAY)
                   );
        myRegister(TMConstants.SOAP_ARRAY, Object[].class,
                   new ArraySerializerFactory(Object[].class,
                                              TMConstants.SOAP_ARRAY),
                   new ArrayDeserializerFactory(Object[].class,
                                                TMConstants.SOAP_ARRAY)
                   );
        myRegister(TMConstants.SOAP_VECTOR, java.util.Vector.class,
                   new VectorSerializerFactory(java.util.Vector.class,
                                               TMConstants.SOAP_VECTOR),
                   new VectorDeserializerFactory(java.util.Vector.class,
                                                 TMConstants.SOAP_VECTOR)
                   );
                myRegister(TMConstants.SOAP_MAP, java.util.Hashtable.class,
                   new MapSerializerFactory(java.util.Hashtable.class,
                                            TMConstants.SOAP_MAP),
                   new MapDeserializerFactory(java.util.Hashtable.class,
                                              TMConstants.SOAP_MAP)
                   );
        myRegister(TMConstants.SOAP_MAP, java.util.Map.class,
                   new MapSerializerFactory(java.util.Map.class,
                                            TMConstants.SOAP_MAP),
                   new MapDeserializerFactory(java.util.Map.class,
                                              TMConstants.SOAP_MAP)
                   );
        myRegister(TMConstants.SOAP_MAP, java.util.HashMap.class,
                   new MapSerializerFactory(java.util.Map.class,
                                            TMConstants.SOAP_MAP),
                   new MapDeserializerFactory(java.util.HashMap.class,
                                              TMConstants.SOAP_MAP)
                   );
        myRegisterSimple(TMConstants.XSD_BOOLEAN, java.lang.Boolean.class);
        myRegisterSimple(TMConstants.XSD_DOUBLE, java.lang.Double.class);
        myRegisterSimple(TMConstants.XSD_FLOAT, java.lang.Float.class);
        myRegisterSimple(TMConstants.XSD_INTEGER, java.math.BigInteger.class);
        myRegisterSimple(TMConstants.XSD_DECIMAL, java.math.BigDecimal.class
                         );
        myRegisterSimple(TMConstants.XSD_LONG, java.lang.Long.class);
        myRegisterSimple(TMConstants.XSD_SHORT, java.lang.Short.class);
        myRegisterSimple(TMConstants.XSD_BYTE, java.lang.Byte.class);
        myRegisterSimple(TMConstants.XSD_INT, java.lang.Integer.class);
                myRegisterSimple(TMConstants.XSD_STRING, char.class);
        myRegisterSimple(TMConstants.XSD_BOOLEAN, boolean.class);
        myRegisterSimple(TMConstants.XSD_DOUBLE, double.class);
        myRegisterSimple(TMConstants.XSD_FLOAT, float.class);
        myRegisterSimple(TMConstants.XSD_INT, int.class);
        myRegisterSimple(TMConstants.XSD_LONG, long.class);
        myRegisterSimple(TMConstants.XSD_SHORT, short.class);
        myRegisterSimple(TMConstants.XSD_BYTE, byte.class);
        myRegisterSimple(TMConstants.XSD_STRING, java.lang.String.class);
        myRegister(TMConstants.QNAME_TYPE_DATAHANDLER, javax.activation.DataHandler.class ,
                   new DataHandlerSerializerFactory(javax.activation.DataHandler.class,
                                              TMConstants.QNAME_TYPE_DATAHANDLER),
                   new DataHandlerDeserializerFactory(javax.activation.DataHandler.class,
                                                TMConstants.QNAME_TYPE_DATAHANDLER)
                   );
    }
    
    private void registerSOAPTypes() {
        myRegisterSimple(TMConstants.SOAP_STRING, java.lang.String.class);
        myRegisterSimple(TMConstants.SOAP_BOOLEAN, java.lang.Boolean.class);
        myRegisterSimple(TMConstants.SOAP_DOUBLE, java.lang.Double.class);
        myRegisterSimple(TMConstants.SOAP_FLOAT, java.lang.Float.class);
        myRegisterSimple(TMConstants.SOAP_INT, java.lang.Integer.class);
        myRegisterSimple(TMConstants.SOAP_INTEGER, java.math.BigInteger.class);
        myRegisterSimple(TMConstants.SOAP_DECIMAL, java.math.BigDecimal.class);
        myRegisterSimple(TMConstants.SOAP_LONG, java.lang.Long.class);
        myRegisterSimple(TMConstants.SOAP_SHORT, java.lang.Short.class);
        myRegisterSimple(TMConstants.SOAP_BYTE, java.lang.Byte.class);
        myRegister(TMConstants.SOAP_BASE64, byte[].class,
                   new Base64SerializerFactory(byte[].class,
                                               TMConstants.SOAP_BASE64),
                   new Base64DeserializerFactory(byte[].class,
                                                 TMConstants.SOAP_BASE64)
                   );
        myRegister(TMConstants.SOAP_BASE64BINARY, byte[].class,
                   new Base64SerializerFactory(byte[].class,
                                               TMConstants.SOAP_BASE64),
                   new Base64DeserializerFactory(byte[].class,
                                                 TMConstants.SOAP_BASE64)
                   );
    }
    
    private void myRegisterSimple(QName xmlType, Class javaType) {
        SerializerFactory sf = new SimpleSerializerFactory(javaType, xmlType);
        DeserializerFactory df = new SimpleDeserializerFactory(javaType,
            xmlType);
        myRegister(xmlType, javaType, sf, df);
    }
    
    private void myRegister(QName xmlType, Class javaType,
                            SerializerFactory sf, DeserializerFactory df) {
        try {
            internalRegister(javaType, xmlType, sf, df);
        }
        catch (JAXRPCException e) {
                        log.error(e);
        }
    }
}
