package org.act.soap.encoding;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import javax.xml.rpc.JAXRPCException;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.rpc.encoding.TypeMappingRegistry;
import org.act.soap.util.i18n.Message;

public class TypeMappingRegistryImpl
    implements TypeMappingRegistry {
    
    private HashMap mappings;
    
    private TypeMapping defaultMapping;
    
    public TypeMappingRegistryImpl () {
                mappings = new HashMap();
                defaultMapping = new TypeMappingImpl();
    }
    
    public void clear () {
        mappings.clear();
    }
    
    public String[] getRegisteredEncodingStyleURIs () {
        Set namespaceSet = mappings.keySet();
        return (String[]) namespaceSet.toArray(new String[namespaceSet.size()]);
    }
    
    public TypeMapping createTypeMapping () {
        return new TypeMappingImpl();
    }
    
    public TypeMapping getDefaultTypeMapping () {
        return defaultMapping;
    }
    
    public void registerDefault (TypeMapping mapping) {
        defaultMapping = mapping;
    }
    
    public boolean removeTypeMapping (TypeMapping mapping) {
                if (null == mapping) {
            return false;
        }
                        boolean result = false;
        Iterator eachEntry = mappings.entrySet().iterator();
        while (eachEntry.hasNext()) {
            java.util.Map.Entry currentEntry = (java.util.Map.Entry) eachEntry.
                next();
            if (mapping.equals(currentEntry.getValue())) {
                eachEntry.remove();
                result = true;
            }
        }
                return result;
    }
    
    public TypeMapping getTypeMapping (String namespaceURI) {
        if (null == namespaceURI) {
            throw new JAXRPCException(Message.getMessage(
                "ERR_TM_mappingIsNull"));
        }
        TypeMapping mapping = (TypeMapping) mappings.get(namespaceURI);
                        if (null == mapping) {
            mapping = defaultMapping;
        }
        return mapping;
    }
    
    public TypeMapping unregisterTypeMapping (String namespaceURI) {
        return (TypeMapping) mappings.remove(namespaceURI);
    }
    
    public TypeMapping register (String namespaceURI, TypeMapping mapping) {
        if ( (null == mapping) || (null == namespaceURI)) {
            throw new JAXRPCException(Message.getMessage(
                "ERR_TM_mappingOrURIIsNull"));
        }
                        if (!mappingSupportsEncoding(mapping, namespaceURI)) {
            throw new JAXRPCException(Message.getMessage(
                "ERR_TM_mappingDoesNotSupportEncoding",
                namespaceURI));
        }
        else {
                        TypeMapping oldMapping = (TypeMapping) mappings.get(namespaceURI);
                        mappings.put(namespaceURI, mapping);
                        return oldMapping;
        }
    }
    
    private static boolean mappingSupportsEncoding (TypeMapping mapping,
        String namespaceURI) {
        String[] encodings = mapping.getSupportedEncodings();
        if (null == encodings) {
          return false;
        }
        for (int i = 0; i < encodings.length; i++) {
            if (encodings[i].equals(namespaceURI)) {
                return true;
            }
        }
        return false;
    }
}
