package org.act.soap.encoding.ser;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.DeserializerFactory;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import org.act.soap.encoding.DeserializationContextImpl;
import org.act.soap.encoding.DeserializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.util.JavaUtils;
import org.act.soap.util.TMConstants;
import org.act.soap.util.XMLUtils;
import org.act.soap.util.i18n.Message;

public class ArrayDeserializer
    extends DeserializerImpl {
    
    public ArrayDeserializer() {
    }
    
    public Object deserialize(SOAPElement element, QName xmlType,
                              Class javaType,
                              DeserializationContextImpl context) throws
        TypeMappingException {
                if (super.isNil(element)) {
            return null;
        }
                if (javaType.isArray()) {
                        return this.deserializeArray(element, xmlType, javaType, context);
        }
        else {
                        return this.deserializeNonArray(element, xmlType, javaType, context);
        }
    }
    
    private Object deserializeNonArray(SOAPElement element, QName xmlType,
                                       Class javaType,
                                       DeserializationContextImpl context) throws
        TypeMappingException {
                Name nameSoapencArrayType = TMConstants.NAME_SOAPENCARRAY_TYPE;
                        String arrayTypeValueLocalPart = element.getAttributeValue(
            nameSoapencArrayType);
        final int leftBracketIndex = arrayTypeValueLocalPart.lastIndexOf('[');
        final int rightBracketIndex = arrayTypeValueLocalPart.lastIndexOf(']');
        if ( ( -1 == leftBracketIndex)
            || ( -1 == rightBracketIndex)
            || (rightBracketIndex < leftBracketIndex)) {
            throw new TypeMappingException(
                Message.getMessage("ERR_TM_badArrayType"));
        }
                String dimString = arrayTypeValueLocalPart.substring(leftBracketIndex +
            1,
            rightBracketIndex);
        final int arraySize = Integer.parseInt(dimString);
                List list = new ArrayList(arraySize);
                TypeMapping tm = context.getTypeMappingRegistry().getDefaultTypeMapping();
                        Object rtnObject = null;
        Iterator iteratorItem = element.getChildElements();
        while (iteratorItem.hasNext()) {
                        Object childObject = iteratorItem.next();
            if (! (childObject instanceof SOAPElement)) {
                continue;
            }
            SOAPElement soapChildElementItem = (SOAPElement) childObject;
            if (super.isNil(soapChildElementItem)) {
                list.add(null);
                continue;
            }
                        QName innerQName = super.getXsitypeAttr(soapChildElementItem);
                        Class clsInner = context.getJavaType(innerQName);
                        DeserializerImpl di = (DeserializerImpl) tm.getDeserializer(
                clsInner,
                innerQName).
                getDeserializerAs(TMConstants.ENCODING_MECHANISMTYPE);
            rtnObject = di.deserialize(soapChildElementItem, innerQName,
                                       clsInner,
                                       context);
                        list.add(rtnObject);
        }
                return JavaUtils.convert(list, javaType);
    }
    
    private Object deserializeArray(SOAPElement element, QName xmlType,
                                    Class javaType,
                                    DeserializationContextImpl context) throws
        TypeMappingException {
                        String arrayTypeValueLocalPart = element.getAttributeValue(
            TMConstants.NAME_SOAPENCARRAY_TYPE);
        final int leftBracketIndex = arrayTypeValueLocalPart.lastIndexOf('[');
        final int rightBracketIndex = arrayTypeValueLocalPart.lastIndexOf(']');
        if ( ( -1 == leftBracketIndex)
            || ( -1 == rightBracketIndex)
            || (rightBracketIndex < leftBracketIndex)) {
            throw new TypeMappingException(
                Message.getMessage("ERR_TM_badArrayType"));
        }
                String dimString = arrayTypeValueLocalPart.substring(leftBracketIndex +
            1,
            rightBracketIndex);
        int arraySize = 0;
        try {
            arraySize = Integer.parseInt(dimString);
        }
        catch (NumberFormatException ex) {
            throw new TypeMappingException(ex);
        }
                int dimlen = 0;         StringBuffer dims = new StringBuffer();         int pos = arrayTypeValueLocalPart.indexOf('[');
        while ( -1 < pos) {
            dimlen++;
            dims.append("[]");
            int pos1 = arrayTypeValueLocalPart.indexOf('[', pos + 1);
            pos = pos1;
        }
                Class partArrayClass = javaType.getComponentType();
        String strClassName = null;
        Class tempArrayClass = partArrayClass;
        while (tempArrayClass.isArray()) {
            tempArrayClass = tempArrayClass.getComponentType();
        }
        if (tempArrayClass == java.lang.Object.class) {
                                                                                    String strQNameType = arrayTypeValueLocalPart.substring(0,
                arrayTypeValueLocalPart.indexOf("["));
                        String strPrefix = XMLUtils.getPrefixFromQName(strQNameType);
                        String strLocalName = XMLUtils.getLocalPartFromQName(strQNameType);
                        QName compXmlType = null;
            if (null != strPrefix) {
                String strURI = element.getNamespaceURI(strPrefix);
                compXmlType = new QName(strURI, strLocalName, strPrefix);
            }
            else {
                compXmlType = new QName(strLocalName);
            }
                        partArrayClass = context.getJavaType(compXmlType);
                                    strClassName = JavaUtils.getTextClassName(partArrayClass.getName());
            strClassName += dims.toString();
        }
        else {
            strClassName = JavaUtils.getTextClassName(partArrayClass.getName());
            strClassName += "[]";
        }
                Class arrayClass = null;
        try {
                                                String loadableClassName
                = JavaUtils.getLoadableClassName(strClassName);
            ClassLoader classLoader
                = Thread.currentThread().getContextClassLoader();
            arrayClass = classLoader.loadClass(loadableClassName);
                    }
        catch (Exception ex) {
            throw new TypeMappingException(
                Message.getMessage("ERR_TM_noComponent",
                                   "" + strClassName), ex);
        }
        if (partArrayClass == java.lang.Object.class) {
                        return this.deseriaizeObjectArray(element, context, arraySize,
                                              arrayClass);
        }
                List list = new ArrayList(arraySize);
                TypeMapping tm = context.getTypeMappingRegistry().getDefaultTypeMapping();
                QName innerQName = null;
        if (dimlen > 1) {
            innerQName = TMConstants.SOAP_ARRAY;
        }
        else {
            innerQName = context.getXmlType(partArrayClass);
        }
                DeserializerFactory df =
            tm.getDeserializer(arrayClass.getComponentType(), innerQName);
        if (df == null) {
            throw new TypeMappingException(
                Message.getMessage("ERR_TM_missingDeserializer",
                                   innerQName.toString(),
                                   arrayClass.getComponentType().getName()));
        }
        DeserializerImpl di =
            (DeserializerImpl) df.getDeserializerAs(TMConstants.
            ENCODING_MECHANISMTYPE);
                Iterator iteratorItem = element.getChildElements();
        while (iteratorItem.hasNext()) {
                        Object childObject = iteratorItem.next();
            if (! (childObject instanceof SOAPElement)) {
                continue;
            }
            SOAPElement soapChildElementItem = (SOAPElement) childObject;
            if (super.isNil(soapChildElementItem)) {
                list.add(null);
                continue;
            }
                        Object rtnObject = di.deserialize(soapChildElementItem,
                                              innerQName,
                                              arrayClass.getComponentType(),
                                              context);
                        list.add(rtnObject);
        }
                return JavaUtils.convert(list, arrayClass);
    }
    
    private Object deseriaizeObjectArray(SOAPElement element,
                                         DeserializationContextImpl context,
                                         int arraySize, Class arrayClass) throws
        TypeMappingException {
                List list = new ArrayList(arraySize);
                TypeMapping tm = context.getTypeMappingRegistry().getDefaultTypeMapping();
                Iterator iteratorItem = element.getChildElements();
        while (iteratorItem.hasNext()) {
                        Object childObject = iteratorItem.next();
            if (! (childObject instanceof SOAPElement)) {
                continue;
            }
            SOAPElement soapChildElementItem = (SOAPElement) childObject;
            if (super.isNil(soapChildElementItem)) {
                list.add(null);
                continue;
            }
                        QName innerQName = super.getXsitypeAttr(soapChildElementItem);
                        Class clsInner = context.getJavaType(innerQName);
                        DeserializerFactory df =
                tm.getDeserializer(clsInner, innerQName);
            if (df == null) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_missingDeserializer",
                                       innerQName.toString(),
                                       clsInner.getName()));
            }
            DeserializerImpl di =
                (DeserializerImpl) df.getDeserializerAs(TMConstants.
                ENCODING_MECHANISMTYPE);
                        Object rtnObject = di.deserialize(soapChildElementItem,
                                              innerQName,
                                              clsInner,
                                              context);
                        list.add(rtnObject);
        }
                return JavaUtils.convert(list, arrayClass);
    }
}
