package org.act.soap.encoding.ser;
import java.lang.reflect.Array;
import java.util.Collection;
import java.util.Iterator;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPFactory;
import org.act.soap.encoding.SerializationContextImpl;
import org.act.soap.encoding.SerializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.util.TMConstants;
import org.act.soap.util.XMLUtils;
import org.act.soap.util.i18n.Message;
import org.act.soap.wsdl.java2wsdl.Types;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ArraySerializer
    extends SerializerImpl {
    
    public ArraySerializer () {
    }
    
    public SOAPElement serialize (Object value, QName valueName, Class javaType,
                                  QName xmlType,
                                  SerializationContextImpl context) throws
        TypeMappingException {
        try {
                        SOAPFactory sef = TMConstants.SOAPFACTORY_INSTANCE;
            SOAPElement soapElement = sef.createElement(valueName.getLocalPart());
                        super.addXsitypeAttr(soapElement,xmlType);
                        if (null == value) {
                super.addNilAttr(soapElement);
                return soapElement;
            }
                        Class cls = value.getClass();
            if (cls.isArray()) {
                                return mySerializeArray(value, valueName, xmlType, context,
                                        soapElement);
            }
            else if (value instanceof Collection) {
                                return mySerializeNonArray(value, valueName, xmlType, context,
                                           soapElement);
            }
            else {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_cantSerialize", javaType.getName()));
            }
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    private static SOAPElement mySerializeArray (Object value, QName valueName,
                                                 QName xmlType,
                                                 SerializationContextImpl
                                                 context,
                                                 SOAPElement soapElement) throws
        TypeMappingException {
        try {
                        Class cls = value.getClass();
                        Class componentType = cls.getComponentType();
                                    String dims = "";
            while (componentType.isArray()) {
                componentType = componentType.getComponentType();
                dims += "[]";
            }
                        QName componentQName = context.getXmlType(componentType);
            
                        if (null == componentQName) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_noType", componentType.getName()));
            }
                                    String prefix = componentQName.getPrefix();
            String compType = "";
            if ( (null == prefix) || "".equals(prefix)) {
                compType = componentQName.getLocalPart();
            }
            else {
                compType = prefix + ':' + componentQName.getLocalPart();
                
                                if(!prefix.equals(TMConstants.URI_DEFAULT_SCHEMA_XSD_PREFIX)){
                    soapElement.addNamespaceDeclaration(prefix,componentQName.getNamespaceURI());
                }
            }
            final int len = Array.getLength(value);
            String arrayType = dims + '[' + len + ']';
            soapElement.addAttribute(TMConstants.NAME_SOAPENCARRAY_TYPE,
                                     compType + arrayType);
                        TypeMapping tm = context.getTypeMappingRegistry().
                getDefaultTypeMapping();
            QName elementName = new QName(TMConstants.LITERAL_ITEM);
                        SerializerImpl si = null;
            QName innerQName = null;
            for (int index = 0; index < len; index++) {
                SOAPElement childElement = null;
                                Object aValue = Array.get(value, index);
                                if (null == aValue) {
                    childElement = soapElement.addChildElement(TMConstants.LITERAL_ITEM);
                    SerializerImpl.addNilAttr(childElement);
                    continue;
                }
                                Class aClass = aValue.getClass();
                if (aClass.isArray()) {
                    innerQName = TMConstants.SOAP_ARRAY;
                }
                else {
                    if(componentType==java.lang.Object.class){
                        innerQName=context.getXmlType(aClass);
                        si = (SerializerImpl) tm.getSerializer(aClass,
                        innerQName).
                        getSerializerAs(TMConstants.ENCODING_MECHANISMTYPE);
                    }
                    else
                    {
                        innerQName = componentQName;
                    }
                }
                                                if (null == si) {
                    si = (SerializerImpl) tm.getSerializer(aClass,
                        innerQName).
                        getSerializerAs(TMConstants.ENCODING_MECHANISMTYPE);
                }
                                childElement = si.serialize(aValue, elementName, aClass,
                                            innerQName,
                                            context);
                soapElement.addChildElement(childElement);
            }
                        return soapElement;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    private static SOAPElement mySerializeNonArray (Object value,
        QName valueName,
        QName xmlType,
        SerializationContextImpl context,
        SOAPElement soapElement) throws
        TypeMappingException {
        try {
                        Collection list = (Collection) value;
                                    String prefix = TMConstants.URI_DEFAULT_SCHEMA_XSD_PREFIX;
            String compType = prefix + ':' + "object";
            final int len = list.size();
            String arrayType = null;
            arrayType = "[" + len + ']';
            Name nameSoapencArrayType = TMConstants.NAME_SOAPENCARRAY_TYPE;
            soapElement.addAttribute(nameSoapencArrayType,
                                     compType + arrayType);
                        TypeMapping tm = context.getTypeMappingRegistry().
                getDefaultTypeMapping();
            QName elementName = new QName(TMConstants.LITERAL_ITEM);
                        for (Iterator iterator = list.iterator(); iterator.hasNext(); ) {
                                Object aValue = iterator.next();
                SOAPElement childElement = null;
                                if (null == aValue) {
                    childElement = soapElement.addChildElement(TMConstants.LITERAL_ITEM);
                    SerializerImpl.addNilAttr(childElement);
                    continue;
                }
                                Class aClass = aValue.getClass();
                                                QName aXmlType = context.getXmlType(aClass);
                                SerializerImpl si = (SerializerImpl) tm.getSerializer(aClass,
                    aXmlType).
                    getSerializerAs(TMConstants.ENCODING_MECHANISMTYPE);
                                childElement = si.serialize(aValue, elementName, aClass,
                                            aXmlType, context);
                soapElement.addChildElement(childElement);
            }
                        return soapElement;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    public Element writeSchema (Class javaType, QName xmlType, Types types) throws
        TypeMappingException {
        try {
                        Document doc = XMLUtils.getDOMBuilder().newDocument();
                        String componentTypeName = null;
            if (javaType.isArray()) {
                Class componentType = null;
                String dimString = "[]";
                componentType = javaType.getComponentType();
                while (componentType.isArray()) {
                    dimString += "[]";
                    componentType = componentType.getComponentType();
                }
                                                QName componentTypeQName = types.getTypeQName(componentType);
                componentTypeName = types.getQNameString(componentTypeQName);
                                componentTypeName += dimString;
            }
                        Element complexType = doc.createElement("complexType");
                                                complexType.setAttribute("name", xmlType.getLocalPart());
                        Element complexContent = doc.createElement("complexContent");
            complexType.appendChild(complexContent);
            Element restriction = doc.createElement("restriction");
            complexContent.appendChild(restriction);
            restriction.setAttribute("base",
                                     TMConstants.URI_DEFAULT_SOAP_ENC_PREFIX +
                                     ":Array");
            Element attribute = doc.createElement("attribute");
            restriction.appendChild(attribute);
            attribute.setAttribute("ref",
                                   TMConstants.URI_DEFAULT_SOAP_ENC_PREFIX +
                                   ":arrayType");
            attribute.setAttribute(TMConstants.NS_PREFIX_WSDL + ":arrayType",
                                   componentTypeName);
            attribute.setAttribute("xmlns:" + TMConstants.NS_PREFIX_WSDL,
                                   TMConstants.NS_URI_WSDL);
                                                                        return complexType;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
}
