package org.act.soap.encoding.ser;
import java.lang.reflect.Modifier;
import java.util.List;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPFactory;
import org.act.soap.encoding.SerializationContextImpl;
import org.act.soap.encoding.SerializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.encoding.ser.bean.BeanPropertyDescriptor;
import org.act.soap.encoding.ser.bean.BeanUtils;
import org.act.soap.util.TMConstants;
import org.act.soap.util.XMLUtils;
import org.act.soap.wsdl.java2wsdl.Types;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.act.soap.wsdl.java2wsdl.TypesException;

public class BeanSerializer
    extends SerializerImpl {
    
    public BeanSerializer () {
    }
    
    public SOAPElement serialize (Object value, QName valueName, Class javaType,
                                  QName xmlType,
                                  SerializationContextImpl context) throws
        TypeMappingException {
        BeanPropertyDescriptor[] propertyDescriptor = BeanUtils.getPd(javaType);
        try {
                        SOAPFactory sef = TMConstants.SOAPFACTORY_INSTANCE;
            SOAPElement soapElement = sef.createElement(valueName.getLocalPart());
                        super.addXsitypeAttr(soapElement,xmlType);
                        if (null == value) {
                super.addNilAttr(soapElement);
                return soapElement;
            }
                        TypeMapping tm = context.getTypeMappingRegistry().
                getDefaultTypeMapping();
            for (int i = 0; i < propertyDescriptor.length; i++) {
                                String propName = propertyDescriptor[i].getName();
                if ("class".equals(propName)) {
                    continue;
                }
                if (propertyDescriptor[i].isReadable()) {
                    if (!propertyDescriptor[i].isIndexed()) {
                                                Object propValue = propertyDescriptor[i].get(value);
                                                Class aClass = propertyDescriptor[i].getType();
                                                QName innerXmlType = context.getXmlType(aClass);
                        SerializerImpl si = (SerializerImpl) tm.getSerializer(
                            aClass,
                            innerXmlType).
                            getSerializerAs(TMConstants.ENCODING_MECHANISMTYPE);
                                                QName childElementName = new QName(xmlType.getNamespaceURI(),propName,xmlType.getPrefix());
                                                SOAPElement childElement = si.serialize(propValue,
                            childElementName,
                            aClass,
                            innerXmlType,
                            context);
                        soapElement.addChildElement(childElement);
                    }
                }
            }
                        return soapElement;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    public Element writeSchema (Class javaType, QName xmlType, Types types) throws
        TypeMappingException {
        try {
                        Document doc = XMLUtils.getDOMBuilder().newDocument();
                        BeanPropertyDescriptor[] propertyDescriptor = BeanUtils.getPd(javaType);
                        Element complexType = doc.createElement("complexType");
            complexType.setAttribute("name", xmlType.getLocalPart());
                        Element e = null;
            Class superClass = javaType.getSuperclass();
            BeanPropertyDescriptor[] superPd = null;
            List stopClasses = null;
            if ( (null != superClass) &&
                (superClass != java.lang.Object.class) &&
                (superClass != java.lang.Exception.class) &&
                (superClass != java.lang.Throwable.class) &&
                (superClass != java.rmi.RemoteException.class) &&
                (null == stopClasses)) {
                                                                QName baseQName = types.getTypeQName(superClass);
                String base = types.getQNameString(baseQName);
                                Element complexContent = doc.createElement("complexContent");
                complexType.appendChild(complexContent);
                Element extension = doc.createElement("extension");
                complexContent.appendChild(extension);
                extension.setAttribute("base", base);
                e = extension;
                superPd = BeanUtils.getPd(superClass);
            }
            else {
                e = complexType;
            }
                        Element all = doc.createElement("sequence");
            e.appendChild(all);
            if (Modifier.isAbstract(javaType.getModifiers())) {
                complexType.setAttribute("abstract", "true");
            }
                        for (int i = 0; i < propertyDescriptor.length; i++) {
                String propName = propertyDescriptor[i].getName();
                                boolean writeProperty = true;
                if ("class".equals(propName)) {
                    writeProperty = false;
                }
                                if ( (null != superPd) && writeProperty) {
                    for (int j = 0; (j < superPd.length) && writeProperty; j++) {
                        if (propName.equals(superPd[j].getName())) {
                            writeProperty = false;
                        }
                    }
                }
                if (!writeProperty) {
                    continue;
                }
                                Class fieldType = propertyDescriptor[i].getType();
                writeField(types,
                           propName,
                           fieldType,
                           propertyDescriptor[i].isIndexed(), false, all);
            }
                        types.putSchemaElement(xmlType, complexType);
                        return complexType;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    private void writeField (Types types, String fieldName,
                             Class fieldType,
                             boolean isUnbounded,
                             boolean isOmittable,
                             Element where) throws TypeMappingException, TypesException {
        Element elem = null;
                        QName elementTypeQName = types.getTypeQName(fieldType);
        String elementType = types.getQNameString(elementTypeQName);
                if (null == elementType) {
            QName anyQN = TMConstants.XSD_ANYTYPE;
            types.putNamespace(anyQN.getPrefix(), anyQN.getNamespaceURI());
            String prefix = anyQN.getPrefix();
            elementType = prefix + ':' + anyQN.getLocalPart();
        }
        elem = createElement(fieldName,
                             elementType,
                                                                                       Types.isNullable(fieldType),
                                                          isOmittable,
                             where.getOwnerDocument());
        if (isUnbounded) {
            elem.setAttribute("maxOccurs", "unbounded");
        }
        where.appendChild(elem);
    }
    
    private static Element createElement (String elementName,
                                          String elementType,
                                          boolean nullable,
                                          boolean omittable,
                                          Document docHolder) {
        Element element = docHolder.createElement("element");
        element.setAttribute("name", elementName);
        if (nullable) {
            element.setAttribute("nillable", "true");
        }
        if (omittable) {
            element.setAttribute("minOccurs", "0");
            element.setAttribute("maxOccurs", "1");
        }
        if (null != elementType) {
            element.setAttribute("type", elementType);
        }
        return element;
    }
}
