package org.act.soap.encoding.ser;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import org.act.soap.encoding.DeserializationContextImpl;
import org.act.soap.encoding.DeserializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.util.i18n.Message;

public class CalendarDeserializer
    extends DeserializerImpl {
    
    public CalendarDeserializer () {
    }
    
    private static final SimpleDateFormat ZULU =
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
    static {
        ZULU.setTimeZone(TimeZone.getTimeZone("GMT"));
    }
    
    public Object deserialize (SOAPElement element, QName xmlType,
                               Class javaType,
                               DeserializationContextImpl context) throws
        TypeMappingException {
                if (super.isNil(element)) {
            return null;
        }
                String strNodeValue = element.getValue();
                return makeValue(strNodeValue, javaType);
    }
    
    private static Object makeValue (String source, Class javaType) throws
        TypeMappingException {
        Calendar calendar = Calendar.getInstance();
        Date date = null;
        boolean bc = false;
                        if (null != source) {
            if ('+' == source.charAt(0)) {
                source = source.substring(1);
            }
            if ('-' == source.charAt(0)) {
                source = source.substring(1);
                bc = true;
            }
                        if (19 > source.length()) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_badTime"));
            }
            if ( ('-' != source.charAt(4)) || ('-' != source.charAt(7)) ||
                ('T' != source.charAt(10))) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_badDate"));
            }
            if ( (':' != source.charAt(13)) || (':' != source.charAt(16))) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_badTime"));
            }
        }
                try {
            synchronized (ZULU) {
                date = ZULU.parse(null == source ? null :
                                  (source.substring(0, 19) + ".000Z"));
            }
        }
        catch (Exception e) {
            throw new TypeMappingException(e);
        }
                if (null != source) {
            int pos = 19;
                        if ( (pos < source.length()) && ('.' == source.charAt(pos))) {
                int milliseconds = 0;
                int start = ++pos;
                while ( (pos < source.length()) &&
                       Character.isDigit(source.charAt(pos))) {
                    pos++;
                }
                String decimal = source.substring(start, pos);
                if (3 == decimal.length()) {
                    milliseconds = Integer.parseInt(decimal);
                }
                else if (3 > decimal.length()) {
                    milliseconds = Integer.parseInt( (decimal + "000")
                        .substring(0, 3));
                }
                else {
                    milliseconds = Integer.parseInt(decimal.substring(0, 3));
                    if ('5' <= decimal.charAt(3)) {
                        ++milliseconds;
                    }
                }
                                date.setTime(date.getTime() + milliseconds);
            }
                                                                                    if ( (pos + 5 < source.length()) &&
                ( ('+' == source.charAt(pos)) || ('-' == source.charAt(pos)))) {
                if (!Character.isDigit(source.charAt(pos + 1)) ||
                    !Character.isDigit(source.charAt(pos + 2)) ||
                    (':' != source.charAt(pos + 3)) ||
                    !Character.isDigit(source.charAt(pos + 4)) ||
                    !Character.isDigit(source.charAt(pos + 5))) {
                    throw new TypeMappingException(
                        Message.getMessage("ERR_TM_badTimezone"));
                }
                int hours = (source.charAt(pos + 1) - '0') * 10
                    + source.charAt(pos + 2) - '0';
                int mins = (source.charAt(pos + 4) - '0') * 10
                    + source.charAt(pos + 5) - '0';
                int milliseconds = (hours * 60 + mins) * 60 * 1000;
                                if ('+' == source.charAt(pos)) {
                    milliseconds = -milliseconds;
                }
                date.setTime(date.getTime() + milliseconds);
                pos += 6;
            }
            if ( (pos < source.length()) && ('Z' == source.charAt(pos))) {
                pos++;
                calendar.setTimeZone(TimeZone.getTimeZone("GMT"));
            }
            if (pos < source.length()) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_badChars"));
            }
        }
        calendar.setTime(date);
                if (bc) {
            calendar.set(Calendar.ERA, GregorianCalendar.BC);
        }
                if (javaType == Date.class) {
            return date;
        }
        else {
            return calendar;
        }
    }
}
