package org.act.soap.encoding.ser;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPFactory;
import org.act.soap.encoding.SerializationContextImpl;
import org.act.soap.encoding.SerializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.util.JavaUtils;
import org.act.soap.util.TMConstants;
import org.act.soap.util.XMLUtils;
import org.act.soap.wsdl.java2wsdl.Types;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class EnumSerializer
    extends SerializerImpl {
    
    public EnumSerializer () {
    }
    
    public SOAPElement serialize (Object value, QName valueName, Class javaType,
                                  QName xmlType,
                                  SerializationContextImpl context) throws
        TypeMappingException {
        try {
                        SOAPFactory sef = TMConstants.SOAPFACTORY_INSTANCE;
            SOAPElement soapElement = sef.createElement(valueName.getLocalPart());
                        super.addXsitypeAttr(soapElement,xmlType);
                                    if (null == value) {
                super.addNilAttr(soapElement);
                return soapElement;
            }
                        soapElement.addTextNode(getValueAsString(value, javaType));
            return soapElement;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    private static String getValueAsString (Object value, Class javaType) throws
        TypeMappingException {
        try {
            Method toStringMethod = javaType.getMethod("toString", null);
            return (String) toStringMethod.invoke(value, null);
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    public Element writeSchema (Class javaType, QName xmlType, Types types) throws
        TypeMappingException {
        try {
                        Document doc = XMLUtils.getDOMBuilder().newDocument();
                        if (!JavaUtils.isEnumClass(javaType)) {
                return null;
            }
                                    java.lang.reflect.Method m = javaType.getMethod("getValue", null);
            Class base = m.getReturnType();
                        Element simpleType = doc.createElement("simpleType");
            simpleType.setAttribute("name", xmlType.getLocalPart());
            Element restriction = doc.createElement("restriction");
            simpleType.appendChild(restriction);
                                                QName baseTypeQName = types.getTypeQName(base);
            String baseType = types.getQNameString(baseTypeQName);
                        restriction.setAttribute("base", baseType);
                        Field[] fields = javaType.getDeclaredFields();
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
                final int mod = field.getModifiers();
                                if (Modifier.isPublic(mod) &&
                    Modifier.isStatic(mod) &&
                    Modifier.isFinal(mod) &&
                    (field.getType() == base)) {
                                        Element enumeration = doc.createElement("enumeration");
                    enumeration.setAttribute("value", field.get(null).toString());
                    restriction.appendChild(enumeration);
                }
            }
                        types.putSchemaElement(xmlType, simpleType);
                        return simpleType;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
}
