package org.act.soap.encoding.ser;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPFactory;
import org.act.soap.encoding.SerializationContextImpl;
import org.act.soap.encoding.SerializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.util.TMConstants;
import org.act.soap.util.XMLUtils;
import org.act.soap.util.i18n.Message;
import org.act.soap.wsdl.java2wsdl.Types;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class MapSerializer
    extends SerializerImpl {
    
    public MapSerializer () {
    }
    
    public SOAPElement serialize (Object value, QName valueName, Class javaType,
                                  QName xmlType,
                                  SerializationContextImpl context) throws
        TypeMappingException {
        try {
                        SOAPFactory sef = TMConstants.SOAPFACTORY_INSTANCE;
            SOAPElement soapElement = sef.createElement(valueName.getLocalPart());
                        super.addXsitypeAttr(soapElement,xmlType);
                                    if (null == value) {
                super.addNilAttr(soapElement);
                return soapElement;
            }
                        if (! (value instanceof Map)) {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_noMap",
                                       value.getClass().getName()));
            }
                        Map map = (Map) value;
            for (Iterator iterator = map.entrySet().iterator();
                 iterator.hasNext(); ) {
                Map.Entry entry = (Map.Entry) iterator.next();
                Object key = entry.getKey();
                Object val = entry.getValue();
                SOAPElement itemSoapElement =soapElement.addChildElement(TMConstants.
                    QNAME_MAP_ITEM.getLocalPart());
                                itemSoapElement.addChildElement(mySerialize(key,
                    TMConstants.QNAME_MAP_KEY, null, context));
                                itemSoapElement.addChildElement(mySerialize(val,
                    TMConstants.QNAME_MAP_VALUE, null, context));
            }
                        return soapElement;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
    
    private static SOAPElement mySerialize (Object value, QName valueName,
                                     QName xmlType,
                                     SerializationContextImpl context) throws
        TypeMappingException {
        SOAPElement childElement = null;
                try {
            if (null == value) {
                childElement = TMConstants.SOAPFACTORY_INSTANCE.createElement(
                    valueName.
                    getLocalPart());
                SerializerImpl.addNilAttr(childElement);
                return childElement;
            }
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
                TypeMapping tm = context.getTypeMappingRegistry().
            getDefaultTypeMapping();
                Class aClass = value.getClass();
                QName aXmlType = context.getXmlType(aClass);
                SerializerImpl si = (SerializerImpl) tm.getSerializer(aClass, aXmlType).
            getSerializerAs(TMConstants.ENCODING_MECHANISMTYPE);
                childElement = si.serialize(value, valueName, aClass,
                                    aXmlType, context);
        return childElement;
    }
    
    public Element writeSchema (Class javaType, QName xmlType, Types types) throws
        TypeMappingException {
        try {
                        Document doc = XMLUtils.getDOMBuilder().newDocument();
                        Element complexType = doc.createElement(TMConstants.
                SCHEMA_COMPLEXTYPE);
            complexType.setAttribute(TMConstants.SCHEMA_NAME, "Map");
            Element seq = doc.createElement(TMConstants.SCHEMA_SEQUENCE);
            complexType.appendChild(seq);
            Element element = doc.createElement(TMConstants.SCHEMA_ELEMENT);
            element.setAttribute(TMConstants.SCHEMA_NAME,
                                 TMConstants.SCHEMA_ITEM);
            element.setAttribute(TMConstants.SCHEMA_MINOCCURS,
                                 TMConstants.SCHEMA_0);
            element.setAttribute(TMConstants.SCHEMA_MAXOCCURS,
                                 TMConstants.SCHEMA_UNBOUNDED);
            element.setAttribute(TMConstants.SCHEMA_TYPE, "mapItem");
            seq.appendChild(element);
            Element itemType = doc.createElement(TMConstants.SCHEMA_COMPLEXTYPE);
            itemType.setAttribute(TMConstants.SCHEMA_NAME, "mapItem");
            Element seq2 = doc.createElement(TMConstants.SCHEMA_SEQUENCE);
            itemType.appendChild(seq2);
            Element element2 = doc.createElement(TMConstants.SCHEMA_ELEMENT);
            element2.setAttribute(TMConstants.SCHEMA_NAME, "key");
            element2.setAttribute(TMConstants.SCHEMA_NILLABLE, "true");
            element2.setAttribute(TMConstants.SCHEMA_TYPE,
                                  TMConstants.SCHEMA_ANYTYPE);
            seq2.appendChild(element2);
            Element element3 = doc.createElement(TMConstants.SCHEMA_ELEMENT);
            element3.setAttribute(TMConstants.SCHEMA_NAME, "value");
            element3.setAttribute(TMConstants.SCHEMA_NILLABLE, "true");
            element3.setAttribute(TMConstants.SCHEMA_TYPE,
                                  TMConstants.SCHEMA_ANYTYPE);
            seq2.appendChild(element3);
                                    QName qname = new QName(xmlType.getNamespaceURI(), "mapItem");
            types.putSchemaElement(qname, itemType);
                        return complexType;
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
}
