package org.act.soap.encoding.ser;
import java.lang.reflect.Constructor;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import org.act.soap.encoding.DeserializationContextImpl;
import org.act.soap.encoding.DeserializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.util.JavaUtils;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.TMConstants;

public class SimpleDeserializer
    extends DeserializerImpl {
    
    public SimpleDeserializer() {
    }
    
    public Object deserialize(SOAPElement element, QName xmlType,
                              Class javaType,
                              DeserializationContextImpl context) throws
        TypeMappingException {
                        if (super.isNil(element)) {
            if (javaType.isPrimitive()) {
                throw new TypeMappingException(Message.getMessage(
                    "ERR_TM_badPrimitiveNull"));
            }
            else {
                return null;
            }
        }
                String strNodeValue = element.getValue();
                return makeValue(strNodeValue, javaType, xmlType);
    }
    
    private static Object makeValue(String source, Class javaType,
                                    QName xmlType) throws
        TypeMappingException {
        if (source == null) {
            if (xmlType.equals(TMConstants.XSD_STRING)) {
                source = "";
            }
            else {
                throw new TypeMappingException(Message.getMessage(
                    "ERR_TM_simpleTypeIsNull"));
            }
        }
                if ( (javaType == boolean.class) || (javaType == Boolean.class)) {
            if (source.equals("0") || source.equalsIgnoreCase("false")) {
                return Boolean.FALSE;
            }
            else if (source.equals("1") || source.equalsIgnoreCase("true")) {
                return Boolean.TRUE;
            }
            else {
                throw new TypeMappingException(
                    Message.getMessage("ERR_TM_badBool"));
            }
        }
                if ( (javaType == float.class) || (javaType == java.lang.Float.class)) {
            if ("NaN".equals(source)) {
                return new Float(Float.NaN);
            }
            else if ("INF".equals(source)) {
                return new Float(Float.POSITIVE_INFINITY);
            }
            else if ("-INF".equals(source)) {
                return new Float(Float.NEGATIVE_INFINITY);
            }
        }
                if ( (javaType == double.class) || (javaType == java.lang.Double.class)) {
            if ("NaN".equals(source)) {
                return new Double(Double.NaN);
            }
            else if ("INF".equals(source)) {
                return new Double(Double.POSITIVE_INFINITY);
            }
            else if ("-INF".equals(source)) {
                return new Double(Double.NEGATIVE_INFINITY);
            }
        }
                if ( (javaType == char.class) || (javaType == java.lang.Character.class)) {
            return new Character(source.charAt(0));
        }
                try {
            Constructor constructor = null;
            if (!javaType.isPrimitive()) {
                constructor =
                    javaType.getDeclaredConstructor(new Class[] {String.class});
            }
            else {
                Class wrapper = JavaUtils.getWrapperClass(javaType);
                if (null != wrapper) {
                    constructor =
                        wrapper.getDeclaredConstructor(new Class[] {String.class});
                }
            }
            return constructor.newInstance(new Object[] {source});
        }
        catch (Exception ex) {
            throw new TypeMappingException(ex);
        }
    }
}
