package org.act.soap.encoding.ser.bean;
import java.beans.IndexedPropertyDescriptor;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import org.act.soap.util.i18n.Message;

public class BeanPropertyDescriptor {
    
    protected PropertyDescriptor myPD = null;
    
    public BeanPropertyDescriptor (PropertyDescriptor pd) {
        myPD = pd;
    }
    
    protected BeanPropertyDescriptor () {
    }
    
    public String getName () {
        return myPD.getName();
    }
    
    public boolean isReadable () {
        return null != myPD.getReadMethod();
    }
    
    public boolean isWriteable () {
        return null != myPD.getWriteMethod();
    }
    
    public boolean isIndexed () {
        return myPD instanceof IndexedPropertyDescriptor;
    }
    
    public Object get (Object obj) throws InvocationTargetException,
        IllegalAccessException {
        if (null != myPD.getReadMethod()) {
            return myPD.getReadMethod().invoke(obj, new Object[] {});
        }
        throw new IllegalAccessException(Message.getMessage("ERR_TM_badGetter"));
    }
    
    public Object get (Object obj, int i) throws InvocationTargetException,
        IllegalAccessException {
        if (!isIndexed()) {
            return Array.get(get(obj), i);
        }
        else {
            IndexedPropertyDescriptor id = (IndexedPropertyDescriptor) myPD;
            return id.getIndexedReadMethod().invoke(obj,
                new Object[] {
                new Integer(i)});
        }
    }
    
    public void set (Object obj, Object newValue) throws
        InvocationTargetException,
        IllegalAccessException {
        if (null != myPD.getWriteMethod()) {
            myPD.getWriteMethod().invoke(obj, new Object[] {newValue});
        }
        else {
            throw new IllegalAccessException(Message.getMessage(
                "ERR_TM_badSetter"));
        }
    }
    
    public void set (Object obj, int i, Object newValue) throws
        InvocationTargetException, IllegalAccessException {
        if (isIndexed()) {
            IndexedPropertyDescriptor id = (IndexedPropertyDescriptor) myPD;
            growArrayToSize(obj, id.getIndexedPropertyType(), i);
            id.getIndexedWriteMethod().invoke(obj,
                                              new Object[] {
                                              new Integer(i), newValue});
        }
        else {
            Array.set(get(obj), i, newValue);
        }
    }
    
    public Class getType () {
        if (isIndexed()) {
            return ( (IndexedPropertyDescriptor) myPD).getIndexedPropertyType();
        }
        else {
            return myPD.getPropertyType();
        }
    }
    
    protected void growArrayToSize (Object obj, Class componentType, int i) throws
        InvocationTargetException, IllegalAccessException {
        Object array = get(obj);
        if ( (null == array) || (Array.getLength(array) <= i)) {
            Object newArray = Array.newInstance(componentType, i + 1);
            if (null != array) {
                System.arraycopy(array, 0, newArray, 0, Array.getLength(array));
            }
            set(obj, newArray);
        }
    }
}
