package org.act.soap.message;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import javax.activation.CommandMap;
import javax.activation.DataHandler;
import javax.activation.MailcapCommandMap;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MimeHeader;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPException;
import org.act.soap.util.i18n.Message;

final class AttachmentPartImpl extends AttachmentPart {
    
    static {
        CommandMap map = CommandMap.getDefaultCommandMap();
        if (map instanceof MailcapCommandMap) {
            MailcapCommandMap mailMap = (MailcapCommandMap) map;
            String hndlrStr = ";;x-java-content-handler=";
            mailMap.addMailcap("text/xml" +
                                hndlrStr +
                                "org.act.soap.message.dch.XmlDCH");
            mailMap.addMailcap("application/soap+xml" +
                                hndlrStr +
                                "org.act.soap.message.dch.XmlDCH");
                        mailMap.addMailcap("text/plain" +
                                hndlrStr +
                                "org.act.soap.message.dch.PlainTextDCH");
        }
        else {
            throw new IllegalArgumentException(
                        Message.getMessage("SAAJ_E_ATTACH_INVALID_COMMANDMAP"));
        }
    }
    
    private MimeHeaders headers;
    
    private DataHandler dataHandler;
    
    AttachmentPartImpl () {
        headers = new MimeHeaders();
        dataHandler = null;
    }
    
    
    public void removeAllMimeHeaders () {
        headers.removeAllHeaders();
    }
    
    public void removeMimeHeader (String header) {
        headers.removeHeader(header);
    }
    
    public String[] getMimeHeader (String name) {
        return headers.getHeader(name);
    }
    
    public Iterator getMatchingMimeHeaders (String[] names) {
        return headers.getMatchingHeaders(names);
    }
    
    public void setMimeHeader (String name, String value) {
        headers.setHeader(name, value);
    }
    
    public Iterator getAllMimeHeaders () {
        return headers.getAllHeaders();
    }
    
    public Iterator getNonMatchingMimeHeaders (String[] names) {
        return headers.getNonMatchingHeaders(names);
    }
    
    public void addMimeHeader (String name, String value) {
        headers.addHeader(name, value);
    }
    
    public DataHandler getDataHandler () throws SOAPException {
        if (null == dataHandler) {
            throw new SOAPException(
                        Message.getMessage("SAAJ_E_ATTACH_NO_CONTENT"));
        } else {
            return dataHandler;
        }
    }
    
    public void setContent (Object object, String contentType) {
        dataHandler = new DataHandler(object, contentType);
    }
    
    public void setDataHandler (DataHandler dh) {
        if (null == dh) {
            throw new IllegalArgumentException(
                        Message.getMessage("SAAJ_E_ATTACH_NULL_DH"));
        }
        dataHandler = dh;
    }
    
    public Object getContent () throws SOAPException {
        if (null == dataHandler) {
            throw new SOAPException(
                        Message.getMessage("SAAJ_E_ATTACH_NO_CONTENT"));
        }
        try {
            return dataHandler.getContent();
        } catch (java.io.IOException ioe) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_ATTACH_CANNOT_GETCONTENT"), ioe);
        }
    }
    
    public int getSize () throws SOAPException {
        if (null == dataHandler) {
            return 0;
        }
        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        try {
            dataHandler.writeTo(bout);
        } catch (IOException ioe) {
            throw new SOAPException("SAAJ_E_IOE", ioe);
        }
        return bout.size();
    }
    
    public void clearContent () {
        dataHandler = null;
    }
    
    
    
    boolean hasHeaders (MimeHeaders hdrs) {
        for (Iterator iter = hdrs.getAllHeaders(); iter.hasNext(); ) {
            MimeHeader hdr = (MimeHeader) iter.next();
            String name = hdr.getName();
            String value = hdr.getValue();
            String[] values = this.headers.getHeader(name);
            boolean found = false;
            if (null != values) {
                for (int j = 0; j < values.length; j++) {
                    if (!value.equalsIgnoreCase(values[j])) {
                        continue;
                    }
                    found = true;
                    break;
                }
            }
            if (!found) {
                return false;
            }
        }
        return true;
    }
    
    MimeBodyPart getMimePart () throws SOAPException{
        MimeBodyPart mbp = new MimeBodyPart();
        try {
            mbp.setDataHandler(dataHandler);
            Utils.copyMimeHeaders(headers, mbp);
        } catch (MessagingException me) {
            throw new SOAPException(
            Message.getMessage("SAAJ_E_ATTACH_CANNOT_CONSTRUCT_MIMEBODYPART"),
            me);
        }
        return mbp;
    }
    
}
