package org.act.soap.message;
import java.util.Locale;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFault;
import org.w3c.dom.Document;
import org.act.soap.util.i18n.Message;

public abstract class BodyImpl extends ElementImpl implements SOAPBody {
    
    protected BodyImpl (SOAPDocumentImpl ownerDoc, Name bodyName) {
        super(ownerDoc, bodyName);
    }
    
    
    public final SOAPFault addFault () throws SOAPException {
        
        if (hasFault()) {  
            throw new SOAPException(
                        Message.getMessage("SAAJ_E_BODY_FAULT_EXIST"));
        }
        else {  
            return (SOAPFault) appendChild(createFault());
        }
    }
    
    
    public final SOAPFault addFault (Name faultCode, 
                                     String faultString, 
                                     Locale locale)
        throws SOAPException {
        SOAPFault fault = addFault();
        fault.setFaultCode(faultCode);
        fault.setFaultString(faultString, locale);
        return fault;
    }
    
    
    public final SOAPFault addFault (Name faultCode, String faultString)
        throws SOAPException {
        SOAPFault fault = addFault();
        fault.setFaultCode(faultCode);
        fault.setFaultString(faultString);
        return fault;
    }
    
    public final boolean hasFault () {
        SOAPFault fault = (SOAPFault) _getFirstChildElement(getFaultName());
        return null != fault;
    }
    
    public final SOAPFault getFault () {
        return (SOAPFault) _getFirstChildElement(getFaultName());
    }
    
    public final SOAPBodyElement addBodyElement (Name name) 
        throws SOAPException {
        SOAPBodyElement bodyEle = createBodyElement(name);
        appendChild(bodyEle);
        return bodyEle;
    }
    
    public final SOAPBodyElement addDocument (Document document) 
        throws SOAPException {
        SOAPBodyElement e = (SOAPBodyElement) getOwnerDocument().importNode(document.getDocumentElement(), true);
        appendChild(e);
        return e;
    }
    
    
    
    protected abstract SOAPFault createFault();
    
    
    protected abstract SOAPBodyElement createBodyElement(Name name);
    
    
    protected abstract Name getFaultName();
}
