package org.act.soap.message;
import java.util.HashSet;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Set;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.xerces.dom.ElementNSImpl;
import org.act.soap.util.i18n.Message;

public class ElementImpl extends ElementNSImpl
                         implements SOAPElement, SOAPBodyElement {
            
    public ElementImpl (SOAPDocumentImpl ownerDoc, Name name) {
        super(ownerDoc, name.getURI(),
                        name.getQualifiedName(),
                        name.getLocalName());
    }
            
    public final SOAPElement addChildElement (Name name) throws SOAPException {
        return _addElement(name.getURI(), name.getQualifiedName());
    }
    
    public final SOAPElement addChildElement (String localName)
        throws SOAPException {
        return _addElement(null, localName);
    }
    
    public final SOAPElement addChildElement (String localName, String prefix)
        throws SOAPException {
        
        String uri = getNamespaceURI(prefix);
        if ( null == uri ) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_ELE_UNABLE_LOCATE_NS", prefix));
        } else {
            return addChildElement(localName, prefix, uri);
        }
    }
    
    public final SOAPElement addChildElement (String localName,
                                              String prefix,
                                              String uri)
        throws SOAPException {
        return _addElement(uri, prefix+ ':' + localName);
    }
    
    public final SOAPElement addChildElement (SOAPElement element)
        throws SOAPException {
        
        
        Document document = getOwnerDocument();
        Document oldDocument = element.getOwnerDocument();
        if (oldDocument != document) {
            return (SOAPElement)appendChild(document.importNode(element, true));
        } else {
            return (SOAPElement) appendChild(element);
        }
    }
    
    public final SOAPElement addTextNode (String text) throws SOAPException {
        
        appendChild( getOwnerDocument().createTextNode(text) );
        return this;
    }
    
    public final SOAPElement addAttribute (Name name, String value)
        throws SOAPException {
        String uri = name.getURI();
        String qName = name.getQualifiedName();
        setAttributeNS(uri, qName, value);
        return this;
    }
    
    public final SOAPElement addNamespaceDeclaration (String prefix, String uri)
        throws SOAPException {
        if ( (null != prefix) && (0 < prefix.length()) ) {
            setAttributeNS(Constants.XMLNS_URI, "xmlns:" + prefix, uri);
        } else {
            setAttributeNS(Constants.XMLNS_URI, "xmlns", uri);
        }
        return this;
    }
    
    public final String getAttributeValue (Name name) {
        String uri = name.getURI();
        String localName = name.getLocalName();
        return getAttributeNS(uri, localName);
    }
    
    public final Iterator getAllAttributes () {
        return new Iterator () {
            
            Iterator eachAttr = _getAttrNodes(false);
            public void remove () {
                eachAttr.remove();
            }
            public boolean hasNext () {
                return eachAttr.hasNext();
            }
            public Object next () {
                Attr attr = (Attr) eachAttr.next();
                return new NameImpl(attr.getName(),
                                    attr.getNamespaceURI());
            }
        };
    }
    
    public final String getNamespaceURI (String prefix) {
        if ( Constants.NS_PREFIX.equals(prefix) ) {
            return Constants.XMLNS_URI;
        }
        
        String attrQName = (null==prefix || 0==prefix.length())
            ? Constants.NS_PREFIX : Constants.NS_PREFIX + ':' + prefix;
        
                Attr attr = getAttributeNode(attrQName);
        if ( null != attr )
            return attr.getNodeValue();
        
        SOAPElement parent = getParentElement();
        if ( null != parent ) {
            return parent.getNamespaceURI(prefix);
        }
        
        return null;
    }
    
    public final Iterator getNamespacePrefixes () {
        return new Iterator () {
            
            private Iterator eachNSAttr = _getAttrNodes(true);
            public void remove () {
                eachNSAttr.remove();
            }
            public boolean hasNext () {
                return eachNSAttr.hasNext();
            }
            public Object next () {
                Attr attr = (Attr) eachNSAttr.next();
                String prefix = attr.getPrefix();
                String localName = attr.getLocalName();
                                return null == prefix ? Constants.EMPTY_STRING : localName;
            }
        };
    }
    
    public final Name getElementName () {
        return new NameImpl(getLocalName(), getPrefix(), getNamespaceURI());
    }
    
    public final boolean removeAttribute (Name name) {
        Attr attr = getAttributeNodeNS(name.getURI(), name.getLocalName());
        if ( null == attr ) {
            return false;
        } else {
            removeAttributeNode(attr);
            return true;
        }
    }
    
    public final boolean removeNamespaceDeclaration (String prefix) {
        
        String attrQName = (null==prefix || 0==prefix.length())
            ? Constants.NS_PREFIX : Constants.NS_PREFIX + ':' + prefix;
        Attr attr = getAttributeNode(attrQName);
        if ( null == attr )
            return false;
        else {
            removeAttributeNode(attr);
            return true;
        }
    }
    
    public final Iterator getChildElements () {
        return new Iterator () {
            
            private NodeList children = getChildNodes();
            
            private Node next;
            
            private Node last;
            
            private int index = 0;
            
            private boolean findNext () {
                if ( index < children.getLength() ) {
                    next = children.item(index++);
                    return true;
                }
                else {
                    return false;
                }
            }
            
            public boolean hasNext () {
                return null != next ? true : findNext();
            }
            public Object next () {
                if ( !hasNext() ) {
                    throw new NoSuchElementException();
                }
                last = next;
                next = null;
                return last;
            }
            public void remove () {
                if ( null == last ) {
                    throw new IllegalStateException(
                                Message.getMessage("SAAJ_E_ITERATOR_REMOVE"));
                }
                removeChild(last);
                index--;
                last = null;
            }
            
        };
    }
    
    public final Iterator getChildElements (final Name name) {
        return new Iterator () {
            
            private Iterator childEles = _getChildElements();
            
            private SOAPElement next;
            
            private SOAPElement last;
            
            private boolean findNext () {
                Object child = null;
                while ( childEles.hasNext() ) {
                    child = childEles.next();
                    if ( ((SOAPElement) child).getElementName().equals(name) ) {
                        next = (SOAPElement) child;
                        return true;
                    }
                }
                return false;
            }
            
            public boolean hasNext () {
                return null != next ? true : findNext();
            }
            public Object next () {
                if ( !hasNext() ) {
                    throw new NoSuchElementException();
                }
                last = next;
                next = null;
                return last;
            }
            public void remove () {
                if ( null == last ) {
                    throw new IllegalStateException(
                                Message.getMessage("SAAJ_E_ITERATOR_REMOVE"));
                }
                childEles.remove();
                last = null;
            }
            
        };
    }
    
    public final void setEncodingStyle (String encodingStyle)
        throws SOAPException {
        setAttributeNS(_getSOAPEnvURI(),
                       _getSOAPEnvPrefix()+':'+Constants.ATTR_ENCODING_STYLE,
                       encodingStyle);
    }
    
    public final String getEncodingStyle () {
        return getAttributeNS(_getSOAPEnvURI(), Constants.ATTR_ENCODING_STYLE);
    }
    
    public final void removeContents () {
        for (Iterator iter = getChildElements(); iter.hasNext(); ) {
            iter.next();
            iter.remove();
        }
    }
    
    public final Iterator getVisibleNamespacePrefixes () {
        Set set = new HashSet();
        _getVisibleNamespacePrefixes(set);
        return set.iterator();
    }
    
    public final String getValue () {
        normalize();
        for (Iterator iter = getChildElements(); iter.hasNext(); ) {
            org.w3c.dom.Node n = (org.w3c.dom.Node) iter.next();
            if ( n instanceof javax.xml.soap.Text ) {
                return n.getNodeValue();
            }
        }
        return null;
    }
    
    public final void setParentElement (SOAPElement parent)
        throws SOAPException {
        if ( null == parent ) {
            throw new IllegalArgumentException(
                        Message.getMessage("SAAJ_E_ELE_NULL_PARENT"));
        } else {
            parent.addChildElement(this);
        }
    }
    
    public final SOAPElement getParentElement () {
        org.w3c.dom.Node parentNode = getParentNode();
        if (null == parentNode ||
            parentNode.getNodeType() == Node.DOCUMENT_NODE ) {
            return null;
        } else {
            return (SOAPElement) parentNode;
        }
    }
    
    public final void detachNode () {
        org.w3c.dom.Node parent = getParentNode();
        if (null != parent) {
            parent.removeChild(this);
        }
    }
    
    public final void recycleNode () {
        detachNode();
    }
    
    public final void setValue (String value) {
        normalize();
        Node firstChild = getFirstChild();
        if ( null == firstChild ) {
            try {
                addTextNode(value);
            }
            catch (SOAPException soape) {
                throw new IllegalStateException(soape.getMessage());
            }
        }
        else if (firstChild instanceof javax.xml.soap.Text &&
                 null == firstChild.getNextSibling()) {
            firstChild.setNodeValue(value);
        }
        else {
            throw new IllegalStateException(
                        Message.getMessage("SAAJ_E_ELE_UNABLE_SETVALUE"));
        }
    }
            
    private static final boolean isNSAttr (Attr attr) {
        String prefix = attr.getPrefix();
        String name = attr.getName();
        if ("xmlns".equals(prefix) ||
            null == prefix && "xmlns".equals(name)) {
            return true;
        } else {
            return false;
        }
    }
    
    public static final void _replaceWith (SOAPElement oldEle, SOAPElement newEle) {
        
        NamedNodeMap attrs = oldEle.getAttributes();
        final int attrCount = attrs.getLength();
        for ( int i = 0; i < attrCount; i++ ) {
            newEle.setAttributeNodeNS((Attr) attrs.item(i).cloneNode(false));
        }
        
        while( oldEle.getFirstChild() != null )
            newEle.appendChild(oldEle.getFirstChild());
        
        
        Node parent = oldEle.getParentNode();
        parent.replaceChild(newEle, oldEle);
    }
    
    private final SOAPElement _addElement (String uri, String qName) {
        SOAPElement child =
            (SOAPElement) getOwnerDocument().createElementNS(uri, qName);
        super.appendChild(child);
        return child;
    }
    
    protected final SOAPElement _getFirstChildElement (Name name) {
        Iterator iter = getChildElements(name);
        return iter.hasNext() ? (SOAPElement) iter.next() : null;
    }
    
    protected final String _getSOAPEnvPrefix () {
        Element env = getOwnerDocument().getDocumentElement();
        return null == env ? Constants.NS_PREFIX_SOAP_ENV : env.getPrefix();
    }
    
    private final String _getSOAPEnvURI () {
        Element env = getOwnerDocument().getDocumentElement();
        return null == env ? Constants.URI_SOAP11_ENV : env.getNamespaceURI();
    }
    
    private final Iterator _getAttrNodes (final boolean getNSAttrs) {
        final NamedNodeMap attrs = getAttributes();
        return new Iterator () {
            
            private Attr next;
            
            private Attr last;
            
            private int index = 0;
            
            private boolean findNext () {
                final int length = attrs.getLength();
                Attr attr = null;
                while ( index < length ) {
                    attr = (Attr) attrs.item(index);
                    if ( getNSAttrs ) {
                        if (isNSAttr(attr)) {
                            next = attr;
                            index++;
                            return true;
                        }
                    }
                    else {
                        if (!isNSAttr(attr)) {
                            next = attr;
                            index++;
                            return true;
                        }
                    }
                    index++;
                }
                return false;
            }
            
            public boolean hasNext () {
                return null != next ? true : findNext();
            }
            public Object next () {
                if ( !hasNext() ) {
                    throw new NoSuchElementException();
                }
                last = next;
                next = null;
                return last;
            }
            public void remove () {
                if ( null == last ) {
                    throw new IllegalStateException(
                            Message.getMessage("SAAJ_E_ITERATOR_REMOVE"));
                }
                removeAttributeNode(last);
                index--;
                last = null;
            }
            
        };
    }
    
    final Iterator _getChildElements () {
        return new Iterator () {
            
            private Iterator childNodes = getChildElements();
            
            private SOAPElement next;
            
            private SOAPElement last;
            
            private boolean findNext () {
                Object child = null;
                while ( childNodes.hasNext() ) {
                    child = childNodes.next();
                    if ( child instanceof SOAPElement ) {
                        next = (SOAPElement) child;
                        return true;
                    }
                }
                return false;
            }
            
            public boolean hasNext () {
                return null != next ? true : findNext();
            }
            public Object next () {
                if ( !hasNext() ) {
                    throw new NoSuchElementException();
                }
                last = next;
                next = null;
                return last;
            }
            public void remove () {
                if ( null == last ) {
                    throw new IllegalStateException(
                            Message.getMessage("SAAJ_E_ITERATOR_REMOVE"));
                }
                childNodes.remove();
                last = null;
            }
            
        };
    }
    
    private final void _getVisibleNamespacePrefixes (Set set) {
        
        for (Iterator iter = getNamespacePrefixes(); iter.hasNext(); ) {
            set.add(iter.next());
        }
        
        ElementImpl parent = (ElementImpl) getParentElement();
        if ( null != parent ) {
            parent._getVisibleNamespacePrefixes(set);
        }
    }
}
