package org.act.soap.message;
import java.io.OutputStream;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.act.soap.util.i18n.Message;

public abstract class EnvelopeImpl extends ElementImpl implements SOAPEnvelope {
    
    private String omitXmlDecl;
    
    private String charset;
    
    private String outputIndent;
            
    protected EnvelopeImpl (SOAPDocumentImpl ownerDoc, Name name,
                            boolean createHeader, boolean createBody) {  
        this(ownerDoc, name);
        if (createHeader) {
            appendChild(createHeader(getPrefix()));
        }
        if (createBody) {
            appendChild(createBody(getPrefix()));
        }
    }
    
    protected EnvelopeImpl (SOAPDocumentImpl ownerDoc, Name name) {  
        super(ownerDoc, name);
        omitXmlDecl = "yes";
        charset = "utf-8";
        outputIndent = "no";
    }
            
    public final Name createName (String localName, String prefix, String uri)
        throws SOAPException {  
        return new NameImpl(localName, prefix, uri);
    }
    
    public final Name createName (String localName) throws SOAPException {  
        return new NameImpl(localName);
    }
    
    public final SOAPHeader getHeader () throws SOAPException {  
        return (SOAPHeader) _getFirstChildElement(getHeaderName(null));
    }
    
    public final SOAPBody getBody () throws SOAPException {  
        return (SOAPBody) _getFirstChildElement(getBodyName(null));
    }
    
    public final SOAPHeader addHeader () throws SOAPException {  
                SOAPHeader header = getHeader();
        if ( null != header ) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_ENV_HEADER_EXIST"));
        }
                String prefix = getPrefix();
        header = createHeader(prefix);
        insertBefore(header, getFirstChild());
        return header;
    }
    
    public final SOAPBody addBody () throws SOAPException {  
                SOAPBody body = getBody();
        if ( null != body ) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_ENV_BODY_EXIST"));
        }
                String prefix = getPrefix();
        body = createBody(prefix);
        appendChild(body);
        return body;
    }
            
    protected abstract Name getBodyName(String prefix);
    
    protected abstract Name getHeaderName(String prefix);
    
    protected abstract SOAPHeader createHeader(String prefix);
    
    protected abstract SOAPBody createBody(String prefix);
            
    final void setCharsetEncoding (String value) {
        charset = value;
    }
    
    final void setOmitXmlDecl (String value) {
        omitXmlDecl = value;
    }
    
    final void setOutputIndent (String value) {
        outputIndent = value;
    }
    
    final void writeTo (OutputStream out) throws SOAPException {
        try {
                        Transformer transformer = Utils.getTransformer();
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, 
                                          omitXmlDecl);
            transformer.setOutputProperty(OutputKeys.ENCODING, charset);
            transformer.setOutputProperty(OutputKeys.INDENT, outputIndent);
                        transformer.transform(new DOMSource(getOwnerDocument()), 
                                  new StreamResult(out));
        } catch (TransformerConfigurationException tce) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_TRANS_CONFIG"), tce);
        } catch (TransformerException te) {
            throw new SOAPException(Message.getMessage("SAAJ_E_TRANSFORM"), te);
        }
    }
}
