package org.act.soap.message;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import org.act.soap.util.i18n.Message;

public abstract class HeaderImpl extends ElementImpl implements SOAPHeader {
    
    protected HeaderImpl (SOAPDocumentImpl ownerDoc, Name name) {
        super(ownerDoc, name);
    }
            
    public final SOAPHeaderElement addHeaderElement (Name name)
        throws SOAPException {
        
        String uri = name.getURI();
        if ( null == uri || 0 == uri.length() ) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_HEADER_ENTRY_MUST_QUALIFIED"));
        }
        SOAPHeaderElement headerEntry = createHeaderElement(name);
        appendChild(headerEntry);
        return headerEntry;
    }
    
    public final Iterator examineHeaderElements (String actor) {
        return getHeaderElements(true, actor, false);
    }
    
    public final Iterator extractHeaderElements (String actor) {
        Iterator iter = getHeaderElements(true, actor, false);
        List list = new ArrayList();
        while (iter.hasNext()) {
            list.add(iter.next());
            iter.remove();
        }
        return list.iterator();
    }
    
    public final Iterator examineMustUnderstandHeaderElements (String actor) {
        return getHeaderElements(true, actor, true);
    }
    
    public final Iterator examineAllHeaderElements () {
        return getHeaderElements(false, null, false);
    }
    
    public final Iterator extractAllHeaderElements () {
        List list = new ArrayList();
        for (Iterator iter = getChildElements(); iter.hasNext(); ) {
            list.add(iter.next());
            iter.remove();
        }
        return list.iterator();
    }
            
    protected abstract SOAPHeaderElement createHeaderElement(Name name);
            
    private final Iterator getHeaderElements(final boolean check,
                                             final String actor,
                                             final boolean checkMustUnderstand){
        return new Iterator () {
            
            private Iterator childEles = _getChildElements();
            
            private SOAPHeaderElement next;
            
            private SOAPHeaderElement last;
            
            private boolean findNext () {
                SOAPHeaderElement headerEntry = null;
                while ( childEles.hasNext() ) {
                    headerEntry =
                        convertToHeaderElement((SOAPElement) childEles.next());
                    if ( !check ) {
                        next = headerEntry;
                        return true;
                    }
                    if (actor.equals(headerEntry.getActor())) {
                        if ( checkMustUnderstand ) {
                            if (headerEntry.getMustUnderstand()) {
                                next = headerEntry;
                                return true;
                            }
                        }
                        else {
                            next = headerEntry;
                            return true;
                        }
                    }
                }
                return false;
            }
            
            public boolean hasNext () {
                return null != next ? true : findNext();
            }
            public Object next () {
                if ( !hasNext() ) {
                    throw new NoSuchElementException();
                }
                last = next;
                next = null;
                return last;
            }
            public void remove () {
                if ( null == last ) {
                    throw new IllegalStateException(
                            Message.getMessage("SAAJ_E_ITERATOR_REMOVE"));
                }
                childEles.remove();
                last = null;
            }
            
        };
    }
    
    private final SOAPHeaderElement convertToHeaderElement(SOAPElement oldEle) {
        if (oldEle instanceof SOAPHeaderElement)
            return (SOAPHeaderElement) oldEle;
        SOAPHeaderElement newEle = createHeaderElement(oldEle.getElementName());
        _replaceWith(oldEle, newEle);
        return newEle;
    }
}
