package org.act.soap.message;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;
import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.MessagingException;
import javax.mail.internet.ContentType;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.ParseException;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.transform.stream.StreamSource;
import org.act.soap.message.ver1_1.SOAPPart1_1Impl;
import org.act.soap.message.ver1_2.SOAPPart1_2Impl;
import org.act.soap.util.i18n.Message;

public final class MessageImpl extends SOAPMessage {
    
    public static final String SOAP_VERSION = "org.act.soap.soap-version";
    
    public static final String OUTPUT_INDENT = "org.act.soap.output-indent";
    
    private MimeHeaders headers;
    
    private SOAPPart soapPart;
    
    private List attachments;
    
    private MimeMultipart mpMsg;
    
    private Set properties;
    
    private Map propsUsed;
    
    private boolean saveRequired;
            
    public MessageImpl () {
        headers = new MimeHeaders();
        soapPart = null;
        attachments = null;
        mpMsg = null;
        saveRequired = true;
        initProperties();
    }
    
    MessageImpl (MimeHeaders hds, final InputStream input)
        throws SOAPException {
        saveRequired = true;
        headers = hds;
        initProperties();
        
        String[] values = headers.getHeader(Constants.HEADER_CONTENT_TYPE);
        if (null == values || 0 == values.length) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_MSG_ABSENT_CTYPE"));
        }
        final String ctStr = values[0];
        ContentType ct = null;
        try {
            ct = new ContentType(ctStr);
        }
        catch ( ParseException pe ) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_MSG_UNABLE_PARSE_CTYPE"), pe);
        }
        String primaryType = ct.getPrimaryType();
        String subType = ct.getSubType();
        
        if ("text".equalsIgnoreCase(primaryType) &&
            "xml".equalsIgnoreCase(subType)) {
            soapPart = new SOAPPart1_1Impl();
            soapPart.setContent(new StreamSource(input));
            return;
        }
        
        if ("application".equalsIgnoreCase(primaryType) &&
            "soap+xml".equalsIgnoreCase(subType) ) {
            soapPart = new SOAPPart1_2Impl();
            soapPart.setContent(new StreamSource(input));
            return;
        }
        
        if ("multipart".equalsIgnoreCase(primaryType) &&
            "related".equalsIgnoreCase(subType)) {
                        String typeParam = ct.getParameter("type");
            if ( typeParam.toLowerCase().startsWith("text/xml") ) {
                soapPart = new SOAPPart1_1Impl();
            } else {
                if (typeParam.toLowerCase().startsWith("application/soap+xml")){
                    soapPart = new SOAPPart1_2Impl();
                } else {
                    throw new SOAPException(
                    Message.getMessage("SAAJ_E_MSG_INVALID_SOAP_MIME_TYPE",
                                       typeParam));
                }
            }
            try {
                                DataSource ds = new DataSource () {
                    public InputStream getInputStream () { return input; }
                    public OutputStream getOutputStream () { return null; }
                    public String getContentType () { return ctStr; }
                    public String getName () { return ""; }
                };
                                MimeMultipart msg = new MimeMultipart(ds);
                MimeBodyPart soapMsg = (MimeBodyPart) msg.getBodyPart(0);
                soapPart.setContent(new StreamSource(soapMsg.getInputStream()));
                                final int partCount = msg.getCount();
                for (int i = 1; i < partCount; i++) {
                                        MimeBodyPart bp = (MimeBodyPart) msg.getBodyPart(i);
                    AttachmentPartImpl attach = new AttachmentPartImpl();
                    DataHandler dh = bp.getDataHandler();
                    attach.setDataHandler(dh);
                                        Utils.copyMimeHeaders(bp, attach);
                    addAttachmentPart(attach);
                }
            } catch (MessagingException me) {
                throw new SOAPException(
                        Message.getMessage("SAAJ_E_MSG_UNABLE_PARSE_MSG"), me);
            } catch (IOException ioe) {
                throw new SOAPException(
                        Message.getMessage("SAAJ_E_MSG_UNABLE_PARSE_MSG"), ioe);
            }
            return;
        }
        
        throw new SOAPException(
                    Message.getMessage("SAAJ_E_MSG_INVALID_MSG_TYPE", ctStr));
    }
            
    public final SOAPPart getSOAPPart () {
        if (null != soapPart) {
            return soapPart;
        }
                String version = (String) propsUsed.get(SOAP_VERSION);
                if ( "1.2".equals(version) ) {
            soapPart = new SOAPPart1_2Impl();
        } else {
            soapPart = new SOAPPart1_1Impl();
        }
        return soapPart;
    }
    
    public final Iterator getAttachments () {
        return null == attachments ? Utils.NULL_ITERATOR
                                   : attachments.iterator();
    }
    
    public Iterator getAttachments (final MimeHeaders hds) {
        if ( null == attachments ) {
            return Utils.NULL_ITERATOR;
        }
        return new Iterator () {
            
            private Iterator iter;
            private MimeHeaders mheaders;
            
            private Object nextAttach;
            {
                mheaders = hds;
                iter = attachments.iterator();
            }
            public void remove () {
                iter.remove();
            }
            public boolean hasNext () {
                if (null == nextAttach) {
                    nextAttach = nextMatch();
                }
                return null != nextAttach;
            }
            public Object next () {
                if ( hasNext() ) {
                    Object ret = nextAttach;
                    nextAttach = null;
                    return ret;
                }
                else {
                    throw new NoSuchElementException();
                }
            }
            Object nextMatch () {
                while ( iter.hasNext() ) {
                    AttachmentPartImpl ap = (AttachmentPartImpl) iter.next();
                    if (ap.hasHeaders(headers)) {
                        return ap;
                    }
                }
                return null;
            }
        };
    }
    
    public final void setContentDescription (String description) {
        headers.setHeader(Constants.HEADER_CONTENT_DESCRIPTION, description);
    }
    
    public final boolean saveRequired () {
        return saveRequired;
    }
    
    public final SOAPHeader getSOAPHeader () throws SOAPException {
        return getSOAPPart().getEnvelope().getHeader();
    }
    
    public final void saveChanges () throws SOAPException {
        if ( !saveRequired() ) {
            return ;
        }
                String charset = (String) getProperty(CHARACTER_SET_ENCODING);
        String write = (String) getProperty(WRITE_XML_DECLARATION);
        String indent = (String) getProperty(OUTPUT_INDENT);
        EnvelopeImpl env = (EnvelopeImpl) getSOAPPart().getEnvelope();
        env.setCharsetEncoding(charset);
        if ("true".equalsIgnoreCase(write)) {
            env.setOmitXmlDecl("no");
        } else {
            env.setOmitXmlDecl("yes");
        }
        if ("true".equalsIgnoreCase(indent)) {
            env.setOutputIndent("yes");
        } else {
            env.setOutputIndent("no");
        }
                StringBuffer buf = new StringBuffer(50);
        if ( 0 == countAttachments() ) {
            buf.append(((SOAPPartImpl) getSOAPPart())._getContentType())
               .append("; charset=\"")
               .append(charset)
               .append('"');
        }
        else {
            initMPMsg();
                        buf.append(mpMsg.getContentType())
               .append(';')
               .append("type=\"")
               .append(((SOAPPartImpl) getSOAPPart())._getContentType())
               .append('"');
        }
        headers.setHeader(Constants.HEADER_CONTENT_TYPE, buf.toString());
                String[] soapAction = headers.getHeader(Constants.HEADER_SOAP_ACTION);
        if (null == soapAction || 0 == soapAction.length) {
            headers.setHeader(Constants.HEADER_SOAP_ACTION, "\"\"");
        }
        saveRequired = false;
    }
    
    public final void setProperty (String property, Object value)
        throws SOAPException {
        
        if (!properties.contains(property)) {
            throw new SOAPException(
                Message.getMessage("SAAJ_E_MSG_ILLEGAL_PROPERTY", property));
        }
        propsUsed.put(property, value);
        saveRequired = true;
    }
    
    public final SOAPBody getSOAPBody () throws SOAPException {
        return getSOAPPart().getEnvelope().getBody();
    }
    
    public final MimeHeaders getMimeHeaders () {
        return headers;
    }
    
    public final void writeTo (OutputStream out) throws SOAPException, IOException {
        saveChanges();
        
        if (0 == countAttachments()) {
            ((EnvelopeImpl) getSOAPPart().getEnvelope()).writeTo(out);
        }
        else {
            try {
                loadMPMsg();
                mpMsg.writeTo(out);
                unloadMPMsg();
            } catch (MessagingException me) {
                throw new SOAPException(
                        Message.getMessage("SAAJ_E_MSG_UNABLE_OUTPUT"), me);
            } catch (IOException ioe) {
                throw new SOAPException(
                        Message.getMessage("SAAJ_E_MSG_UNABLE_OUTPUT"), ioe);
            }
        }
    }
    
    public final int countAttachments () {
        return (null == attachments) ? 0 : attachments.size();
    }
    
    public final void removeAllAttachments () {
        if (null != attachments) {
            attachments.clear();
            saveRequired = true;
        }
    }
    
    public final Object getProperty (String property) throws SOAPException {
        
        if (!properties.contains(property)) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_MSG_ILLEGAL_PROPERTY"));
        }
        return propsUsed.get(property);
    }
    
    public final AttachmentPart createAttachmentPart () {
        return new AttachmentPartImpl();
    }
    
    public final String getContentDescription () {
        return Utils.getFirstMimeHeader(
                        headers, Constants.HEADER_CONTENT_DESCRIPTION);
    }
    
    public final void addAttachmentPart (AttachmentPart attachmentPart) {
        if (null == attachmentPart) {
            throw new IllegalArgumentException(
                        Message.getMessage("SAAJ_E_NULL_ARG"));
        }
        if (null == attachments) {
            attachments = new ArrayList();
        }
        attachments.add(attachmentPart);
        saveRequired = true;
    }
            
    private void initProperties () {
        properties = new HashSet();
        properties.add(WRITE_XML_DECLARATION);
        properties.add(CHARACTER_SET_ENCODING);
        properties.add(SOAP_VERSION);
        properties.add(OUTPUT_INDENT);
        propsUsed = new HashMap(properties.size());
        propsUsed.put(WRITE_XML_DECLARATION, "false");
        propsUsed.put(CHARACTER_SET_ENCODING, "utf-8");
        propsUsed.put(SOAP_VERSION, "1.1");
        propsUsed.put(OUTPUT_INDENT, "true");
    }
    
    private MimeMultipart loadMPMsg () throws SOAPException {
        try {
                        mpMsg.addBodyPart( ( (SOAPPartImpl) getSOAPPart()).getMimePart());
                        for (Iterator iter = getAttachments(); iter.hasNext(); ) {
                mpMsg.addBodyPart(
                    ((AttachmentPartImpl) iter.next()).getMimePart());
            }
        } catch ( MessagingException me ) {
            throw new SOAPException(
                    Message.getMessage("SAAJ_E_MSG_UNABLE_TO_MULTIPART"), me);
        }
        return mpMsg;
    }
    
    private void unloadMPMsg () throws SOAPException {
        try {
            final int partCount = mpMsg.getCount();
            for (int i = 0; i < partCount; i++) {
                mpMsg.removeBodyPart(0);
            }
        } catch ( MessagingException me ) {
            throw new SOAPException(me);
        }
    }
    
    private void initMPMsg () {
        if ( null == mpMsg ) {
            mpMsg = new MimeMultipart("related");
        }
    }
}
