package org.act.soap.message;
import javax.xml.soap.Name;
import org.act.soap.util.i18n.Message;

public final class NameImpl implements Name {
    
    private String namespaceURI;
    
    private String localName;
    
    private String prefix;
    
    private volatile int hashCode = 0;
            
    public NameImpl (String localPart, String pre, String uri) {
        initName(localPart, pre, uri);
    }
    
    NameImpl (String localPart) {
        this(localPart, null, null);
    }
    
    NameImpl (String qName, String uri) {
        
        String[] qn = Utils.splitQName(qName);
        initName(qn[1], qn[0], uri);
    }
            
    public static final Name createEnvelope1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_ENVELOPE,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name createEnvelope1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_ENVELOPE,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createBody1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_BODY,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name createBody1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_BODY,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createHeader1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_HEADER,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name createHeader1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_HEADER,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFault1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name createFault1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultCode1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_CODE_SOAP11,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name FAULT_CODE11_NAME = new NameImpl(Constants.ELEM_FAULT_CODE_SOAP11);
    
    public static final Name createFaultCode1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_CODE_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultValue1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_VALUE_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultString1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_STRING_SOAP11,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name FAULT_STRING11_NAME = new NameImpl(Constants.ELEM_FAULT_STRING_SOAP11);
    
    public static final Name createFaultReason1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_REASON_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultText1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_TEXT_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultActor1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_ACTOR_SOAP11,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name FAULT_ACTOR_11_NAME = new NameImpl(Constants.ELEM_FAULT_ACTOR_SOAP11);
    
    public static final Name createFaultNode1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_NODE_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultRole1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_ROLE_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createFaultDetail1_1Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_DETAIL_SOAP11,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name FAULT_DETAIL_11_NAME = new NameImpl(Constants.ELEM_FAULT_DETAIL_SOAP11);
    
    public static final Name createFaultDetail1_2Name (String pre) {
        return new NameImpl(Constants.ELEM_FAULT_DETAIL_SOAP12,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createAttr_Actor1_1Name (String pre) {
        return new NameImpl(Constants.ATTR_ACTOR,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name createAttr_Role1_2Name (String pre) {
        return new NameImpl(Constants.ATTR_ROLE,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createAttr_MustUnderstand1_1Name (String pre) {
        return new NameImpl(Constants.ATTR_MUST_UNDERSTAND,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP11_ENV);
    }
    
    public static final Name createAttr_MustUnderstand1_2Name (String pre) {
        return new NameImpl(Constants.ATTR_MUST_UNDERSTAND,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
    
    public static final Name createAttr_Relay1_2Name (String pre) {
        return new NameImpl(Constants.ATTR_RELAY,
                            null == pre ? Constants.NS_PREFIX_SOAP_ENV:pre,
                            Constants.URI_SOAP12_ENV);
    }
            
    public final boolean equals (Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof NameImpl)) {
            return false;
        }
        if (namespaceURI != ((NameImpl) obj).namespaceURI) {
            return false;
        }
        return localName == ((NameImpl) obj).localName;
    }
    
    public final String toString () {
        return null == namespaceURI ? localName
                                    : '{' + namespaceURI + '}' + localName;
    }
    
    public final int hashCode () {
        if (0 == hashCode) {
            int result = 17;
            result = 37*result + localName.hashCode();
            if (null != namespaceURI) {
                result = 37*result + namespaceURI.hashCode();
            }
            hashCode = result;
        }
        return hashCode;
    }
            
    public final String getLocalName () {
        return localName;
    }
    
    public final String getQualifiedName () {
        if ( null == namespaceURI ) {
            return localName;
        }
        else {
            return null == prefix ? localName : prefix + ':' + localName;
        }
    }
    
    public final String getPrefix () {
        return prefix;
    }
    
    public final String getURI () {
        return namespaceURI;
    }
            
    private void initName (String localPart, String pre, String uri) {
        if ((null == localPart) || (0 == localPart.length())) {
            throw new IllegalArgumentException(
                        Message.getMessage("SAAJ_E_NAME_NULL_LOCAL"));
        } else {
            localName = localPart.intern();
        }
        if ( null != pre ) {
            prefix = pre.intern();
        }
        if ( null != uri ) {
            namespaceURI = uri.intern();
        }
    }
}
