package org.act.soap.message;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.apache.xerces.dom.DocumentImpl;
import org.act.soap.message.ver1_1.Body1_1Impl;
import org.act.soap.message.ver1_1.Detail1_1Impl;
import org.act.soap.message.ver1_1.Envelope1_1Impl;
import org.act.soap.message.ver1_1.Fault1_1Impl;
import org.act.soap.message.ver1_1.FaultActor1_1Impl;
import org.act.soap.message.ver1_1.FaultCode1_1Impl;
import org.act.soap.message.ver1_1.FaultString1_1Impl;
import org.act.soap.message.ver1_1.Header1_1Impl;
import org.act.soap.message.ver1_2.Body1_2Impl;
import org.act.soap.message.ver1_2.Detail1_2Impl;
import org.act.soap.message.ver1_2.Envelope1_2Impl;
import org.act.soap.message.ver1_2.Fault1_2Impl;
import org.act.soap.message.ver1_2.FaultCode1_2Impl;
import org.act.soap.message.ver1_2.FaultNode1_2Impl;
import org.act.soap.message.ver1_2.FaultReason1_2Impl;
import org.act.soap.message.ver1_2.FaultRole1_2Impl;
import org.act.soap.message.ver1_2.Header1_2Impl;
import org.act.soap.util.i18n.Message;

public final class SOAPDocumentImpl extends DocumentImpl {
    
    public final DocumentFragment createDocumentFragment () {  
        return new DocumentFragmentImpl(this);
    }
    
    public final DocumentType getDoctype () {  
        return null;
    }
    
    public final CDATASection createCDATASection (String data)
        throws DOMException {  
        return new CDATAImpl(this, data);
    }
    
    public final Comment createComment (String data) {  
        return new CommentImpl(this, data);
    }
    
    public final Element createElement (String tagName)
        throws DOMException {  
        return new ElementImpl(this, new NameImpl(tagName, null));
    }
    
    public final EntityReference createEntityReference (String name)
        throws DOMException {  
        throw new UnsupportedOperationException(
                Message.getMessage("SAAJ_E_ENV_DISALLOW_ENTITYREF"));
    }
    
    public final Text createTextNode (String data) {  
        return new TextImpl(this, data);
    }
    
    public final Element createElementNS (String namespaceURI, 
                                          String qualifiedName)
        throws DOMException {  
        return createElement(this, new NameImpl(qualifiedName, namespaceURI));
    }
    
    public final ProcessingInstruction createProcessingInstruction (String target, String data)
        throws DOMException {  
        throw new UnsupportedOperationException(
                    Message.getMessage("SAAJ_E_ENV_DISALLOW_PI"));
    }
    
    
    
    private static Element createElement (SOAPDocumentImpl ownerDoc,
                                           NameImpl name) {
        String localName = name.getLocalName();
        String prefix = name.getPrefix();
        String uri = name.getURI();
        
        if (Constants.URI_SOAP11_ENV.equals(uri)) {
            if (localName.equals(Constants.ELEM_ENVELOPE))
                return new Envelope1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_BODY))
                return new Body1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_HEADER))
                return new Header1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT))
                return new Fault1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_CODE_SOAP11))
                return new FaultCode1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_STRING_SOAP11))
                return new FaultString1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_ACTOR_SOAP11))
                return new FaultActor1_1Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_DETAIL_SOAP11))
                return new Detail1_1Impl(ownerDoc, prefix);
        }
        
        if (Constants.URI_SOAP12_ENV.equals(uri)) {
            if (localName.equals(Constants.ELEM_ENVELOPE))
                return new Envelope1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_BODY))
                return new Body1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_HEADER))
                return new Header1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT))
                return new Fault1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_CODE_SOAP12))
                return new FaultCode1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_REASON_SOAP12))
                return new FaultReason1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_NODE_SOAP12))
                return new FaultNode1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_ROLE_SOAP12))
                return new FaultRole1_2Impl(ownerDoc, prefix);
            if (localName.equals(Constants.ELEM_FAULT_DETAIL_SOAP12))
                return new Detail1_2Impl(ownerDoc, prefix);
        }
        
        
        return new ElementImpl(ownerDoc, name);
    }
}
