package org.act.soap.message;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Iterator;
import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPPart;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.act.soap.util.i18n.Message;

public abstract class SOAPPartImpl
    extends SOAPPart {
  
  private MimeHeaders headers;
  
  private Source source = null;
  
  protected SOAPDocumentImpl document;
      
  protected SOAPPartImpl() {
    document = new SOAPDocumentImpl();
    headers = new MimeHeaders();
    headers.setHeader(Constants.HEADER_CONTENT_TYPE, _getContentType());
  }
      
  public final Iterator getAllMimeHeaders() {
    return headers.getAllHeaders();
  }
  
  public final void setMimeHeader(String name, String value) {
    headers.setHeader(name, value);
  }
  
  public final void removeAllMimeHeaders() {
    headers.removeAllHeaders();
  }
  
  public final Iterator getMatchingMimeHeaders(String[] names) {
    return headers.getMatchingHeaders(names);
  }
  
  public final void removeMimeHeader(String header) {
    headers.removeHeader(header);
  }
  
  public final String[] getMimeHeader(String name) {
    return headers.getHeader(name);
  }
  
  public final void addMimeHeader(String name, String value) {
    headers.addHeader(name, value);
  }
  
  public final Iterator getNonMatchingMimeHeaders(String[] names) {
    return headers.getNonMatchingHeaders(names);
  }
  
  public final void setContent(Source src) throws SOAPException {
    source = src;
  }
  
  public final Source getContent() throws SOAPException {
    return null != source ? source : new DOMSource(document);
  }
  
  public final SOAPEnvelope getEnvelope() throws SOAPException {
    if (null != source) {
      _createEnvFromSource(document, source);
      source = null;
      return (SOAPEnvelope) document.getDocumentElement();
    }
    SOAPEnvelope env = (SOAPEnvelope) document.getDocumentElement();
    if (null == env) {
      env = createEmptyEnvelope();
      env.addNamespaceDeclaration(env.getPrefix(), env.getNamespaceURI());
      document.insertBefore(env, null);
    }
    return env;
  }
      
  public final DOMImplementation getImplementation() {
    return document.getImplementation();
  }
  
  public final Node getNextSibling() {
    return document.getNextSibling();
  }
  
  public final String getPrefix() {
    return document.getPrefix();
  }
  
  public final boolean isSupported(String feature, String version) {
    return document.isSupported(feature, version);
  }
  
  public final Attr createAttribute(String name) throws DOMException {
    return document.createAttribute(name);
  }
  
  public final Element getElementById(String elementId) {
    handleNewSource();
    return document.getElementById(elementId);
  }
  
  public final Attr createAttributeNS(String uri, String qualifiedName) throws
      DOMException {
    return document.createAttributeNS(uri, qualifiedName);
  }
  
  public final NodeList getChildNodes() {
    handleNewSource();
    return document.getChildNodes();
  }
  
  public final NodeList getElementsByTagName(String tagname) {
    handleNewSource();
    return document.getElementsByTagName(tagname);
  }
  
  public final Node getPreviousSibling() {
    return document.getPreviousSibling();
  }
  
  public final NamedNodeMap getAttributes() {
    return document.getAttributes();
  }
  
  public final short getNodeType() {
    return document.getNodeType();
  }
  
  public final Comment createComment(String data) {
    return document.createComment(data);
  }
  
  public final Node getLastChild() {
    handleNewSource();
    return document.getLastChild();
  }
  
  public final Document getOwnerDocument() {
    return document.getOwnerDocument();
  }
  
  public final Node getFirstChild() {
    handleNewSource();
    return document.getFirstChild();
  }
  
  public final void normalize() {
    document.normalize();
  }
  
  public final boolean hasChildNodes() {
    handleNewSource();
    return document.hasChildNodes();
  }
  
  public final Node insertBefore(Node newChild, Node refChild) throws
      DOMException {
    return document.insertBefore(newChild, refChild);
  }
  
  public final Node importNode(Node importedNode, boolean deep) throws
      DOMException {
    return document.importNode(importedNode, deep);
  }
  
  public final DocumentType getDoctype() {
    return document.getDoctype();
  }
  
  public final Node getParentNode() {
    return document.getParentNode();
  }
  
  public final ProcessingInstruction createProcessingInstruction(String target,
      String data) throws DOMException {
    return document.createProcessingInstruction(target, data);
  }
  
  public final Node cloneNode(boolean deep) {
    handleNewSource();
    return document.cloneNode(deep);
  }
  
  public final void setPrefix(String prefix) throws DOMException {
    document.setPrefix(prefix);
  }
  
  public final Element createElement(String tagName) throws DOMException {
    return document.createElement(tagName);
  }
  
  public final String getNodeValue() throws DOMException {
    return document.getNodeValue();
  }
  
  public final Node appendChild(Node newChild) throws DOMException {
    handleNewSource();
    return document.appendChild(newChild);
  }
  
  public final void setNodeValue(String nodeValue) throws DOMException {
    document.setNodeValue(nodeValue);
  }
  
  public final Node replaceChild(Node newChild, Node oldChild) throws
      DOMException {
    handleNewSource();
    return document.replaceChild(newChild, oldChild);
  }
  
  public final CDATASection createCDATASection(String data) throws DOMException {
    return document.createCDATASection(data);
  }
  
  public final Node removeChild(Node oldChild) throws DOMException {
    handleNewSource();
    return document.removeChild(oldChild);
  }
  
  public final String getNamespaceURI() {
    return document.getNamespaceURI();
  }
  
  public final DocumentFragment createDocumentFragment() {
    return document.createDocumentFragment();
  }
  
  public final boolean hasAttributes() {
    return document.hasAttributes();
  }
  
  public final NodeList getElementsByTagNameNS(String uri, String localName) {
    handleNewSource();
    return document.getElementsByTagNameNS(uri, localName);
  }
  
  public final String getLocalName() {
    return document.getLocalName();
  }
  
  public final String getNodeName() {
    return document.getNodeName();
  }
  
  public final EntityReference createEntityReference(String name) throws
      DOMException {
    return document.createEntityReference(name);
  }
  
  public final Text createTextNode(String data) {
    return document.createTextNode(data);
  }
  
  public final Element createElementNS(String uri, String qualifiedName) throws
      DOMException {
    return document.createElementNS(uri, qualifiedName);
  }
  
  public final Element getDocumentElement() {
    handleNewSource();
    return document.getDocumentElement();
  }
      
  protected abstract String _getContentType();
  
  protected abstract SOAPEnvelope createEmptyEnvelope();
      
  private static final void _createEnvFromSource(Document doc, Source source) throws
      SOAPException {
    try {
      org.xml.sax.InputSource is = SAXSource.sourceToInputSource(source);
      SAXSource src = new SAXSource(is);
      Transformer transformer = Utils.getTransformer();
      DOMResult result = new DOMResult(doc);
      transformer.transform(src, result);
    }
    catch (TransformerConfigurationException tce) {
      throw new SOAPException(
          Message.getMessage("SAAJ_E_TRANS_CONFIG"), tce);
    }
    catch (TransformerException te) {
      throw new SOAPException(Message.getMessage("SAAJ_E_TRANSFORM"), te);
    }
  }
  
  private final DataHandler getDataHandler() {
        DataSource ds = new DataSource() {
      public InputStream getInputStream() throws IOException {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
          ( (EnvelopeImpl) getEnvelope()).writeTo(bos);
        }
        catch (SOAPException soape) {
          throw new IOException(soape.getMessage());
        }
        return new ByteArrayInputStream(bos.toByteArray());
      }
      public OutputStream getOutputStream() throws IOException {
        throw new IOException();
      }
      public String getContentType() {
        return _getContentType();
      }
      public String getName() {
        return getContentId();
      }
    };
        return new DataHandler(ds);
  }
  
  final MimeBodyPart getMimePart() throws SOAPException {
    MimeBodyPart soapMimePart = new MimeBodyPart();
    try {
            soapMimePart.setDataHandler(getDataHandler());
            Utils.copyMimeHeaders(headers, soapMimePart);
    }
    catch (MessagingException me) {
      throw new SOAPException(
          Message.getMessage("SAAJ_E_UNABLE_SP_TO_MBP"), me);
    }
    return soapMimePart;
  }
  
  private void handleNewSource() {
    try {
      getEnvelope();
    }
    catch (SOAPException soape) {
      throw new DOMException(DOMException.INVALID_STATE_ERR,
                             soape.getMessage());
    }
  }
}
