package org.act.soap.message;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Locale;
import java.util.NoSuchElementException;
import javax.activation.MimeType;
import javax.activation.MimeTypeParseException;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MimeHeader;
import javax.xml.soap.MimeHeaders;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;

public final class Utils {
    
    private Utils () {
    }
    
    public static final String[] splitQName (String qname) {
        int division = qname.indexOf(':');
        if (division < 0)
            return new String[] {"", qname};
        return new String[] { qname.substring(0, division),
                              qname.substring(division+1) };
    }
    
    static final String getFirstMimeHeader (MimeHeaders hds, String header) {
        String[] values = hds.getHeader(header);
        if ( (null != values) && (0 < values.length))
            return values[0];
        return null;
    }
    
    public static final String getCharset (String contentType) {
        String charset = null;
        try {
            MimeType mt = new MimeType(contentType);
            charset = mt.getParameter("charset");
        } catch (MimeTypeParseException e) {
            throw new IllegalArgumentException(e.getMessage());
        }
        if (null == charset)
            charset = System.getProperty("file.encoding");
        return charset;
    }
    
    public static final Transformer getTransformer ()
        throws TransformerConfigurationException {
        
        TransformerFactory tf = TransformerFactory.newInstance();
        return tf.newTransformer();
    }
    
    static final void copyMimeHeaders (MimeHeaders headers, MimeBodyPart mbp)
        throws MessagingException {
        for (Iterator iter = headers.getAllHeaders(); iter.hasNext(); ) {
            MimeHeader mh = (MimeHeader) iter.next();
            mbp.addHeader(mh.getName(), mh.getValue());
        }
    }
    
    static final void copyMimeHeaders (MimeBodyPart mbp, AttachmentPart attach)
        throws MessagingException {
        Header hdr = null;
        for (Enumeration enum1 = mbp.getAllHeaders(); enum1.hasMoreElements(); ) {
            hdr = (Header) enum1.nextElement();
            attach.addMimeHeader(hdr.getName(), hdr.getValue());
        }
    }
    
    public static final String localeToXmlLang (Locale locale) {
        String xmlLang = locale.getLanguage();
        String country = locale.getCountry();
        if (!"".equals(country)) {
            xmlLang += '-' + country;
        }
        return xmlLang;
    }
    
    public static final Locale xmlLangToLocale (String xmlLang) {
        if (null == xmlLang) {
            return null;
        }
        int index = xmlLang.indexOf("-");
        if (-1 == index) {
            index = xmlLang.indexOf("_");
        }
        if (-1 == index) {
            return new Locale(xmlLang, "");
        }
        else {
            String language = xmlLang.substring(0, index);
            String country = xmlLang.substring(index + 1);
            return new Locale(language, country);
        }
    }
    
    static final Iterator NULL_ITERATOR = new Iterator () {
        public void remove () {
            throw new IllegalStateException();
        }
        public boolean hasNext () {
            return false;
        }
        public Object next () {
            throw new NoSuchElementException();
        }
    };
}
