package org.act.soap.message.dch;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import javax.activation.ActivationDataFlavor;
import javax.activation.DataContentHandler;
import javax.activation.DataSource;
import org.act.soap.message.Constants;
import org.act.soap.message.Utils;
import org.act.soap.util.i18n.Message;

public class PlainTextDCH implements DataContentHandler {
    
    private static final Class JAVA_LANG_STRING = String.class;
    
    private static final ActivationDataFlavor DATA_FLAVOR = 
                        new ActivationDataFlavor(JAVA_LANG_STRING,
                                                 Constants.MIME_TYPE_TEXT_PLAIN, 
                                                 "Plain Text");
    
    public final DataFlavor[] getTransferDataFlavors () {
        return new DataFlavor[] { DATA_FLAVOR };
    }
    
    public final Object getTransferData (DataFlavor df, DataSource ds)
        throws IOException, UnsupportedFlavorException {  
        if (DATA_FLAVOR.equals(df)) {
            return getContent(ds);
        } else {
            throw new UnsupportedFlavorException(df);
        }
    }
    
    public final Object getContent (DataSource ds) throws IOException {
        
        String charset = Utils.getCharset(ds.getContentType());
        
        InputStreamReader isr = new InputStreamReader(ds.getInputStream(), 
                                                      charset);
        
        StringBuffer buf = new StringBuffer(100);
        final char[] data = new char[1024];
        int bytes_read = 0;
        try {
            do {
                bytes_read = isr.read(data);
                if (bytes_read <= 0) {
                    break;
                }
                buf.append(data, 0, bytes_read);
            } while (true);
        } catch (IOException ioe) {
            throw ioe;
        } finally {
            if ( null != isr ) {
                isr.close();
            }
        }
        return buf.toString();
    }
    
    public final void writeTo (Object obj, String mimeType, OutputStream os)
        throws IOException {
                if ( !(obj instanceof String) )
            throw new IllegalArgumentException(
                Message.getMessage("SAAJ_E_DCH_UNEXPECTED_OBJ", 
                                   obj.getClass().getName(),
                                   JAVA_LANG_STRING.getName()));
                if (!mimeType.startsWith(Constants.MIME_TYPE_TEXT_PLAIN))
            throw new IllegalArgumentException(
                Message.getMessage("SAAJ_E_DCH_UNEXPECTED_MTYPE", 
                                   mimeType,
                                   Constants.MIME_TYPE_TEXT_PLAIN));
                String charset = Utils.getCharset(mimeType);
                Writer out = new OutputStreamWriter(os, charset);
        out.write((String) obj);
        out.flush();
        return;
    }
}
