package org.act.soap.message.dch;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.OutputStream;
import javax.activation.ActivationDataFlavor;
import javax.activation.DataContentHandler;
import javax.activation.DataSource;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.act.soap.message.Constants;
import org.act.soap.message.Utils;
import org.act.soap.util.i18n.Message;

public class XmlDCH implements DataContentHandler {
    
    private static final Class XML_SOURCE = Source.class;
    
    private final static DataFlavor[] DATA_FLAVORS = 
                new DataFlavor[] {
                        new ActivationDataFlavor(XML_SOURCE,
                                                 Constants.MIME_TYPE_TEXT_XML,
                                                 "XML Text"),
                        new ActivationDataFlavor(XML_SOURCE,
                                                 Constants.MIME_TYPE_APP_SOAP,
                                                 "XML Text")};
    
    public final DataFlavor[] getTransferDataFlavors () {
        return DATA_FLAVORS;
    }
    
    public final Object getTransferData (DataFlavor df, DataSource ds)
        throws IOException, UnsupportedFlavorException {  
        if (DATA_FLAVORS[0].equals(df) || DATA_FLAVORS[1].equals(df)) {
            return getContent(ds);
        } else {
            throw new UnsupportedFlavorException(df);
        }
    }
    
    public final Object getContent (DataSource ds) throws IOException {
        return new StreamSource(ds.getInputStream());
    }
    
    public final void writeTo (Object obj, String mimeType, OutputStream os)
        throws IOException {
                if ( !(obj instanceof Source) )
            throw new IllegalArgumentException(
                Message.getMessage("SAAJ_E_DCH_UNEXPECTED_OBJ", 
                                   obj.getClass().getName(),
                                   XML_SOURCE.getName()));
                if (!(mimeType.startsWith(Constants.MIME_TYPE_TEXT_XML) || 
              mimeType.startsWith(Constants.MIME_TYPE_APP_SOAP)))
            throw new IllegalArgumentException(
                Message.getMessage("SAAJ_E_DCH_UNEXPECTED_MTYPE", 
                                   mimeType,
                                   Constants.MIME_TYPE_TEXT_XML,
                                   Constants.MIME_TYPE_APP_SOAP));
                try {
            Transformer transformer = Utils.getTransformer();
            StreamResult result = new StreamResult(os);
            transformer.transform( (Source) obj, result);
        } catch (TransformerConfigurationException tce) {
            throw new IOException(Message.getMessage("SAAJ_E_TRANS_CONFIG") +
                                  tce.getMessage());
        } catch (TransformerException te) {
            throw new IOException(Message.getMessage("SAAJ_E_TRANSFORM") +
                                  te.getMessage());
        }
    }
}
