package org.act.soap.message.transport;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.Iterator;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeader;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import org.act.soap.message.Constants;
import org.act.soap.message.MessageFactoryImpl;
import org.act.soap.util.i18n.Message;

final class ConnectionImpl extends SOAPConnection {
    
    public SOAPMessage call (SOAPMessage request, Object to)
        throws SOAPException {
                if (null == request || null == to)
            throw new IllegalArgumentException(
                    Message.getMessage("SAAJ_E_NULL_ARG"));
                URL url = null;
        if ( to instanceof String )
            try {
                url = new URL( (String) to);
            } catch ( MalformedURLException e) {
                throw new SOAPException(
                    Message.getMessage("SAAJ_E_CON_MAL_URL", (String) to), e);
            }
        else
            if ( to instanceof URL )
                url = (URL) to;
            else
                throw new SOAPException(
                        Message.getMessage("SAAJ_E_CON_UNK_EP_TYPE",
                                           to.getClass().getName()));
                String protocol = url.getProtocol();
        if ( "http".equals(protocol) )
            return sendByHttp(request, url);
        if ( "https".equals(protocol) )
            return sendByHttps(request, url);
        if ( "mailto".equals(protocol) )
            return sendBySmtp(request, url);
                throw new SOAPException(
                Message.getMessage("SAAJ_E_CON_UNSUP_PRO", protocol));
    }
    
    public void close () throws SOAPException {
    }
    
    private static SOAPMessage sendByHttp (SOAPMessage reqMsg, URL to)
        throws SOAPException {
        
        reqMsg.saveChanges();
        
        HttpURLConnection con = null;
        try {
            con = (HttpURLConnection) to.openConnection();
            con.setRequestMethod(Constants.HEADER_POST);
        } catch (IOException ioe) {
            throw new SOAPException(Message.getMessage("SAAJ_E_IOE"), ioe);
        }
        con.setDoOutput(true);
        con.setDoInput(true);
                        
        con.setRequestProperty(Constants.HEADER_USER_AGENT, "ACTSOAP 1.0");
        con.setRequestProperty(Constants.HEADER_ACCEPT,
                        "application/soap+xml, multipart/related, text/xml");
        MimeHeaders headers = reqMsg.getMimeHeaders();
        for (Iterator iter = headers.getAllHeaders(); iter.hasNext(); ) {
            MimeHeader header = (MimeHeader) iter.next();
            con.setRequestProperty(header.getName(), header.getValue());
        }
        
        try {
            con.connect();
        } catch (IOException ioe) {
            throw new SOAPException(
                Message.getMessage("SAAJ_E_CON_UNABLE_EST_CON"), ioe);
        }
        
        try {
            OutputStream out = con.getOutputStream();
            reqMsg.writeTo(out);
            out.flush();
            out.close();
        } catch (IOException ioe) {
            throw new SOAPException(Message.getMessage("SAAJ_E_IOE"), ioe);
        }
        
        InputStream resStream = null;          try {
            final int resCode = con.getResponseCode();
            if (resCode == HttpURLConnection.HTTP_INTERNAL_ERROR)
                resStream = con.getErrorStream();
            else {
                if (resCode / 100 != 2)
                    throw new SOAPException(
                        Message.getMessage("SAAJ_E_CON_SEND_FAIL",
                                           Integer.toString(resCode)));
                resStream = con.getInputStream();
            }
        } catch(IOException ioe) {
            throw new SOAPException(Message.getMessage("SAAJ_E_IOE"), ioe);
        }
        
        MimeHeaders resHeaders = new MimeHeaders();
                String ct = con.getHeaderField(Constants.HEADER_CONTENT_TYPE);
        if ( null == ct )
            throw new SOAPException(
                        Message.getMessage("SAAJ_E_MSG_ABSENT_CTYPE"));
        resHeaders.addHeader(Constants.HEADER_CONTENT_TYPE, ct);
                String cd =con.getHeaderField(Constants.HEADER_CONTENT_DESCRIPTION);
        if ( null != cd )
            resHeaders.addHeader(Constants.HEADER_CONTENT_DESCRIPTION, cd);
                String cl = con.getHeaderField(Constants.HEADER_CONTENT_LOCATION);
        if ( null != cl )
            resHeaders.addHeader(Constants.HEADER_CONTENT_LOCATION, cl);
        
        SOAPMessage resMsg = null;
        try {
            MessageFactory mf = new MessageFactoryImpl();
            resMsg = mf.createMessage(resHeaders, resStream);
                        resMsg.getSOAPBody();
        } catch(IOException ioe) {
            throw new SOAPException(Message.getMessage("SAAJ_E_IOE"), ioe);
        }
        
        
        con.disconnect();
        return resMsg;
    }
    
    private static SOAPMessage sendByHttps (SOAPMessage reqMsg, URL to)
        throws SOAPException {
        
        throw new SOAPException(
                Message.getMessage("SAAJ_E_CON_UNSUP_PRO", to.getProtocol()));
    }
    
    private static SOAPMessage sendBySmtp (SOAPMessage reqMsg, URL to)
        throws SOAPException {
        
        throw new SOAPException(
                Message.getMessage("SAAJ_E_CON_UNSUP_PRO", to.getProtocol()));
    }
}
