package org.act.soap.server;
import java.util.HashMap;
import javax.security.auth.Subject;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.xml.rpc.encoding.TypeMappingRegistry;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPMessage;
import org.w3c.dom.*;
import org.act.soap.util.i18n.Message;

public class MessageContext {
  
  private String mTransportProtocol = null;
  
  private BindingOperation mBindingOperation = null;
  
  private HashMap mPropertyTable = null;
  
  private SOAPMessage mRequestMessage = null;
  
  private SOAPMessage mResponseMessage = null;
  
  private String mTransportUserName = null;
  
  private Subject mUserSubject = null;
  
  private MimeHeaders requestMimeHeaders = null;
  
  private WebServiceObject wso = null;
  
  public MessageContext() {
  }
  
  public MessageContext(WebServiceObject newWSO) {
    this.wso = newWSO;
  }
  
  protected void setWebServiceObject(WebServiceObject newWSO) {
    this.wso = newWSO;
  }
  
  public void clear() {
    this.mTransportProtocol = null;
    this.mBindingOperation = null;
    this.mRequestMessage = null;
    this.mResponseMessage = null;
    this.mTransportUserName = null;
    this.mUserSubject = null;
    if (null != this.mPropertyTable) {
      this.mPropertyTable.clear();
    }
    this.mPropertyTable = null;
    this.requestMimeHeaders = null;
    this.wso = null;
  }
  
  public String getTransportProtocol() {
    return this.mTransportProtocol;
  }
  
  public BindingOperation getBindingOperation() {
    return this.mBindingOperation;
  }
  
  public MimeHeaders getMimeHeaders() {
    return this.requestMimeHeaders;
  }
  
  public Object getProperty(String name) {
    if ( (null != name) && (null != mPropertyTable)) {
      return mPropertyTable.get(name);
    }
    return null;
  }
  
  public SOAPMessage getRequestMessage() {
    return this.mRequestMessage;
  }
  
  public SOAPMessage getResponseMessage() {
    return this.mResponseMessage;
  }
  
  public String getTransportUserName() {
    return this.mTransportUserName;
  }
  
  public TypeMappingRegistry getTypeMappingRegistry() {
    if (null != this.wso) {
      return this.wso.getTypeMappingRegistry();
    }
    return null;
  }
  
  public Subject getUserSubject() {
    return this.mUserSubject;
  }
  
  public String getWebServiceName() {
    if (null != this.wso) {
      return this.wso.getServiceName();
    }
    return null;
  }
  public String getWorkingPath() {
    if (null != this.wso) {
      return this.wso.getWorkingPath();
    }
    return null;
  }
  
  public Definition getWSDLDefinition() {
    if (null != this.wso) {
      return this.wso.getWSDLDefinition();
    }
    return null;
  }
  
  public void setTransportProtocol(String transportProtocol) {
    this.mTransportProtocol = transportProtocol;
  }
  
  public void setBindingOperation(BindingOperation newBindingOperation) {
    this.mBindingOperation = newBindingOperation;
  }
  
  public void setMimeHeaders(MimeHeaders newRequestMimeHeaders) {
    this.requestMimeHeaders = newRequestMimeHeaders;
  }
  
  public void setProperty(String name, Object value) {
    if ( (null == name) || (null == value)) {
      throw new IllegalArgumentException(Message.
                                         getMessage("invalidPropety"));
    }
    if (null == mPropertyTable) {
      mPropertyTable = new HashMap();
    }
    mPropertyTable.put(name, value);
  }
  
  public void setRequestMessage(SOAPMessage reqMsg) {
    mRequestMessage = reqMsg;
  }
  
  public void setResponseMessage(SOAPMessage respMsg) {
    mResponseMessage = respMsg;
  }
  
  public void setTransportUserName(String sUserName) {
    this.mTransportUserName = sUserName;
  }
  
  public void setUserSubject(Subject userSubject) {
    this.mUserSubject = userSubject;
  }
}
