package org.act.soap.server;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Input;
import javax.wsdl.Operation;
import javax.wsdl.Part;
import javax.wsdl.extensions.soap.SOAPBinding;
import javax.wsdl.extensions.soap.SOAPOperation;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import org.act.soap.server.config.ConfigManager;
import org.act.soap.server.deploy.DeployEventListener;
import org.act.soap.server.deploy.DeployException;
import org.act.soap.server.deploy.DeployInfo;
import org.act.soap.server.exception.ExceptionHandler;
import org.act.soap.server.exception.ExceptionHandlerFactory;
import org.act.soap.server.handler.Handler;
import org.act.soap.server.handler.HandlerFactory;
import org.act.soap.server.provider.Provider;
import org.act.soap.server.provider.ProviderFactory;
import org.act.soap.server.rbac.RBACFactory;
import org.act.soap.server.rbac.RBAC;
import org.act.soap.server.transport.Transport;
import org.act.soap.server.transport.TransportContext;
import org.act.soap.server.transport.TransportException;
import org.act.soap.server.transport.TransportFactory;
import org.act.soap.util.Constants;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;
import org.act.soap.util.TMConstants;
import org.act.soap.util.XMLUtils;
import org.act.soap.wsdd.Deployment;
import org.act.soap.wsdd.Parameter;
import org.act.soap.wsdd.Transports;
import org.w3c.dom.*;
import java.util.Vector;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.UnknownExtensibilityElement;

public class SOAPEngine
    implements DeployEventListener {
    
    public static final int SOAPENGINE_STATE_STARTING = 0;
    
    public static final int SOAPENGINE_STATE_RUNNING = 1;
    
    public static final int SOAPENGINE_STATE_SUSPEND = 2;
    
    public static final int SOAPENGINE_STATE_STOPING = -1;
    
    public static final int SOAPENGINE_STATE_STOPPED = -2;
    
    private static Log log = LogFactory.getLog(SOAPEngine.class.getName());
    
    private static SOAPEngine soapEngine = null;
    
    private int soapEngineState = SOAPENGINE_STATE_STOPPED;
    
    private ServiceManager serviceManager = null;
    
    private ExceptionHandlerFactory exceptionHandlerFactory = null;
    
    private ProviderFactory providerFactory = null;
    
    private HandlerFactory handlerFactory = null;
    
    private RBACFactory rbacFactory = null;
    
    private TransportFactory transportFactory = null;
    
    private HashMap messageEntries = new HashMap();
    
    public static SOAPEngine getInstance() {
                if (null == soapEngine) {
            soapEngine = new SOAPEngine();
        }
        return soapEngine;
    }
    
    private SOAPEngine() {
    }
    
    public void start() throws SOAPEngineException {
                log.info(Message.getMessage("INFO_SE_starting"));
                if (this.soapEngineState != SOAPEngine.SOAPENGINE_STATE_STOPPED) {
            throw new SOAPEngineException(Message.getMessage(
                "ERR_SE_cannotStart"));
        }
        try {
                        this.soapEngineState = SOAPEngine.SOAPENGINE_STATE_STARTING;
                        this.serviceManager = new ServiceManager();
                        ConfigManager configManager = ConfigManager.getInstance();
            Element elmConfiguration = XMLUtils.getElementFromFile(
                configManager.getConfigFileName());
                        HashMap configEHF = configManager.getExceptionHandlerFactory();
            HashMap configPF = configManager.getProviderFactory();
            HashMap configRF = configManager.getRBACFactory();
            HashMap configHF = configManager.getHandlerFactory();
            HashMap configTF = configManager.getTransportFactory();
                        Iterator iterator = null;
                        
            exceptionHandlerFactory = new
                ExceptionHandlerFactory();
            iterator = configEHF.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry entry = (Map.Entry) iterator.next();
                exceptionHandlerFactory.registerExceptionHandler( (String)
                    entry.getKey(),
                    (String) entry.getValue());
            }
                        providerFactory = ProviderFactory.newInstance();
            iterator = configPF.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry entry = (Map.Entry) iterator.next();
                providerFactory.registerProvider( (String) entry.getKey(),
                                                 (String) entry.getValue());
            }
                        rbacFactory = RBACFactory.newInstance();
            iterator = configRF.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry entry = (Map.Entry) iterator.next();
                rbacFactory.registerRBAC( (String) entry.getKey(),
                                         (String) entry.getValue());
            }
                        handlerFactory = HandlerFactory.newInstance();
            iterator = configHF.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry entry = (Map.Entry) iterator.next();
                handlerFactory.registerHandler( (String) entry.getKey(),
                                               (String) entry.getValue());
            }
                        transportFactory = TransportFactory.newInstance();
            iterator = configTF.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry entry = (Map.Entry) iterator.next();
                transportFactory.registerTransport( (String) entry.getKey(),
                    (String) entry.getValue());
            }
                        this.soapEngineState = SOAPEngine.SOAPENGINE_STATE_RUNNING;
                        log.info(Message.getMessage("INFO_SE_startSuccess"));
        }
        catch (Exception ex) {
        	ex.printStackTrace();
            throw new SOAPEngineException(Message.getMessage(
                "ERR_SE_wrongStart"), ex);
        }
    }
    
    public void stop() throws SOAPEngineException {
                if ( (this.soapEngineState != SOAPEngine.SOAPENGINE_STATE_RUNNING) &&
            (this.soapEngineState != SOAPEngine.SOAPENGINE_STATE_SUSPEND)) {
            throw new SOAPEngineException(Message.getMessage(
                "ERR_SE_cannotStop"));
        }
                this.soapEngineState = SOAPEngine.SOAPENGINE_STATE_STOPING;
                this.serviceManager = null;
                exceptionHandlerFactory = null;
        providerFactory = null;
        handlerFactory = null;
        transportFactory = null;
                this.soapEngineState = SOAPEngine.SOAPENGINE_STATE_STOPPED;
    }
    
    public TransportContext invoke(TransportContext transportContext) throws
        TransportException, SOAPEngineException {
                        ClassLoader originClassLoader = Thread.currentThread().
            getContextClassLoader();
                Transport transport = transportFactory.createTransport(transportContext.
            getProtocolName());
                MessageContext messageContext = new MessageContext();
        try {
                        if (this.soapEngineState != SOAPEngine.SOAPENGINE_STATE_RUNNING) {
                throw new SOAPEngineException(Message.getMessage(
                    "ERR_SE_cannotInvoke"));
            }
                        
            String requestURL = transportContext.getRequestURL();
            Iterator iterator = this.messageEntries.keySet().iterator();
            if(iterator != null){
                while(iterator.hasNext()){
                    Object obj = iterator.next();
                    System.err.println(obj + "--------" + this.messageEntries.get(obj));
                }
            }
            String serviceName = (String)this.messageEntries.get(requestURL);
            if (null == serviceName) {
                throw new SOAPEngineException(Message.getMessage(
                    "ERR_SE_cannotFindService", requestURL));
            }
                        WebServiceObject wso = this.serviceManager.getWebServiceObject(
                serviceName);
                        Thread.currentThread().setContextClassLoader(wso.getClassLoader());
                        messageContext.setWebServiceObject(wso);
                                    transport.process(transportContext, messageContext);
                        System.err.println(messageContext.getRequestMessage().toString());
                                    List handersList = wso.getHandlersList();
            for (int i = 0; i < handersList.size(); i++) {
                Handler handler = (Handler) handersList.get(i);
                handler.process(messageContext);
            }
                                    BindingOperation bindingOperation = getBindingOperation(
                messageContext);
            messageContext.setBindingOperation(bindingOperation);
                                    RBAC rbac = wso.getRBAC();
            if (null != rbac) {
                rbac.process(messageContext);
            }
                                                Provider provider = wso.getProvider();
            provider.process(messageContext);
                                    for (int i = handersList.size() - 1; 0 <= i; i--) {
                Handler handler = (Handler) handersList.get(i);
                handler.postProcess(messageContext);
            }
        }
        catch (Exception ex) {
            processException(ex, messageContext);
        }
        finally {
                        Thread.currentThread().setContextClassLoader(originClassLoader);
        }
                        transport.postProcess(transportContext, messageContext);
                return transportContext;
    }
    
    public Definition getWSDLDefination(String requestURL) throws
        SOAPEngineException {
                if (this.soapEngineState != SOAPEngine.SOAPENGINE_STATE_RUNNING) {
            throw new SOAPEngineException(Message.getMessage(
                "ERR_SE_cannotInvoke"));
        }
                
        String serviceName = (String)this.messageEntries.get(requestURL);
        if (null == serviceName) {
            throw new SOAPEngineException(Message.getMessage(
                "ERR_SE_cannotFindService", requestURL));
        }
                try {
            WebServiceObject wso = this.serviceManager.getWebServiceObject(
                serviceName);
            return wso.getWSDLDefinition();
        }
        catch (InvalidWSOException ex) {
            throw new SOAPEngineException(ex);
        }
    }
    
    public void suspend() throws SOAPEngineException {
                if (this.soapEngineState != SOAPEngine.SOAPENGINE_STATE_RUNNING) {
            throw new SOAPEngineException(Message.getMessage(
                "ERR_SE_cannotSuspend"));
        }
        this.soapEngineState = this.SOAPENGINE_STATE_SUSPEND;
    }
    
    public void onDeploy(DeployInfo deployInfo) throws DeployException {
                String serviceName = deployInfo.getServiceName();
                Deployment wsddDeployment = deployInfo.getWSDDDeployment();
                Transports transports = wsddDeployment.getTransports();
        if (null == transports) {
            throw new DeployException(Message.getMessage("ERR_SE_noWSDD"));
        }
        List vecTransport = transports.getAllTransport();
        if (0 == vecTransport.size()) {
            throw new DeployException(Message.getMessage("ERR_SE_noWSDD"));
        }
                for (int i = 0; i < vecTransport.size(); i++) {
            org.act.soap.wsdd.Transport transport
                = (org.act.soap.wsdd.Transport) vecTransport.get(i);
            
            Parameter parameter = transport.getParameter("URLPattern");
            String urlPattern = parameter.getValue();
                        synchronized (messageEntries) {
                this.messageEntries.put(urlPattern, serviceName);
            }
        }
    }
    
    public void onUndeploy(DeployInfo deployInfo) throws DeployException {
                String serviceName = deployInfo.getServiceName();
                synchronized (messageEntries) {
            Iterator iterator = this.messageEntries.values().iterator();
            while (iterator.hasNext()) {
                String strServiceName = (String) iterator.next();
                if (strServiceName.equals(serviceName)) {
                    iterator.remove();
                }
            }
        }
        if (this.serviceManager != null) {
            this.serviceManager.removeWebServiceObject(serviceName);
        }
    }
    
    public ServiceManager getServiceManager() {
        return this.serviceManager;
    }
    
    public int getEngineState() {
        return this.soapEngineState;
    }
    
    public HandlerFactory getHandlerFactory() {
        return this.handlerFactory;
    }
    
    public ProviderFactory getProviderFactory() {
        return this.providerFactory;
    }
    
    public RBACFactory getRBACFactory() {
        return this.rbacFactory;
    }
    
    private void processException(Exception ex, MessageContext messageContext) {
        ExceptionHandler exHandler = this.exceptionHandlerFactory.
            getExceptionHandler(ex);
        exHandler.process(ex, messageContext);
    }
        
    private BindingOperation getBindingOperation(MessageContext messageContext) throws
        SOAPEngineException {
                SOAPElement firstSOAPElement =
            getFirstElementInBody(messageContext.getRequestMessage());
                String operationName
            = firstSOAPElement.getElementName().getLocalName();
                List parameterElements = getSOAPParameterElements(firstSOAPElement);
                List operations =
            getBindingOperations(messageContext.getWSDLDefinition());
                String soapBindingStyle =
            getSOAPBindingStyle(messageContext.getWSDLDefinition());
                for (int i = 0; i < operations.size(); i++) {
            BindingOperation bindingOperation =
                (BindingOperation) operations.get(i);
                        if (!bindingOperation.getName().equals(operationName))continue;
                        String style = getOperationStyle(soapBindingStyle, bindingOperation);
            if (style.equals(Constants.PROCESS_STYLE_DOCUMENT))
                return bindingOperation;
                        if (isMatchedOperation(
                bindingOperation.getOperation(), parameterElements,
                messageContext.getWSDLDefinition())) {
                return bindingOperation;
            }
        }
        return null;
    }
    
    private SOAPElement getFirstElementInBody(SOAPMessage requestMessage) throws
        SOAPEngineException {
        SOAPBody soapBody = null;
        try {
            soapBody = requestMessage.getSOAPBody();
        }
        catch (SOAPException ex) {
            throw new SOAPEngineException(
                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
        }
        Iterator iterator = soapBody.getChildElements();
        while (iterator.hasNext()) {
            Object subElement = iterator.next();
            if (subElement instanceof SOAPElement) {
                return (SOAPElement) subElement;
            }
        }
        throw new SOAPEngineException(Message.getMessage(
            "noSOAPElementInSOAPBody"));
    }
    
    private List getSOAPParameterElements(SOAPElement firstSOAPElement) {
        Iterator subElements = firstSOAPElement.getChildElements();
        List parameterElements = new ArrayList();
        while (subElements.hasNext()) {
            Object subElement = subElements.next();
            if (subElement instanceof SOAPElement) {
                parameterElements.add( (SOAPElement) subElement);
            }
        }
        return parameterElements;
    }
    
    private List getBindingOperations(Definition definition) throws
        SOAPEngineException {
        if (definition == null) {
            throw new SOAPEngineException(
                Message.getMessage("nullDefinition"));
        }
        if (!definition.getBindings().values().iterator().hasNext()) {
            throw new SOAPEngineException(
                Message.getMessage("wsdlFileException"));
        }
        Binding binding =
            (Binding) definition.getBindings().values().iterator().next();
        return binding.getBindingOperations();
    }
    
    private String getSOAPBindingStyle(Definition definition) throws
        SOAPEngineException {
        Binding binding =
            (Binding) definition.getBindings().values().iterator().next();
        if (1 > binding.getExtensibilityElements().size()) {
            throw new SOAPEngineException(
                Message.getMessage("wsdlFileException"));
        }
        SOAPBinding soapBinding =
            (SOAPBinding) binding.getExtensibilityElements().get(0);
        return soapBinding.getStyle();
    }
    
    private String getOperationStyle(String soapBindingStyle,
                                     BindingOperation bindingOperation) {
        SOAPOperation soapOperation =
            (SOAPOperation) bindingOperation.getExtensibilityElements().get(0);
        String style = Constants.PROCESS_STYLE_DOCUMENT;
        if (soapOperation.getStyle() != null) {
            style = soapOperation.getStyle();
        }
        else if (soapBindingStyle != null) {
            style = soapBindingStyle;
        }
        return style;
    }
    
    private boolean isMatchedOperation(Operation operation,
                                       List parameterElements,
                                       Definition wsdlDefinition) throws
        SOAPEngineException {
        Input input = operation.getInput();
        javax.wsdl.Message inputMessage = input.getMessage();
        List parts =
            inputMessage.getOrderedParts(operation.getParameterOrdering());
        if (parts.size() != parameterElements.size())return false;
        for (int j = 0; j < parts.size(); j++) {
            SOAPElement parameterElement =
                (SOAPElement) parameterElements.get(j);
            if (null == parameterElement) {
                return false;
            }
            String parameterType =
                parameterElement.getAttributeValue(TMConstants.NAME_XSITYPE);
            String parameterTypePrefix =
                XMLUtils.getPrefixFromQName(parameterType);
            String parameterTypeLocalPart =
                XMLUtils.getLocalPartFromQName(parameterType);
            String parameterTypeURI =
                parameterElement.getNamespaceURI(parameterTypePrefix);
            QName paramType =
                new QName(parameterTypeURI, parameterTypeLocalPart);
            Part part = (Part) parts.get(j);
            QName partType = part.getTypeName();
            
                        if (TMConstants.SOAP_ARRAY.equals(paramType)) {
                if (isArrayTypeMatch(parameterElement, partType, wsdlDefinition)) {
                    return true;
                }
            }
            if (!paramType.equals(partType)) {
                return false;
            }
        }
        return true;
    }
            private boolean isArrayTypeMatch(SOAPElement parameterElement,
                                     QName partType, Definition wsdlDefinition) {
                        List schemas = wsdlDefinition.getTypes().getExtensibilityElements();
        if (schemas == null) {
            return false;
        }
        for (Iterator i = schemas.iterator(); i.hasNext(); ) {
                        UnknownExtensibilityElement schemaExtElement = (
                UnknownExtensibilityElement) i.next();
            Element schemaElement = schemaExtElement.getElement();
            if (!TMConstants.SCHEMA.equals(schemaElement.getLocalName())) {
                continue;             }
            String targetNamespace = schemaElement.getAttribute(TMConstants.
                TARGET_NAMESPACE);
                        Vector children
                = XMLUtils.getChildElementsByNS(schemaElement,
                                                
                                                TMConstants.URI_2001_SCHEMA_XSD,
                                                TMConstants.SCHEMA_COMPLEXTYPE);
            if (children == null) {
                continue;
            }
            for (int j = 0; j < children.size(); j++) {
                Element complexElement = (Element) children.get(j);
                String typeName = complexElement.getAttribute(TMConstants.
                    SCHEMA_NAME);
                QName typeQName = new QName(targetNamespace, typeName);
                                if (partType.equals(typeQName)) {
                                        Element complexContent = XMLUtils.getFirstChildElement(
                        complexElement);
                    if (complexContent != null) {
                        Element restriction = XMLUtils.getFirstChildElement(
                            complexContent);
                        if (restriction != null) {
                            Element attribute = XMLUtils.getFirstChildElement(
                                restriction);
                            if (attribute != null) {
                                                                String soapArrayType =
                                    parameterElement.getAttributeValue(
                                    TMConstants.NAME_SOAPENCARRAY_TYPE);
                                String soapPrefix = XMLUtils.getPrefixFromQName(
                                    soapArrayType);
                                String soapLocalPart = XMLUtils.
                                    getLocalPartFromQName(soapArrayType);
                                                                soapLocalPart = soapLocalPart.substring(0,
                                    soapLocalPart.lastIndexOf('['));
                                String soapURI = parameterElement.
                                    getNamespaceURI(soapPrefix);
                                QName soapArrayTypeQName = new QName(soapURI,
                                    soapLocalPart);
                                                                String wsdlArrayType
                                    = attribute.getAttributeNS(TMConstants.
                                    NS_URI_WSDL, TMConstants.ATTR_ARRAY_TYPE);
                                String wsdlPrefix = XMLUtils.getPrefixFromQName(
                                    wsdlArrayType);
                                String wsdlLocalPart = XMLUtils.
                                    getLocalPartFromQName(wsdlArrayType);
                                                                wsdlLocalPart = wsdlLocalPart.substring(0,
                                    wsdlLocalPart.lastIndexOf('['));
                                String wsdlURI = wsdlDefinition.getNamespace(
                                    wsdlPrefix);
                                QName wsdlArrayTypeQName = new QName(wsdlURI,
                                    wsdlLocalPart);
                                                                if (soapArrayTypeQName.equals(
                                    wsdlArrayTypeQName)) {
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
        }
        return false;
    }
}
