package org.act.soap.server;
import java.io.File;
import java.util.HashMap;
import java.util.List;
import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.WSDLException;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.DeserializerFactory;
import javax.xml.rpc.encoding.SerializerFactory;
import javax.xml.rpc.encoding.TypeMappingRegistry;
import org.act.soap.encoding.TypeMappingRegistryImpl;
import org.act.soap.server.handler.Handler;
import org.act.soap.server.rbac.*;
import org.act.soap.server.handler.HandlerFactory;
import org.act.soap.server.deploy.*;
import org.act.soap.server.provider.*;
import org.act.soap.util.ClassLoaderUtils;
import org.act.soap.util.i18n.Message;
import org.act.soap.wsdd.Deployment;
import org.act.soap.wsdd.WSDDException;

public class ServiceManager {
    
    protected HashMap wsoMap = new HashMap();
    
    public ServiceManager () {
    }
    
    public void removeWebServiceObject (String serviceName) {
                synchronized (this.wsoMap) {
            WebServiceObject wso = (WebServiceObject)this.wsoMap.remove(
                serviceName);
            if (wso != null) {
                wso.clearHandlersList();
            }
        }
    }
    
    public WebServiceObject getWebServiceObject (String
                                                 serviceName) throws
        InvalidWSOException {
        if (null == serviceName) {
            InvalidWSOException ex = new InvalidWSOException("invalidWSO");
            ex.setCode("Server");
            throw ex;
        }
                WebServiceObject wso = (WebServiceObject)this.wsoMap.get(serviceName);
        System.err.println(1);
        if (null != wso)
            return wso;
                        try {
            wso = this.createWebServiceObject(serviceName);
            synchronized (this.wsoMap) {
                        System.err.println(3);
                this.wsoMap.put(serviceName, wso);
                        System.err.println(4);
                return wso;
            }
        }
        catch (InvalidWSOException ex) {
                        throw new InvalidWSOException(Message.getMessage("ERR_GET_WSO",
                ex.getMessage()),
                                          ex,
                                          "Server");
        }
    }
    
    private WebServiceObject createWebServiceObject (String serviceName) throws
        InvalidWSOException {
        ClassLoader originClassLoader = null;
        originClassLoader = Thread.currentThread().getContextClassLoader();
        try {
            WebServiceObject wso = new WebServiceObject();
            wso.setServiceName(serviceName);
                                    String workingPath = DeployManager.getInstance().
                getServiceWorkingPath(
                serviceName);
                        wso.setWorkingPath(workingPath);
                        ClassLoader classLoader = ClassLoaderUtils.getWebServiceClassLoader(
                workingPath, originClassLoader);
            wso.setClassLoader(classLoader);
                        Thread.currentThread().setContextClassLoader(classLoader);
                        Deployment deployment = DeployManager.getInstance().getWSDDDeployment(serviceName);
            wso.setWSDDDeployment(deployment);
                                    String wsdlLocation = deployment.getWSDLLocation();
            File wsdlFile = new File(wsdlLocation);
            if (!wsdlFile.isAbsolute()) {
                wsdlFile = new File(workingPath + File.separator + "META-INF" +
                                    File.separator + wsdlLocation);
            }
                        WSDLFactory factory = WSDLFactory.newInstance();
            WSDLReader wsdlReader = factory.newWSDLReader();
            wsdlReader.setFeature("javax.wsdl.verbose", false);
            Definition definition = wsdlReader.readWSDL(wsdlFile.toString());
            wso.setWSDLDefinition(definition);
                        Provider provider = SOAPEngine.getInstance().getProviderFactory().
                createProvider(deployment.getProvider().getType());
                                    provider.init(deployment.getProvider());
                        wso.setProvider(provider);
                                    org.act.soap.wsdd.RBAC wsddRbac = deployment.getRBAC();
            if( null != wsddRbac ){
                RBACFactory rbacFactory = SOAPEngine.getInstance().getRBACFactory();
                wsddRbac.setDeployment( deployment );
                String strType = wsddRbac.getType();
                RBAC rbac = rbacFactory.createRBAC( strType );
                rbac.init( wsddRbac );
                wso.setRBAC( rbac );
            }
                                    this.setWSOHandlerList(wso, deployment);
                        this.setWSOTypeMappingRegistry(wso, deployment);
            return wso;
        }
        catch (WSDDException ex_wsdd) {
            throw new InvalidWSOException(Message.getMessage(
                "ERR_CREAT_WSO_WRONG_WSDD", ex_wsdd.getMessage()),
                                          ex_wsdd,
                                          "Server");
        }
        catch (DeployException ex_deploy) {
            throw new InvalidWSOException(Message.getMessage(
                "ERR_CREAT_WSO_DEPLOY", ex_deploy.getMessage()),
                                          ex_deploy,
                                          "Server");
        }
        catch (ProviderException ex_provider) {
            throw new InvalidWSOException(Message.getMessage(
                "ERR_CREAT_WSO_PROVIDER", ex_provider.getMessage()),
                                          ex_provider,
                                          "Server");
        }
        catch (WSDLException ex_wsdl) {
            throw new InvalidWSOException(Message.getMessage(
                "ERR_CREAT_WSO_WSDL", ex_wsdl.getMessage()),
                                          ex_wsdl,
                                          "Server");
        }
        catch (Exception e) {
            throw new InvalidWSOException(Message.getMessage("ERR_CREAT_WSO",
                e.getMessage()),
                                          e,
                                          "Server");
        }
        finally {
                        Thread.currentThread().setContextClassLoader(originClassLoader);
        }
    }
    
    private void setWSOTypeMappingRegistry (WebServiceObject wso,
                                            Deployment deployment) throws
        InvalidWSOException {
        TypeMappingRegistry tmr = new TypeMappingRegistryImpl();
        List typeMappingList = null;
        if (deployment.getTypeMappings() != null &&
            deployment.getTypeMappings().getAllTypeMappings() != null) {
            typeMappingList = deployment.getTypeMappings().getAllTypeMappings();
        }
                if (typeMappingList != null) {
            try {
                ClassLoader classLoader = Thread.currentThread().
                    getContextClassLoader();
                for (int i = 0; i < typeMappingList.size(); i++) {
                    org.act.soap.wsdd.TypeMapping wsddTypeMapping
                        = (org.act.soap.wsdd.TypeMapping) typeMappingList.
                        get(
                        i);
                    QName qName = wsddTypeMapping.getQName();
                                        String javaTypeName = wsddTypeMapping.
                        getLanguageSpecificType();
                    Class javaType = classLoader.loadClass(javaTypeName);
                                        String serializerName = wsddTypeMapping.getSerializer();
                    Class serializerClass = classLoader.loadClass(
                        serializerName);
                    SerializerFactory serializer = (SerializerFactory)
                        serializerClass.getConstructor(new Class[] {Class.class,
                        QName.class}).
                        newInstance(new Object[] {javaType, qName});
                                        String deserializerName = wsddTypeMapping.getDeserializer();
                    Class deserializerClass = classLoader.loadClass(
                        deserializerName);
                    DeserializerFactory deserializer = (DeserializerFactory)
                        deserializerClass.getConstructor(new Class[] {Class.class,
                        QName.class}).
                        newInstance(new Object[] {javaType, qName});
                    
                    javax.xml.rpc.encoding.TypeMapping typeMapping = tmr.
                        getDefaultTypeMapping();
                    typeMapping.register(javaType,
                                         qName,
                                         serializer,
                                         deserializer);
                }
            }
            catch (Exception e) {
                throw new InvalidWSOException(Message.getMessage("invalidWSO",
                    e.getMessage()),
                                              e,
                                              "Server");
            }
        }
                wso.setTypeMappingRegistry(tmr);
    }
    
    private void setWSOHandlerList (WebServiceObject wso,
                                    org.act.soap.wsdd.Deployment deployment) throws
        Exception {
        org.act.soap.wsdd.Handlers handlers = deployment.getHandlers();
                if (null == handlers || null == handlers.getAllHandler()) {
            wso.clearHandlersList();
        }
        else {
                        List handlersList = handlers.getAllHandler();
            HandlerFactory handlerFactory = SOAPEngine.getInstance().
                getHandlerFactory();
            for (int i = 0; i < handlersList.size(); i++) {
                org.act.soap.wsdd.Handler wsddHandler
                    = (org.act.soap.wsdd.Handler) handlersList.get(i);
                Handler handler = handlerFactory.createHandler(wsddHandler.
                    getType());
                handler.init(wsddHandler);
                wso.addHandler(handler);
            }
        }
    }
}
