package org.act.soap.server.config;
import java.io.File;
import java.io.FileWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.Map;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.act.soap.util.Constants;
import org.act.soap.util.XMLUtils;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;
import org.act.soap.server.secmanager.keymanager.KeyStoreImpl;
import org.act.soap.server.config.RBACDevice;
import org.act.soap.server.config.UserDevice;
import org.act.soap.server.secmanager.UserManager;
import org.act.soap.server.secmanager.RBACManager;
import org.act.soap.server.secmanager.CredManager;
import java.util.MissingResourceException;
import java.security.Key;
import java.security.cert.Certificate;

public class ConfigManager {
    
    private static Log log = LogFactory.getLog( ConfigManager.class.getName() );
    
    public static final String TAG_NAME_CONFIGURATION = "Configuration";
    
    public static final String TAG_NAME_ENGINE = "Engine";
    
    public static final String TAG_NAME_NAME = "Name";
    
    public static final String TAG_NAME_DESCRIPTION = "Description";
    
    public static final String TAG_NAME_MAXPROCESSORS = "MaxProcessors";
    
    public static final String TAG_NAME_DEPLOYER = "Deployer";
    
    public static final String TAG_NAME_DEPLOYERUPDATEINTERVAL =
        "DeployerUpdateInterval";
    
    public static final String TAG_NAME_DEPLOYSPACE = "DeploySpace";
    
    public static final String TAG_NAME_WORKSAPCE = "WorkSpace";
    
    public static final String TAG_NAME_TRANSPORTFACTORY = "TransportFactory";
    
    public static final String TAG_NAME_TRANSPORT = "Transport";
    
    public static final String TAG_NAME_PROVIDERFACTORY = "ProviderFactory";
    
    public static final String TAG_NAME_PROVIDER = "Provider";
    
    public static final String TAG_NAME_HANDLERFACTORY = "HandlerFactory";
    
    public static final String TAG_NAME_HANDLER = "Handler";
    
    public static final String TAG_NAME_RBACFACTORY = "RBACFactory";
    
    public static final String TAG_NAME_RBAC = "RBAC";
    
    public static final String TAG_NAME_EXCEPTIONHANDLERFACTORY =
        "ExceptionHandlerFactory";
    
    public static final String TAG_NAME_EXCEPTIONHANDLER = "ExceptionHandler";
    
    public static final String TAG_NAME_CREDMANAGER = "CredManager";
    
    public static final String TAG_NAME_CREDDEVICE = "CredDevice";
    
    public static final String TAG_NAME_USERDEVICE = "UserDevice";
    
    public static final String TAG_NAME_RBACDEVICE = "RBACDevice";
    
    public static final String TAG_NAME_DEVICENAME = "DeviceName";
    
    public static final String TAG_NAME_DEVICECONFIG = "DeviceConfig";
    
    public static final String TAG_NAME_KEYSTOREFILE = "KeyStoreFile";
    
    public static final String TAG_NAME_KEYSTOREPASSWORD = "KeyStorePassWord";
    
    public static final String TAG_NAME_SUPPORTEDCREDTYPES =
        "SupportedCredTypes";
    
    public static final String TAG_NAME_CREDTYPE = "CredType";
    
    public static final String TAG_NAME_RBACMANAGER = "RBACManager";
    
    public static final String TAG_NAME_DRIVER = "Driver";
    
    public static final String TAG_NAME_FILENAME = "FileName";
    
    public static final String TAG_NAME_URL = "URL";
    
    public static final String TAG_NAME_USERNAME = "Username";
    
    public static final String TAG_NAME_PASSWORD = "Password";
    
    public static final String TAG_NAME_USERMANAGER = "UserManager";
    
    public static final String TAG_NAME_KEYINFO = "KeyInfo";
    
    public static final String TAG_NAME_AUTHENKEY = "AuthenKey";
    
    public static final String TAG_NAME_SIGNKEY = "SignKey";
    
    public static final String TAG_NAME_AUTHENALIAS = "AuthenAlias";
    
    public static final String TAG_NAME_AUTHENPASSWORD = "AuthenPassword";
    
    public static final String TAG_NAME_SIGNALIAS = "SignAlias";
    
    public static final String TAG_NAME_SIGNPASSWORD = "SignPassword";
    
    public static final String ATTR_NAME_TYPE = "Type";
    
    public static final String ATTR_NAME_CLASSNAME = "ClassName";
    
    private static ConfigManager mConfigManager = null;
    
    private String gEngineName = null;
    
    private String gEngineDescription = null;
    
    private int gMaxProcessors = 0;
    
    private String gAuthenAlias = null;
    
    private String gAuthenPassword = null;
    
    private String gSignAlias = null;
    
    private String gSignPassword = null;
    
    private String gDeployPath = null;
    
    private String gWorkingPath = null;
    
    private long gDeployerUpdateInterval = 0;
    
    private Key authenPrivateKey = null;
    
    private Certificate authenCert = null;
    
    private Key signPrivateKey = null;
    
    private String signPrivateKeyName = null;
    
    private Certificate signCert = null;
    
    private HashMap gTransportFactory = new HashMap();
    
    private HashMap gProviderFactory = new HashMap();
    
    private HashMap gHandlerFactory = new HashMap();
    
    private HashMap gRBACFactory = new HashMap();
    
    private HashMap gExceptionHandlerFactory = new HashMap();
    
    private String configFileName = null;
    
    private HashMap credDevices = new HashMap();
    
    private HashMap userDevices = new HashMap();
    
    private HashMap rbacDevices = new HashMap();
    
    private UserManager userManager;
    
    private CredManager credManager;
    
    private RBACManager rbacManager;
    
    private ConfigManager () {
                        configFileName =
            System.getProperty( Constants.PROPERTY_NAME_CONFIG_FILE_PATH );
        if ( null == configFileName ) {
            configFileName = Constants.DEFAULT_CONFIG_FILE_PATH;
        }
        File file = new File( configFileName );
            }
    
    public static ConfigManager getInstance () {
        if ( null == mConfigManager ) {
            mConfigManager = new ConfigManager();
        }
        return mConfigManager;
    }
    
    private static boolean isValidElement ( Node node, String tagName ) {
        boolean result = true;
        if ( null == node ) {
            result = false;
        }
        if ( node.getNodeType() != Node.ELEMENT_NODE ) {
            result = false;
        } else if ( !node.getNodeName().equals( tagName ) ) {
            result = false;
        }
        return result;
    }
    
    public void load () throws ConfigException {
        Node childNode = null;
        int childNodeLength = 0;
        Element eConfiguration = XMLUtils.getElementFromFile( configFileName );
        if ( null == eConfiguration ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_File",
                configFileName ) );
        }
        
        if ( !isValidElement( eConfiguration, TAG_NAME_CONFIGURATION ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_wrongConfiguration",
                TAG_NAME_CONFIGURATION ) );
        }
        childNodeLength = eConfiguration.getChildNodes().getLength();
        for ( int i = 0; i < childNodeLength; i++ ) {
            childNode = eConfiguration.getChildNodes().item( i );
                        if ( isValidElement( childNode, TAG_NAME_ENGINE ) ) {
                initEngineElement( ( Element ) childNode );
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_DEPLOYER ) ) {
                initDeployerElement( ( Element ) childNode );
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_TRANSPORTFACTORY ) ) {
                initTransportFactoryElement( ( Element ) childNode );
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_PROVIDERFACTORY ) ) {
                initProviderFactoryElement( ( Element ) childNode );
                continue;
            }
                                    if ( isValidElement( childNode, TAG_NAME_RBACFACTORY ) ) {
                initRBACFactoryElement( ( Element ) childNode );
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_HANDLERFACTORY ) ) {
                initHandlerFactoryElement( ( Element ) childNode );
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_EXCEPTIONHANDLERFACTORY ) ) {
                initExceptionHandlerFactoryElement( ( Element ) childNode );
                continue;
            }
                                    if ( isValidElement( childNode, TAG_NAME_CREDMANAGER ) ) {
                initCredManagerElement( ( Element ) childNode );
                continue;
            }
                                    if ( isValidElement( childNode, TAG_NAME_USERMANAGER ) ) {
                initUserManagerElement( ( Element ) childNode );
                continue;
            }
                                    if ( isValidElement( childNode, TAG_NAME_RBACMANAGER ) ) {
                initRBACManagerElement( ( Element ) childNode );
                continue;
            }
                                    if ( isValidElement( childNode, TAG_NAME_KEYINFO ) ) {
                initKeyInfoElement( ( Element ) childNode );
                continue;
            }
        }
    }
    
    public void save () throws ConfigException {
        FileWriter fw = null;
        try {
                        File file = null;
            Document doc1 = XMLUtils.createDocument();
            Element eConfiguration = this.makeElementConfiguration( doc1 );
            doc1.appendChild( eConfiguration );
                        file = new File( configFileName );
                        fw = new FileWriter( file );
            fw.write( XMLUtils.nodeToPrettyString( doc1 ) );
        } catch ( Exception ex ) {
            throw new ConfigException( Message.getMessage( "ERR_CONF_wrongSave" ),
                                       ex );
        } finally {
            try {
                fw.close();
            } catch ( Exception ex ) {
                throw new ConfigException( Message.getMessage(
                    "ERR_CONF_wrongSave" ), ex );
            }
        }
    }
    
    public String getConfigFileName () {
        return this.configFileName;
    }
    
    public UserManager getUserManager () {
        return this.userManager;
    }
    
    void setUserManager ( UserManager userManager ) {
        this.userManager = userManager;
    }
    
    public CredManager getCredManager () {
        CredManager credManager = new CredManager( this.getConfigFileName() );
        return credManager;
    }
    
    void setCredManager ( CredManager credManager ) {
        this.credManager = credManager;
    }
    
    public RBACManager getRBACManager () {
        return this.rbacManager;
    }
    
    void setRBACManager ( RBACManager rbacManager ) {
        this.rbacManager = rbacManager;
    }
    
    public String getEngineName () {
        return this.gEngineName;
    }
    
    public void setEngineName ( String engineName ) throws ConfigException {
        if ( ( null == engineName ) || ( "".equals( engineName ) ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                TAG_NAME_NAME ) );
        }
        this.gEngineName = engineName;
    }
    
    public String getEngineDescription () {
        return this.gEngineDescription;
    }
    
    public void setEngineDescription ( String engineDescription ) {
        if ( null == engineDescription ) {
            this.gEngineDescription = "";
        } else {
            this.gEngineDescription = engineDescription;
        }
    }
    
    public Key getAuthenPrivateKey () {
        return authenPrivateKey;
    }
    
    public void setAuthenPrivateKey ( Key key ) {
        this.authenPrivateKey = key;
    }
    
    public Certificate getAuthenCert () {
        return authenCert;
    }
    
    public void setAuthenCert ( Certificate cert ) {
        this.authenCert = cert;
    }
    
    public Key getSignPrivateKey () {
        return signPrivateKey;
    }
    
    public void setSignPrivateKey ( Key key ) {
        this.signPrivateKey = key;
    }
    
    public String getSignPrivateKeyName () {
        return signPrivateKeyName;
    }
    
    public void setSignPrivateKeyName ( String name ) {
        this.signPrivateKeyName = name;
    }
    
    public Certificate getSignCert () {
        return signCert;
    }
    
    public void setSignCert ( Certificate cert ) {
        this.signCert = cert;
    }
    
    public int getMaxProcessors () {
        return this.gMaxProcessors;
    }
    
    public void setMaxProcessors ( int maxProcessors ) throws ConfigException {
                if ( 0 >= maxProcessors ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                TAG_NAME_MAXPROCESSORS ) );
        }
        this.gMaxProcessors = maxProcessors;
    }
    
    public String getDeployPath () {
        if ( !new File( this.gDeployPath ).isAbsolute() ) {
            File prefix = new File( this.configFileName ).getParentFile().
                getParentFile();
            return new File( prefix, this.gDeployPath ).getAbsolutePath();
        } else {
            return this.gDeployPath;
        }
    }
    
    public void setDeployPath ( String deployPath ) throws ConfigException {
        if ( ( null == deployPath ) || "".equals( deployPath ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                TAG_NAME_DEPLOYSPACE ) );
        }
        this.gDeployPath = deployPath;
    }
    
    public String getWorkingPath () {
        if ( !new File( this.gDeployPath ).isAbsolute() ) {
            File prefix = new File( this.configFileName ).getParentFile().
                getParentFile();
            return new File( prefix, this.gWorkingPath ).getAbsolutePath();
        } else {
            return this.gWorkingPath;
        }
    }
    
    public void setWorkingPath ( String workingPath ) throws ConfigException {
        if ( ( null == workingPath ) || "".equals( workingPath ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                TAG_NAME_WORKSAPCE ) );
        }
        this.gWorkingPath = workingPath;
    }
    
    public long getDeployerUpdateInterval () {
        return this.gDeployerUpdateInterval;
    }
    
    public void setDeployerUpdateInterval ( long deployerUpdateInterval ) throws
        ConfigException {
                if ( 0 >= deployerUpdateInterval ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                TAG_NAME_DEPLOYERUPDATEINTERVAL ) );
        }
        this.gDeployerUpdateInterval = deployerUpdateInterval;
    }
    
    public HashMap getTransportFactory () {
        return this.gTransportFactory;
    }
    
    public void setTransportFactory ( HashMap transportFactory ) {
        this.gTransportFactory = transportFactory;
    }
    
    public HashMap getProviderFactory () {
        return this.gProviderFactory;
    }
    
    public void setProviderFactory ( HashMap providerFactory ) {
        this.gProviderFactory = providerFactory;
    }
    
    public HashMap getRBACFactory () {
        return this.gRBACFactory;
    }
    
    public void setRBACFactory ( HashMap RBACFactory ) {
        this.gRBACFactory = RBACFactory;
    }
    
    public HashMap getHandlerFactory () {
        return this.gHandlerFactory;
    }
    
    public void setHandlerFactory ( HashMap handlerFactory ) {
        this.gHandlerFactory = handlerFactory;
    }
    
    public HashMap getExceptionHandlerFactory () {
        return this.gExceptionHandlerFactory;
    }
    
    public void setExceptionHandlerFactory ( HashMap exceptionHandlerFactory ) {
        this.gExceptionHandlerFactory = exceptionHandlerFactory;
    }
    
    private void initEngineElement ( Element e ) throws ConfigException {
        Node childNode = null;
        String nodeValue = null;
        final int childNodesLength = e.getChildNodes().getLength();
        for ( int i = 0; i < childNodesLength; i++ ) {
            childNode = e.getChildNodes().item( i );
                        if ( isValidElement( childNode, TAG_NAME_NAME ) ) {
                nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                    childNode );
                                if ( null != nodeValue ) {
                    this.setEngineName( nodeValue );
                } else {
                    throw new ConfigException( Message.getMessage(
                        "ERR_CONF_nullConfigElement",
                        TAG_NAME_NAME ) );
                }
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_DESCRIPTION ) ) {
                nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                    childNode );
                this.setEngineDescription( null != nodeValue ? nodeValue : "" );
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_MAXPROCESSORS ) ) {
                nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                    childNode );
                if ( null != nodeValue ) {
                    try {
                        final int intTemp = Integer.parseInt( nodeValue );
                                                if ( 0 >= intTemp ) {
                            throw new ConfigException( Message.getMessage(
                                "ERR_CONF_badConfigFormat",
                                TAG_NAME_MAXPROCESSORS ) );
                        }
                        this.setMaxProcessors( intTemp );
                    } catch ( NumberFormatException ex ) {
                        throw new ConfigException( Message.getMessage(
                            "ERR_CONF_badConfigFormat",
                            TAG_NAME_MAXPROCESSORS ), ex );
                    }
                } else {
                    throw new ConfigException( Message.getMessage(
                        "ERR_CONF_nullConfigElement",
                        TAG_NAME_MAXPROCESSORS ) );
                }
                continue;
            }
        }
    }
    
    private void initDeployerElement ( Element e ) throws ConfigException {
        Node childNode = null;
        String nodeValue = null;
        final int childNodesLength = e.getChildNodes().getLength();
        for ( int i = 0; i < childNodesLength; i++ ) {
            childNode = e.getChildNodes().item( i );
                        if ( isValidElement( childNode, TAG_NAME_DEPLOYERUPDATEINTERVAL ) ) {
                nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                    childNode );
                if ( null != nodeValue ) {
                    try {
                        final long lngTemp = Long.parseLong( nodeValue );
                                                if ( 0 >= lngTemp ) {
                            throw new ConfigException( Message.getMessage(
                                "ERR_CONF_badConfigFormat",
                                TAG_NAME_DEPLOYERUPDATEINTERVAL ) );
                        }
                        this.setDeployerUpdateInterval( lngTemp );
                    } catch ( NumberFormatException ex ) {
                        throw new ConfigException( Message.getMessage(
                            "ERR_CONF_badConfigFormat",
                            TAG_NAME_DEPLOYERUPDATEINTERVAL ), ex );
                    }
                } else {
                    throw new ConfigException( Message.getMessage(
                        "ERR_CONF_nullConfigElement",
                        TAG_NAME_DEPLOYERUPDATEINTERVAL ) );
                }
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_DEPLOYSPACE ) ) {
                nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                    childNode );
                                if ( null != nodeValue ) {
                    this.setDeployPath( nodeValue );
                } else {
                    throw new ConfigException( Message.getMessage(
                        "ERR_CONF_nullConfigElement",
                        TAG_NAME_DEPLOYSPACE ) );
                }
                continue;
            }
                        if ( isValidElement( childNode, TAG_NAME_WORKSAPCE ) ) {
                nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                    childNode );
                                if ( null != nodeValue ) {
                    this.setWorkingPath( nodeValue );
                } else {
                    throw new ConfigException( Message.getMessage(
                        "ERR_CONF_nullConfigElement",
                        TAG_NAME_WORKSAPCE ) );
                }
                continue;
            }
        }
    }
    
    private void initTransportFactoryElement ( Element e ) {
                NodeList nodeList = e.getElementsByTagName( TAG_NAME_TRANSPORT );
        final int nodeListLength = nodeList.getLength();
        for ( int i = 0; i < nodeListLength; i++ ) {
            Element childElement = ( Element ) nodeList.item( i );
            String type = childElement.getAttribute( ATTR_NAME_TYPE );
            String className = childElement.getAttribute( ATTR_NAME_CLASSNAME );
            this.gTransportFactory.put( type, className );
        }
    }
    
    private void initProviderFactoryElement ( Element e ) {
                NodeList nodeList = e.getElementsByTagName( TAG_NAME_PROVIDER );
        for ( int i = 0; i < nodeList.getLength(); i++ ) {
            Element childElement = ( Element ) nodeList.item( i );
            String type = childElement.getAttribute( ATTR_NAME_TYPE );
            String className = childElement.getAttribute( ATTR_NAME_CLASSNAME );
            this.gProviderFactory.put( type, className );
        }
    }
    
        private void initRBACFactoryElement ( Element e ) {
                NodeList nodeList = e.getElementsByTagName( TAG_NAME_RBAC );
        for ( int i = 0; i < nodeList.getLength(); i++ ) {
            Element childElement = ( Element ) nodeList.item( i );
            String type = childElement.getAttribute( ATTR_NAME_TYPE );
            String className = childElement.getAttribute( ATTR_NAME_CLASSNAME );
            this.gRBACFactory.put( type, className );
        }
    }
    
    private void initHandlerFactoryElement ( Element e ) {
                NodeList nodeList = e.getElementsByTagName( TAG_NAME_HANDLER );
        for ( int i = 0; i < nodeList.getLength(); i++ ) {
            Element childElement = ( Element ) nodeList.item( i );
            String type = childElement.getAttribute( ATTR_NAME_TYPE );
            String className = childElement.getAttribute( ATTR_NAME_CLASSNAME );
            this.gHandlerFactory.put( type, className );
        }
    }
    
    private void initExceptionHandlerFactoryElement ( Element e ) {
                NodeList nodeList = e.getElementsByTagName( TAG_NAME_EXCEPTIONHANDLER );
        for ( int i = 0; i < nodeList.getLength(); i++ ) {
            Element childElement = ( Element ) nodeList.item( i );
            String type = childElement.getAttribute( ATTR_NAME_TYPE );
            String className = childElement.getAttribute( ATTR_NAME_CLASSNAME );
            this.gExceptionHandlerFactory.put( type, className );
        }
    }
    
        private void initCredManagerElement ( Element e ) throws
        MissingResourceException, ConfigException {
                NodeList nodeList = e.getElementsByTagName( TAG_NAME_CREDDEVICE );
        for ( int i = 0; i < nodeList.getLength(); i++ ) {
            Element eCredDevice = ( Element ) nodeList.item( i );
                        String type = eCredDevice.getAttribute( ATTR_NAME_TYPE );
            String className = eCredDevice.getAttribute( ATTR_NAME_CLASSNAME );
            KeyStoreImpl credDevice = new KeyStoreImpl();
            this.credDevices.put( type, credDevice );
            credDevice.setType( type );
            credDevice.setClassName( className );
                        Node childNode = null;
            String nodeValue = null;
            final int childNodesLength = eCredDevice.getChildNodes().getLength();
            for ( int j = 0; j < childNodesLength; j++ ) {
                childNode = eCredDevice.getChildNodes().item( j );
                                if ( isValidElement( childNode, TAG_NAME_DEVICENAME ) ) {
                    nodeValue = XMLUtils.getTextValueOfElement( ( Element )
                        childNode );
                                        if ( null != nodeValue ) {
                        credDevice.setDeviceName( nodeValue );
                    } else {
                        throw new ConfigException( Message.getMessage(
                            "ERR_CONF_nullConfigElement",
                            TAG_NAME_DEVICENAME ) );
                    }
                    continue;
                }
                                if ( isValidElement( childNode, TAG_NAME_DEVICECONFIG ) ) {
                    Element eDeviceConfig = ( Element ) childNode;
                    final int childlength = eDeviceConfig.getChildNodes().
                        getLength();
                    for ( int m = 0; m < childlength; m++ ) {
                        Node DeviceConfigchild = eDeviceConfig.getChildNodes().
                            item( m );
                                                if ( isValidElement( DeviceConfigchild,
                                             TAG_NAME_KEYSTOREFILE ) ) {
                            nodeValue = XMLUtils.getTextValueOfElement( (
                                Element )
                                DeviceConfigchild );
                                                        if ( null != nodeValue ) {
                                credDevice.setKeyStoreFile( nodeValue );
                            } else {
                                throw new ConfigException( Message.getMessage(
                                    "ERR_CONF_nullConfigElement",
                                    TAG_NAME_KEYSTOREFILE ) );
                            }
                            continue;
                        }
                                                if ( isValidElement( DeviceConfigchild,
                                             TAG_NAME_KEYSTOREPASSWORD ) ) {
                            nodeValue = XMLUtils.getTextValueOfElement( (
                                Element )
                                DeviceConfigchild );
                                                        if ( null != nodeValue ) {
                                credDevice.setKeyStorePassWord( nodeValue );
                            } else {
                                throw new ConfigException( Message.getMessage(
                                    "ERR_CONF_nullConfigElement",
                                    TAG_NAME_KEYSTOREPASSWORD ) );
                            }
                            continue;
                        }
                    }
                }
                                if ( isValidElement( childNode, TAG_NAME_SUPPORTEDCREDTYPES ) ) {
                                        Vector strCredTypes = new Vector();
                    Vector credTypes = XMLUtils.getAllChildElements( childNode );
                    for ( int k = 0; k < credTypes.size(); k++ ) {
                        Element eCredType = ( Element ) credTypes.elementAt( k );
                        if ( isValidElement( eCredType, TAG_NAME_CREDTYPE ) ) {
                            nodeValue = XMLUtils.getTextValueOfElement(
                                eCredType );
                                                        if ( null != nodeValue ) {
                                                                strCredTypes.addElement( nodeValue );
                            } else {
                                throw new ConfigException( Message.getMessage(
                                    "ERR_CONF_nullConfigElement",
                                    TAG_NAME_CREDTYPE ) );
                            }
                            continue;
                        }
                    }
                    credDevice.setCredTypes( strCredTypes );
                }
            }
        }
    }
    
        private void initUserManagerElement ( Element e ) throws
        ConfigException {
                try {
                        Element elmUserDevice = ( Element ) e.getElementsByTagName( this.
                TAG_NAME_USERDEVICE ).item( 0 );
            String sType = elmUserDevice.getAttribute( this.ATTR_NAME_TYPE );
            String sClassName = elmUserDevice.getAttribute( this.
                ATTR_NAME_CLASSNAME );
            if ( sType.equalsIgnoreCase( "Database" ) ) {
                String sDatabaseURL = "", sDatabaseUser = "",
                    sDatabasePass = "", sDatabaseDriver = "";
                Text txtDriver = ( Text ) elmUserDevice.getElementsByTagName( this.
                    TAG_NAME_DRIVER ).item( 0 ).getFirstChild();
                if ( txtDriver != null ) {
                    sDatabaseDriver = txtDriver.getNodeValue();
                }
                Text txtURL = ( Text ) elmUserDevice.getElementsByTagName( this.
                    TAG_NAME_URL ).item( 0 ).getFirstChild();
                if ( txtURL != null ) {
                    sDatabaseURL = txtURL.getNodeValue();
                }
                Text txtUsername = ( Text ) elmUserDevice.getElementsByTagName( this.
                    TAG_NAME_USERNAME ).item( 0 ).getFirstChild();
                if ( txtUsername != null ) {
                    sDatabaseUser = txtUsername.getNodeValue();
                }
                Text txtPassword = ( Text ) elmUserDevice.getElementsByTagName( this.
                    TAG_NAME_PASSWORD ).item( 0 ).getFirstChild();
                if ( txtPassword != null ) {
                    sDatabasePass = txtPassword.getNodeValue();
                                        
                                                        }
                this.userManager = new UserManager( sDatabaseDriver,
                    sDatabaseURL, sDatabaseUser, sDatabasePass );
            } else if ( sType.equalsIgnoreCase( "XML" ) ) {
                String fileName = null;
                Text txtFileName = ( Text ) elmUserDevice.getElementsByTagName( this.
                    TAG_NAME_FILENAME ).item( 0 ).getFirstChild();
                if ( txtFileName != null ) {
                    fileName = txtFileName.getNodeValue();
                }
                                                int indexWin = this.configFileName.lastIndexOf( '\\' );
                int indexLin = this.configFileName.lastIndexOf( '/' );
                StringBuffer buffer = new StringBuffer( 80 );
                buffer.append( this.configFileName );
                if( 0 > indexLin ){
                                        buffer.delete( indexWin + 1, this.configFileName.length() );
                }else{
                                        buffer.delete( indexLin + 1, this.configFileName.length() );
                }
                buffer.append( fileName );
                String filePath = buffer.toString();
                this.userManager = new UserManager( filePath );
            } else {
                throw new ConfigException( "Undefined user manager Type!" );             }
        } catch ( Exception ex ) {
            throw new ConfigException(
                "Malformed user manager config file or it is not found!" );
        }
    }
    
        private void initRBACManagerElement ( Element e ) throws
        ConfigException {
        try {
                        Element elmRBACDevice = ( Element ) e.getElementsByTagName( this.
                TAG_NAME_RBACDEVICE ).item( 0 );
            String sType = elmRBACDevice.getAttribute( this.ATTR_NAME_TYPE );
            String sClassName = elmRBACDevice.getAttribute( this.
                ATTR_NAME_CLASSNAME );
            if ( sType.equalsIgnoreCase( "Database" ) ) {
                String sDatabaseURL = "", sDatabaseUser = "",
                    sDatabasePass = "", sDatabaseDriver = "";
                Text txtDriver = ( Text ) elmRBACDevice.getElementsByTagName( this.
                    TAG_NAME_DRIVER ).item( 0 ).getFirstChild();
                if ( txtDriver != null ) {
                    sDatabaseDriver = txtDriver.getNodeValue();
                }
                Text txtURL = ( Text ) elmRBACDevice.getElementsByTagName( this.
                    TAG_NAME_URL ).item( 0 ).getFirstChild();
                if ( txtURL != null ) {
                    sDatabaseURL = txtURL.getNodeValue();
                }
                Text txtUsername = ( Text ) elmRBACDevice.getElementsByTagName( this.
                    TAG_NAME_USERNAME ).item( 0 ).getFirstChild();
                if ( txtUsername != null ) {
                    sDatabaseUser = txtUsername.getNodeValue();
                }
                Text txtPassword = ( Text ) elmRBACDevice.getElementsByTagName( this.
                    TAG_NAME_PASSWORD ).item( 0 ).getFirstChild();
                if ( txtPassword != null ) {
                    sDatabasePass = txtPassword.getNodeValue();
                                        
                                                        }
                this.rbacManager = new RBACManager
                    ( sDatabaseDriver, sDatabaseURL, sDatabaseUser,
                      sDatabasePass );
            } else if ( sType.equalsIgnoreCase( "XML" ) ) {
                String fileName = null;
                Text txtFileName = ( Text ) elmRBACDevice.getElementsByTagName( this.
                    TAG_NAME_FILENAME ).item( 0 ).getFirstChild();
                if ( txtFileName != null ) {
                    fileName = txtFileName.getNodeValue();
                }
                                                int indexWin = this.configFileName.lastIndexOf( '\\' );
                int indexLin = this.configFileName.lastIndexOf( '/' );
                StringBuffer buffer = new StringBuffer( 80 );
                buffer.append( this.configFileName );
                if( 0 > indexLin ){
                                        buffer.delete( indexWin + 1, this.configFileName.length() );
                }else{
                                        buffer.delete( indexLin + 1, this.configFileName.length() );
                }
                buffer.append( fileName );
                String filePath = buffer.toString();
                this.rbacManager = new RBACManager( filePath );
            } else {
                throw new ConfigException( "Undefined rbac manager Type!" );             }
        } catch ( Exception ex ) {
            throw new ConfigException(
                "Malformed rbac manager config file or it is not found!" );
        }
    }
    
    private void initKeyInfoElement ( Element e ) throws ConfigException {
        String sAuthenAlias = null;
        String sAuthenPassword = null;
        String sSignAlias = null;
        String sSignPassword = null;
        if ( e != null ) {
                        Element elmAuthenKey = ( Element ) e.getElementsByTagName( this.
                TAG_NAME_AUTHENKEY ).item( 0 );
            if ( elmAuthenKey != null ) {
                Element elmAuthenAlias = ( Element ) elmAuthenKey.
                    getElementsByTagName( this.TAG_NAME_AUTHENALIAS ).item( 0 );
                Text txtAuthenAlias = ( Text ) elmAuthenKey.
                    getElementsByTagName( this.TAG_NAME_AUTHENALIAS ).item( 0 ).
                    getFirstChild();
                if ( txtAuthenAlias != null ) {
                    sAuthenAlias = txtAuthenAlias.getNodeValue();
                }
                Element elmAuthenPassword = ( Element ) elmAuthenKey.
                    getElementsByTagName( this.TAG_NAME_AUTHENPASSWORD ).item(
                    0 );
                Text txtAuthenPassword = ( Text ) elmAuthenKey.
                    getElementsByTagName( this.TAG_NAME_AUTHENPASSWORD ).item(
                    0 ).getFirstChild();
                if ( txtAuthenPassword != null ) {
                    sAuthenPassword = txtAuthenPassword.getNodeValue();
                }
                CredManager credManager = this.getCredManager();
                Key prikey = credManager.getPrivateKey( sAuthenAlias,
                    sAuthenPassword );
                Certificate cert = this.getCredManager().getX509Cert(
                    sAuthenAlias );
                this.setAuthenPrivateKey( prikey );
                this.setAuthenCert( cert );
                log.info( Message.getMessage( "successautheninfo" ) );
            } else {
                log.info( Message.getMessage( "failautheninfo" ) );
                            }
            Element elmSignKey = ( Element ) e.getElementsByTagName( this.
                TAG_NAME_SIGNKEY ).item( 0 );
            if ( elmSignKey != null ) {
                Element elmSignAlias = ( Element ) elmSignKey.
                    getElementsByTagName( this.TAG_NAME_SIGNALIAS ).item( 0 );
                Text txtSignAlias = ( Text ) elmSignKey.getElementsByTagName( this.
                    TAG_NAME_SIGNALIAS ).item( 0 ).getFirstChild();
                if ( txtSignAlias != null ) {
                    sSignAlias = txtSignAlias.getNodeValue();
                }
                Element elmSignPassword = ( Element ) elmSignKey.
                    getElementsByTagName( this.TAG_NAME_SIGNPASSWORD ).item( 0 );
                Text txtSignPassword = ( Text ) elmSignKey.getElementsByTagName( this.
                    TAG_NAME_SIGNPASSWORD ).item( 0 ).getFirstChild();
                if ( txtSignPassword != null ) {
                    sSignPassword = txtSignPassword.getNodeValue();
                }
                Key prikey = this.getCredManager().getPrivateKey( sSignAlias,
                    sSignPassword );
                Certificate cert = this.getCredManager().getX509Cert(
                    sSignAlias );
                this.setSignPrivateKey( prikey );
                this.setSignPrivateKeyName( sSignAlias );
                this.setSignCert( cert );
                log.info( Message.getMessage( "successsigninfo" ) );
            } else {
                log.info( Message.getMessage( "failsigninfo" ) );
            }
        }
    }
    
    private Element makeElementConfiguration ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_CONFIGURATION );
                result.appendChild( this.makeElementEngine( doc ) );
                result.appendChild( this.makeElementDeployer( doc ) );
                if ( null != this.gTransportFactory ) {
            result.appendChild( this.makeElementTransportFactory( doc ) );
        }
                if ( null != this.gProviderFactory ) {
            result.appendChild( this.makeElementProviderFactory( doc ) );
        }
                if ( null != this.gRBACFactory ) {
            result.appendChild( this.makeElementRBACFactory( doc ) );
        }
                if ( null != this.gHandlerFactory ) {
            result.appendChild( this.makeElementHandlerFactory( doc ) );
        }
                if ( null != this.gExceptionHandlerFactory ) {
            result.appendChild( this.makeElementExceptionHandlerFactory( doc ) );
        }
                if ( null != this.credDevices ) {
            result.appendChild( this.makeElementCredManager( doc ) );
        }
                if ( null != this.userDevices ) {
            result.appendChild( this.makeElementUserManager( doc ) );
        }
                if ( null != this.rbacDevices ) {
            result.appendChild( this.makeElementRBACManager( doc ) );
        }
                result.appendChild( this.makeElementKeyInfo( doc ) );
        return result;
    }
    
    private Element makeElementEngine ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_ENGINE );
                Element childElement = doc.createElement( TAG_NAME_NAME );
        childElement.appendChild( doc.createTextNode( this.gEngineName ) );
        result.appendChild( childElement );
                childElement = doc.createElement( TAG_NAME_DESCRIPTION );
        childElement.appendChild( doc.createTextNode( this.gEngineDescription ) );
        result.appendChild( childElement );
                childElement = doc.createElement( TAG_NAME_MAXPROCESSORS );
        childElement.appendChild( doc.createTextNode( Integer.toString( this.
            gMaxProcessors ) ) );
        result.appendChild( childElement );
        return result;
    }
    
    private Element makeElementDeployer ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_DEPLOYER );
                Element childElement = doc.createElement(
            TAG_NAME_DEPLOYERUPDATEINTERVAL );
        childElement.appendChild( doc.createTextNode( Long.toString( this.
            gDeployerUpdateInterval ) ) );
        result.appendChild( childElement );
                childElement = doc.createElement( TAG_NAME_DEPLOYSPACE );
        childElement.appendChild( doc.createTextNode( this.gDeployPath ) );
        result.appendChild( childElement );
                childElement = doc.createElement( TAG_NAME_WORKSAPCE );
        childElement.appendChild( doc.createTextNode( this.gWorkingPath ) );
        result.appendChild( childElement );
        return result;
    }
    
    private Element makeElementTransportFactory ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_TRANSPORTFACTORY );
        Element childElement = null;
        Iterator it = this.gTransportFactory.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            childElement = doc.createElement( TAG_NAME_TRANSPORT );
            childElement.setAttribute( ATTR_NAME_TYPE, ( String ) entry.getKey() );
            childElement.setAttribute( ATTR_NAME_CLASSNAME,
                                       ( String ) entry.getValue() );
            result.appendChild( childElement );
        }
        return result;
    }
    
    private Element makeElementProviderFactory ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_PROVIDERFACTORY );
        Element childElement = null;
        Iterator it = this.gProviderFactory.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            childElement = doc.createElement( TAG_NAME_PROVIDER );
            childElement.setAttribute( ATTR_NAME_TYPE, ( String ) entry.getKey() );
            childElement.setAttribute( ATTR_NAME_CLASSNAME,
                                       ( String ) entry.getValue() );
            result.appendChild( childElement );
        }
        return result;
    }
    
        private Element makeElementRBACFactory ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_RBACFACTORY );
        Element childElement = null;
        Iterator it = this.gRBACFactory.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            childElement = doc.createElement( TAG_NAME_RBAC );
            childElement.setAttribute( ATTR_NAME_TYPE, ( String ) entry.getKey() );
            childElement.setAttribute( ATTR_NAME_CLASSNAME,
                                       ( String ) entry.getValue() );
            result.appendChild( childElement );
        }
        return result;
    }
    
    private Element makeElementExceptionHandlerFactory ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_EXCEPTIONHANDLERFACTORY );
        Element childElement = null;
        Iterator it = this.gExceptionHandlerFactory.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            childElement = doc.createElement( TAG_NAME_EXCEPTIONHANDLER );
            childElement.setAttribute( ATTR_NAME_TYPE, ( String ) entry.getKey() );
            childElement.setAttribute( ATTR_NAME_CLASSNAME,
                                       ( String ) entry.getValue() );
            result.appendChild( childElement );
        }
        return result;
    }
    
    private Element makeElementHandlerFactory ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_HANDLERFACTORY );
        Element childElement = null;
        Iterator it = this.gHandlerFactory.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            childElement = doc.createElement( TAG_NAME_HANDLER );
            childElement.setAttribute( ATTR_NAME_TYPE, ( String ) entry.getKey() );
            childElement.setAttribute( ATTR_NAME_CLASSNAME,
                                       ( String ) entry.getValue() );
            result.appendChild( childElement );
        }
        return result;
    }
    
        private Element makeElementCredManager ( Document doc ) {
        Element eCredManager = doc.createElement( TAG_NAME_CREDMANAGER );
        Iterator it = this.credDevices.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            Element eCredDevice = doc.createElement( TAG_NAME_CREDDEVICE );
                        CredDevice credDevice = ( CredDevice ) entry.getValue();
            eCredDevice.setAttribute( ATTR_NAME_TYPE, credDevice.getType() );
            eCredDevice.setAttribute( ATTR_NAME_CLASSNAME,
                                      credDevice.getClassName() );
                        Element eDeviceName = doc.createElement( TAG_NAME_DEVICENAME );
            eDeviceName.appendChild( doc.createTextNode( credDevice.
                getDeviceName() ) );
            eCredDevice.appendChild( eDeviceName );
                        Element eDeviceConfig = doc.createElement( TAG_NAME_DEVICECONFIG );
            Element eKeyStoreFile = doc.createElement( TAG_NAME_KEYSTOREFILE );
            eKeyStoreFile.appendChild( doc.createTextNode( credDevice.
                getKeyStoreFile() ) );
            eDeviceConfig.appendChild( eKeyStoreFile );
            Element eKeyStorePassWord = doc.createElement(
                TAG_NAME_KEYSTOREPASSWORD );
            eKeyStorePassWord.appendChild( doc.createTextNode( credDevice.
                getKeyStorePassWord() ) );
            eDeviceConfig.appendChild( eKeyStorePassWord );
            eCredDevice.appendChild( eDeviceConfig );
                        Element eSupportedCredTypes = doc.createElement(
                TAG_NAME_SUPPORTEDCREDTYPES );
                        Vector strCredTypes = credDevice.getCredTypes();
            for ( int i = 0; i < strCredTypes.size(); i++ ) {
                Element eCredType = doc.createElement( TAG_NAME_CREDTYPE );
                eCredType.appendChild( doc.createTextNode( ( String )
                    strCredTypes.
                    elementAt( i ) ) );
                eSupportedCredTypes.appendChild( eCredType );
            }
            eCredDevice.appendChild( eSupportedCredTypes );
                        eCredManager.appendChild( eCredDevice );
        }
        return eCredManager;
    }
    
        private Element makeElementUserManager ( Document doc ) {
        Element eUserManager = doc.createElement( TAG_NAME_USERMANAGER );
        Iterator it = this.userDevices.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            Element eUserDevice = doc.createElement( TAG_NAME_USERDEVICE );
                        UserDevice userDevice = ( UserDevice ) entry.getValue();
            eUserDevice.setAttribute( ATTR_NAME_TYPE, userDevice.getType() );
            eUserDevice.setAttribute( ATTR_NAME_CLASSNAME,
                                      userDevice.getClassName() );
                        Element eDriver = doc.createElement( TAG_NAME_DRIVER );
            eDriver.appendChild( doc.createTextNode( userDevice.getDriver() ) );
            eUserDevice.appendChild( eDriver );
                        Element eURL = doc.createElement( TAG_NAME_URL );
            eURL.appendChild( doc.createTextNode( userDevice.getURL() ) );
            eUserDevice.appendChild( eURL );
                        Element eUsername = doc.createElement( TAG_NAME_USERNAME );
            eUsername.appendChild( doc.createTextNode( userDevice.getUsername() ) );
            eUserDevice.appendChild( eUsername );
                        Element ePassword = doc.createElement( TAG_NAME_PASSWORD );
            ePassword.appendChild( doc.createTextNode( userDevice.getPassword() ) );
            eUserDevice.appendChild( ePassword );
                        eUserManager.appendChild( eUserDevice );
        }
        return eUserManager;
    }
    
        private Element makeElementRBACManager ( Document doc ) {
        Element eRBACManager = doc.createElement( TAG_NAME_RBACMANAGER );
        Iterator it = this.userDevices.entrySet().iterator();
        while ( it.hasNext() ) {
            Map.Entry entry = ( Map.Entry ) it.next();
            Element eRBACDevice = doc.createElement( TAG_NAME_RBACDEVICE );
                        RBACDevice rbacDevice = ( RBACDevice ) entry.getValue();
            eRBACDevice.setAttribute( ATTR_NAME_TYPE, rbacDevice.getType() );
            eRBACDevice.setAttribute( ATTR_NAME_CLASSNAME,
                                      rbacDevice.getClassName() );
                        Element eDriver = doc.createElement( TAG_NAME_DRIVER );
            eDriver.appendChild( doc.createTextNode( rbacDevice.getDriver() ) );
            eRBACDevice.appendChild( eDriver );
                        Element eURL = doc.createElement( TAG_NAME_URL );
            eURL.appendChild( doc.createTextNode( rbacDevice.getURL() ) );
            eRBACDevice.appendChild( eURL );
                        Element eUsername = doc.createElement( TAG_NAME_USERNAME );
            eUsername.appendChild( doc.createTextNode( rbacDevice.getUsername() ) );
            eRBACDevice.appendChild( eUsername );
                        Element ePassword = doc.createElement( TAG_NAME_PASSWORD );
            ePassword.appendChild( doc.createTextNode( rbacDevice.getPassword() ) );
            eRBACDevice.appendChild( ePassword );
                        eRBACManager.appendChild( eRBACDevice );
        }
        return eRBACManager;
    }
    
        private Element makeElementKeyInfo ( Document doc ) {
        Element result = doc.createElement( TAG_NAME_KEYINFO );
                Element eAuthenKey = doc.createElement(
            TAG_NAME_AUTHENKEY );
                Element eAuthenAlias = doc.createElement( TAG_NAME_AUTHENALIAS );
        eAuthenAlias.appendChild( doc.createTextNode( this.gAuthenAlias ) );
        eAuthenKey.appendChild( eAuthenAlias );
                Element eAuthenPassword = doc.createElement( TAG_NAME_AUTHENPASSWORD );
        eAuthenPassword.appendChild( doc.createTextNode( this.gAuthenPassword ) );
        eAuthenKey.appendChild( eAuthenPassword );
        result.appendChild( eAuthenKey );
                Element eSignKey = doc.createElement(
            TAG_NAME_SIGNKEY );
                Element eSignAlias = doc.createElement( TAG_NAME_SIGNALIAS );
        eSignAlias.appendChild( doc.createTextNode( this.gSignAlias ) );
        eSignKey.appendChild( eSignAlias );
                Element eSignPassword = doc.createElement( TAG_NAME_SIGNPASSWORD );
        eSignPassword.appendChild( doc.createTextNode( this.gSignPassword ) );
        eSignKey.appendChild( eSignPassword );
        result.appendChild( eSignKey );
        return result;
    }
    
    
    public void clear () {
        this.mConfigManager = null;
    }
}
