package org.act.soap.server.deploy;
import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import org.act.soap.util.Constants;
import org.act.soap.util.i18n.Message;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;

public class AutoDeployer
    implements Runnable {
    
    private static Log log = LogFactory.getLog(AutoDeployer.class.getName());
    
    private final FilenameFilter filter = new FilenameFilter() {
        public boolean accept (File dir, String filename) {
            File file = new File(dir, filename);
            if (null != file && file.isFile() ) {
                String fn = file.getName();
                final int len = fn.length();
                if (5 < len) {
                    String ext = fn.substring(len - 5, len);
                    return ext.equalsIgnoreCase(Constants.WSAR_FILE_SUFFIX);
                }
            }
            return false;
        }
    };
    
    private DeployManager gDeployManager = null;
    
    private HashMap watchedWSARs = new HashMap();
    
    private boolean running = false;
    
    public AutoDeployer () {
    }
    
    public AutoDeployer (DeployManager deployManager) {
        this.gDeployManager = deployManager;
    }
    
    public void start () {
                initDeploy();
                        this.running = true;
        new Thread(this, Constants.AUTODEPLOYER_THREAD_NAME).start();
    }
    
    public void stop () {
        this.running = false;
    }
    
    public synchronized void run () {
        long lm = 0;         DeployInfo deployInfo = null;         DeployManager deployManager = this.gDeployManager;
        while (running) {
                        final long checkInterval = DeployManager.getCheckInterval();
            if (0 < checkInterval) {
                try {
                    Thread.sleep(checkInterval);
                }
                catch (InterruptedException ex) {
                    log.error(ex);
                }
            }
            else {
                this.running = false;
                break;
            }
            try {
                                File deployDir = new File(DeployManager.getDeployDir());
                File[] wsarFiles = null;
                Iterator iterator = null;
                if (null == deployDir) {
                    continue;
                }
                                                wsarFiles = deployDir.listFiles(filter);
                if (null == wsarFiles) {
                    continue;
                }
                for (int i = 0; i < wsarFiles.length; i++) {
                    String wsarFileName = wsarFiles[i].getName();
                    String serviceName = wsarFileName.substring(0,
                        wsarFileName.lastIndexOf(Constants.WSAR_FILE_SUFFIX));
                    if (null == watchedWSARs.get(serviceName)) {
                                                URL wsarDeploySpaceUrl = wsarFiles[i].toURL();
                        URL wsarWorkSpaceURL = new File(new File(DeployManager.
                            getWorkingDir()),
                            wsarFiles[i].getName()).toURL();
                        deployInfo = new DeployInfo();
                        deployInfo.setServiceName(serviceName);
                        deployInfo.setSource(wsarDeploySpaceUrl);
                        deployInfo.setDestination(wsarWorkSpaceURL);
                        watchedWSARs.put(serviceName, deployInfo);
                    }
                }
                                iterator = watchedWSARs.values().iterator();
                while (iterator.hasNext()) {
                    URL url = null;
                    File currentFile = null;
                    deployInfo = (DeployInfo) iterator.next();
                    url = deployInfo.getSource();
                    currentFile = new File(url.getPath());
                                        if (!currentFile.exists()) {
                        if (deployManager.isServiceDeployed(deployInfo.
                            getServiceName())) {
                                                        gDeployManager.undeploy(deployInfo.getServiceName());
                            iterator.remove();
                        }
                    }
                    else {
                                                lm = currentFile.lastModified();
                                                if ( (0 == deployInfo.getLastModified()) ||
                            (deployInfo.getLastModified() < lm)) {
                                                                                    if (deployManager.isServiceDeployed(
                                deployInfo.
                                getServiceName())) {
                                                                gDeployManager.undeploy(deployInfo.
                                    getServiceName());
                            }
                            log.info(Message.getMessage("Info_SD_deploy0",
                                deployInfo.getServiceName()));
                            gDeployManager.deploy(currentFile);
                            deployInfo.setLastModified(lm);
                        }
                    }
                }
            }
            catch (MalformedURLException ex) {
                log.error(ex);
            }
            catch (DeployException ex) {
                                processDeployException(ex, deployInfo, lm);
            }
            catch (Exception ex) {
                log.fatal(ex);
            }
        }     }
    
    public boolean isRunning () {
        return this.running;
    }
    
    private void initDeploy () {
        long lm = 0;         DeployInfo deployInfo = null;         try {
            HashMap firstSkipedWSARs = gDeployManager.getDeployedWSARs();
            File[] wsarFiles = null;
            DeployManager deployManager = this.gDeployManager;
                        File deployDir = new File(DeployManager.getDeployDir());
            if (null == deployDir) {
                return;
            }
                                    wsarFiles = deployDir.listFiles(filter);
            if (null == wsarFiles) {
                return;
            }
            for (int i = 0; i < wsarFiles.length; i++) {
                File currentFile = wsarFiles[i];
                String wsarFileName = currentFile.getName();
                String serviceName = wsarFileName.substring(0,
                    wsarFileName.lastIndexOf(Constants.WSAR_FILE_SUFFIX));
                URL wsarDeploySpaceUrl = currentFile.toURL();
                URL wsarWorkSpaceURL = new File(new File(DeployManager.
                    getWorkingDir()),
                                                currentFile.getName()).toURL();
                                if (!firstSkipedWSARs.containsKey(serviceName)) {
                    log.info(Message.getMessage("Info_SD_deploy0", serviceName));
                    gDeployManager.deploy(currentFile);
                }
                deployInfo = new DeployInfo();
                deployInfo.setServiceName(serviceName);
                deployInfo.setSource(wsarDeploySpaceUrl);
                deployInfo.setDestination(wsarWorkSpaceURL);
                                lm = currentFile.lastModified();
                deployInfo.setLastModified(lm);
                                watchedWSARs.put(serviceName, deployInfo);
            }
        }
        catch (MalformedURLException ex) {
            log.error(ex);
        }
        catch (DeployException ex) {
                        processDeployException(ex, deployInfo, lm);
        }
        catch (Exception ex) {
            log.fatal(ex);
        }
    }
    
    private void processDeployException (DeployException ex,
                                         DeployInfo deployInfo,
                                         long lastModified) {
        File wsarFailDir = null;
        File wsarFile = null;
        String sTime = null;
        String newFileName = null;
        File wsarFailFile = null;
                deployInfo.setLastModified(lastModified);
                wsarFailDir = new File(DeployManager.
                               getDeployDir() + File.separator +
                               Constants.DEPLORY_FAIL_DIR);
        if (!wsarFailDir.exists()) {
            wsarFailDir.mkdir();
        }
        wsarFile = new File(deployInfo.getSource().getPath());
                sTime = String.valueOf(new Date().getTime());
        newFileName = wsarFailDir.getAbsolutePath() +
            File.separator + deployInfo.getServiceName() + '.' + sTime +
            ".wsar";
        log.error(Message.getMessage("ERR_SD_deploy00",
                                     deployInfo.getServiceName(), newFileName),
                  ex);
        wsarFailFile = new File(newFileName);
        if (!wsarFile.renameTo(wsarFailFile)) {
                        if (wsarFile.delete()) {
                log.error(Message.getMessage("ERR_SD_deploy01",
                                             deployInfo.getServiceName()));
            }
            else {
                log.error(Message.getMessage("ERR_SD_deploy02",
                                             deployInfo.getServiceName()));
            }
        }
    }
}
