package org.act.soap.server.handler;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.*;
import javax.xml.soap.*;
import org.act.soap.server.config.ConfigManager;
import org.act.soap.server.*;
import org.act.soap.soapsec.*;
import org.act.soap.soapsec.Signature;
import org.act.soap.soapsec.securitytoken.*;
import org.act.soap.soapsec.securitytoken.keyprocessor.*;
import org.act.soap.util.*;
import org.act.soap.util.logging.*;
import org.act.soap.wsdd.*;

public class SOAPSecHandler
    extends BasicHandler {
    
    private boolean signResponse = false;
    
    private boolean encryptResponse = false;
    private static Log log = LogFactory.getLog( SOAPSecHandler.class.getName() );
    
    public SOAPSecHandler () {}
    
    public void init ( org.act.soap.wsdd.Handler wsddHandler ) {
                Parameter paramSign = wsddHandler.getParameter( "SignResponse" );
        if ( paramSign != null ) {
            this.signResponse = paramSign.getValue().equalsIgnoreCase( "true" );
        }
        Parameter paramEnc = wsddHandler.getParameter( "EncryptResponse" );
        if ( paramEnc != null ) {
            this.encryptResponse = paramEnc.getValue().equalsIgnoreCase( "true" );
        }
    }
    
    public void cleanup () {
    }
    
    public void process ( MessageContext msgContext ) throws Exception {
                SOAPMessage soapSecMsg = msgContext.getRequestMessage();
                        Iterator attas = soapSecMsg.getAttachments();
        Vector newattas = new Vector();
        while ( attas.hasNext() ) {
            AttachmentPart ap = ( AttachmentPart ) attas.next();
            newattas.addElement( ap );
        }
        soapSecMsg.removeAllAttachments();
                System.out.println( "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" );
        String soapSecMessage = SOAPUtils.soapMessageToString( soapSecMsg );
        SOAPSecurity soapsec = new SOAPSecurity();
        
        SecurityTokenKeyProcessor stkp = new DefaultSecurityKeyProcessor();
        soapsec.setSecurityTokenKeyProcessor( stkp );
        soapsec.setDecKey( ConfigManager.getInstance().getSignPrivateKey() );
        String soapMessage = soapsec.getSoapMessage( soapSecMessage );
        SOAPMessage soapMsg = SOAPUtils.stringToSOAPMessage( soapMessage );
                        boolean hasAttach = false;
        AttachmentPart ap = null;
        for ( int i = 0; i < newattas.size(); i++ ) {
            ap = ( AttachmentPart ) newattas.elementAt( i );
            soapMsg.addAttachmentPart( ap );
            hasAttach = true;
        }
        if ( hasAttach ) {
            soapMsg.saveChanges();
        }
                        msgContext.setRequestMessage( soapMsg );
                msgContext.setProperty( SOAPSecurity.class.getName(), soapsec );
        System.out.println( "-----------------------------------SoapSec Process Result-------------------------------" );
        System.out.println( new String( SOAPUtils.soapMessageToString( soapMsg ) ) );
        System.out.println( "----------------------------------------------------------------------------------------" );
    }
    
    public void postProcess ( MessageContext msgContext ) throws Exception {
        if ( this.signResponse == false && this.encryptResponse == false ) {
            return;         }
        String sUserID = null;
        String sAuthType = "GUEST";
        
        if ( sAuthType != null ) {
                        SOAPSecurity soapsec = null;
            if ( sAuthType.equals( "PASSWORD" ) ||
                 sAuthType.equals( "X509CERTIFICATE" ) ||
                 sAuthType.equals( "GUEST" ) ) {
                                                                soapsec = getSOAPSeccurityByServerCert( msgContext );
            }
            if ( soapsec != null ) {
                                SOAPMessage soapMsg = msgContext.getResponseMessage();
                                                Iterator attas = soapMsg.getAttachments();
                soapMsg.removeAllAttachments();
                                String soapMessage = SOAPUtils.soapMessageToString( soapMsg );
                String soapSecMessage = soapsec.getSoapSecMessage( soapMessage );
                SOAPMessage soapSecMsg = SOAPUtils.stringToSOAPMessage(
                    soapSecMessage );
                                                AttachmentPart ap = null;
                while ( attas.hasNext() ) {
                    ap = ( AttachmentPart ) attas.next();
                    soapSecMsg.addAttachmentPart( ap );
                }
                                                msgContext.setResponseMessage( soapSecMsg );
            }
        }
    }
    
    private SOAPSecurity getSOAPSeccurityByServerCert ( MessageContext
        msgContext ) throws
        Exception {
        SOAPSecurity soapsec = new SOAPSecurity();
                if ( this.signResponse == true ) {
                        ConfigManager configManager = ConfigManager.getInstance();
            if ( configManager != null ) {
                Key signKey = configManager.getSignPrivateKey();
                String signKeyName = configManager.getSignPrivateKeyName();
                X509Certificate signCert = ( X509Certificate ) configManager.
                    getSignCert();
                if ( signKey != null && signCert != null ) {
                    X509CertToken token = new X509CertToken( signCert );
                    token.setSignKey( signKey );
                    token.setSignKeyName( signKeyName );
                    soapsec.addSecurityToken( token );
                    Signature signature = new Signature( token );
                    soapsec.addSignature( signature );
                }
            }
        }
                if ( this.encryptResponse == true ) {
                                    String keystore_location = System.getProperty(
                "org.act.soap.server.engine.rootpath" ) +
                java.io.File.separator + "conf/keystore.jks";
            char[] password = "123456".toCharArray();
            KeyStore keystore = KeyStore.getInstance( "JKS" );
            InputStream in = new FileInputStream( keystore_location );
            keystore.load( in, password );
            in.close();
            Key userPublicKey = keystore.getCertificate( "mykey" ).getPublicKey();
            if ( userPublicKey != null ) {
                Encryption encryption = new Encryption();
                                encryption.setEncKey( true );
                encryption.setEncKeyKey( userPublicKey );
                soapsec.addEncryption( encryption );
            }
                        
        }
        return soapsec;
    }
}
