package org.act.soap.server.management;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.NotificationBroadcasterSupport;
import javax.management.ObjectName;
import javax.wsdl.*;
import org.act.soap.server.SOAPEngine;
import org.act.soap.server.config.ConfigException;
import org.act.soap.server.config.ConfigManager;
import org.act.soap.server.deploy.DeployException;
import org.act.soap.server.deploy.DeployManager;
import org.act.soap.util.i18n.Message;
import org.act.soap.wsdd.Deployment;
import org.act.soap.server.SOAPEngineException;
import java.util.*;

public class EngineManager
    extends NotificationBroadcasterSupport
    implements EngineManagerMBean, MBeanRegistration {
    
    private static final String WSAR_FILE_POSTFIX = ".wsar";
    
    public EngineManager () {
    }
    
    public void saveEngineConfig () throws ConfigException {
        ConfigManager.getInstance().save();
    }
    
    public void loadEngineConfig () throws ConfigException {
        ConfigManager.getInstance().load();
    }
    
    public void setEngineName ( String engineName ) {
        if ( null != engineName ) {
            try {
                ConfigManager.getInstance().setEngineName(
                    engineName );
            } catch ( ConfigException ex ) {
                return;
            }
        }
    }
    
    public String getEngineName () {
        return ConfigManager.getInstance().getEngineName();
    }
    
    public void setEngineDescription ( String engineDescription ) {
        ConfigManager.getInstance().setEngineDescription(
            engineDescription );
    }
    
    public String getEngineDescription () {
        return ConfigManager.getInstance().getEngineDescription();
    }
    
    public void setMaxProcessors ( int maxProcessors ) {
        try {
            ConfigManager.getInstance().setMaxProcessors(
                maxProcessors );
        } catch ( ConfigException ex ) {
            return;
        }
    }
    
    public int getMaxProcessors () {
        return ConfigManager.getInstance().getMaxProcessors();
    }
    
    public boolean isAutoDeployStart () {
        return DeployManager.getInstance().isAutoDeployStart();
    }
    
    public boolean startAutoDeploy () {
        DeployManager.getInstance().startAutoDeploy();
        return DeployManager.getInstance().isAutoDeployStart();
    }
    
    public boolean stopAutoDeploy () {
        DeployManager.getInstance().stopAutoDeploy();
        return! ( DeployManager.getInstance().isAutoDeployStart() );
    }
    
    public void setDeployDir ( String deployPath ) throws DeployException {
        try {
            ConfigManager.getInstance().setDeployPath( deployPath );
        } catch ( ConfigException ex ) {
            throw new DeployException( ex );
        }
    }
    
    public String getDeployDir () {
        return ConfigManager.getInstance().getDeployPath();
    }
    
    public void setWorkingDir ( String dir ) {
        try {
            DeployManager.getInstance().setWorkingDir( dir );
        } catch ( DeployException ex ) {
            return;
        }
    }
    
    public String getWorkingDir () {
        return DeployManager.getInstance().getWorkingDir();
    }
    
    public void setDeployerUpdateInterval ( long interval ) {
        try {
            ConfigManager.getInstance().
                setDeployerUpdateInterval( interval );
        } catch ( ConfigException ex ) {
            return;
        }
    }
    
    public long getDeployerUpdateInterval () {
        return ConfigManager.getInstance().
            getDeployerUpdateInterval();
    }
    
    public boolean isWebServiceDeployed ( String serviceName ) {
        return DeployManager.getInstance().isServiceDeployed( serviceName );
    }
    
    public void deploy ( String serviceName, byte[] src )
                throws DeployException {
        try {
            URL source = new File( serviceName + WSAR_FILE_POSTFIX ).toURL();
            ByteArrayInputStream inputData = new ByteArrayInputStream( src );
            DeployManager.getInstance().deploy( source, inputData );
        } catch (DeployException ex) {
            throw ex;
        } catch ( MalformedURLException ex1 ) {
            throw new DeployException( Message.getMessage(
                "ERR_SD_badServiceName01",
                serviceName ), ex1 );
        } catch ( Exception ex2 ) {
            throw new DeployException( ex2 );
        }
    }
    
    public void undeploy ( String serviceName ) throws DeployException {
        DeployManager.getInstance().undeploy( serviceName );
    }
    
    public HashMap getDeployedWebServices () {
        HashMap webServicesMap = DeployManager.getInstance().getDeployedWSARs();
        return ( HashMap ) webServicesMap.clone();
    }
    
    public Deployment getWSDDDeployment ( String serviceName ) {
        try {
            return DeployManager.getInstance().getWSDDDeployment( serviceName );
        } catch ( Exception ex1 ) {
            return null;
        }
    }
    
    public Definition getWSDLDefinition ( String serviceName ) {
        try {
            return SOAPEngine.getInstance().getServiceManager().
                getWebServiceObject( serviceName ).getWSDLDefinition();
        } catch ( Exception ex ) {
            return null;
        }
    }
    
    public ObjectName preRegister ( MBeanServer mBeanServer,
                                    ObjectName objectName ) {
        return objectName;
    }
    
    public void postRegister ( Boolean stat ) {
        return;
    }
    
    public void preDeregister () {
        return;
    }
    
    public void postDeregister () {
        return;
    }
    
    public void startEngine() throws SOAPEngineException {
        SOAPEngine.getInstance().start();
    }
    
    public void stopEngine() throws SOAPEngineException {
        SOAPEngine.getInstance().stop();
    }
    
    public int getEngineState() {
        return SOAPEngine.getInstance().getEngineState();
    }
    
    public List getOperations(String serviceName) {
        Definition def = getWSDLDefinition(serviceName);
        List ops = new ArrayList();
        PortType porttype = (PortType) def.getPortTypes().values().iterator().next();
        Iterator operations = porttype.getOperations().iterator();
        while (operations.hasNext()) {
            ops.add(((Operation) operations.next()).getName());
        }
        return ops;
    }
}
