package org.act.soap.server.provider;
import org.act.soap.server.MessageContext;
import java.util.Properties;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.lang.reflect.Method;
import org.act.soap.util.TMConstants;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPBody;
import java.lang.reflect.InvocationTargetException;
import javax.xml.soap.MessageFactory;
import org.act.soap.util.SOAPUtils;
import java.util.Hashtable;
public class EJBProvider extends Provider {
    
    private MessageFactory mMessageFactory = null;
    
    private String jndiName;
    
    private String homeName;
    
    private String icf;
    
    private String pURL;
    public EJBProvider() {
        mMessageFactory = SOAPUtils.MESSAGEFACTORY_INSTANCE;
    }
    
    public void init(org.act.soap.wsdd.Provider wsddProvider) throws
        ProviderException {
        jndiName = wsddProvider.getParameter("JNDIName").getValue();
        homeName = wsddProvider.getParameter("HomeName").getValue();
        icf = wsddProvider.getParameter("INITIAL_CONTEXT_FACTORY").getValue();
        pURL = wsddProvider.getParameter("PROVIDER_URL").getValue();
    }
    
    protected void doDocumentProcess(MessageContext msgContext) throws
        ProviderException, ServiceImplementationException {
    }
    
    protected void doRPCProcess(MessageContext msgContext) throws
        ProviderException, ServiceImplementationException {
        try {
                        TypeMappingHandler typeMappingHandler = null;
            Object returnObject = null;
            Class returnType = null;
            Object[] parameters = null;
            try {
                                Properties properties = new Properties();
                properties.put(Context.INITIAL_CONTEXT_FACTORY, icf);
                properties.put(Context.PROVIDER_URL, pURL);
                InitialContext initialContext = new InitialContext(properties);
                Object ref = initialContext.lookup(jndiName);
                initialContext.close();
                                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                Class homeClass = classLoader.loadClass(homeName);
                                Object homeObject = javax.rmi.PortableRemoteObject.narrow(ref, homeClass);
                                Method createMethod = homeClass.getMethod("create", new Class[0]);
                Object ejbObject = createMethod.invoke(homeObject, new Object[0]);
                                Class ejbClass = ejbObject.getClass();
                typeMappingHandler = new TypeMappingHandler(msgContext, ejbClass);
                parameters = typeMappingHandler.deserialize();
                                Method method = typeMappingHandler.getMethod();
                returnObject = method.invoke(ejbObject, parameters);
                returnType = method.getReturnType();
                                ejbClass.getMethod("remove", null).invoke(ejbObject, null);
            } catch (InvocationTargetException ex) {
                Throwable innerEx = ex.getTargetException();
                throw new ServiceImplementationException(
                    org.act.soap.util.i18n.Message.getMessage(
                    "serviceClassInvokeException", innerEx.getMessage()), innerEx);
            } catch (ClassNotFoundException ex) {
                throw new ProviderException(
                    org.act.soap.util.i18n.Message.getMessage(
                    "serviceClassNotFound", homeName));
            }
                        SOAPMessage responseMessage = mMessageFactory.createMessage();
            responseMessage.setProperty(SOAPMessage.CHARACTER_SET_ENCODING, "UTF-8");
            responseMessage.setProperty(SOAPMessage.WRITE_XML_DECLARATION, "true");
            SOAPPart soapPart = responseMessage.getSOAPPart();
            SOAPEnvelope envelope = soapPart.getEnvelope();
            SOAPBody body = envelope.getBody();
                        envelope.addNamespaceDeclaration(
                TMConstants.NS_XSI_TYPE_PREFIX,
                TMConstants.URI_2001_SCHEMA_XSI);
            envelope.addNamespaceDeclaration(
                TMConstants.URI_DEFAULT_SCHEMA_XSD_PREFIX,
                TMConstants.URI_DEFAULT_SCHEMA_XSD);
            envelope.addNamespaceDeclaration(
                TMConstants.URI_DEFAULT_SOAP_ENC_PREFIX,
                TMConstants.URI_DEFAULT_SOAP_ENC);
                                    typeMappingHandler.serialize(returnObject, returnType, parameters, responseMessage);
                        msgContext.setResponseMessage(responseMessage);
        } catch (SOAPException ex) {
            throw new ProviderException(
                org.act.soap.util.i18n.Message.getMessage(
                "javaSOAPException", ex.getMessage()), ex);
        } catch (Exception ex) {
            throw new ServiceImplementationException(
                org.act.soap.util.i18n.Message.getMessage(
                "serviceClassInvokeException", ex.getMessage()), ex);
        }
    }
}
