package org.act.soap.server.provider;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import org.act.soap.server.MessageContext;
import org.act.soap.server.WebServiceObject;
import org.act.soap.util.TMConstants;
import org.act.soap.util.SOAPUtils;
import org.act.soap.server.*;
import java.lang.reflect.InvocationTargetException;
import javax.activation.DataHandler;
import org.act.soap.util.Constants;
import java.util.Arrays;
import javax.xml.soap.AttachmentPart;

public class JavaProvider extends Provider {
    
    private Class mServiceClass = null;
    
    private Object mServiceObject = null;
    
    private MessageFactory mMessageFactory = null;
    
    public JavaProvider () {
          mMessageFactory = SOAPUtils.MESSAGEFACTORY_INSTANCE;
    }
    
    public void init (org.act.soap.wsdd.Provider wsddProvider) throws ProviderException {
                String serviceClassName = "";
        try {
            serviceClassName = wsddProvider.getParameter("ClassName").getValue();
            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
            mServiceClass = classLoader.loadClass(serviceClassName);
                        mServiceObject = mServiceClass.newInstance();
                        Method initMethod = null;
            try {
                initMethod = mServiceClass.getMethod("init", new Class[0]);
                if (null != initMethod) {
                    initMethod.invoke(mServiceObject, new Object[0]);
                }
            } catch (Exception ex) {
                            }
        } catch (ClassNotFoundException ex) {
            throw new ProviderException(
                org.act.soap.util.i18n.Message.getMessage(
                "serviceClassNotFound", serviceClassName));
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new ProviderException(
                org.act.soap.util.i18n.Message.getMessage(
                "serviceClassInvokeException", ex.getMessage()), ex);
        }
    }
    
    public void doRPCProcess (MessageContext msgContext) throws
        ProviderException, ServiceImplementationException {
        try {
            TypeMappingHandler typeMappingHandler =
                new TypeMappingHandler(msgContext, mServiceClass);
                        Object[] parameters = typeMappingHandler.deserialize();
                        Object returnObject = null;
            Class returnType = null;
            try {
                                Method method = typeMappingHandler.getMethod();
                returnObject = method.invoke(mServiceObject, parameters);
                returnType = method.getReturnType();
            } catch (InvocationTargetException ex) {
                Throwable innerEx = ex.getTargetException();
                throw new ServiceImplementationException(
                    org.act.soap.util.i18n.Message.getMessage(
                    "serviceClassInvokeException", innerEx.getMessage()), innerEx);
            } catch (Exception ex) {
                throw new ServiceImplementationException(
                    org.act.soap.util.i18n.Message.getMessage(
                    "serviceClassInvokeException", ex.getMessage()), ex);
            }
                        SOAPMessage responseMessage = mMessageFactory.createMessage();
            responseMessage.setProperty(SOAPMessage.CHARACTER_SET_ENCODING, "UTF-8");
            responseMessage.setProperty(SOAPMessage.WRITE_XML_DECLARATION, "true");
            SOAPPart soapPart = responseMessage.getSOAPPart();
            SOAPEnvelope envelope = soapPart.getEnvelope();
                        envelope.addNamespaceDeclaration(
                TMConstants.NS_XSI_TYPE_PREFIX,
                TMConstants.URI_2001_SCHEMA_XSI);
            envelope.addNamespaceDeclaration(
                TMConstants.URI_DEFAULT_SCHEMA_XSD_PREFIX,
                TMConstants.URI_DEFAULT_SCHEMA_XSD);
            envelope.addNamespaceDeclaration(
                TMConstants.URI_DEFAULT_SOAP_ENC_PREFIX,
                TMConstants.URI_DEFAULT_SOAP_ENC);
                                    typeMappingHandler.serialize(returnObject, returnType, parameters, responseMessage);
                        msgContext.setResponseMessage(responseMessage);
                        System.err.println(msgContext.getResponseMessage().getSOAPBody().toString());
                    } catch (SOAPException ex) {
            throw new ProviderException(
                org.act.soap.util.i18n.Message.getMessage(
                "javaSOAPException", ex.getMessage()), ex);
        }
    }
    
    public void doDocumentProcess (MessageContext msgContext) throws
        ProviderException, ServiceImplementationException {
        try {
                        String methodName = msgContext.getBindingOperation().getName();
            Method method = getDocumentMethod(mServiceClass, methodName);
                        SOAPMessage requestMessage = msgContext.getRequestMessage();
            SOAPBody requestBody = requestMessage.getSOAPBody();
            List requestElements = new ArrayList(1);
            for (Iterator iterator = requestBody.getChildElements();
                iterator.hasNext(); ) {
                Object subElement = iterator.next();
                if (subElement instanceof SOAPElement) {
                    requestElements.add( (SOAPElement) subElement);
                }
            }
            Class[] javaTypes = method.getParameterTypes();
            Object[] parameters = null;
            if (javaTypes.length == 2 && javaTypes[0] == DataHandler[].class) {
                parameters = new Object[2];
                DataHandler[] dataHandlers = new DataHandler[Constants.MAX_ATTACHMENT_COUNT];
                Arrays.fill(dataHandlers, null);
                Iterator attachmentsIterator = requestMessage.getAttachments();
                int j = 0;
                while (attachmentsIterator.hasNext()) {
                    AttachmentPart attachment = (AttachmentPart) attachmentsIterator.next();
                    try {
                        dataHandlers[j++] = attachment.getDataHandler();
                    } catch (SOAPException ex) {
                        throw new ProviderException(
                            org.act.soap.util.i18n.Message.getMessage(
                            "javaSOAPException", ex.getMessage()), ex);
                    }
                }
                parameters[0] = dataHandlers;
                parameters[1] =
                    (SOAPElement[]) requestElements.toArray(
                    new SOAPElement[requestElements.size()]);
            } else {
                parameters = new Object[] {
                    (SOAPElement[]) requestElements.toArray(
                    new SOAPElement[requestElements.size()])};
            }
                        SOAPElement[] returnElements = null;
            try {
                Object serviceObject = mServiceClass.newInstance();
                returnElements =
                    (SOAPElement[]) method.invoke(
                        serviceObject, parameters);
            } catch (Exception ex) {
                throw new ServiceImplementationException(
                    org.act.soap.util.i18n.Message.getMessage(
                    "serviceClassInvokeException", ex.getMessage()), ex);
            }
                        SOAPMessage responseMessage = mMessageFactory.createMessage();
            responseMessage.setProperty(SOAPMessage.CHARACTER_SET_ENCODING, "UTF-8");
            responseMessage.setProperty(SOAPMessage.WRITE_XML_DECLARATION, "true");
            SOAPBody body = responseMessage.getSOAPBody();
            for (int i = 0; i < returnElements.length; i++) {
                body.addChildElement(returnElements[i]);
            }
                        if (parameters[0].getClass() == DataHandler[].class) {
                DataHandler[] dataHandlers = (DataHandler[]) parameters[0];
                for (int i = 0, n = dataHandlers.length; i < n; i++) {
                    if (null == dataHandlers[i]) break;
                    AttachmentPart attachment =
                        responseMessage.createAttachmentPart(dataHandlers[i]);
                    responseMessage.addAttachmentPart(attachment);
                }
            }
            responseMessage.saveChanges();
            msgContext.setResponseMessage(responseMessage);
        } catch (SOAPException ex) {
            throw new ProviderException(
                org.act.soap.util.i18n.Message.getMessage(
                "javaSOAPException", ex.getMessage()), ex);
        }
    }
    
    public static Method getDocumentMethod (Class aClass, String methodName)
        throws ProviderException {
        
        Method[] methods = aClass.getMethods();
                                                for (int i = 0; i < methods.length; ++i) {
            if (!methodName.equals(methods[i].getName())) continue;
            Class[] parameterTypes = methods[i].getParameterTypes();
            if (1 == parameterTypes.length) {
                if (parameterTypes[0] != SOAPElement[].class) continue;
            } else if (2 == parameterTypes.length) {
                if (parameterTypes[0] != DataHandler[].class) continue;
                if (parameterTypes[1] != SOAPElement[].class) continue;
            } else continue;
            if (methods[i].getReturnType() != SOAPElement[].class) continue;
            return methods[i];
        }
        throw new ProviderException(
            org.act.soap.util.i18n.Message.getMessage(
            "noServiceMethod", methodName));
    }
    
    public String getServiceClassName () {
        return mServiceClass.getName();
    }
}
