
package org.act.soap.server.provider;
import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.extensions.soap.SOAPBinding;
import javax.wsdl.extensions.soap.SOAPOperation;
import org.act.soap.server.MessageContext;
import org.act.soap.server.SOAPEngineException;
import org.act.soap.server.WebServiceObject;
import org.act.soap.util.Constants;
import org.act.soap.util.i18n.Message;

public abstract class Provider {
    
    public Provider () {
    }
    
    protected static String getSOAPBindingStyle (Definition definition)
        throws SOAPEngineException {
        Binding binding =
            (Binding) definition.getBindings().values().iterator().next();
        if (1 > binding.getExtensibilityElements().size()) {
            throw new SOAPEngineException(
                Message.getMessage("wsdlFileException"));
        }
        SOAPBinding soapBinding =
            (SOAPBinding) binding.getExtensibilityElements().get(0);
        return soapBinding.getStyle();
    }
    
    protected static String getOperationStyle (String soapBindingStyle,
        BindingOperation bindingOperation) {
        SOAPOperation soapOperation =
            (SOAPOperation) bindingOperation.getExtensibilityElements().get(0);
        String style = Constants.PROCESS_STYLE_DOCUMENT;
        if (null != soapOperation.getStyle()) {
            style = soapOperation.getStyle();
        }
        else if (null != soapBindingStyle) {
            style = soapBindingStyle;
        }
        return style;
    }
    
    public abstract void init (org.act.soap.wsdd.Provider wsddProvider) throws ProviderException;
    
    public void process (MessageContext messageContext)
        throws ProviderException, ServiceImplementationException {
        BindingOperation bindingOperation =
            messageContext.getBindingOperation();
                if (null == bindingOperation) {
            throw new ProviderException(
            Message.getMessage("noWSDLOperation"));
        }
                if (1 > bindingOperation.getExtensibilityElements().size()) {
            throw new ProviderException(
            Message.getMessage("wsdlFileException"));
        }
                String soapBindingStyle = null;
        try {
            soapBindingStyle =
                getSOAPBindingStyle(messageContext.getWSDLDefinition());
        } catch (SOAPEngineException ex) {
            throw new ProviderException(
                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
        }
                String processStyle =
            getOperationStyle(soapBindingStyle, bindingOperation);
        if (processStyle.equals(Constants.PROCESS_STYLE_RPC)) {
            doRPCProcess(messageContext);
        } else if (processStyle.equals(Constants.PROCESS_STYLE_DOCUMENT)) {
            doDocumentProcess(messageContext);
        } else {
            throw new ProviderException(
                Message.getMessage("unsupportedProcessType", processStyle));
        }
    }
    
    protected abstract void doRPCProcess (MessageContext msgContext )
        throws ProviderException, ServiceImplementationException;
    
    protected abstract void doDocumentProcess (MessageContext msgContext )
        throws ProviderException, ServiceImplementationException;
}
