package org.act.soap.server.provider;
import java.util.HashMap;
import java.util.Map;
import org.act.soap.util.i18n.Message;

public class ProviderFactory {
    
    private HashMap mProviderRegistryMap;
    
    private ProviderFactory () {
        mProviderRegistryMap = new HashMap(1);
        mProviderRegistryMap.put("java", JavaProvider.class.getName());
    }
    
    public static ProviderFactory newInstance () {
        return new ProviderFactory();
    }
    
    public Provider createProvider (String providerType)
        throws ProviderException {
        String providerName = (String) mProviderRegistryMap.get(providerType);
        if (null == providerName) {
            throw new ProviderException(
                Message.getMessage("unknownProviderType", providerType));
        }
        Provider provider = null;
        try {
            ClassLoader classLoader =
                Thread.currentThread().getContextClassLoader();
            Class providerImpl = classLoader.loadClass(providerName);
            provider = (Provider) providerImpl.newInstance();
        } catch (Exception e) {
            throw new ProviderException(
                Message.getMessage("newInstanceException", e.getMessage()), e);
        }
        return provider;
    }
    
    public void registerProvider (String providerType, String providerClassName)
        throws ProviderException {
        try {
            ClassLoader classLoader =
                Thread.currentThread().getContextClassLoader();
            classLoader.loadClass(providerClassName);
        } catch (ClassNotFoundException ex) {
            throw new ProviderException(
                Message.getMessage("javaClassNotFound", ex.getMessage()));
        }
        mProviderRegistryMap.put(providerType, providerClassName);
    }
    
    public Map getProviderRegistryMap () {
        return mProviderRegistryMap;
    }
    
    public void removeProvider (String providerType)
        throws ProviderException {
        String providerName = (String) mProviderRegistryMap.get(providerType);
        if (null == providerName) {
            throw new ProviderException(
                Message.getMessage("unknownProviderType", providerType));
        }
        mProviderRegistryMap.remove(providerType);
    }
}
