package org.act.soap.server.provider;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.wsdl.BindingOperation;
import javax.wsdl.Operation;
import javax.wsdl.Part;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.Deserializer;
import javax.xml.rpc.encoding.DeserializerFactory;
import javax.xml.rpc.encoding.SerializerFactory;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import org.act.soap.encoding.DeserializationContextImpl;
import org.act.soap.encoding.DeserializerImpl;
import org.act.soap.encoding.SerializationContextImpl;
import org.act.soap.encoding.SerializerImpl;
import org.act.soap.encoding.TypeMappingException;
import org.act.soap.server.MessageContext;
import org.act.soap.util.Constants;
import org.act.soap.util.TMConstants;
import org.act.soap.util.i18n.Message;
import javax.activation.DataHandler;
import javax.xml.soap.AttachmentPart;

public class TypeMappingHandler {
    
    private MessageContext mMessageContext = null;
    
    private Class mServiceClass = null;
    
    private TypeMapping mTypeMapping = null;
    
    private Method mMethod = null;
    
    private int mMaxAttachmentCount = Constants.MAX_ATTACHMENT_COUNT;
    
    public TypeMappingHandler (MessageContext msgContext, Class serviceClass)
        throws ProviderException {
        mMessageContext = msgContext;
        if (null == mMessageContext) {
            throw new ProviderException(
            Message.getMessage("nullMessageContext"));
        }
        mServiceClass = serviceClass;
        if (null == mServiceClass) {
            throw new ProviderException(Message.getMessage("nullServiceClass"));
        }
        
        mTypeMapping =
            mMessageContext.getTypeMappingRegistry().getDefaultTypeMapping();
    }
        public TypeMappingHandler(MessageContext msgContext)throws ProviderException{
        mMessageContext = msgContext;
        if (null == mMessageContext) {
            throw new ProviderException(
            Message.getMessage("nullMessageContext"));
        }
        
       mTypeMapping =
           mMessageContext.getTypeMappingRegistry().getDefaultTypeMapping();
    }
        
    private TypeMappingHandler () {
    }
    
    private static QName[] getXMLTypes (Operation operation) {
        List paramOrdering = operation.getParameterOrdering();
        List inputParts =
            operation.getInput().getMessage().getOrderedParts(paramOrdering);
        List inputTypes = new ArrayList(inputParts.size());
        for (int i = 0; i < inputParts.size(); i++) {
            Part part = (Part) inputParts.get(i);
            inputTypes.add(part.getTypeName());
        }
        return (QName[]) inputTypes.toArray(new QName[inputTypes.size()]);
    }
    
    private static QName getReturnXMLType (Operation operation) {
        javax.wsdl.Message outputMessage = operation.getOutput().getMessage();
        Iterator parts = outputMessage.getParts().values().iterator();
        return parts.hasNext() ? ((Part) parts.next()).getTypeName() : null;
    }
    
    private static QName getReturnName (Operation operation) {
        javax.wsdl.Message outputMessage = operation.getOutput().getMessage();
        Iterator parts = outputMessage.getParts().values().iterator();
        return
            parts.hasNext() ? new QName(((Part) parts.next()).getName()) : null;
    }
    
    private static QName getResponseName (Operation operation) {
        return operation.getOutput().getMessage().getQName();
    }
    
    private static SOAPElement[] getParamElements (SOAPMessage requestMsg)
        throws ProviderException {
        try {
            javax.xml.soap.SOAPBody body = requestMsg.getSOAPBody();
            SOAPElement opElement = null;
            for (Iterator elements = body.getChildElements();
                elements.hasNext(); ) {
                Object subElement = elements.next();
                if (subElement instanceof SOAPElement) {
                    opElement = (SOAPElement) subElement;
                    break;
                }
            }
            if (null == opElement) {
                throw new ProviderException(
                Message.getMessage("noSOAPElementInSOAPBody"));
            }
            List paramList = new ArrayList(0);
            for (Iterator elements = opElement.getChildElements();
                elements.hasNext(); ) {
                Object subElement = elements.next();
                if (subElement instanceof SOAPElement) {
                    paramList.add( (SOAPElement) subElement);
                }
            }
            return (SOAPElement[]) paramList.toArray(
                new SOAPElement[paramList.size()]);
        } catch (SOAPException ex) {
            throw new ProviderException(
                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
        }
    }
    
    private static boolean isMatchedMethod (Method method,
                                    QName[] xmlTypes,
                                    TypeMapping typeMapping,
                                    List deserializers) {
        Class[] paramTypes = method.getParameterTypes();
        for (int i = 0; i < paramTypes.length; i++) {
            DeserializerFactory deserializerFactory =
                typeMapping.getDeserializer(paramTypes[i], xmlTypes[i]);
            if (null == deserializerFactory) {
                return false;
            }
            Deserializer deserializer =
                deserializerFactory.getDeserializerAs(
                TMConstants.ENCODING_MECHANISMTYPE);
            deserializers.add(deserializer);
        }
        return true;
    }
    
    private static Method findMethod (Class aClass,
                                String methodName,
                                QName[] xmlTypes,
                                TypeMapping typeMapping,
                                List deserializers) {
        Method[] methods = aClass.getMethods();
        for (int i = 0; i < methods.length; i++) {
            if (!methods[i].getName().equals(methodName)) {
                continue;
            }
            deserializers.clear();
            if (isMatchedMethod(methods[i],
                xmlTypes, typeMapping, deserializers)) {
                return methods[i];
            }
        }
        return null;
    }
    
    public void serialize (Object returnObject, Class returnType,
        Object[] parameters, SOAPMessage responseMessage)
        throws ProviderException {
        SOAPBody soapBody = null;
        try {
            soapBody = responseMessage.getSOAPBody();
        } catch (SOAPException ex) {
            throw new ProviderException(
                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
        }
                BindingOperation bindingOperation =
            mMessageContext.getBindingOperation();
        Operation operation = bindingOperation.getOperation();
        SOAPElement responseSOAPElement = null;
        try {
            QName responseQName = getResponseName(operation);
            responseSOAPElement =
                soapBody.addChildElement(responseQName.getLocalPart());
        } catch (SOAPException ex) {
            throw new ProviderException(
                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
        }
                QName xmlType = getReturnXMLType(operation);
        if (null == xmlType) {
            return ;
        }
                SerializerFactory serializerFactory =
            mTypeMapping.getSerializer(returnType, xmlType);
        if (null == serializerFactory) {
            String methodName = operation.getName();
            throw new ProviderException(
            Message.getMessage("noServiceMethod", methodName));
        }
        List returnElements = new ArrayList(1);
        try {
            SerializerImpl serializer =
                (SerializerImpl) serializerFactory.getSerializerAs(
                TMConstants.ENCODING_MECHANISMTYPE);
            SerializationContextImpl serializeContext =
                new SerializationContextImpl(
                    null, mMessageContext.getTypeMappingRegistry());
            QName returnName = getReturnName(operation);
            returnElements.add(
                serializer.serialize(returnObject,
                                    returnName,
                                    returnType,
                                    xmlType,
                                    serializeContext));
            SOAPElement[] serializeTrailingBlocks =
                serializeContext.serializeTrailingBlocks();
            if (null != serializeTrailingBlocks) {
                returnElements.addAll(Arrays.asList(serializeTrailingBlocks));
            }
        } catch (TypeMappingException ex) {
            throw new ProviderException(
                Message.getMessage(
                "actTypeMappingException", ex.getMessage()), ex);
        }
                try {
            for (int i = 0; i < returnElements.size(); i++) {
                responseSOAPElement.addChildElement(
                    (SOAPElement) returnElements.get(i));
            }
                        if (parameters.length > 0) {
                if (parameters[0].getClass() == DataHandler[].class) {
                    DataHandler[] dataHandlers = (DataHandler[]) parameters[0];
                    for (int i = 0, n = dataHandlers.length; i < n; i++) {
                        if (null == dataHandlers[i]) break;
                        AttachmentPart attachment =
                            responseMessage.createAttachmentPart(dataHandlers[i]);
                        responseMessage.addAttachmentPart(attachment);
                    }
                    responseMessage.saveChanges();
                }
            }
        } catch (SOAPException ex) {
            throw new ProviderException(
                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
        }
    }
   
    public Object[] deserialize () throws ProviderException {
        BindingOperation bindingOperation =
            mMessageContext.getBindingOperation();
        Operation operation = bindingOperation.getOperation();
                String methodName = operation.getName();
                QName[] xmlTypes = getXMLTypes(operation);
                List deserializers = new ArrayList(xmlTypes.length);
        mMethod = findMethod(mServiceClass,
                            methodName,
                            xmlTypes,
                            mTypeMapping,
                            deserializers);
        if (null == mMethod) {
            throw new ProviderException(
            Message.getMessage("noServiceMethod", methodName));
        }
        try {
            SOAPMessage requestMessage = mMessageContext.getRequestMessage();
                        SOAPElement[] paramElements = getParamElements(requestMessage);
                        Class[] javaTypes = mMethod.getParameterTypes();
            Object[] paramObjects = new Object[xmlTypes.length];
            DeserializationContextImpl deserializeContext =
                new DeserializationContextImpl(
                    mMessageContext.getRequestMessage(),
                    mMessageContext.getTypeMappingRegistry());
            for (int i = 0; i < xmlTypes.length; i++) {
                if (javaTypes[i] == DataHandler[].class) {
                    if (0 != i) {
                        throw new ProviderException(
                            Message.getMessage("attachmentParameterException"));
                    }
                                        DataHandler[] dataHandlers = new DataHandler[mMaxAttachmentCount];
                    Arrays.fill(dataHandlers, null);
                    Iterator attachmentsIterator = requestMessage.getAttachments();
                    int j = 0;
                    while (attachmentsIterator.hasNext()) {
                        AttachmentPart attachment = (AttachmentPart) attachmentsIterator.next();
                        try {
                            dataHandlers[j++] = attachment.getDataHandler();
                        } catch (SOAPException ex) {
                            throw new ProviderException(
                                Message.getMessage("javaSOAPException", ex.getMessage()), ex);
                        }
                    }
                    paramObjects[i] = dataHandlers;
                    continue;
                }
                DeserializerImpl deserializer =
                    (DeserializerImpl) deserializers.get(i);
                paramObjects[i] = deserializer.deserialize(paramElements[i],
                                                            xmlTypes[i],
                                                            javaTypes[i],
                                                            deserializeContext);
            }
            return paramObjects;
        } catch (TypeMappingException ex) {
            throw new ProviderException(
                Message.getMessage("actTypeMappingException", ex.getMessage()),
                ex);
        }
    }
    
    public Method getMethod () {
        return mMethod;
    }
}
