package org.act.soap.server.rbac;
import java.security.*;
import java.util.*;
import javax.security.auth.*;
import javax.xml.soap.*;
import org.act.rbac.ri0.*;
import org.act.soap.server.*;
import org.act.soap.server.secmanager.*;
import org.act.soap.util.i18n.*;
import org.act.soap.util.logging.*;
import org.act.soap.wsdd.*;
import org.w3c.dom.*;
import org.act.soap.server.config.ConfigManager;

public class RBAC {
    private static Log log = LogFactory.getLog( RBAC.class.getName() );
    private static final String PARAM_NAME = "RBACConfig";
    private static final String TAG_ROLES = "Roles";
    private static final String TAG_ROLE = "Role";
    private static final String ATTR_NAME = "Name";
    private static final String TAG_DESCRIPTION = "Description";
    private static final String TAG_GRANTED_PERMISSIONS = "GrantedPermissions";
    private static final String TAG_PERMISSION = "Permission";
    private static final String ATTR_OPERATION_NAME = "OperationName";
    private static final String TAG_IMMEDIATE_DESCENDANTS =
        "ImmediateDescendants";
    private static final String TAG_ROLE_NAME = "RoleName";
    private static final String TAG_CARDINALITY_CONSTRAINTS =
        "CardinalityConstraints";
    private static final String TAG_CARDINALITY_CONSTRAINT =
        "CardinalityConstraint";
    private static final String ATTR_ROLE_NAME = "RoleName";
    private static final String ATTR_CARDINALITY = "Cardinality";
    private static final String TAG_STATIC_SEPARATION_OF_DUTIES =
        "StaticSeparationOfDuties";
    private static final String TAG_STATIC_SEPARATION_OF_DUTY =
        "StaticSeparationOfDuty";
    private static final String TAG_ROLE_SET = "RoleSet";
    private static final String TAG_DYNAMIC_SEPARATION_OF_DUTIES =
        "DynamicSeparationOfDuties";
    private static final String TAG_DYNAMIC_SEPARATION_OF_DUTY =
        "DynamicSeparationOfDuty";
    
    private GeneralHierarchicalRBACWithSSDAndCCAndDSD rbacRT;
    
    private Random random = new Random( System.currentTimeMillis() );
    
    protected org.act.soap.wsdd.RBAC wsddRBAC;
    
    public void init ( org.act.soap.wsdd.RBAC wsddRBAC ) throws Exception {
        this.wsddRBAC = wsddRBAC;
                this.rbacRT = new GeneralHierarchicalRBACWithSSDAndCCAndDSD();
                log.info( Message.getMessage( "INFO_BEGIAN_RBAC" ) );
        try {
                        Parameter param = wsddRBAC.getParameter( PARAM_NAME );
            if ( param != null ) {
                Element eConfig = param.getParameterElement();
                                initRoles( eConfig );
                initCC( eConfig );
                initSSD( eConfig );
                initDSD( eConfig );
            }
        }
        catch ( Exception ex ) {
            log.debug( ex );
            throw new RBACHandlerException( "Init rbac runtime failed. Reason:"
                                            + ex.getMessage() );
        }
        finally {
                        log.info( Message.getMessage( "INFO_SUCCESS_RBAC" ) );
                    }
    }
    
    private void initRoles ( Element eConfig ) throws Exception {
        ConfigManager configManager = ConfigManager.getInstance();
        RBACManager rbacManager = configManager.getRBACManager();
        NodeList nl = eConfig.getElementsByTagName( TAG_ROLES );
        if ( nl == null || nl.getLength() < 0 ) {
            throw new RBACHandlerException( "ȱٽɫϢ" );
        }
        Element eRoles = ( Element ) nl.item( 0 );
        NodeList nlRole = eRoles.getElementsByTagName( TAG_ROLE );
        for ( int i = 0; i < nlRole.getLength(); i++ ) {
                        Element eRole = ( Element ) nlRole.item( i );
            String sRoleName = eRole.getAttribute( ATTR_NAME );
            if ( sRoleName == null || sRoleName.equals( "" ) ) {
                throw new RBACHandlerException( "ɫ" + i + "ȱơ" );
            }
            RBACRole role = new RBACRole( sRoleName );
            rbacRT.AddRole( role );
            String sWSARName = this.wsddRBAC.getDeployment().getName();
                        int len = sWSARName.length();
            if ( sWSARName != null && len > 5 ) {
                String ext = sWSARName.substring( len - 5, len );
                if ( ext.equalsIgnoreCase( ".wsar" ) ) {
                    sWSARName = sWSARName.substring( 0, len - 5 );
                }
            }
            String sWSName = this.wsddRBAC.getDeployment().getServiceName();
                        String sRoleNameCom = sWSARName + "." + sWSName + "." + sRoleName;
                        int[] userID = rbacManager.getUserIDsInRole( sRoleNameCom );
            for ( int j = 0; j < userID.length; j++ ) {
                RBACUser user = new RBACUser( String.valueOf( userID[j] ) );
                try {
                    rbacRT.AddUser( user );
                }
                catch ( Exception e ) {
                                    }
                rbacRT.AssignUser( user, role );
            }
                        Element eGrantedPermissions = ( Element ) eRole.
                getElementsByTagName( TAG_GRANTED_PERMISSIONS ).item( 0 );
            NodeList nlPermission = eGrantedPermissions.getElementsByTagName(
                TAG_PERMISSION );
            for ( int j = 0; j < nlPermission.getLength(); j++ ) {
                Element ePermission = ( Element ) nlPermission.item( j );
                String sOperationName = ePermission.getAttribute(
                    ATTR_OPERATION_NAME );
                if ( sOperationName == null || sOperationName.equals( "" ) ) {
                    continue;
                }
                                RBACPermission permission = new RBACPermission( sOperationName );
                try {
                    rbacRT.AddPermission( permission );
                }
                catch ( Exception e ) {
                                    }
                rbacRT.GrantPermission( permission, role );
            }
        }
                for ( int i = 0; i < nlRole.getLength(); i++ ) {
            Element eRole = ( Element ) nlRole.item( i );
            String sRoleName = eRole.getAttribute( ATTR_NAME );
                        NodeList nlImmediateDescendants = eRole.getElementsByTagName(
                TAG_IMMEDIATE_DESCENDANTS );
            if ( nlImmediateDescendants == null
                 || nlImmediateDescendants.getLength() <= 0 ) {
                continue;
            }
            Element eImmediateDescendants = ( Element ) nlImmediateDescendants.
                item( 0 );
            NodeList nlRoleName = eImmediateDescendants.getElementsByTagName(
                TAG_ROLE_NAME );
            for ( int j = 0; j < nlRoleName.getLength(); j++ ) {
                Element eRoleName = ( Element ) nlRoleName.item( j );
                String sDescRoleName = eRoleName.getFirstChild().getNodeValue();
                rbacRT.AddInheritance( new RBACRole( sRoleName ),
                                       new RBACRole( sDescRoleName ) );
            }
        }
    }
    
    private void initCC ( Element eConfig ) throws Exception {
        NodeList nl = eConfig.getElementsByTagName( TAG_CARDINALITY_CONSTRAINTS );
        if ( nl == null || nl.getLength() < 1 ) {
            return;
        }
        Element eCCs = ( Element ) nl.item( 0 );
                                NodeList nlCC = eCCs.getElementsByTagName( TAG_CARDINALITY_CONSTRAINT );
        for ( int i = 0; i < nlCC.getLength(); i++ ) {
            Element eCC = ( Element ) nlCC.item( i );
            String sCCName = eCC.getAttribute( ATTR_NAME );
            String sRoleName = eCC.getAttribute( ATTR_ROLE_NAME );
            String sCardinality = eCC.getAttribute( ATTR_CARDINALITY );
            int nCardinality = Integer.parseInt( sCardinality );
            rbacRT.AddCardinalityConstraint( sCCName, new RBACRole( sRoleName ),
                                             nCardinality );
                                            }
                            }
    
    private void initSSD ( Element eConfig ) throws Exception {
        NodeList nl = eConfig.getElementsByTagName(
            TAG_STATIC_SEPARATION_OF_DUTIES );
        if ( nl == null || nl.getLength() < 1 ) {
            return;
        }
        Element eSSDs = ( Element ) nl.item( 0 );
                                NodeList nlSSD = eSSDs.getElementsByTagName(
            TAG_STATIC_SEPARATION_OF_DUTY );
        for ( int i = 0; i < nlSSD.getLength(); i++ ) {
            Element eSSD = ( Element ) nlSSD.item( i );
            String sSSDName = eSSD.getAttribute( ATTR_NAME );
            String sCardinality = eSSD.getAttribute( ATTR_CARDINALITY );
            int nCardinality = Integer.parseInt( sCardinality );
            Set roleSet = new HashSet();
            Element eRoleSet = ( Element ) eSSD.getElementsByTagName(
                TAG_ROLE_SET ).item( 0 );
            NodeList nlRoleName = eRoleSet.getElementsByTagName( TAG_ROLE_NAME );
            if ( nlRoleName != null && nlRoleName.getLength() > 0 ) {
                for ( int j = 0; j < nlRoleName.getLength(); j++ ) {
                    Element eRoleName = ( Element ) nlRoleName.item( j );
                    String sRoleName = eRoleName.getFirstChild().getNodeValue();
                                        roleSet.add( new RBACRole( sRoleName ) );
                }
                rbacRT.CreateSsdSet( sSSDName, roleSet, nCardinality );
            }
        }
                            }
    
    private void initDSD ( Element eConfig ) throws Exception {
        NodeList nl = eConfig.getElementsByTagName(
            TAG_DYNAMIC_SEPARATION_OF_DUTIES );
        if ( nl == null || nl.getLength() < 1 ) {
            return;
        }
        Element eDSDs = ( Element ) nl.item( 0 );
        NodeList nlDSD = eDSDs.getElementsByTagName(
            TAG_DYNAMIC_SEPARATION_OF_DUTY );
        for ( int i = 0; i < nlDSD.getLength(); i++ ) {
            Element eDSD = ( Element ) nlDSD.item( i );
            String sDSDName = eDSD.getAttribute( ATTR_NAME );
            String sCardinality = eDSD.getAttribute( ATTR_CARDINALITY );
            int nCardinality = Integer.parseInt( sCardinality );
                        Set roleSet = new HashSet();
            Element eRoleSet = ( Element ) eDSD.getElementsByTagName(
                TAG_ROLE_SET ).item( 0 );
            NodeList nlRoleName = eRoleSet.getElementsByTagName( TAG_ROLE_NAME );
            if ( nlRoleName != null && nlRoleName.getLength() > 0 ) {
                for ( int j = 0; j < nlRoleName.getLength(); j++ ) {
                    Element eRoleName = ( Element ) nlRoleName.item( j );
                    String sRoleName = eRoleName.getFirstChild().getNodeValue();
                                        roleSet.add( new RBACRole( sRoleName ) );
                }
                rbacRT.CreateDsdSet( sDSDName, roleSet, nCardinality );
            }
        }
    }
    public void process ( MessageContext msgContext ) throws
        RBACHandlerException {
        String sWebServiceName = msgContext.getWebServiceName();
        String sUserID = null;
        String sUserName = null;
        try {
            Subject subjectUser = msgContext.getUserSubject();
            Principal principalUser = ( Principal ) subjectUser.getPrincipals().
                iterator().next();
            sUserID = principalUser.getName();
            UserManager um = null;
            ConfigManager configManager = ConfigManager.getInstance();
            if ( configManager != null ) {
                um = configManager.getUserManager();
            }
            sUserName = um.getUserName( Integer.parseInt( sUserID ) );
        }
        catch ( Exception ex ) {
            log.debug( ex );
            throw new RBACHandlerException(
                "Authentication failed. Can not get user id." );
        }
                
                                String sOperationName = null;
        try {
                                    SOAPMessage m = msgContext.getRequestMessage();
            m.getSOAPPart().getEnvelope().getBody();
            sOperationName = msgContext.getBindingOperation().getName();
        }
        catch ( Exception ex ) {
                        System.out.println( Message.getMessage(
                "*************************************************************************" ) );
            System.out.println( Message.getMessage(
                "                   Can't get requesting operation." ) );
            System.out.println( Message.getMessage(
                "*************************************************************************" ) );
                        throw new RBACHandlerException( "Can't get requesting operation." );
        }
        if ( sOperationName == null ) {
                        System.out.println( Message.getMessage(
                "*************************************************************************" ) );
            System.out.println( Message.getMessage(
                "                   No Operation requested." ) );
            System.out.println( Message.getMessage(
                "*************************************************************************" ) );
                        return;
        }
        boolean bResult = false;
        try {
                        RBACPermission requestingPermission = new RBACPermission(
                sOperationName );
            String sSessionID = String.valueOf( random.nextInt() );
            RBACSession currentSession = new RBACSession( sSessionID );
            RBACUser currentUser = new RBACUser( sUserID );
                                                Set stRoleSet = rbacRT.AuthorizedRoles( currentUser );
            rbacRT.CreateSession( currentUser, stRoleSet, currentSession );
            bResult = rbacRT.CheckAccess( currentSession, requestingPermission );
        }
        catch ( Exception ex ) {
            log.debug( ex );
            bResult = false;
        }
        if ( bResult ) {
                        System.out.println( Message.getMessage( "OperationCallOK",
                sOperationName ) );
                    }
        else {
                                    System.out.println( Message.getMessage( "OperationCallFailed",
                sOperationName ) );
            System.out.println( Message.getMessage( "MsgEnd" ) );
                        throw new RBACHandlerException( "Access denied. Operation = "
                                            + sOperationName );
        }
    }
    public void postProcess ( MessageContext msgContext ) throws
        RBACHandlerException {
    }
}
