package org.act.soap.server.rbac;
import java.util.*;
import org.act.soap.util.i18n.*;

public class RBACFactory {
    
    private HashMap mRBACRegistryMap;
    
    private RBACFactory () {
        mRBACRegistryMap = new HashMap( 1 );
        mRBACRegistryMap.put( "rbac", RBAC.class.getName() );
    }
    
    public static RBACFactory newInstance () {
        return new RBACFactory();
    }
    
    public RBAC createRBAC ( String rbacType ) throws RBACHandlerException {
        String rbacName = ( String ) mRBACRegistryMap.get( rbacType );
        if ( null == rbacName ) {
            throw new RBACHandlerException(
                Message.getMessage( "unknownRBACType", rbacType ) );
        }
        RBAC rbac = null;
        try {
            ClassLoader classLoader =
                Thread.currentThread().getContextClassLoader();
            Class rbacImpl = classLoader.loadClass( rbacName );
            rbac = ( RBAC ) rbacImpl.newInstance();
        }
        catch ( Exception e ) {
            throw new RBACHandlerException(
                Message.getMessage( "newInstanceException", e.getMessage() ) );
        }
        return rbac;
    }
    
    public void registerRBAC ( String rbacType, String rbacClassName ) throws
        RBACHandlerException {
        if ( null == rbacType || null == rbacClassName ) {
            throw new RBACHandlerException( Message.getMessage(
                "ERR_TP_NullArgument" ) );
        }
        try {
            ClassLoader classLoader =
                Thread.currentThread().getContextClassLoader();
            classLoader.loadClass( rbacClassName );
        }
        catch ( ClassNotFoundException ex ) {
            throw new RBACHandlerException(
                Message.getMessage( "ClassNotFound", ex.getMessage() ) );
        }
        mRBACRegistryMap.put( rbacType, rbacClassName );
    }
    
    public Map getRBACRegistryMap () {
        return mRBACRegistryMap;
    }
    
    public void removeRBAC ( String rbacType ) throws RBACHandlerException {
        String rbacName = ( String ) mRBACRegistryMap.get( rbacType );
        if ( null == rbacName ) {
            throw new RBACHandlerException(
                Message.getMessage( "unknownRBACType", rbacType ) );
        }
        mRBACRegistryMap.remove( rbacType );
    }
}
