package org.act.soap.server.secmanager;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.*;
import javax.crypto.spec.*;
import javax.xml.parsers.*;
import org.act.soap.server.config.*;
import org.act.soap.server.secmanager.keymanager.*;
import org.w3c.dom.*;

public class CredManager {
    
    public final static String CREDTYPE_PUBLICKEYNAME = "PublicKeyName";
    public final static String CREDTYPE_PRIVATEKEYNAME = "PrivateKeyName";
    public final static String CREDTYPE_CERTIFICATENAME = "X509CertName";
    public final static String CREDTYPE_X509_DN = "X509_DN";
    public final static String CREDTYPE_PRIVATEKEYDN = "PrivateKey_DN";
    public final static String CREDTYPE_SAML = "SAML";
    public final static String CREDTYPE_VERIFYCERTVALIDATE =
        "VerifyCertificate";
    public final static String CREDTYPE_IMPORTKEY = "ImportKey";
    
    Hashtable hDeviceWithCapability = null;     
    Vector keyDevices = null;
                ConfigManager configManager = ConfigManager.getInstance();
    
    public CredManager ( String configFile ) {
        Element config = null;
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware( true );
            DocumentBuilder db = dbf.newDocumentBuilder();
            File file = new File( configFile );
            Document doc = db.parse( file );
            Element elt = doc.getDocumentElement();
            if ( elt.getNodeName().equalsIgnoreCase( "Configuration" ) ) {
                config = elt;
            }
            else {
                config = ( Element ) elt.getElementsByTagName( "Configuration" ).
                    item( 0 );
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
        hDeviceWithCapability = new Hashtable();
        init( config );
    }
    
    
     
      public Key getPublicKey ( String keyTag ) {
          String[] keyTags = new String[1];
          keyTags[0] = keyTag;
          Object cred = this.getCredential( this.CREDTYPE_PUBLICKEYNAME,
                                            keyTags );
          if ( cred instanceof java.security.Key ) {
              return ( Key ) cred;
          }
          return null;
      }
    
    public Key getPublicKey ( String keyTag, String keydevice ) {
        String[] keyTags = new String[1];
        keyTags[0] = keyTag;
        Object cred = this.getCredential( this.CREDTYPE_PUBLICKEYNAME,
                                          keyTags, keydevice );
        if ( cred instanceof java.security.Key ) {
            return ( Key ) cred;
        }
        return null;
    }
    
    public Key getPrivateKey ( String keyTags[] ) {
        Object cred = this.getCredential( this.CREDTYPE_PRIVATEKEYNAME, keyTags );
        if ( cred instanceof java.security.Key ) {
            return ( Key ) cred;
        }
        cred = this.getCredential( this.CREDTYPE_PRIVATEKEYDN, keyTags );
        if ( cred instanceof java.security.Key ) {
            return ( Key ) cred;
        }
        return null;
    }
    
    public Key getPrivateKey ( String alias, String password ) {
        String[] keyTags = {
            alias, password};
        return getPrivateKey( keyTags );
    }
    
    public Key getPrivateKey ( String keyTags[], String keydevice ) {
        Object cred = this.getCredential( this.CREDTYPE_PRIVATEKEYNAME,
                                          keyTags, keydevice );
        if ( cred instanceof java.security.Key ) {
            return ( Key ) cred;
        }
        cred = this.getCredential( this.CREDTYPE_PRIVATEKEYDN,
                                   keyTags, keydevice );
        if ( cred instanceof java.security.Key ) {
            return ( Key ) cred;
        }
        return null;
    }
    
    public Key getPrivateKey ( String alias, String password, String keydevice ) {
        String[] keyTags = {
            alias, password};
        return getPrivateKey( keyTags, keydevice );
    }
    
    public X509Certificate getX509Cert ( String keyTag ) {
        String[] keyTags = new String[1];
        keyTags[0] = keyTag;
        Object cred = this.getCredential( this.CREDTYPE_CERTIFICATENAME,
                                          keyTags );
        if ( cred instanceof java.security.cert.X509Certificate ) {
            return ( X509Certificate ) cred;
        }
        cred = this.getCredential( this.CREDTYPE_X509_DN, keyTags );
        if ( cred instanceof java.security.cert.X509Certificate ) {
            return ( X509Certificate ) cred;
        }
        return null;
    }
    
    public X509Certificate getX509Cert ( String keyTag, String keydevice ) {
        String[] keyTags = new String[1];
        keyTags[0] = keyTag;
        Object cred = this.getCredential( this.CREDTYPE_CERTIFICATENAME,
                                          keyTags, keydevice );
        if ( cred instanceof java.security.cert.X509Certificate ) {
            return ( X509Certificate ) cred;
        }
        cred = this.getCredential( this.CREDTYPE_X509_DN,
                                   keyTags, keydevice );
        if ( cred instanceof java.security.cert.X509Certificate ) {
            return ( X509Certificate ) cred;
        }
        return null;
    }
    
    public Object getCredential ( String credType, String keyTags[] ) {
        Object cred = null;
                Vector devices = ( Vector )this.hDeviceWithCapability.get( credType );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                cred = device.getCredential( credType, keyTags );
                if ( cred != null ) {
                    break;
                }
            }
        }
        return cred;
    }
    
    public Object getCredential ( String credType, String keyTag[],
                                  String keydevice ) {
        Object cred = null;
                Vector devices = ( Vector )this.hDeviceWithCapability.get( credType );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                if ( device.getDeviceName().equalsIgnoreCase( keydevice ) ) {
                    cred = device.getCredential( credType, keyTag );
                    break;
                }
            }
        }
        return cred;
    }
    
    private void init ( org.w3c.dom.Element config ) {
        String sDeviceName = null;
        String sKeyStoreFile = null;
        String sKeyStoreFilePassWord = null;
        String sCredType = null;
        if ( config == null ) {
            return;
        }
        Element elmCredManager = ( Element ) config.getElementsByTagName(
            configManager.TAG_NAME_CREDMANAGER ).item( 0 );
        NodeList devices = elmCredManager.getElementsByTagName( configManager.
            TAG_NAME_CREDDEVICE );
                                for ( int i = 0; i < devices.getLength(); i++ ) {
            Element device = ( Element ) devices.item( i );
                                    String sType = device.getAttribute( configManager.ATTR_NAME_TYPE );
                        String sClassName = device.getAttribute( configManager.
                ATTR_NAME_CLASSNAME );
                        Element elmDeviceName = ( Element ) device.getElementsByTagName(
                configManager.TAG_NAME_DEVICENAME ).item( 0 );
            Text txtDeviceName = ( Text ) device.getElementsByTagName(
                configManager.TAG_NAME_DEVICENAME ).item( 0 ).getFirstChild();
            if ( txtDeviceName != null ) {
                sDeviceName = txtDeviceName.getNodeValue();
            }
                        Element elmDeviceConfig = ( Element ) device.getElementsByTagName(
                configManager.TAG_NAME_DEVICECONFIG ).item( 0 );
                        Element elmKeyStoreFile = ( Element ) elmDeviceConfig.
                getElementsByTagName( configManager.TAG_NAME_KEYSTOREFILE ).
                item( 0 );
            Text txtKeyStoreFile = ( Text ) elmDeviceConfig.
                getElementsByTagName( configManager.TAG_NAME_KEYSTOREFILE ).
                item( 0 ).getFirstChild();
            if ( txtKeyStoreFile != null ) {
                sKeyStoreFile = txtKeyStoreFile.getNodeValue();
            }
                        Element elmKeyStoreFilePassWord = ( Element ) elmDeviceConfig.
                getElementsByTagName( configManager.TAG_NAME_KEYSTOREPASSWORD ).
                item( 0 );
            Text txtKeyStoreFilePassWord = ( Text ) elmDeviceConfig.
                getElementsByTagName( configManager.TAG_NAME_KEYSTOREPASSWORD ).
                item( 0 ).getFirstChild();
            if ( txtKeyStoreFilePassWord != null ) {
                sKeyStoreFilePassWord = txtKeyStoreFilePassWord.getNodeValue();
            }
                        KeyDevice keyDevice = this.createKeyDevice( sType, sClassName,
                sDeviceName, sKeyStoreFile, sKeyStoreFilePassWord );
                                    Element elmSupportedCredTypes = ( Element ) device.
                getElementsByTagName( configManager.TAG_NAME_SUPPORTEDCREDTYPES ).
                item( 0 );
            NodeList elmCredTypes = elmSupportedCredTypes.getElementsByTagName(
                configManager.TAG_NAME_CREDTYPE );
            for ( int k = 0; k < elmCredTypes.getLength(); k++ ) {
                Element elmCredType = ( Element ) elmCredTypes.item( k );
                Text txtCredType = ( Text ) elmSupportedCredTypes.
                    getElementsByTagName( configManager.TAG_NAME_CREDTYPE ).
                    item( k ).getFirstChild();
                if ( txtCredType != null ) {
                    sCredType = txtCredType.getNodeValue();
                }
                this.addCredTypeAndDevice( sCredType, keyDevice );
            }
        }
    }
    
    private org.act.soap.server.secmanager.keymanager.KeyDevice createKeyDevice (
        String deviceType, String deviceClass, String deviceName,
        String keyStorePath, String keyStorePwd ) {
        KeyDevice credDevice = null;
        try {
            credDevice = new KeyStoreImpl();
            credDevice.setType( deviceType );
            credDevice.setDeviceName( deviceName );
            credDevice.setClassName( deviceClass );
            credDevice.setKeyStoreFile( keyStorePath );
            credDevice.setKeyStorePassWord( keyStorePwd );
        }
        catch ( Exception ex ) {
            ex.printStackTrace();
        }
        return credDevice;
    }
    
    private void addCredTypeAndDevice ( String sCredType,
                                        org.act.soap.server.secmanager.
                                        keymanager.KeyDevice device ) {
        if ( keyDevices == null ) {
            keyDevices = new Vector();
        }
        if ( !keyDevices.contains( device ) ) {
            keyDevices.addElement( device );
        }
        Vector devices = ( Vector )this.hDeviceWithCapability.get( sCredType );
                if ( devices == null ) {             devices = new Vector();
            devices.add( device );
            hDeviceWithCapability.put( sCredType, devices );
        }
        else {             if ( !devices.contains( device ) ) {
                devices.add( device );
            }
        }
    }
    
    public boolean validateCert ( X509Certificate cert ) {
        boolean validate = false;
                Vector devices = ( Vector )this.hDeviceWithCapability.get(
            CREDTYPE_VERIFYCERTVALIDATE );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                validate = device.validateCert( cert );
                if ( validate ) {
                    break;
                }
            }
        }
        return validate;
    }
    
    public boolean validateCert ( X509Certificate cert, String keydevice ) {
        boolean validate = false;
                Vector devices = ( Vector )this.hDeviceWithCapability.get(
            this.CREDTYPE_VERIFYCERTVALIDATE );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                if ( device.getDeviceName().equalsIgnoreCase( keydevice ) ) {
                    validate = device.validateCert( cert );
                    break;
                }
            }
        }
        return validate;
    }
    
    public static Key getKey ( byte[] key, String AlgName ) {
        SecretKeySpec mykey = new SecretKeySpec( key, AlgName );
        return mykey;
    }
    
    public boolean importKeyPair ( String alias, KeyPair kp,
                                   X509Certificate cert,
                                   String passwd, String keydevice ) {
        boolean issuccess = false;
                Vector devices = ( Vector )this.hDeviceWithCapability.get(
            this.CREDTYPE_IMPORTKEY );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                if ( device.getDeviceName().equalsIgnoreCase( keydevice ) ) {
                    issuccess = device.importKeyPair( alias, kp, cert, passwd );
                    break;
                }
            }
        }
        return issuccess;
    }
    
    public boolean importCertificate ( String alias, X509Certificate cert,
                                       String keydevice ) {
        boolean issuccess = false;
                Vector devices = ( Vector )this.hDeviceWithCapability.get(
            this.CREDTYPE_IMPORTKEY );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                if ( device.getDeviceName().equalsIgnoreCase( keydevice ) ) {
                    issuccess = device.importCertificate( alias, cert );
                    break;
                }
            }
        }
        return issuccess;
    }
    
    public Vector EnumerationKey () {
        Vector keyInfo = new Vector();
                Vector devices = ( Vector )this.hDeviceWithCapability.get(
            this.CREDTYPE_VERIFYCERTVALIDATE );
        if ( devices != null ) {                         for ( int i = 0; i < devices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice device = (
                    org.act.soap.server.secmanager.keymanager.KeyDevice )
                    devices.elementAt( i );
                Vector item = device.EnumerationKey();
                if ( item != null ) {
                    for ( int j = 0; j < item.size(); j++ ) {
                        keyInfo.addElement( item.elementAt( j ) );
                    }
                }
            }
        }
        return keyInfo;
    }
    
    public Vector EnumerationKeyDevice () {
        Vector data = null;
        if ( keyDevices != null ) {
            data = new Vector();
            for ( int i = 0; i < keyDevices.size(); i++ ) {
                org.act.soap.server.secmanager.keymanager.KeyDevice kd = ( org.
                    act.soap.server.secmanager.keymanager.KeyDevice )
                    keyDevices.elementAt( i );
                Vector item = new Vector();
                item.addElement( kd.getDeviceName() );
                if ( kd.getClass().getName().equalsIgnoreCase(
                    "org.act.soap.server.secmanager.keymanager.XKMSImpl" ) ) {
                                    }
                else {
                    if ( kd.getClass().getName().equalsIgnoreCase(
                        "org.act.soap.server.secmanager.keymanager.KeyStoreImpl" ) ) {
                        item.addElement( "KeyStore" );
                    }
                }
                if ( item.size() == 2 ) {
                    data.addElement( item );
                }
            }
        }
        return data;
    }
    
    public boolean importCertFromTS ( String keyname, String xkms,
                                      String keystore ) {
        boolean issuccess = false;
        boolean isxkms = false;
        boolean iskeystore = false;
                for ( int i = 0; i < this.keyDevices.size(); i++ ) {
            org.act.soap.server.secmanager.keymanager.KeyDevice kd = ( org.act.
                soap.server.secmanager.keymanager.KeyDevice )this.keyDevices.
                elementAt( i );
            if ( kd.getDeviceName().equalsIgnoreCase( xkms ) &&
                 kd.getClass().getName().equalsIgnoreCase(
                "org.act.soap.server.secmanager.keymanager.XKMSImpl" ) ) {
                isxkms = true;
                break;
            }
        }
                for ( int i = 0; i < this.keyDevices.size(); i++ ) {
            org.act.soap.server.secmanager.keymanager.KeyDevice kd = ( org.act.
                soap.server.secmanager.keymanager.KeyDevice )this.keyDevices.
                elementAt( i );
            if ( kd.getDeviceName().equalsIgnoreCase( keystore ) &&
                 kd.getClass().getName().equalsIgnoreCase(
                "org.act.soap.server.secmanager.keymanager.KeyStoreImpl" ) ) {
                iskeystore = true;
                break;
            }
        }
        if ( isxkms && iskeystore ) {
            X509Certificate cert = this.getX509Cert( keyname, xkms );
            if ( cert == null ) {
                return false;
            }
            issuccess = this.importCertificate( keyname, cert, keystore );
        }
        return issuccess;
    }
    
    public boolean removeKey ( String keyname, String keydevice ) {
        boolean issuccess = false;
                for ( int i = 0; i < this.keyDevices.size(); i++ ) {
            org.act.soap.server.secmanager.keymanager.KeyDevice kd = ( org.act.
                soap.server.secmanager.keymanager.KeyDevice )this.keyDevices.
                elementAt( i );
            if ( kd.getDeviceName().equalsIgnoreCase( keydevice ) ) {
                issuccess = kd.removeKey( keyname );
                break;
            }
        }
        return issuccess;
    }
}
