package org.act.soap.server.secmanager.keymanager;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.*;
import org.act.soap.server.config.*;
import org.act.soap.server.secmanager.*;
import org.act.soap.util.*;
import org.act.soap.util.i18n.*;
import com.sun.net.ssl.*;

public class KeyStoreImpl
    implements KeyDevice {
            
    private String sKeyStorePath = null;
    private String sKeyStorePwd = null;
    private String type = "";
    private String className = "";
    private String deviceName = "KeyStore";
    private String credType = "";
    private Vector credTypes = new Vector();
    ConfigManager configManager = ConfigManager.getInstance();
    public KeyStoreImpl () {
    }
    
    
     
      
       public String getType () {
           return this.type;
       }
    
    public void setType ( String type ) throws ConfigException {
        if ( ( null == type ) || "".equals( type ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                ConfigManager.ATTR_NAME_TYPE ) );
        }
        this.type = type;
    }
    public String getClassName () {
        return this.className;
    }
    public void setClassName ( String className ) throws ConfigException {
        if ( ( null == className ) || "".equals( className ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                ConfigManager.ATTR_NAME_CLASSNAME ) );
        }
        this.className = className;
    }
    public String getDeviceName () {
        return this.deviceName;
    }
    public void setDeviceName ( String deviceName ) {
        this.deviceName = deviceName;
    }
    public String getKeyStoreFile () {
        return this.sKeyStorePath;
    }
    public void setKeyStoreFile ( String keyStoreFile ) throws ConfigException {
        if ( ( null == keyStoreFile ) || "".equals( keyStoreFile ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                ConfigManager.TAG_NAME_KEYSTOREFILE ) );
        }
        this.sKeyStorePath = keyStoreFile;
    }
    public String getKeyStorePassWord () {
        return this.sKeyStorePwd;
    }
    public void setKeyStorePassWord ( String keyStorePassWord ) throws
        ConfigException {
        if ( ( null == keyStorePassWord ) || "".equals( keyStorePassWord ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                ConfigManager.TAG_NAME_KEYSTOREPASSWORD ) );
        }
        this.sKeyStorePwd = keyStorePassWord;
    }
    public String getCredType () {
        return this.credType;
    }
    public void setCredType ( String credType ) throws ConfigException {
        if ( ( null == credType ) || "".equals( credType ) ) {
            throw new ConfigException( Message.getMessage(
                "ERR_CONF_badConfigFormat",
                ConfigManager.TAG_NAME_CREDTYPE ) );
        }
        this.credType = credType;
    }
    public Vector getCredTypes () {
        return this.credTypes;
    }
    public void setCredTypes ( Vector credTypes ) {
        this.credTypes = credTypes;
    }
    
    public Object getCredential ( String credType, String[] keyTags ) {
        if ( keyTags == null ) {
            return null;
        }
        if ( credType.equals( CredManager.CREDTYPE_PRIVATEKEYNAME ) ) {
            if ( keyTags.length > 1 ) {
                return getPrivateKeyWithKeyName( keyTags[0], keyTags[1] );
            }
            else {
                return null;
            }
        }
        else if ( credType.equals( CredManager.CREDTYPE_PUBLICKEYNAME ) ) {
            return getPublicKeyWithKeyName( keyTags[0] );
        }
        else if ( credType.equals( CredManager.CREDTYPE_CERTIFICATENAME ) ) {
            return getCertificateWithName( keyTags[0] );
        }
        else if ( credType.equals( CredManager.CREDTYPE_PRIVATEKEYDN ) ) {
            return getPrivateKeyWithDN( keyTags[0], keyTags[1] );
        }
        else if ( credType.equals( CredManager.CREDTYPE_X509_DN ) ) {
            return getCertificateWithDN( keyTags[0] );
        }
        return null;
    }
    
    private Key getPrivateKeyWithKeyName ( String keyName, String psword ) {
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return null;
        }
        char[] keyStorePwd = this.sKeyStorePwd.toCharArray();
        char[] keyPwd = psword.toCharArray();
        String alias = keyName;
        KeyStore keyStore = null;
        Key key = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            String strKeyStorePath = null;
            String rootPath = System.getProperty( Constants.
                                                  PROPERTY_NAME_ROOT_PATH );
            if ( null == rootPath ) {
                                rootPath = System.getProperty( "clientConfigPath" );
            }
            strKeyStorePath = rootPath + java.io.File.separator + keyStorePath;
            keyStore.load( new FileInputStream( strKeyStorePath ), keyStorePwd );
            key = keyStore.getKey( alias, keyPwd );
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }
        return key;
    }
    
    private Key getPublicKeyWithKeyName ( String keyName ) {
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return null;
        }
        String alias = keyName;
        char[] keyStorePass = new String( this.sKeyStorePwd ).toCharArray();
        KeyStore keyStore = null;
        Key key = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            String strKeyStorePath = null;
            String rootPath = System.getProperty( Constants.
                                                  PROPERTY_NAME_ROOT_PATH );
            if ( null == rootPath ) {
                                rootPath = ".";
            }
            strKeyStorePath = rootPath + java.io.File.separator + keyStorePath;
            keyStore.load( new FileInputStream( strKeyStorePath ), keyStorePass );
            key = keyStore.getCertificate( alias ).getPublicKey();
        }
        catch ( Exception e ) {
            return null;
        }
        return key;
    }
    
    private X509Certificate getCertificateWithName ( String certName ) {
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return null;
        }
        char[] keyStorePwd = this.sKeyStorePwd.toCharArray();
        String alias = certName;
        KeyStore keyStore = null;
        X509Certificate cert = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            String strKeyStorePath = null;
            String rootPath = System.getProperty( Constants.
                                                  PROPERTY_NAME_ROOT_PATH );
            if ( null == rootPath ) {
                                rootPath = ".";
            }
            strKeyStorePath = rootPath + java.io.File.separator + keyStorePath;
            keyStore.load( new FileInputStream( strKeyStorePath ), keyStorePwd );
            cert = ( X509Certificate ) keyStore.getCertificate( alias );
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }
        return cert;
    }
    
    private X509Certificate getCertificateWithDN ( String DN ) {
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return null;
        }
        char[] keyStorePwd = this.sKeyStorePwd.toCharArray();
        KeyStore keyStore = null;
        X509Certificate cert = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            keyStore.load( new FileInputStream( keyStorePath ), keyStorePwd );
            Enumeration aliases = keyStore.aliases();
            while ( aliases.hasMoreElements() ) {
                cert = ( X509Certificate ) ( keyStore.getCertificate( ( String ) (
                    aliases.nextElement() ) ) );
                if ( cert.getSubjectDN().toString().equalsIgnoreCase( DN ) ) {
                    return cert;
                }
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }
        return null;
    }
    
    private Key getPrivateKeyWithDN ( String DN, String password ) {
        Key key = null;
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return null;
        }
        char[] keyStorePwd = this.sKeyStorePwd.toCharArray();
        KeyStore keyStore = null;
        X509Certificate cert = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            keyStore.load( new FileInputStream( keyStorePath ), keyStorePwd );
            Enumeration aliases = keyStore.aliases();
            while ( aliases.hasMoreElements() ) {
                String alias = ( String ) ( aliases.nextElement() );
                cert = ( X509Certificate ) ( keyStore.getCertificate( alias ) );
                if ( cert.getSubjectDN().toString().equalsIgnoreCase( DN ) ) {
                    key = keyStore.getKey( alias, password.toCharArray() );
                    if ( key != null ) {
                        return key;
                    }
                }
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }
        return null;
    }
    
    public boolean validateCert ( X509Certificate cert ) {
        boolean validate = false;
        try {
                        cert.checkValidity();
                        String KeyStoreName = this.sKeyStorePath;
            String KeyStorePassword = this.sKeyStorePwd;
            KeyStore store = KeyStore.getInstance( "JKS", "SUN" );
            String strKeyStorePath = System.getProperty( Constants.
                PROPERTY_NAME_ROOT_PATH ) + java.io.File.separator
                + KeyStoreName;
            InputStream in = new FileInputStream( strKeyStorePath );
            char[] password = KeyStorePassword.toCharArray();
            store.load( in, password );
            in.close();
            String trustManagerAlgorithm = "SunX509";
            TrustManagerFactory tmf = TrustManagerFactory.getInstance(
                trustManagerAlgorithm );
            tmf.init( store );
            X509TrustManager x509TM = ( X509TrustManager ) tmf.getTrustManagers()[
                0];
            X509Certificate[] certChain = new X509Certificate[] {
                cert
            };
            validate = x509TM.isClientTrusted( certChain );
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return false;
        }
        return validate;
    }
    
    public boolean importCertificate ( String alias, X509Certificate cert ) {
        boolean issuccess = false;
        try {
            String KeyStoreName = this.sKeyStorePath;
            String KeyStorePassword = this.sKeyStorePwd;
            KeyStore store = KeyStore.getInstance( "JKS", "SUN" );
            InputStream in = new FileInputStream( KeyStoreName );
            char[] password = KeyStorePassword.toCharArray();
            store.load( in, password );
            in.close();
            store.deleteEntry( alias );
            store.setCertificateEntry( alias, cert );
            FileOutputStream fos = new FileOutputStream( KeyStoreName );
            store.store( fos, password );
            fos.close();
            issuccess = true;
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return false;
        }
        return issuccess;
    }
    
    public boolean importKeyPair ( String alias, KeyPair kp,
                                   X509Certificate cert, String passwd ) {
        boolean issuccess = false;
        try {
            String KeyStoreName = this.sKeyStorePath;
            String KeyStorePassword = this.sKeyStorePwd;
            KeyStore store = KeyStore.getInstance( "JKS", "SUN" );
            InputStream in = new FileInputStream( KeyStoreName );
            char[] password = KeyStorePassword.toCharArray();
            store.load( in, password );
            in.close();
            X509Certificate[] certchain = {
                cert};
            store.deleteEntry( alias );
            store.setKeyEntry( alias, kp.getPrivate(), passwd.toCharArray(),
                               certchain );
            FileOutputStream fos = new FileOutputStream( KeyStoreName );
            store.store( fos, password );
            fos.close();
            issuccess = true;
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return false;
        }
        return issuccess;
    }
    
    public Vector EnumerationKey () {
        Vector vecKey = new Vector();
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return null;
        }
        char[] keyStorePwd = this.sKeyStorePwd.toCharArray();
        KeyStore keyStore = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            keyStore.load( new FileInputStream( keyStorePath ), keyStorePwd );
            Enumeration aliases = keyStore.aliases();
            while ( aliases.hasMoreElements() ) {
                Vector item = new Vector();
                item.addElement( this.deviceName );
                String alias = ( String ) aliases.nextElement();
                item.addElement( alias );
                if ( keyStore.isKeyEntry( alias ) ) {
                    String strAlg = keyStore.getCertificate( alias ).
                        getPublicKey().getAlgorithm();
                    item.addElement( strAlg );
                    item.addElement( "KeyPair" );
                }
                else {
                    String strAlg = keyStore.getCertificate( alias ).
                        getPublicKey().getAlgorithm();
                    item.addElement( strAlg );
                    item.addElement( "Certificate" );
                }
                if ( item.size() == 4 ) {
                    vecKey.addElement( item );
                }
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }
        return vecKey;
    }
    
    public boolean removeKey ( String keyname ) {
        boolean issuccess = false;
        String keyStorePath = this.sKeyStorePath;
        if ( this.sKeyStorePwd == null || keyStorePath == null ) {
            return false;
        }
        char[] keyStorePwd = this.sKeyStorePwd.toCharArray();
        KeyStore keyStore = null;
        try {
            keyStore = KeyStore.getInstance( "JKS" );
            keyStore.load( new FileInputStream( keyStorePath ), keyStorePwd );
            keyStore.deleteEntry( keyname );
            FileOutputStream fos = new FileOutputStream( keyStorePath );
            keyStore.store( fos, keyStorePwd );
            fos.close();
            issuccess = true;
        }
        catch ( Exception e ) {
            e.printStackTrace();
            return false;
        }
        return issuccess;
    }
}
