package org.act.soap.server.transport;
import java.util.HashMap;
import org.act.soap.server.transport.http.HTTPTransport;
import org.act.soap.util.Constants;
import org.act.soap.util.i18n.Message;

public class TransportFactory {
  
  private HashMap transportRegistryMap;
  
  private TransportFactory() {
    transportRegistryMap = new HashMap();
    transportRegistryMap.put(Constants.HTTP_PROTOCOL_NAME,
                             HTTPTransport.class.getName());
  }
  
  public static TransportFactory newInstance() {
    return new TransportFactory();
  }
  
  public Transport createTransport(String type) throws TransportException {
    if (null == type) {
      throw new TransportException(Message.getMessage(
          "ERR_TP_NullArgument"));
    }
    String transportName = (String) transportRegistryMap.get(type);
    if (null == transportName) {
      throw new TransportException(
          Message.getMessage("unknownTransportType", type));
    }
    Transport transport = null;
    try {
      ClassLoader classLoader =
          Thread.currentThread().getContextClassLoader();
      Class transportImpl = classLoader.loadClass(transportName);
      transport = (Transport) transportImpl.newInstance();
    }
    catch (Exception ex) {
      throw new TransportException(
          Message.getMessage("newInstanceException", ex.getMessage()), ex);
    }
    return transport;
  }
  
  public void registerTransport(String type, String className) throws
      TransportException {
    if (null == type || null == className) {
      throw new TransportException(Message.getMessage(
          "ERR_TP_NullArgument"));
    }
    try {
      ClassLoader classLoader =
          Thread.currentThread().getContextClassLoader();
      classLoader.loadClass(className);
    }
    catch (ClassNotFoundException ex) {
      throw new TransportException(
          Message.getMessage("classNotFound", className), ex);
    }
    transportRegistryMap.put(type, className);
  }
  
  public HashMap getTransportRegistryMap() {
    return this.transportRegistryMap;
  }
  
  public void removeTransport(String transportType) throws
      TransportException {
    String transportName = (String) transportRegistryMap.get(transportType);
    if (null == transportName) {
      throw new TransportException(
          Message.getMessage("unknownTransportType", transportType));
    }
    transportRegistryMap.remove(transportType);
  }
}
