package org.act.soap.server.transport.http;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Iterator;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeader;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPMessage;
import org.act.soap.server.MessageContext;
import org.act.soap.server.transport.Transport;
import org.act.soap.server.transport.TransportContext;
import org.act.soap.server.transport.TransportException;
import org.act.soap.util.Constants;
import org.act.soap.util.SOAPUtils;

public class HTTPTransport
    implements Transport {
  
  public HTTPTransport() {
  }
  
  public static final int ERR_STATE_SENDER = 400;
  
  public static final int ERR_STATE_OTHER = 500;
  
  public void process(TransportContext transportContext,
                      MessageContext messageContext) throws
      TransportException {
    HTTPTransportContext httpTC = (HTTPTransportContext) transportContext;
    HttpServletRequest req = httpTC.getHttpServletRequest();
        messageContext.setTransportProtocol(httpTC.getProtocolName());
        MimeHeaders mimeHeaders = new MimeHeaders();
    Enumeration enumHeaderName = req.getHeaderNames();
    if (null != enumHeaderName) {
            while (enumHeaderName.hasMoreElements()) {
        String strHeaderName = (String) enumHeaderName.nextElement();
        mimeHeaders.addHeader(strHeaderName,
                              req.getHeader(strHeaderName));
      }
    }
    messageContext.setMimeHeaders(mimeHeaders);
        InputStream in = null;
        try {
      in = req.getInputStream();
      
      MessageFactory mf = SOAPUtils.MESSAGEFACTORY_INSTANCE;
      SOAPMessage soapMsg = mf.createMessage(mimeHeaders, in);
      messageContext.setRequestMessage(soapMsg);
      SOAPBody soapBody = soapMsg.getSOAPBody();
      Iterator iterator = soapMsg.getAttachments();
      
    }
    catch (Exception ex) {
      throw new TransportException(ex);
    }
  }
  
  public void postProcess(TransportContext transportContext,
                          MessageContext messageContext) throws
      TransportException {
    HTTPTransportContext httpTC = (HTTPTransportContext) transportContext;
    HttpServletResponse res = httpTC.getHttpServletResponse();
    OutputStream outputStream = null;
    try {
            SOAPMessage soapMessage = messageContext.getResponseMessage();
            if (soapMessage.getSOAPBody().hasFault()) {
        int err_state = -1;
        SOAPFault soapFault = soapMessage.getSOAPBody().getFault();
        Name faultCodeName = soapFault.getFaultCodeAsName();
                if (faultCodeName.getLocalName().equals(Constants.
                                                FAULT_CODE_SENDER)) {
          err_state = ERR_STATE_SENDER;
        }
        else {
          err_state = ERR_STATE_OTHER;
        }
        res.setStatus(err_state);
      }
            String strContentType="multipart/related;type=text/xml";
      if(soapMessage.getAttachments().hasNext()){
                res.setContentType(strContentType);
      }
      else{
        res.setContentType("text/xml;charset=UTF-8");
      }
      
     
                  MimeHeaders mimeHeaders = soapMessage.getMimeHeaders();
      Iterator itMimeHeaders = mimeHeaders.getAllHeaders();
      while (itMimeHeaders.hasNext()) {
        MimeHeader mimeHeader = (MimeHeader) itMimeHeaders.next();
        res.addHeader(mimeHeader.getName(), mimeHeader.getValue());
      }
            outputStream = res.getOutputStream();
      soapMessage.writeTo(outputStream);
      outputStream.close();
    }
    catch (javax.xml.soap.SOAPException ex) {
      throw new TransportException(ex);
    }
    catch (java.io.IOException ex) {
      throw new TransportException(ex);
    }
    catch (Exception ex) {
      throw new TransportException(ex);
    }
  }
}
