package org.act.soap.server.transport. http;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import java.net.URL;
import java.util.HashMap;
import java.util.Properties;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogFactory;
import java.util.Enumeration;
import org.act.soap.server.SOAPEngine;
import javax.xml.soap.MimeHeaders;
import org.act.soap.util.Constants;
import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLWriter;
import java.util.Iterator;
import javax.xml.soap.MimeHeader;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;


public class StandAloneWebServer implements Runnable {
            
    private static Log log = LogFactory.getLog(StandAloneWebServer.class.getName());
    
    private int port = 8084;
    
    private InetAddress bindAddress;
    
    private int backlog = 50;
    
        
    private ServerSocket server = null;
    
       
        
    private ThreadPool threadPool = new ThreadPool();
        public StandAloneWebServer() throws Exception
    {
                
        this.setBindAddress( "localhost" );
                this.start();
        log.info("Started webserver with address: " + this.getBindAddress() + " port: "+this.getPort());
    }
    
    public void setPort(int p)
    {
        port = p;
    }
    
    public int getPort()
    {
        return port;
    }
    public String getBindAddress()
    {
        String address = null;
        if( bindAddress != null )
            address = bindAddress.getHostAddress();
        return address;
    }
    public void setBindAddress(String host)
    {
        try
        {
            if (host != null)
                bindAddress = InetAddress.getByName(host);
        }
        catch(UnknownHostException e)
        {
            String msg = "Invalid host address specified: " + host;
            log.error(msg, e);
        }
    }
    
    public int getBacklog()
    {
        return backlog;
    }
    
    public void setBacklog(int backlog)
    {
        if( backlog <= 0 )
            backlog = 50;
        this.backlog = backlog;
    }

    public void start() throws IOException
    {
        try
        {
            server = new ServerSocket(port, backlog, bindAddress);
            log.debug("Started server: "+server);
            listen();
        }
        catch (IOException e)
        {
            throw e;
        }
    }
    
    public void stop()
    {
        try
        {
            ServerSocket srv = server;
            server = null;
            srv.close();
        }
        catch (Exception e)
        {
        }
    }
    
    public void run()
    {
                if (server == null)
            return;
                Socket socket = null;
        try
        {
            socket = server.accept();
        }
        catch (IOException e)
        {
                        if( server != null )
                log.error("Failed to accept connection", e);
            return;
        }
                listen();
        try
        {
                        OutputStream out = socket.getOutputStream();
            try
            {
                                InputStream in = socket.getInputStream();
                                String line = readLine(in);
                log.trace("raw request="+line);
                                int start = line.indexOf(' ');
                int end = line.indexOf(' ', start+1);
                String method = line.substring(0, start);
                log.trace( "REQUEST_TYPE = " + method );
                                String rawPath = line.substring(start+2, end);
                if( method.equals("GET") )
                {
                    int qmIndex = rawPath.indexOf('?');
                    String queryString = rawPath.substring( qmIndex + 1 );
                    if( queryString.equalsIgnoreCase(Constants.QUERYSTRING_WSDL))
                    {
                                                int separator = rawPath.indexOf('/');
                        String filePath = rawPath.substring(separator, qmIndex);
                        log.trace( "filePath = " + filePath);
                        synchronized (this) {
                                                        SOAPEngine soapEngine = SOAPEngine.getInstance();
                            Definition definition = soapEngine.getWSDLDefination(filePath);
                            WSDLWriter wsdlWriter = WSDLFactory.newInstance().newWSDLWriter();
                            ByteArrayOutputStream bos = new ByteArrayOutputStream();
                            wsdlWriter.writeWSDL(definition, bos);
                            byte[] bytes = bos.toByteArray();
                            out.write("HTTP/1.0 200 OK\r\n".getBytes());
                            out.write(("Content-Length: " + (bytes.length+200) + "\r\n").getBytes());
                            out.write("Content-Type: text/xml;charset=UTF-8".getBytes());
                            out.write("\r\n\r\n".getBytes());
                                                        int offset = 0;
                            int bufSize = 32;
                            while( offset < bytes.length )
                            {
                                int len = bytes.length - offset;
                                len = len > bufSize ? bufSize : len;
                                out.write(bytes, offset, len);
                                offset += len;
                            }
                            out.flush();
                        }
                    }
                }
                else if( method.equals("POST") )
                {
                                        String httpVersion = line.substring(end+1);
                    log.trace( "HTTP_VERSION = " + httpVersion );
                    int separator = rawPath.indexOf('/');
                    String filePath = rawPath.substring(separator);
                    HttpServletRequestImpl req = new HttpServletRequestImpl();
                    MimeHeaders reqHeaders = this.getMimeHeaders(in);
                    req.setMimeHeaders(reqHeaders);
                    String[] cl = reqHeaders.getHeader("Content-Length");
                    int contentLength = 0;
                    if ( cl != null && cl.length > 0 )
                    {
                        contentLength = Integer.parseInt( cl[0] );
                    }
                    ByteArrayOutputStream bos = new ByteArrayOutputStream();
                    int c = in.read();
                    while( c != -1 && bos.size() < contentLength - 1 )
                    {
                        bos.write( c );
                        c = in.read();
                    }
                    req.setInputStream(new ByteArrayInputStream( bos.toByteArray() ));
                    HttpServletResponseImpl res = new HttpServletResponseImpl();
                    HTTPTransportContext reqCtx = new HTTPTransportContext();
                    reqCtx.setHttpServletRequest(req);
                    reqCtx.setHttpServletResponse(res);
                    reqCtx.setRequestURL(filePath);
                                        SOAPEngine soapEngine = SOAPEngine.getInstance();
                    soapEngine.invoke(reqCtx);
                    byte[] resContent = res.getMessageContent();
                    MimeHeaders resHeaders = res.getMimeHeaders();
                    int resStatus = res.getStatus();
                    String httpCode = null;
                    if( resStatus == 500 )
                    {
                       httpCode = "500 Internal Server Error";
                    }
                                                                                                    else
                    {
                        httpCode = "200 OK";
                    }
                    log.trace("HTTP Response = " + httpCode);
                    try
                    {
                        out.write(("HTTP/1.0 "+httpCode+"\r\n").getBytes());
                        out.write(("Content-Length: " + (resContent.length) + "\r\n").getBytes());
                        for( Iterator i = resHeaders.getAllHeaders(); i.hasNext(); )
                        {
                            MimeHeader header = (MimeHeader) i.next();
                            out.write(header.getName().getBytes());
                            out.write(':');
                            out.write(' ');
                            out.write(header.getValue().getBytes());
                            out.write("\r\n".getBytes());
                        }
                        out.write("\r\n".getBytes());
                                                out.write(resContent);
                        out.flush();
                    }
                    catch (IOException ie)
                    {
                        return;
                    }
                }
            }
            catch(Throwable e)
            {
                try
                {
                    log.trace("HTTP code=404 " + e.getMessage());
                                        out.write(("HTTP/1.0 404 " + e.getMessage() + "\r\n").getBytes());
                    out.write("Content-Type: text/html\r\n\r\n".getBytes());
                    out.flush();
                } catch (IOException ex)
                {
                                    }
            }
        }
        catch (IOException ex)
        {
                                    log.debug("error writing response: " + ex.getMessage());
            ex.printStackTrace();
        }
        finally
        {
                        try
            {
                socket.close();
            }
            catch (IOException e)
            {
            }
        }
    }
        
    protected void listen()
    {
        threadPool.run(this);
    }
    
    protected MimeHeaders getMimeHeaders(InputStream in) throws IOException
    {
        
        MimeHeaders mimeHeaders = new MimeHeaders();
        String line = readLine(in);
        while( line != null && line.trim().length() > 0 )
        {
            log.trace("header line : "+line);
                        int start = line.indexOf(':');
            if( start > 0 ) {
                String headerName = line.substring(0, start).trim();
                String headerValue = line.substring(start + 1).trim();
                while (headerValue.endsWith(";")) {
                                        headerValue += readLine(in).trim();
                }
                log.trace("headerName = [" + headerName + "]   headerValue = [" + headerValue + "]");
                mimeHeaders.addHeader(headerName, headerValue);
            }
            line = readLine(in);
        }
        return mimeHeaders;
    }
    public static String readLine(InputStream rawIn) throws IOException
    {
        StringBuffer sb = new StringBuffer();
        int c = rawIn.read();
        while( c != -1 )
        {
            if( c == '\n' )
            {
                                break;
            }
            sb.append((char)c);
            c = rawIn.read();
        }
        return sb.toString();
    }
    public static void main(String[] args) throws Exception
    {
        new StandAloneWebServer();
    }
}
