package org.act.soap.soapsec;
import java.security.*;
import java.util.*;
import javax.crypto.*;
import javax.xml.namespace.*;
import org.act.soap.server.secmanager.*;
import org.act.soap.soapsec.securitytoken.*;
import org.act.soap.util.*;
import org.act.soap.util.XMLUtils;
import org.act.xdsec.core.enc.*;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.syntax.enc.*;
import org.act.xdsec.syntax.keyinfo.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class Encryption
    extends SOAPSecurityElement {
    
    private List vEncParts = new Vector();
    
    private boolean bEncElemContent = true;
    
    private boolean bEncKey = true;
    
    
    private String sEncKeyAlg = Constant.ALG_RSA15_URI;
    
    private Key kEncKeyKey = null;
    
    private Key kDecKeyKey = null;
    
    private String sEncDataAlg = Constant.ALG_DES_URI;
    
    private Key kEncDataKey = null;
    
    private Key kDecDataKey = null;
    
    private String sKeyName = null;
    
    private Element mSecurity = null;
        
    public Encryption () {
        this.namespaceURI = Constants.NS_URI_ENCRYPTION;
        this.prefix = Constants.NS_PRE_ENCRYPTION;
        this.localName = Constants.ELEM_ENCRYPTEDKEY;
        this.vEncParts.add( Constants.PARTINFO_BODY );     }
    
    public Encryption ( SecurityToken token ) {
        this();
    }
    
    public Encryption ( Element eEncryption ) throws SOAPException {
        super( eEncryption );
                if ( Constants.ELEM_REFERENCELIST.equals( this.localName ) ) {
                        this.bEncKey = false;
        }
        else if ( Constants.ELEM_ENCRYPTEDKEY.equals( this.localName ) ) {
                        this.bEncKey = true;
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "ǺϷļԪ" );
        }
    }
                
    public void addEncPart ( QName partInfo ) {
        
        if ( !vEncParts.contains( partInfo ) ) {
            this.vEncParts.add( partInfo );
        }
    }
    
    public QName[] getEncParts () {
        QName[] pis = new QName[this.vEncParts.size()];
        this.vEncParts.toArray( pis );
        return pis;
    }
    
    public boolean isEncryptingPart ( QName part ) {
        for ( int i = 0; i < this.vEncParts.size(); i++ ) {
            if ( this.vEncParts.get( i ).equals( part ) ) {
                return true;
            }
        }
        return false;
    }
    
    public QName removeEncPart ( QName partInfo ) {
        return this.vEncParts.remove( partInfo ) ? partInfo : null;
    }
    
    public boolean isEncElemContent () {
        return bEncElemContent;
    }
    
    public void setEncElemContent ( boolean mEncElemContent ) {
        this.bEncElemContent = mEncElemContent;
    }
    
    public boolean isEncKey () {
        return bEncKey;
    }
    
    public void setEncKey ( boolean mEncKey ) {
        this.bEncKey = mEncKey;
    }
    
    public String getEncKeyAlg () {
        return sEncKeyAlg;
    }
    
    public void setEncKeyAlg ( String mEncKeyAlg ) {
        this.sEncKeyAlg = mEncKeyAlg;
    }
    
    public Key getEncKeyKey () {
        return this.kEncKeyKey;
    }
    
    public void setEncKeyKey ( Key key ) {
        this.kEncKeyKey = key;
    }
    
    public Key getDecKeyKey () {
        return this.kDecKeyKey;
    }
    
    public void setDecKeyKey ( Key key ) {
        this.kDecKeyKey = key;
    }
    
    public String getEncDataAlg () {
        return this.sEncDataAlg;
    }
    
    public void setEncDataAlg ( String alg ) {
        this.sEncDataAlg = alg;
    }
    
    public Key getEncDataKey () {
        return this.kEncDataKey;
    }
    
    public void setEncDataKey ( Key key ) {
        this.kEncDataKey = key;
    }
    
    public Key getDecDataKey () {
        return this.kDecDataKey;
    }
    
    public void setDecDataKey ( Key key ) {
        this.kDecDataKey = key;
    }
    
    public String getKeyName () {
        return this.sKeyName;
    }
    
    public void setKeyName ( String keyName ) {
        this.sKeyName = keyName;
    }
        
    public Element toElement ( Document doc ) throws SOAPException {
                if ( null == this.kEncDataKey ) {
            this.kEncDataKey = Encryption.createEncDataKey( this.sEncDataAlg );
        }
                                if ( this.bEncKey ) {             this.localName = Constants.ELEM_ENCRYPTEDKEY;
                        XEncryptedKey xEncKey = new XEncryptedKey();
            xEncKey.setEncryptionMethod( this.sEncKeyAlg );
            if ( null != this.sKeyName ) {
                xEncKey.addKeyName( this.sKeyName );
            }
            xEncKey.setCipherData( new XCipherData() );
            xEncKey.setCipherValue( "" );
            xEncKey.setReferenceList( new XReferenceList() );
            this.element = xEncKey.toElement( doc );
                        try {
                EncryptedKey oEncKey = new EncryptedKey();
                oEncKey.setEncKey( this.element );
                oEncKey.encrypt( this.kEncDataKey, this.kEncKeyKey );
            }
            catch ( Exception ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ݼԿмʱ", ex );
            }
        }
        else {
                        this.localName = Constants.ELEM_REFERENCELIST;
            this.element = new XReferenceList().toElement( doc );
            this.element.setAttribute( Constants.NS_PRE_XMLNS + ':'
                                       + Constants.NS_PRE_ENCRYPTION,
                                       Constants.NS_URI_ENCRYPTION );
        }
        return this.element;
    }
    
    public Element toElement ( Document doc, Element eSecurity ) throws SOAPException {
        this.mSecurity = eSecurity;
        return this.toElement( doc );
    }
        
    public void encrypt () throws SOAPException {
                Element eReferenceList = null;
        if ( this.bEncKey ) {
            List ReferenceLists = XMLUtils.getChildElementsByNS( this.element,
                Constant.NS_XMLENC_NAME, Constant.ENC_REFERENCELIST_TAG );
            eReferenceList = ( Element ) ( ( Vector ) ReferenceLists ).
                firstElement();
        }
        else {
            eReferenceList = this.element;
        }
        try {
            for ( int i = 0; i < this.vEncParts.size(); i++ ) {
                QName partInfo = ( QName ) ( this.vEncParts.get( i ) );
                                Document doc = this.element.getOwnerDocument();
                NodeList nlParts = doc.getElementsByTagNameNS( partInfo.
                    getNamespaceURI(), partInfo.getLocalPart() );
                                                if ( nlParts == null || nlParts.getLength() <= 0 ) {
                    nlParts = this.mSecurity.getElementsByTagNameNS(
                        partInfo.getNamespaceURI(), partInfo.getLocalPart() );
                }
                if ( null == nlParts || 0 >= nlParts.getLength() ) {
                    throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                             "Ԫ" + partInfo.getNamespaceURI() +
                                             ':' + partInfo.getLocalPart() +
                                             "" );
                }
                Element ePart = ( Element ) nlParts.item( 0 );
                                XEncryptedData xEncData = new XEncryptedData();
                xEncData.setEncryptionMethod( this.sEncDataAlg );
                if ( !this.bEncKey && null != this.sKeyName ) {
                    XKeyInfo xKeyInfo = new XKeyInfo();
                    xKeyInfo.addKeyName( this.sKeyName );
                    xEncData.setKeyInfo( xKeyInfo );
                }
                xEncData.setCipherData( new XCipherData() );
                xEncData.setCipherValue( "" );
                Element eEncData = xEncData.toElement( doc );
                                String partId = UUID.generate().toString();
                eEncData.setAttribute( Constants.ATTR_WSU_ID, partId );
                                String sTagName = Constant.PREFIX_ENCRYPTION + ':' +
                    Constant.ENC_DATAREFERENCE_TAG;
                Element eDataRef = doc.createElementNS( Constant.NS_XMLENC_NAME,
                    sTagName );
                eDataRef.setAttribute( Constant.ATT_URI,
                                       Constants.S_Ref + partId );
                eReferenceList.appendChild( eDataRef );
                                EncryptedData oEncData = new EncryptedData();
                oEncData.setEncData( eEncData );
                oEncData.encrypt( ePart, this.kEncDataKey, this.bEncElemContent, true );             }
        }
        catch ( Exception ex ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ݴ", ex );
        }
    }
    
    public void decrypt ( Key decKey ) throws SOAPException {
        Element eReferenceList = null;         if ( this.bEncKey ) {
            List ReferenceLists = XMLUtils.getChildElementsByNS( this.element,
                Constant.NS_XMLENC_NAME, Constant.ENC_REFERENCELIST_TAG );
            eReferenceList = ( Element ) ( ( Vector ) ReferenceLists ).
                firstElement();
            if ( null != decKey ) {
                this.kDecKeyKey = decKey;
            }
                        try {
                EncryptedKey oEncKey = new EncryptedKey();
                oEncKey.setEncKey( this.element );
                final byte[] decdatakey = oEncKey.decrypt( this.kDecKeyKey, "" );                 
                this.kDecDataKey = CredManager.getKey( decdatakey,
                    Constants.ALG_DESEDE );
                this.sEncKeyAlg = oEncKey.getEncAlgName();
            }
            catch ( Exception ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ܼԿʱ", ex );
            }
        }
        else {
            eReferenceList = this.element;
            if ( null != decKey ) {
                this.kDecDataKey = decKey;
            }
        }
        if ( null != eReferenceList ) {
            Document doc = eReferenceList.getOwnerDocument();
            List eReferences = XMLUtils.getChildElementsByNS( eReferenceList,
                Constant.NS_XMLENC_NAME, Constant.ENC_DATAREFERENCE_TAG );
            String id = null;
            try {
                for ( int i = 0; i < eReferences.size(); i++ ) {
                    Element eReference = ( Element ) eReferences.get( i );
                    String uri = eReference.getAttribute( Constant.ATT_URI );
                    if ( uri.charAt( 0 ) == '#' ) {
                                                id = uri.substring( 1 );                         Element eEncData = XMLUtils.getElementById( doc, id );
                        if ( Constant.ENC_TYPE_CONTENT.equals( eEncData.
                            getAttribute(
                                Constant.ATT_TYPE ) ) ) {
                            this.bEncElemContent = true;
                            Element parent = ( Element ) eEncData.getParentNode();
                            this.vEncParts.add( new QName( parent.
                                getNamespaceURI(),
                                parent.getLocalName() ) );
                        }
                        else {
                            this.bEncElemContent = false;
                            
                            this.vEncParts.add( new QName( eEncData.
                                getNamespaceURI(),
                                eEncData.getLocalName() ) );
                        }
                                                EncryptedData oEncData = new EncryptedData();
                        oEncData.setEncData( eEncData );
                        oEncData.decrypt( this.kDecDataKey, "", true );
                        this.sEncDataAlg = oEncData.getEncAlgName();
                    }
                }             }
            catch ( Exception ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ʱId = " + id, ex );
            }
        }
    }
    
    private static Key createEncDataKey ( String keyAlgURI ) throws
        SOAPException {
        try {
            BlockEncryptionAlgorithm alg = new BlockEncryptionAlgImpl();
            alg.setURI( keyAlgURI );
            String sProvider = alg.getProvider();
            String sAlgName = alg.getAlgId();
            final int index = sAlgName.indexOf( "/" );
            if ( 0 < index ) {
                sAlgName = sAlgName.substring( 0, index );
            }
            KeyGenerator keyg = KeyGenerator.getInstance( sAlgName, sProvider );
            return keyg.generateKey();
        }
        catch ( Exception ex ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ݼԿ",
                                     ex );
        }
    }
    
    public static boolean isEncryption ( Element element ) {
        if ( null == element ) {
            return false;
        }
        else if ( Constants.NS_URI_ENCRYPTION.equals( element.getNamespaceURI() )
                  && Constants.ELEM_REFERENCELIST.equals( element.getLocalName() ) ) {
            return true;
        }
        else if ( Constants.NS_URI_ENCRYPTION.equals( element.getNamespaceURI() )
                  && Constants.ELEM_ENCRYPTEDKEY.equals( element.getLocalName() ) ) {
            return true;
        }
        else {
            return false;
        }
    }
}
