package org.act.soap.soapsec;
import java.io.*;
import java.security.*;
import java.util.*;
import org.act.soap.soapsec.securitytoken.*;
import org.act.soap.soapsec.securitytoken.keyprocessor.*;
import org.act.soap.util.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class SOAPSecurity {
    
    private Timestamp timestamp = new Timestamp();     
    private Security security = new Security();
    
    private Key kDecKey = null;
    
    private SecurityTokenKeyProcessor securityTokenKeyProcessor = new
        DefaultSecurityKeyProcessor();
    
    public Document getSoapSecMessage ( Document soapMessage ) throws
        SOAPException {
                Vector Envelopes = XMLUtils.getChildElementsByNS( soapMessage,
            Constants.NS_URI_SOAP_ENV, Constants.ELEM_ENVELOPE );
        if ( Envelopes == null ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "ԭĵһЧSOAPģȱ<Envelope>Ԫ" );
        }
                Element eEnvelope = ( Element ) Envelopes.firstElement();
                Vector Bodys = XMLUtils.getChildElementsByNS( eEnvelope,
            Constants.NS_URI_SOAP_ENV,
            Constants.ELEM_BODY );
        if ( Bodys == null ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "ԭĵһЧSOAPģȱ<Body>Ԫ" );
        }
                Element eBody = ( Element ) Bodys.firstElement();
                Vector Headers = XMLUtils.getChildElementsByNS( eEnvelope,
            Constants.NS_URI_SOAP_ENV, Constants.ELEM_HEADER );
        if ( Headers == null || Headers.isEmpty() ) {
                        String soapPrefix = eEnvelope.getPrefix();
            Element eHeader = soapMessage.createElementNS( Constants.
                NS_URI_SOAP_ENV,
                soapPrefix + ":" + Constants.ELEM_HEADER );
            eEnvelope.insertBefore( eHeader, eBody );
            if ( Headers == null ) {
                Headers = new Vector();
            }
            Headers.addElement( eHeader );
        }
                Element eHeader = ( Element ) Headers.firstElement();
                eEnvelope.setAttribute( "xmlns:" + Constants.NS_PRE_WSU,
                                Constants.NS_URI_WSU );
        eEnvelope.setAttribute( "xmlns:" + Constants.NS_PRE_WSSE,
                                Constants.NS_URI_WSSE );
                if ( this.timestamp != null ) {
            Element eTimestamp = this.timestamp.toElement( soapMessage );
            eHeader.appendChild( eTimestamp );
        }
                Element eSecurity = security.toElement( soapMessage );
        eHeader.appendChild( eSecurity );
        Vector vChildren = this.security.children;
        for ( int m = 0; m < vChildren.size(); m++ ) {
            Object o = vChildren.get( m );
            if ( o instanceof Signature ) {
                ( ( Signature ) o ).sign();
            }
            if ( o instanceof Encryption ) {
                ( ( Encryption ) o ).encrypt();
            }
        }
        return soapMessage;
    }
    
    public String getSoapSecMessage ( String soapMessage ) throws SOAPException {
        try {
            InputSource is = new InputSource( new StringReader( soapMessage ) );
            Document doc = XMLUtils.getDOMBuilder().parse( is );
            doc = getSoapSecMessage( doc );
            return XMLUtils.nodeToString( doc );
        }
        catch ( java.io.IOException ex ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ȡԭʼĴ",
                                     ex );
        }
        catch ( org.xml.sax.SAXException ex ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ԭʼĴ",
                                     ex );
        }
        catch ( javax.xml.parsers.ParserConfigurationException ex ) {
            throw new SOAPException( Constants.FAULT_CODE_SERVER, "", ex );
        }
    }
    
    public Document getSoapMessage ( Document soapSecMessage ) throws
        SOAPException {
                Vector Envelopes = XMLUtils.getChildElementsByNS( soapSecMessage,
            Constants.NS_URI_SOAP_ENV, Constants.ELEM_ENVELOPE );
        if ( Envelopes == null ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "ԭĵһЧSOAPģȱ<Envelope>Ԫ" );
        }
        Element eEnvelope = ( Element ) Envelopes.firstElement();
                Vector Headers = XMLUtils.getChildElementsByNS( eEnvelope,
            Constants.NS_URI_SOAP_ENV, Constants.ELEM_HEADER );
        if ( Headers == null || Headers.isEmpty() ) {
            return soapSecMessage;         }
        Element eHeader = ( Element ) Headers.firstElement();
                Vector Timestamps = XMLUtils.getChildElementsByNS( eHeader,
            Constants.NS_URI_WSU, Constants.ELEM_TIMESTAMP );
        Element eTimestamp = ( Element ) Timestamps.firstElement();
        if ( eTimestamp != null ) {
            this.timestamp = new Timestamp( eTimestamp );
        }
        else {
            this.timestamp = null;
        }
                
        Vector Securitys = XMLUtils.getChildElementsByNS( eHeader,
            Constants.NS_URI_WSSE, Constants.ELEM_SECURITY );
        Element eSecurity = ( Element ) Securitys.firstElement();
        if ( eSecurity == null ) {
            return soapSecMessage;         }
                this.security = new Security( eSecurity );
        int childrenCount = this.security.children.size();
                                                for ( int i = childrenCount - 1; i >= 0; i-- ) {             Object o = this.security.children.get( i );
            if ( o instanceof Signature ) {
                                Signature sig = ( Signature ) o;
                sig.setSecurityTokenKeyProcessor( this.
                                                  securityTokenKeyProcessor );
                if ( !sig.verify( null ) ) {
                    throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                             "ǩ֤ʧ" );
                }
            }
            if ( o instanceof Encryption ) {
                                ( ( Encryption ) o ).decrypt( this.kDecKey );
                                Vector Envelope = XMLUtils.getChildElementsByNS( soapSecMessage,
                    Constants.NS_URI_SOAP_ENV, Constants.ELEM_ENVELOPE );
                Vector Header = XMLUtils.getChildElementsByNS( eEnvelope,
                    Constants.NS_URI_SOAP_ENV, Constants.ELEM_HEADER );
                Vector Timestamp = XMLUtils.getChildElementsByNS( eHeader,
                    Constants.NS_URI_WSU, Constants.ELEM_TIMESTAMP );
                Vector Security = XMLUtils.getChildElementsByNS( eHeader,
                    Constants.NS_URI_WSSE, Constants.ELEM_SECURITY );
                Element elSecurity = ( Element ) Security.firstElement();
                Security newSecurity = new Security( elSecurity );
                                for ( int j = i + 1; j < childrenCount; j++ ) {
                    newSecurity.children.remove( j );
                    newSecurity.children.add( j, this.security.children.get( j ) );
                }
                this.security = security;
            }
        }
                                                                return soapSecMessage;
    }
    
    public String getSoapMessage ( String soapsecMessage ) throws SOAPException {
        try {
            InputSource is = new InputSource( new StringReader( soapsecMessage ) );
            Document doc = XMLUtils.getDOMBuilder().parse( is );
            doc = getSoapMessage( doc );
            return XMLUtils.nodeToString( doc );
        }
        catch ( java.io.IOException ex ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ȡԭʼĴ",
                                     ex );
        }
        catch ( org.xml.sax.SAXException ex ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ԭʼĴ",
                                     ex );
        }
        catch ( javax.xml.parsers.ParserConfigurationException ex ) {
            throw new SOAPException( Constants.FAULT_CODE_SERVER, "", ex );
        }
    }
    
    public void addSecurityToken ( SecurityToken securityToken ) {
        this.security.add( securityToken );
    }
    
    public SecurityToken removeSecurityToken ( SecurityToken securityToken ) {
        return this.security.remove( securityToken );
    }
    
    public SecurityToken[] getSecurityTokens () {
        return this.security.getSecurityTokens();
    }
    
    public void addSignature ( Signature signature ) {
        this.security.add( signature );
    }
    
    public Signature removeSignature ( Signature signature ) {
        return this.security.remove( signature );
    }
    
    public Signature[] getSignatures () {
        return this.security.getSignatures();
    }
    
    public void addEncryption ( Encryption encryption ) {
        this.security.add( encryption );
    }
    
    public Encryption removeEncryption ( Encryption encryption ) {
        return this.security.remove( encryption );
    }
    
    public Encryption[] getEncryptions () {
        return this.security.getEncryptions();
    }
    
    public Timestamp getTimestamp () {
        return this.timestamp;
    }
    
    public void setTimestamp ( Timestamp timestamp ) {
        this.timestamp = timestamp;
    }
    
    public void setDecKey ( Key kDecKey ) {
        this.kDecKey = kDecKey;
    }
    
    public Key getDecKey () {
        return this.kDecKey;
    }
    
    public SecurityTokenKeyProcessor getSecurityTokenKeyProcessor () {
        return this.securityTokenKeyProcessor;
    }
    
    public void setSecurityTokenKeyProcessor ( SecurityTokenKeyProcessor
                                               securityTokenProcessor ) {
        this.securityTokenKeyProcessor = securityTokenProcessor;
    }
}
