package org.act.soap.soapsec;
import java.util.*;
import org.act.soap.soapsec.securitytoken.*;
import org.act.soap.util.*;
import org.w3c.dom.*;

public class Security
    extends SOAPSecurityElement {
    
    private String actor = null;
    
    private String mustUnderstand = null;
    
    Vector children = new Vector();
    
    private Vector securityTokens = new Vector();
    private Vector signatures = new Vector();
    private Vector encryptions = new Vector();
    private Vector otherElements = new Vector();
    
    public Security () {
        this.namespaceURI = Constants.NS_URI_WSSE;
        this.prefix = Constants.NS_PRE_WSSE;
        this.localName = Constants.ELEM_SECURITY;
    }
    
    public Security ( Element eSecurity ) throws SOAPException {
        super( eSecurity );
                Vector childElements = XMLUtils.getAllChildElements( eSecurity );
        for ( int i = 0; i < childElements.size(); i++ ) {
            Element element = ( Element ) childElements.elementAt( i );
            if ( !SecurityToken.getSecTokenType( element ).equals( Constants.
                TYPE_ST_UNKNOWNTOKEN ) ) {
                                this.add( SecurityToken.createToken( element ) );
            }
            else if ( Signature.isSignature( element ) ) {
                                Signature sig = new Signature( element );
                this.add( sig );
            }
            else if ( Encryption.isEncryption( element ) ) {
                                this.add( new Encryption( element ) );
            }
            else {
                this.add( element );
            }
        }
    }
    
    public String getActor () {
        return actor;
    }
    
    public void setActor ( String actor ) {
        this.actor = actor;
    }
    
    public String getMustUnderstand () {
        return mustUnderstand;
    }
    
    public void setMustUnderstand ( String mustUnderstand ) {
        this.mustUnderstand = mustUnderstand;
    }
        
    public void add ( SecurityToken securityToken ) {
        this.children.add( securityToken );
        this.securityTokens.add( securityToken );
    }
    
    public void add ( Signature signature ) {
        this.children.add( signature );
        this.signatures.add( signature );
    }
    
    public void add ( Encryption encryption ) {
        this.children.add( encryption );
        this.encryptions.add( encryption );
    }
    
    public void add ( Element childElement ) {
        this.children.add( childElement );
        this.otherElements.add( childElement );
    }
        
    public SecurityToken remove ( SecurityToken securityToken ) {
        this.securityTokens.remove( securityToken );
        return this.children.remove( securityToken ) ? securityToken : null;
    }
    
    public Signature remove ( Signature signature ) {
        this.signatures.remove( signature );
        return this.children.remove( signature ) ? signature : null;
    }
    
    public Encryption remove ( Encryption encryption ) {
        this.encryptions.remove( encryption );
        return this.children.remove( encryption ) ? encryption : null;
    }
    
    public Element remove ( Element element ) {
        this.otherElements.remove( element );
        return this.children.remove( element ) ? element : null;
    }
        
    public SecurityToken[] getSecurityTokens () {
        SecurityToken[] tokens = new SecurityToken[this.securityTokens.size()];
        this.securityTokens.toArray( tokens );
        return tokens;
    }
    
    public Signature[] getSignatures () {
        Signature[] sigs = new Signature[this.signatures.size()];
        this.signatures.toArray( sigs );
        return sigs;
    }
    
    public Encryption[] getEncryptions () {
        Encryption[] encs = new Encryption[this.encryptions.size()];
        this.encryptions.toArray( encs );
        return encs;
    }
    
    public Element[] getElements () {
        Element[] elms = new Element[this.otherElements.size()];
        this.otherElements.toArray( elms );
        return elms;
    }
        
    public Element toElement ( Document doc ) throws SOAPException {
        Element eSecurity = super.toElement( doc );
        for ( int i = 0; i < children.size(); i++ ) {
            Object o = children.get( i );
            if ( o instanceof SecurityToken ) {
                eSecurity.appendChild( ( ( SecurityToken ) o ).toElement( doc ) );
            }
            if ( o instanceof Signature ) {
                eSecurity.appendChild( ( ( Signature ) o ).toElement( doc, eSecurity ) );
            }
            if ( o instanceof Encryption ) {
                eSecurity.appendChild( ( ( Encryption ) o ).toElement( doc, eSecurity ) );
            }
            else if ( o instanceof Element ) {
                eSecurity.appendChild( ( Element ) o );
            }
        }
        this.element = eSecurity;
        return eSecurity;
    }
}
