package org.act.soap.soapsec;
import java.security.*;
import java.util.*;
import javax.xml.namespace.*;
import org.act.soap.soapsec.securitytoken.*;
import org.act.soap.soapsec.securitytoken.keyprocessor.*;
import org.act.soap.util.*;
import org.act.soap.util.XMLUtils;
import org.act.xdsec.core.com.*;
import org.act.xdsec.syntax.dsig.*;
import org.act.xdsec.syntax.keyinfo.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class Signature
    extends org.act.soap.soapsec.SOAPSecurityElement {
    
    private Vector vSignParts = new Vector();
    
    private String signatureMethod = Constants.ALG_RSAWITHSHA1_URI;
    
    private SecurityToken securityToken = null;
    
    private SecurityTokenKeyProcessor securityTokenKeyProcessor = null;
    
    public Signature () {
        this.namespaceURI = Constants.NS_URI_SIGNATURE;
        this.localName = Constants.ELEM_SIGNATURE;
        this.prefix = Constants.NS_PRE_SIGNATURE;
                this.vSignParts.add( Constants.PARTINFO_BODY );
        this.vSignParts.add( Constants.PARTINFO_TIMESTAMP );
    }
    
    public Signature ( SecurityToken securityToken ) {
        this();
        this.securityToken = securityToken;
        this.signatureMethod = securityToken.getSignAlgorithm();     }
    
    public Signature ( Element eSignature ) throws SOAPException {
        super( eSignature );
                Vector SignInfos = XMLUtils.getChildElementsByNS( eSignature,
            Constant.NS_XMLDSIG_NAME, Constant.TAG_SIGNEDINFO_NAME );
        Element eSignInfo = ( Element ) SignInfos.firstElement();
        Vector SignMethods = XMLUtils.getChildElementsByNS( eSignInfo,
            Constant.NS_XMLDSIG_NAME, Constant.TAG_SIGNATUREMETHOD_NAME );
        Element eSignMethod = ( Element ) SignMethods.firstElement();
        this.signatureMethod = eSignMethod.getAttribute( Constant.ATT_ALGORITHM );
                Vector References = XMLUtils.getChildElementsByNS( eSignInfo,
            Constant.NS_XMLDSIG_NAME, Constant.TAG_REFERENCE_NAME );
        for ( int i = 0; i < References.size(); i++ ) {
            Element eReference = ( Element ) References.get( i );
            String uri = eReference.getAttribute( Constant.ATT_URI );
            if ( uri.startsWith( "#" ) ) {
                Element ePart = XMLUtils.getElementById( eSignature.
                    getOwnerDocument(), uri.substring( 1 ) );
                this.vSignParts.add( new QName( ePart.getNamespaceURI(),
                                                ePart.getLocalName() ) );
            }
        }
    }
    
    public String getSignatureMethod () {
        return this.signatureMethod;
    }
    
    public void setSignatureMethod ( String signatureMethod ) {
        this.signatureMethod = signatureMethod;
    }
    
    public SecurityToken getSecurityToken () {
        return this.securityToken;
    }
    
    public void setSecurityToken ( SecurityToken securityToken ) {
        this.securityToken = securityToken;
    }
    
    public void addSignPart ( QName partInfo ) {
        
        if ( !vSignParts.contains( partInfo ) ) {
            this.vSignParts.add( partInfo );
        }
    }
    
    public QName[] getSignParts () {
        QName[] pis = new QName[this.vSignParts.size()];
        this.vSignParts.toArray( pis );
        return pis;
    }
    
    public boolean isSigningPart ( QName part ) {
        for ( int i = 0; i < this.vSignParts.size(); i++ ) {
            if ( this.vSignParts.get( i ).equals( part ) ) {
                return true;
            }
        }
        return false;
    }
    
    public QName removeSignPart ( QName partInfo ) {
        return this.vSignParts.remove( partInfo ) ? partInfo : null;
    }
    
    public SecurityTokenKeyProcessor getSecurityTokenKeyProcessor () {
        return this.securityTokenKeyProcessor;
    }
    
    public void setSecurityTokenKeyProcessor ( SecurityTokenKeyProcessor
                                               securityTokenProcessor ) {
        this.securityTokenKeyProcessor = securityTokenProcessor;
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
                XSignature xsig = new XSignature( doc );
        xsig.setSignatureMethod( this.signatureMethod );                 for ( int i = 0; i < this.vSignParts.size(); i++ ) {
            QName partInfo = ( QName )this.vSignParts.get( i );
            NodeList nlParts = doc.getElementsByTagNameNS( partInfo.
                getNamespaceURI(), partInfo.getLocalPart() );
            if ( nlParts == null || nlParts.getLength() <= 0 ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ǩԪ" + partInfo.getNamespaceURI() +
                                         ":" + partInfo.getLocalPart() + "" );
            }
            Element ePart = ( Element ) nlParts.item( 0 );
            String partId = XMLUtils.getElementWSUId( ePart );
            if ( partId == null ) {
                partId = UUID.generate().toString();
                String wsuIdTagName = Constants.NS_PRE_WSU + ":" +
                    Constants.ATTR_WSU_ID;
                ePart.setAttribute( wsuIdTagName, partId );
            }
                        XReference xref = xsig.createReference();
            xref.setURI( Constants.S_Ref + partId );
            XTransform xtrans = xref.createTransform();
            xtrans.setURI( Constant.ALG_CANONICALIZATION_URI );
            xref.addTransform( xtrans );
            xsig.addReference( xref );
        }
                XKeyInfo xkeyinfo = xsig.createKeyInfo();
        SecurityTokenReference str = new SecurityTokenReference();
        str.addReference( this.securityToken );
        xkeyinfo.addAnyKey( new SecurityTokenReferenceKeyType( str ) );
        xsig.addKeyInfo( xkeyinfo );
        this.element = xsig.toElement();
        return this.element;
    }
    
    public Element toElement ( Document doc, Element eSecurity ) throws SOAPException {
                XSignature xsig = new XSignature( doc );
        xsig.setSignatureMethod( this.signatureMethod );                 for ( int i = 0; i < this.vSignParts.size(); i++ ) {
            QName partInfo = ( QName )this.vSignParts.get( i );
            NodeList nlParts = doc.getElementsByTagNameNS( partInfo.getNamespaceURI(),
                partInfo.getLocalPart() );
                                    if ( nlParts == null || nlParts.getLength() <= 0 ) {
                nlParts = eSecurity.getElementsByTagNameNS( partInfo.getNamespaceURI(),
                                                            partInfo.getLocalPart() );
            }
            if ( nlParts == null || nlParts.getLength() <= 0 ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ǩԪ" + partInfo.getNamespaceURI() +
                                         ":" + partInfo.getLocalPart() + "" );
            }
            Element ePart = ( Element ) nlParts.item( 0 );
            String partId = XMLUtils.getElementWSUId( ePart );
            if ( partId == null ) {
                partId = UUID.generate().toString();
                String wsuIdTagName = Constants.NS_PRE_WSU + ":" +
                    Constants.ATTR_WSU_ID;
                ePart.setAttribute( wsuIdTagName, partId );
            }
                        XReference xref = xsig.createReference();
            xref.setURI( Constants.S_Ref + partId );
            XTransform xtrans = xref.createTransform();
            xtrans.setURI( Constant.ALG_CANONICALIZATION_URI );
            xref.addTransform( xtrans );
            xsig.addReference( xref );
        }
                XKeyInfo xkeyinfo = xsig.createKeyInfo();
        SecurityTokenReference str = new SecurityTokenReference();
        str.addReference( this.securityToken );
        xkeyinfo.addAnyKey( new SecurityTokenReferenceKeyType( str ) );
        xsig.addKeyInfo( xkeyinfo );
        this.element = xsig.toElement();
        return this.element;
    }
        
    public void sign () throws SOAPException {
                if ( this.element == null ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "ûSignatureԪأȵtoElementɵ<Signature>Ԫطĵʵλúٵñ" );
        }
                org.act.xdsec.core.dsig.Signature sig = new org.act.xdsec.core.dsig.
            Signature();
        try {
                        Key signKey = null;
            if ( this.securityTokenKeyProcessor != null ) {
                this.securityTokenKeyProcessor.getKey( this.securityToken,
                    SecurityTokenKeyProcessor.USAGE_SIGN );
            }
            else {
                signKey = this.securityToken.getSignKey();
            }
            sig.sign( this.element, "", signKey,
                      this.securityToken.getSignKeyName() );
        }
        catch ( Exception ex ) {
            ex.printStackTrace();
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ǩʧ", ex );
        }
    }
        
    public boolean verify ( Key verifyKey ) throws SOAPException {
                if ( this.element == null ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ûǩԪ" );
        }
                org.act.xdsec.core.dsig.Signature sig = new org.act.xdsec.core.dsig.
            Signature();
                SecurityTokenReferenceKeyProcessor keyp
            = new SecurityTokenReferenceKeyProcessor( this.
            securityTokenKeyProcessor );
        sig.addKeyProcessor( keyp );
        try {
            Validity vad = sig.verify( this.element, "", verifyKey );
            this.securityToken = keyp.getSecurityToken();
            return vad.getValidity();
        }
        catch ( Exception ex ) {
            ex.printStackTrace();
            throw new SOAPException( Constants.FAULT_CODE_CLIENT, "֤", ex );
        }
    }
    
    public static boolean isSignature ( Element element ) {
        if ( element == null ) {
            return false;
        }
        else {
            return Constants.NS_URI_SIGNATURE.equals( element.getNamespaceURI() )
                && Constants.ELEM_SIGNATURE.equals( element.getLocalName() );
        }
    }
}
