package org.act.soap.soapsec;
import java.util.*;
import org.act.soap.util.*;
import org.w3c.dom.*;

public class Timestamp
    extends SOAPSecurityElement {
    
    private Date Created = null;
    
    private Date Expires = null;
    
    public Timestamp () {
        this.prefix = Constants.NS_PRE_WSU;
        this.namespaceURI = Constants.NS_URI_WSU;
        this.localName = Constants.ELEM_TIMESTAMP;
                this.wsuId = "Id-" + UUID.generate().toString();
    }
    
    public Timestamp ( long expirePeriod ) {
        this();
        this.Created = new Date();
        this.Expires = new Date( this.Created.getTime() + expirePeriod );
    }
    
    public Timestamp ( Element elmTimestamp ) throws SOAPException {
        super( elmTimestamp );
                Vector Createds = XMLUtils.getChildElementsByNS( elmTimestamp,
            Constants.NS_URI_WSU, Constants.ELEM_CREATED );
        if ( !Createds.isEmpty() ) {
            Element elmCreated = ( Element ) Createds.firstElement();
            String sCreated = XMLUtils.getTextValueOfElement( elmCreated );
            this.Created = TimeUtil.parseTimeStamp( sCreated );
            if ( this.Created == null ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ʱʽ" );
            }
        }
                Vector Expires = XMLUtils.getChildElementsByNS( elmTimestamp,
            Constants.NS_URI_WSU, Constants.ELEM_EXPIRES );
        if ( !Createds.isEmpty() ) {
            Element elmExpires = ( Element ) Expires.firstElement();
            String sExpires = XMLUtils.getTextValueOfElement( elmExpires );
            this.Expires = TimeUtil.parseTimeStamp( sExpires );
            if ( this.Expires == null ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT, "ʱʽ" );
            }
        }
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
        Element elmTimestamp = super.toElement( doc );
        if ( this.Created != null ) {
                        Element elmCreated = doc.createElementNS( Constants.NS_URI_WSU,
                Constants.NS_PRE_WSU + ":" + Constants.ELEM_CREATED );
            Text txtCreated = doc.createTextNode( TimeUtil.getTimeStamp( this.
                Created ) );
            elmCreated.appendChild( txtCreated );
            elmTimestamp.appendChild( elmCreated );
        }
        if ( this.Expires != null ) {
                        Element elmExpires = doc.createElementNS( Constants.NS_URI_WSU,
                Constants.NS_PRE_WSU + ":" + Constants.ELEM_EXPIRES );
            Text txtExpires = doc.createTextNode( TimeUtil.getTimeStamp( this.
                Expires ) );
            elmExpires.appendChild( txtExpires );
            elmTimestamp.appendChild( elmExpires );
        }
        this.element = elmTimestamp;
        return elmTimestamp;
    }
    
    public boolean isExpired () {
        if ( this.Expires != null ) {
            return this.Expires.before( new Date() );
        }
        else {
            return false;
        }
    }
    
    public Date getCreated () {
        return this.Created;
    }
    
    public void setCreated ( Date createdTime ) {
        this.Created = createdTime;
    }
    
    public Date getExpires () {
        return this.Expires;
    }
    
    public void setExpires ( Date expiresTime ) {
        this.Expires = expiresTime;
    }
}
