package org.act.soap.soapsec.securitytoken;
import org.act.soap.soapsec.*;
import org.act.soap.soapsec.Constants;
import org.act.soap.util.*;
import org.w3c.dom.*;

public class BinarySecurityToken
    extends SecurityToken {
            
    public static String TYPE_VALUE_X509V3 = Constants.NS_PRE_WSSE + ":" +
        "X509v3";
    
    public static String TYPE_VALUE_KERB_TGT = Constants.NS_PRE_WSSE + ":" +
        "Kerberosv5TGT";
    
    public static String TYPE_VALUE_KERB_ST = Constants.NS_PRE_WSSE + ":" +
        "Kerberosv5ST";
    
    public static String TYPE_ENCODE_BASE64 = Constants.NS_PRE_WSSE + ":" +
        "Base64Binary";
    
    public static String TYPE_ENCODE_HEX = Constants.NS_PRE_WSSE + ":" +
        "HexBinary";
            
    protected String ValueType = TYPE_VALUE_X509V3;
    
    protected String EncodingType = TYPE_ENCODE_BASE64;
    
    protected String BinarySecurityTokenString = null;
            
    public BinarySecurityToken () {
        super();
        this.namespaceURI = Constants.NS_URI_WSSE;
        this.prefix = Constants.NS_PRE_WSSE;
        this.localName = Constants.ELEM_BINARYSECURITYTOKEN;
    }
    
    public BinarySecurityToken ( Element eToken ) {
        super( eToken );         this.ValueType = eToken.getAttribute( Constants.ATTR_WSSE_VALUETYPE );
        this.EncodingType = eToken.getAttribute( Constants.
                                                 ATTR_WSSE_ENCODINGTYPE );
        this.BinarySecurityTokenString = XMLUtils.getTextValueOfElement( eToken );
    }
    
    public String getValueType () {
        return ValueType;
    }
    
    public void setValueType ( String valuetype ) {
        this.ValueType = valuetype;
    }
    
    public String getEncodingType () {
        return EncodingType;
    }
    
    public void setEncodingType ( String encodingtype ) {
        this.EncodingType = encodingtype;
    }
    
    public String getBinarySecurityTokenString () {
        return BinarySecurityTokenString;
    }
    
    public void setBinarySecurityTokenString ( String strBST ) {
        this.BinarySecurityTokenString = strBST;
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
                Element eBinaryST = super.toElement( doc );                 if ( this.ValueType == null ) {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "ValueType attribute can not be null." );
        }
        eBinaryST.setAttribute( Constants.ATTR_WSSE_VALUETYPE, this.ValueType );
                if ( this.EncodingType == null ) {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "EncodingType attribute can not be null." );
        }
        eBinaryST.setAttribute( Constants.ATTR_WSSE_ENCODINGTYPE,
                                this.EncodingType );
                if ( this.BinarySecurityTokenString != null ) {
            eBinaryST.appendChild( doc.createTextNode( this.
                BinarySecurityTokenString ) );
        }
        this.element = eBinaryST;
        return eBinaryST;
    }
    
    public static String encode ( byte[] source, String encodingType ) throws
        SOAPException {
        if ( TYPE_ENCODE_BASE64.equals( encodingType ) ) {
            return org.act.soap.util.Base64Utils.base64Encode( source );
        }
        else if ( TYPE_ENCODE_HEX.equals( encodingType ) ) {
            return org.act.soap.util.HexBinary.encode( source );
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "Unsupported encoding type." );
        }
    }
    
    public static byte[] decode ( String source, String encodingType ) throws
        SOAPException {
        if ( TYPE_ENCODE_BASE64.equals( encodingType ) ) {
            return org.act.soap.util.Base64Utils.base64Decode( source );
        }
        else if ( TYPE_ENCODE_HEX.equals( encodingType ) ) {
            return org.act.soap.util.HexBinary.decode( source );
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "Unsupported encoding type." );
        }
    }
    
    public static BinarySecurityToken createBinarySecurityToken ( Element
        eToken ) throws
        SOAPException {
        BinarySecurityToken bst = new BinarySecurityToken( eToken );
        if ( BinarySecurityToken.TYPE_VALUE_X509V3.equals( bst.getValueType() ) ) {
            return new X509CertToken( eToken );
        }
        else {
            return bst;
        }
    }
}
