package org.act.soap.soapsec.securitytoken;
import java.security.cert.*;
import javax.security.auth.kerberos.*;
import org.act.soap.soapsec.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class KerberosTicketToken
    extends BinarySecurityToken {
    
    private String ValueType = TYPE_VALUE_X509V3;
    
    private String EncodingType = TYPE_ENCODE_BASE64;
    
    private String BinarySecurityTokenString = null;
    
    private KerberosTicket Kerberosv5TGT = null;
    
    private KerberosTicket Kerberosv5ST = null;
            
    public KerberosTicketToken () {
        super();
    }
    
    public String getValueType () {
        return ValueType;
    }
    
    public void setValueType ( String valuetype ) {
        this.ValueType = valuetype;
    }
    
    public String getEncodingType () {
        return EncodingType;
    }
    
    public void setEncodingType ( String encodingtype ) {
        this.EncodingType = encodingtype;
    }
    
    public String getBinarySecurityTokenString () {
        return BinarySecurityTokenString;
    }
    public static String getBinarySecurityTokenString ( byte[] source,
        String encodingtype ) throws SOAPException {
        if ( encodingtype == TYPE_ENCODE_BASE64 ) {
            return JavaUtils.Base64Encode( source );
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "Unsupported encoding type." );
        }
    }
    public static String getBinarySecurityTokenString ( X509Certificate cert,
        String encodingtype ) {
        try {
            return getBinarySecurityTokenString( cert.getEncoded(),
                                                 encodingtype );
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
        return null;
    }
    public static String getBinarySecurityTokenString ( KerberosTicket
        krbticket,
        String encodingtype ) {
        try {
            return getBinarySecurityTokenString( krbticket.getEncoded(),
                                                 encodingtype );
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
        return null;
    }
    
    public void setBinarySecurityTokenString ( String strBST ) {
        this.BinarySecurityTokenString = strBST;
    }
    
    public KerberosTicket getKerberosv5TGT () {
        return Kerberosv5TGT;
    }
    
    public void setKerberosTicket ( KerberosTicket ticket ) {
        this.Kerberosv5TGT = ticket;
    }
    
    public KerberosTicket getKerberosv5ST () {
        return Kerberosv5ST;
    }
    
    public void setKerberv5ST ( KerberosTicket ticket ) {
        this.Kerberosv5ST = ticket;
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
        Element eBinaryST = doc.createElement( Constants.
                                               Q_ELEM_BINARYSECURITYTOKEN.
                                               toString() );
        if ( this.wsuId != null && this.wsuId != "" ) {
            eBinaryST.setAttribute( Constants.ATTR_WSU_ID, this.wsuId );
        }
        else {
            eBinaryST.setAttribute( Constants.ATTR_WSU_ID, "" );
        }
        if ( ValueType == null ) {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "ValueType attribute can not be null." );
        }
        eBinaryST.setAttribute( Constants.ATTR_WSSE_VALUETYPE, ValueType );
        if ( EncodingType == null ) {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "EncodingType attribute can not be null." );
        }
        eBinaryST.setAttribute( Constants.ATTR_WSSE_ENCODINGTYPE, EncodingType );
        if ( BinarySecurityTokenString == null ) {
            if ( ValueType == TYPE_VALUE_KERB_TGT && Kerberosv5TGT != null ) {
                this.BinarySecurityTokenString = getBinarySecurityTokenString(
                    Kerberosv5TGT, EncodingType );
            }
            else if ( ValueType == TYPE_VALUE_KERB_ST && Kerberosv5ST != null ) {
                this.BinarySecurityTokenString = getBinarySecurityTokenString(
                    Kerberosv5ST, EncodingType );
            }
        }
        if ( BinarySecurityTokenString == null ) {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "BinarySecurityToken's content can not be null." );
        }
        else {
            eBinaryST.appendChild( doc.createTextNode( Constants.S_Line ) );
            eBinaryST.appendChild( doc.createTextNode(
                BinarySecurityTokenString ) );
            eBinaryST.appendChild( doc.createTextNode( Constants.S_Line ) );
        }
        return eBinaryST;
    }
}
