package org.act.soap.soapsec.securitytoken;
import java.security.*;
import java.util.*;
import javax.crypto.spec.*;
import org.act.soap.soapsec.*;
import org.act.soap.soapsec.Constants;
import org.act.soap.util.*;
import org.act.soap.util.XMLUtils;
import org.act.xdsec.core.enc.*;
import org.act.xdsec.syntax.enc.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class SecurityContextToken
    extends SecurityToken {
            
    private String identifier = null;
    
    private Date created = null;
    
    private Date expires = null;
    
    private Element encryptedKey = null;
    
    private Key sessionKey = null;
    
    private String encAlgURI = Constant.ALG_RSA15_URI;
    
    private Key encKey = null;
    
    public SecurityContextToken () {
        super();
    }
    public SecurityContextToken ( String identifier ) {
        super();
        this.namespaceURI = Constants.NS_URI_WSSE;
        this.prefix = Constants.NS_PRE_WSSE;
        this.localName = Constants.ELEM_SECURITYCONTEXTTOKEN;
                this.signAlgorithm = Constant.ALG_HMAC_URI;
        this.encryptAlgorithm = Constant.ALG_DES_URI;
        this.identifier = identifier;
    }
    
    public SecurityContextToken ( Element eToken ) throws SOAPException {
        super( eToken );
                Vector Identifiers = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSU,
            Constants.ELEM_IDENTIFIER );
        Element eIdentifier = ( Element ) Identifiers.firstElement();
        if ( eIdentifier != null ) {
            this.identifier = XMLUtils.getTextValueOfElement( eIdentifier );
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "<SecurityContextToken>ȱ<Identifier>" );
        }
                Vector Createds = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSU,
            Constants.ELEM_CREATED );
        Element eCreated = ( Element ) Createds.firstElement();
        if ( eCreated != null ) {
            String sCreated = XMLUtils.getTextValueOfElement( eCreated );
            this.created = TimeUtil.parseTimeStamp( sCreated );
            if ( this.created == null ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "<Created>ʱʽ" );
            }
        }
                Vector Expires = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSU,
            Constants.ELEM_EXPIRES );
        Element eExpires = ( Element ) Expires.firstElement();
        if ( eExpires != null ) {
            String sExpires = XMLUtils.getTextValueOfElement( eExpires );
            this.expires = TimeUtil.parseTimeStamp( sExpires );
            if ( this.expires == null ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "<Expires>ʱʽ" );
            }
        }
                Vector Keys = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSSE,
            Constants.ELEM_KEY );
        Element eKey = ( Element ) Keys.firstElement();
        if ( eKey != null ) {
            Vector encryptedKeys = XMLUtils.getChildElementsByNS( eKey,
                Constants.NS_URI_ENCRYPTION,
                Constants.ELEM_ENCRYPTEDKEY );
            this.encryptedKey = ( Element ) encryptedKeys.firstElement();
        }
    }
    
    public String getIdentifier () {
        return identifier;
    }
    
    public void setIdentifier ( String identifier ) {
        this.identifier = identifier;
    }
    
    public boolean isExpired () {
        if ( this.expires != null ) {
            return this.expires.before( new Date() );
        }
        else {
            return false;
        }
    }
    
    public Date getCreated () {
        return this.created;
    }
    
    public void setCreated ( Date createdTime ) {
        this.created = createdTime;
    }
    
    public Date getExpires () {
        return this.expires;
    }
    
    public void setExpires ( Date expiresTime ) {
        this.expires = expiresTime;
    }
    
    public Key getSessionKey ( Key decKey ) throws SOAPException {
        if ( this.sessionKey != null ) {
            return this.sessionKey;
        }
        else if ( this.encryptedKey != null ) {
            try {
                EncryptedKey oEncKey = new EncryptedKey();
                oEncKey.setEncKey( this.encryptedKey );
                                byte[] decdatakey = oEncKey.decrypt( decKey, "" );
                                SecretKeySpec mykey = new SecretKeySpec( decdatakey,
                    Constants.ALG_DESEDE );
                return mykey;
            }
            catch ( Exception ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ܻỰԿ" );
            }
        }
        else {
            return null;
        }
    }
    
    public void setSessionKey ( Key sessionKey, Key encKey, String encAlgURI ) throws
        SOAPException {
        this.sessionKey = sessionKey;
        this.encKey = encKey;
        this.encAlgURI = encAlgURI;
    }
    
    public void setSessionKey ( Key sessionKey, Key encKey ) throws
        SOAPException {
        this.sessionKey = sessionKey;
        this.encKey = encKey;
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
                Element eToken = super.toElement( doc );
                if ( this.identifier != null ) {
            Element eIdentifier = doc.createElementNS( Constants.NS_URI_WSU,
                Constants.NS_PRE_WSU + ":" + Constants.ELEM_IDENTIFIER );
            eIdentifier.appendChild( doc.createTextNode( this.identifier ) );
            eToken.appendChild( eIdentifier );
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "SecurityContextTokenIdentifierԲΪ" );
        }
        if ( this.created != null ) {
                        Element eCreated = doc.createElementNS( Constants.NS_URI_WSU,
                Constants.NS_PRE_WSU + ":" + Constants.ELEM_CREATED );
            Text txtCreated = doc.createTextNode( TimeUtil.getTimeStamp( this.
                created ) );
            eCreated.appendChild( txtCreated );
            eToken.appendChild( eCreated );
        }
        if ( this.expires != null ) {
                        Element eExpires = doc.createElementNS( Constants.NS_URI_WSU,
                Constants.NS_PRE_WSU + ":" + Constants.ELEM_EXPIRES );
            Text txtExpires = doc.createTextNode( TimeUtil.getTimeStamp( this.
                expires ) );
            eExpires.appendChild( txtExpires );
            eToken.appendChild( eExpires );
        }
                if ( this.sessionKey != null ) {
                        XEncryptedKey xEncKey = new XEncryptedKey();
            xEncKey.setEncryptionMethod( this.encAlgURI );
            xEncKey.setCipherData( new XCipherData() );
            xEncKey.setCipherValue( "" );
            this.encryptedKey = xEncKey.toElement( doc );
                        try {
                EncryptedKey oEncKey = new EncryptedKey();
                oEncKey.setEncKey( this.encryptedKey );
                oEncKey.encrypt( this.sessionKey, this.encKey );
            }
            catch ( Exception ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                         "ԻỰԿмʱ", ex );
            }
            Element eKey = doc.createElementNS( Constants.NS_URI_WSSE,
                                                Constants.NS_PRE_WSSE + ":" +
                                                Constants.ELEM_KEY );
            eKey.appendChild( this.encryptedKey );
            eToken.appendChild( eKey );
        }
        this.element = eToken;
        return eToken;
    }
}
