package org.act.soap.soapsec.securitytoken;
import java.io.*;
import java.security.*;
import java.util.*;
import javax.crypto.spec.*;
import org.act.soap.soapsec.*;
import org.act.soap.soapsec.Constants;
import org.act.soap.util.*;
import org.act.soap.util.XMLUtils;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class UsernameToken
    extends SecurityToken {
            
    public static String TYPE_PW_TEXT = Constants.NS_PRE_WSSE + ":PasswordText";
    
    public static String TYPE_PW_DIGEST = Constants.NS_PRE_WSSE +
        ":PasswordDigest";
    
    private String Username = null;
    
    private String Password = null;
    
    private String Nonce = null;
    
    private String Created = null;
    
    private String PasswordType = TYPE_PW_TEXT;
            
    public UsernameToken () {
        this.PasswordType = TYPE_PW_TEXT;
    }
    
    public UsernameToken ( String username ) {
        this.Username = username;
        this.PasswordType = TYPE_PW_TEXT;
    }
    
    public UsernameToken ( String username, String password ) {
        this.Username = username;
        this.Password = password;
        this.PasswordType = TYPE_PW_TEXT;
    }
    
    public UsernameToken ( String username, String password,
                           String passwordType ) {
        super();
        this.namespaceURI = Constants.NS_URI_WSSE;
        this.prefix = Constants.NS_PRE_WSSE;
        this.localName = Constants.ELEM_USERNAMETOKEN;
        this.Username = username;
        this.Password = password;
        this.PasswordType = passwordType;
                this.signAlgorithm = Constant.ALG_HMAC_URI;
        this.encryptAlgorithm = Constant.ALG_DES_URI;
    }
    
    public UsernameToken ( Element eToken ) throws SOAPException {
        super( eToken );
                Vector Usernames = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSSE,
            Constants.ELEM_USERNAME );
        Element eUsername = ( Element ) Usernames.firstElement();
        if ( eUsername == null ) {
            throw new SOAPException( Constants.FAULT_CODE_CLIENT,
                                     "<UsernameToken>ȱ<Username>" );
        }
        this.Username = XMLUtils.getTextValueOfElement( eUsername );
                Vector Passwords = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSSE,
            Constants.ELEM_PASSWORD );
        if ( !Passwords.isEmpty() ) {
            Element ePassword = ( Element ) Passwords.firstElement();
            if ( ePassword != null ) {
                String type = ePassword.getAttribute( Constants.ATTR_WSSE_TYPE );
                if ( type != null ) {
                    this.PasswordType = type;
                }
                if ( !TYPE_PW_TEXT.equals( this.PasswordType ) &&
                     !TYPE_PW_DIGEST.equals( this.PasswordType ) ) {
                    throw new SOAPException( Constants.
                                             FAULT_CODE_InvalidSecToken,
                                             "Invalid PasswordType." );
                }
                this.Password = XMLUtils.getTextValueOfElement( ePassword );
            }
        }
                Vector Nonces = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSSE,
            Constants.ELEM_NONCE );
        if ( !Nonces.isEmpty() ) {
            Element eNonce = ( Element ) Nonces.firstElement();
            if ( eNonce != null ) {
                this.Nonce = XMLUtils.getTextValueOfElement( eNonce );
            }
        }
                Vector Createds = XMLUtils.getChildElementsByNS( eToken,
            Constants.NS_URI_WSU,
            Constants.ELEM_CREATED );
        if ( !Createds.isEmpty() ) {
            Element eCreated = ( Element ) Createds.firstElement();
            if ( eCreated != null ) {
                this.Created = XMLUtils.getTextValueOfElement( eCreated );
            }
        }
    }
    
    public String getUsername () {
        return Username;
    }
    
    public void setUsername ( String username ) {
        this.Username = username;
    }
    
    public String getPassword () {
        return Password;
    }
    
    public void setPassword ( String password ) {
        if ( this.PasswordType.equals( UsernameToken.TYPE_PW_TEXT ) ) {
            this.Password = password;
        }
    }
    
    public String getPasswordType () {
        return PasswordType;
    }
    
    public void createNonce () {
        this.Nonce = Base64Utils.base64Encode( UUID.generate().toString().
                                               getBytes() );
    }
    
    public Date getCreated () {
        return TimeUtil.parseTimeStamp( this.Created );
    }
    
    public void setCreated ( Date created ) {
        this.Created = TimeUtil.getTimeStamp( created );
    }
    
    public static byte[] getPasswordDigt ( String password, String nonce,
                                           String created ) {
        AlgorithmFactory algFactory = new AlgorithmFactory( null );
        DigestAlgorithm algDigest = algFactory.createDigestMethod( Constant.
            ALG_SHA1_URI );
        String origin = "";
        if ( nonce != null ) {
            origin += nonce;
        }
        if ( created != null ) {
            origin += created;
        }
        if ( password != null ) {
            origin += password;
        }
        try {
            return algDigest.digest( origin.getBytes( "UTF-8" ) );
        }
        catch ( UnsupportedEncodingException ex ) {
                        return null;
        }
    }
    
    public Key getSignKey () {
        if ( this.signKey == null ) {
            return new SecretKeySpec(
                UsernameToken.getPasswordDigt( this.Password, this.Nonce,
                                               this.Created ),
                Constants.ALG_DES );
        }
        else {
            return this.signKey;
        }
    }
    
    public Key getVerifyKey () {
        if ( this.verifyKey == null ) {
            return createKey();
        }
        else {
            return this.verifyKey;
        }
    }
    
    public Key getEncryptKey () {
        if ( this.verifyKey == null ) {
            return new SecretKeySpec(
                UsernameToken.getPasswordDigt( this.Password, this.Nonce,
                                               this.Created ),
                Constants.ALG_DES );
        }
        else {
            return this.encryptKey;
        }
    }
    
    public Key getDecryptKey () {
        if ( this.verifyKey == null ) {
            return createKey();
        }
        else {
            return this.encryptKey;
        }
    }
    
    private Key createKey () {
        if ( UsernameToken.TYPE_PW_DIGEST.equals( this.PasswordType ) ) {
            return new SecretKeySpec( Base64Utils.base64Decode( this.Password ),
                                      Constants.ALG_DES );
        }
        else {
            return new SecretKeySpec(
                UsernameToken.getPasswordDigt( this.Password, this.Nonce,
                                               this.Created ),
                Constants.ALG_DES );
        }
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
                String tagName = "wsse:UsernameToken";
        Element eUsernameToken = doc.createElementNS( Constants.NS_URI_WSSE,
            tagName );
        if ( this.wsuId != null ) {
            String wsuIdTagName = Constants.NS_PRE_WSU + ":" +
                Constants.ATTR_WSU_ID;
            eUsernameToken.setAttribute( wsuIdTagName, this.wsuId );
        }
                if ( this.Username != null ) {
            Element eUsername = doc.createElementNS( Constants.NS_URI_WSSE,
                Constants.NS_PRE_WSSE + ":" + Constants.ELEM_USERNAME );
            eUsername.appendChild( doc.createTextNode( this.Username ) );
            eUsernameToken.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            eUsernameToken.appendChild( eUsername );
            eUsernameToken.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        else {
            throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                     "Username can not be null." );
        }
                if ( this.Password != null ) {
            if ( !TYPE_PW_TEXT.equals( this.PasswordType ) &&
                 !TYPE_PW_DIGEST.equals( this.PasswordType ) ) {
                throw new SOAPException( Constants.FAULT_CODE_InvalidSecToken,
                                         "Invalid PasswordType." );
            }
            Element ePassword = doc.createElementNS( Constants.NS_URI_WSSE,
                Constants.NS_PRE_WSSE + ":" + Constants.ELEM_PASSWORD );
            ePassword.setAttribute( Constants.ATTR_WSSE_TYPE, this.PasswordType );
            if ( TYPE_PW_TEXT.equals( this.PasswordType ) ) {
                ePassword.appendChild( doc.createTextNode( this.Password ) );
            }
            else {
                if ( this.Nonce == null ) {
                    this.createNonce();
                }
                if ( this.Created == null ) {
                    this.setCreated( new Date() );
                }
                String pwd = Base64Utils.base64Encode( UsernameToken.
                    getPasswordDigt( this.Password,
                                     this.Nonce,
                                     this.Created ) );
                ePassword.appendChild( doc.createTextNode( pwd ) );
            }
            eUsernameToken.appendChild( ePassword );
            eUsernameToken.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
                if ( this.Nonce != null ) {
            Element eNonce = doc.createElementNS( Constants.NS_URI_WSSE,
                                                  Constants.NS_PRE_WSSE + ":" +
                                                  Constants.ELEM_NONCE );
            eNonce.appendChild( doc.createTextNode( this.Nonce ) );
            eUsernameToken.appendChild( eNonce );
        }
                if ( this.Created != null ) {
            Element eCreated = doc.createElementNS( Constants.NS_URI_WSSE,
                Constants.NS_PRE_WSU + ":" + Constants.ELEM_CREATED );
            eCreated.appendChild( doc.createTextNode( this.Created ) );
            eUsernameToken.appendChild( eCreated );
        }
        this.element = eUsernameToken;
        return eUsernameToken;
    }
    
    public boolean verifyPassword ( String password ) {
        if ( TYPE_PW_TEXT.equals( this.PasswordType ) ) {
            if ( this.Password == null ) {
                return ( password == null );
            }
            else {
                return this.Password.equals( password );
            }
        }
        else if ( TYPE_PW_DIGEST.equals( this.PasswordType ) ) {
            if ( this.Password == null ) {
                return ( password == null );
            }
            else {
                String pwd = Base64Utils.base64Encode( UsernameToken.
                    getPasswordDigt( password,
                                     this.Nonce,
                                     this.Created ) );
                return this.Password.equals( pwd );
            }
        }
        else {
            return false;
        }
    }
}
