package org.act.soap.soapsec.securitytoken;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import org.act.soap.soapsec.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class X509CertToken
    extends BinarySecurityToken {
            
    private X509Certificate X509v3Cert = null;
            
    public X509CertToken () {
        super();
        this.ValueType = TYPE_VALUE_X509V3;
        this.EncodingType = TYPE_ENCODE_BASE64;
                this.signAlgorithm = Constant.ALG_RSAWITHSHA1_URI;
        this.encryptAlgorithm = Constant.ALG_RSA15_URI;
    }
    
    public X509CertToken ( X509Certificate cert ) {
        this();
        this.X509v3Cert = cert;
    }
    
    public X509CertToken ( Element eToken ) throws SOAPException {
        super( eToken );
        if ( this.BinarySecurityTokenString != null ) {
            try {
                byte[] certBytes = BinarySecurityToken.decode( this.
                    BinarySecurityTokenString, this.EncodingType );
                CertificateFactory certFact = CertificateFactory.getInstance(
                    Constant.JCA_CERT_ID );
                this.X509v3Cert = ( X509Certificate ) ( certFact.
                    generateCertificate( new ByteArrayInputStream( certBytes ) ) );
            }
            catch ( CertificateException ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT, "֤鹹ʧ",
                                         ex );
            }
        }
    }
    
    public X509Certificate getX509v3Cert () {
        return X509v3Cert;
    }
    
    public void setX509v3Cert ( X509Certificate cert ) {
        this.X509v3Cert = cert;
    }
    
    public Key getVerifyKey () {
        if ( this.verifyKey == null ) {
            return this.X509v3Cert.getPublicKey();
        }
        else {
            return this.verifyKey;
        }
    }
    
    public Key getEncryptKey () {
        if ( this.encryptKey == null ) {
            return this.X509v3Cert.getPublicKey();
        }
        else {
            return this.encryptKey;
        }
    }
    
    public Element toElement ( Document doc ) throws SOAPException {
        ValueType = TYPE_VALUE_X509V3;
        if ( this.X509v3Cert != null ) {
            try {
                byte[] certBytes = this.X509v3Cert.getEncoded();
                this.BinarySecurityTokenString = BinarySecurityToken.encode(
                    certBytes, this.EncodingType );
            }
            catch ( CertificateException ex ) {
                throw new SOAPException( Constants.FAULT_CODE_CLIENT, "֤ʧ",
                                         ex );
            }
        }
        return super.toElement( doc );
    }
}
