package org.act.soap.util;
import java.io.File;
import java.io.FilenameFilter;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Vector;
import org.act.soap.util.logging.LogFactory;
import org.act.soap.util.logging.Log;

public class ClassLoaderUtils {
    
    private static Log log = LogFactory.getLog(ClassLoaderUtils.class.getName());
    
    private static FilenameFilter filter = new FilenameFilter() {
        public boolean accept(File dir, String filename) {
            File file = new File(dir, filename);
            if (file != null && file.isFile()) {
                String fn = file.getName();
                int len = fn.length();
                if (len > 4) {
                    String ext = fn.substring(len - 4, len);
                    return ext.equalsIgnoreCase(".jar") || ext.equalsIgnoreCase(".zip");
                }
            }
            return false;
        }
    };
    
    public static ClassLoader getWebServiceClassLoader(String workingPath,ClassLoader parent) {
        try {
            Vector classpathes = new Vector();
            File classesDir = new File(workingPath, "classes");
            if( classesDir.exists() && classesDir.isDirectory() )
                classpathes.add(classesDir.toURL());
            File libDir = new File(workingPath, "lib");
            if( libDir.exists() && libDir.isDirectory() ) {
                File[] files = libDir.listFiles(filter);
                for(int i=0; i<files.length; i++) {
                    classpathes.add( files[i].toURL() );
                }
            }
            URL[] urls = new URL[classpathes.size()];
            classpathes.toArray(urls);
            return new URLClassLoader(urls,parent);
        }
        catch(Exception ex){
            log.error( ex );
            return null;
        }
    }
    
    public static void printClassPath(URLClassLoader loader) {
        if(loader != null) {
            URL[] urls = loader.getURLs();
            for(int i=0; i<urls.length; i++)
                System.out.println(urls[i]);
        }
    }
}
