package org.act.soap.util;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.*;
import java.io.*;
import java.util.Iterator;

public class SOAPUtils {
    public SOAPUtils() {
    }
    
    public static MessageFactory MESSAGEFACTORY_INSTANCE = null;
    static {
        try {
            System.setProperty("javax.xml.soap.MessageFactory",
                               "org.act.soap.message.MessageFactoryImpl");
            MESSAGEFACTORY_INSTANCE = MessageFactory.newInstance();
        }
        catch (Exception ex) {}
    }
    public static SOAPMessage stringToSOAPMessage(String messageString) {
        InputStream input = new ByteArrayInputStream(messageString.getBytes());
        return stringToSOAPMessage(input);
    }
    public static SOAPMessage stringToSOAPMessage(InputStream input) {
        MimeHeaders headers = new MimeHeaders();
        headers.addHeader("content-type", "text/xml");
        try {
            return MESSAGEFACTORY_INSTANCE.createMessage(headers, input);
        } catch (Exception ex) {
            return null;
        }
    }
    
    private static String serializeStartTag(SOAPElement element) {
        String s = "";
        s += element.getElementName().getQualifiedName();
        Iterator namespaces = element.getNamespacePrefixes();
        for (; namespaces.hasNext(); ) {
            String namespace = (String)namespaces.next();
            if (namespace == null || namespace.equals("")) {
                s += " xmlns" + "=\"" + element.getNamespaceURI(namespace) + "\"";
            } else {
                s += " xmlns:" + namespace +
                    "=\"" + element.getNamespaceURI(namespace) + "\"";
            }
        }
        Iterator attributes = element.getAllAttributes();
        for (; attributes.hasNext(); ) {
            Name attribute = (Name)attributes.next();
            s += " " + attribute.getQualifiedName() + "=\"" +
                element.getAttributeValue(attribute) + "\"";
        }
        return s;
    }
        private static String serializeTree(SOAPElement element) {
        String s = "";
        Iterator childElements = element.getChildElements();
        if (!childElements.hasNext()) {
            s += "<" + serializeStartTag(element) + "/>";
        } else {
            s += "<" + serializeStartTag(element) + ">";
            for (; childElements.hasNext(); ) {
                Object o = childElements.next();
                if (o instanceof Text) {
                    s += ((Text)o).getData();
                } else {
                    s += serializeTree((SOAPElement)o);
                }
            }
            s += "</" + element.getElementName().getQualifiedName() + ">";
        }
        return s;
    }
    
    private static String serializePackedTree(SOAPElement element) {
        String s = "";
        Iterator childElements = element.getChildElements();
        if (!childElements.hasNext()) {
            s += "<" + serializeStartTag(element) + "/>";
        } else {
            s += "<" + serializeStartTag(element) + ">";
            for (; childElements.hasNext(); ) {
                Object o = childElements.next();
                if (o instanceof Text) {
                    s += ((Text)o).getData().trim();
                } else {
                    s += serializeTree((SOAPElement)o);
                }
            }
            s += "</" + element.getElementName().getQualifiedName() + ">";
        }
        return s;
    }
    public static String soapMessageToString(SOAPMessage message) {
        OutputStream output = new ByteArrayOutputStream();
        try {
            message.writeTo(output);
            return output.toString();
        } catch (Exception ex) {
            return null;
        }
    }
    public static String soapElementToPackedString(SOAPElement soapElement) {
        return serializePackedTree(soapElement);
    }
}
