package org.act.soap.util.logging;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

public abstract class LogFactory {
        public static final String FACTORY_PROPERTY =
        "org.act.soap.util.logging.LogFactory";
    public static final String FACTORY_DEFAULT =
        "org.act.soap.util.logging.impl.LogFactoryImpl";
    public static final String FACTORY_PROPERTIES =
        "commons-logging.properties";
    protected static final String SERVICE_ID =
        "META-INF/services/org.act.soap.util.logging.LogFactory";
        protected LogFactory() { }
        
    public abstract Object getAttribute(String name);
    
    public abstract String[] getAttributeNames();
    
    public abstract Log getInstance(Class clazz)
        throws LogConfigurationException;
    
    public abstract Log getInstance(String name)
        throws LogConfigurationException;
    
    public abstract void release();
    
    public abstract void removeAttribute(String name);
    
    public abstract void setAttribute(String name, Object value);
        
    protected static Hashtable factories = new Hashtable();
        
    public static LogFactory getFactory() throws LogConfigurationException {
                ClassLoader contextClassLoader =
            (ClassLoader)AccessController.doPrivileged(
                new PrivilegedAction() {
                    public Object run() {
                        return getContextClassLoader();
                    }
                });
                LogFactory factory = getCachedFactory(contextClassLoader);
        if (factory != null)
            return factory;
                        Properties props=null;
        try {
            InputStream stream = getResourceAsStream(contextClassLoader,
                                                     FACTORY_PROPERTIES);
            if (stream != null) {
                props = new Properties();
                props.load(stream);
                stream.close();
            }
        } catch (IOException e) {
        } catch (SecurityException e) {
        }
                try {
            String factoryClass = System.getProperty(FACTORY_PROPERTY);
            if (factoryClass != null) {
                factory = newFactory(factoryClass, contextClassLoader);
            }
        } catch (SecurityException e) {
            ;          }
                                                if (factory == null) {
            try {
                InputStream is = getResourceAsStream(contextClassLoader,
                                                     SERVICE_ID);
                if( is != null ) {
                                                            BufferedReader rd;
                    try {
                        rd = new BufferedReader(new InputStreamReader(is, "UTF-8"));
                    } catch (java.io.UnsupportedEncodingException e) {
                        rd = new BufferedReader(new InputStreamReader(is));
                    }
                    String factoryClassName = rd.readLine();
                    rd.close();
                    if (factoryClassName != null &&
                        ! "".equals(factoryClassName)) {
                        factory= newFactory( factoryClassName, contextClassLoader );
                    }
                }
            } catch( Exception ex ) {
                ;
            }
        }
                                                                if (factory == null  &&  props != null) {
            String factoryClass = props.getProperty(FACTORY_PROPERTY);
            if (factoryClass != null) {
                factory = newFactory(factoryClass, contextClassLoader);
            }
        }
                if (factory == null) {
            factory = newFactory(FACTORY_DEFAULT, LogFactory.class.getClassLoader());
        }
        if (factory != null) {
            
            cacheFactory(contextClassLoader, factory);
            if( props!=null ) {
                Enumeration names = props.propertyNames();
                while (names.hasMoreElements()) {
                    String name = (String) names.nextElement();
                    String value = props.getProperty(name);
                    factory.setAttribute(name, value);
                }
            }
        }
        return factory;
    }
    
    public static Log getLog(Class clazz)
        throws LogConfigurationException {
        return (getFactory().getInstance(clazz));
    }
    
    public static Log getLog(String name)
        throws LogConfigurationException {
        return (getFactory().getInstance(name));
    }
    
    public static void release(ClassLoader classLoader) {
        synchronized (factories) {
            LogFactory factory = (LogFactory) factories.get(classLoader);
            if (factory != null) {
                factory.release();
                factories.remove(classLoader);
            }
        }
    }
    
    public static void releaseAll() {
        synchronized (factories) {
            Enumeration elements = factories.elements();
            while (elements.hasMoreElements()) {
                LogFactory element = (LogFactory) elements.nextElement();
                element.release();
            }
            factories.clear();
        }
    }
        
    protected static ClassLoader getContextClassLoader()
        throws LogConfigurationException
    {
        ClassLoader classLoader = null;
        try {
                        Method method = Thread.class.getMethod("getContextClassLoader", null);
                        try {
                classLoader = (ClassLoader)method.invoke(Thread.currentThread(), null);
            } catch (IllegalAccessException e) {
                throw new LogConfigurationException
                    ("Unexpected IllegalAccessException", e);
            } catch (InvocationTargetException e) {
                
                if (e.getTargetException() instanceof SecurityException) {
                    ;                  } else {
                                                            throw new LogConfigurationException
                        ("Unexpected InvocationTargetException", e.getTargetException());
                }
            }
        } catch (NoSuchMethodException e) {
                        classLoader = LogFactory.class.getClassLoader();
        }
                return classLoader;
    }
    
    private static LogFactory getCachedFactory(ClassLoader contextClassLoader)
    {
        LogFactory factory = null;
        if (contextClassLoader != null)
            factory = (LogFactory) factories.get(contextClassLoader);
        return factory;
    }
    private static void cacheFactory(ClassLoader classLoader, LogFactory factory)
    {
        if (classLoader != null && factory != null)
            factories.put(classLoader, factory);
    }
    
    protected static LogFactory newFactory(final String factoryClass,
                                           final ClassLoader classLoader)
        throws LogConfigurationException
    {
        Object result = AccessController.doPrivileged(
            new PrivilegedAction() {
                public Object run() {
                    try {
                        if (classLoader != null) {
                            try {
                                                                                                                                return (LogFactory)classLoader.loadClass(factoryClass).newInstance();
                            } catch (ClassNotFoundException ex) {
                                if (classLoader == LogFactory.class.getClassLoader()) {
                                                                        throw ex;
                                }
                                                            } catch (NoClassDefFoundError e) {
                                if (classLoader == LogFactory.class.getClassLoader()) {
                                                                        throw e;
                                }
                            }catch(ClassCastException e){
                              if (classLoader == LogFactory.class.getClassLoader()) {
                                                                        throw e;
                               }
                            }
                                                    }
                        
                                                                        return (LogFactory)Class.forName(factoryClass).newInstance();
                    } catch (Exception e) {
                        return new LogConfigurationException(e);
                    }
                }
            });
        if (result instanceof LogConfigurationException)
            throw (LogConfigurationException)result;
        return (LogFactory)result;
    }
    private static InputStream getResourceAsStream(final ClassLoader loader,
                                                   final String name)
    {
        return (InputStream)AccessController.doPrivileged(
            new PrivilegedAction() {
                public Object run() {
                    if (loader != null) {
                        return loader.getResourceAsStream(name);
                    } else {
                        return ClassLoader.getSystemResourceAsStream(name);
                    }
                }
            });
    }
}
