package org.act.soap.util.logging.impl;import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.act.soap.util.logging.Log;
import org.act.soap.util.logging.LogConfigurationException;
import org.act.soap.util.logging.LogFactory;

public class LogFactoryImpl extends LogFactory {
        
    public LogFactoryImpl() {
        super();
    }
        
    public static final String LOG_PROPERTY =
        "org.act.soap.util.logging.Log";
    
    protected static final String LOG_PROPERTY_OLD =
        "org.act.soap.util.logging.log";
        
    protected Hashtable attributes = new Hashtable();
    
    protected Hashtable instances = new Hashtable();
    
    private String logClassName;
    
    protected Constructor logConstructor = null;
    
    protected Class logConstructorSignature[] =
    { java.lang.String.class };
    
    protected Method logMethod = null;
    
    protected Class logMethodSignature[] =
    { LogFactory.class };
        
    public Object getAttribute(String name) {
        return (attributes.get(name));
    }
    
    public String[] getAttributeNames() {
        Vector names = new Vector();
        Enumeration keys = attributes.keys();
        while (keys.hasMoreElements()) {
            names.addElement((String) keys.nextElement());
        }
        String results[] = new String[names.size()];
        for (int i = 0; i < results.length; i++) {
            results[i] = (String) names.elementAt(i);
        }
        return (results);
    }
    
    public Log getInstance(Class clazz) throws LogConfigurationException {
        return (getInstance(clazz.getName()));
    }
    
    public Log getInstance(String name) throws LogConfigurationException {
        Log instance = (Log) instances.get(name);
        if (instance == null) {
            instance = newInstance(name);
            instances.put(name, instance);
        }
        return (instance);
    }
    
    public void release() {
        instances.clear();
    }
    
    public void removeAttribute(String name) {
        attributes.remove(name);
    }
    
    public void setAttribute(String name, Object value) {
        if (value == null) {
            attributes.remove(name);
        } else {
            attributes.put(name, value);
        }
    }
        
    protected String getLogClassName() {
                if (logClassName != null) {
            return logClassName;
        }
        logClassName = (String) getAttribute(LOG_PROPERTY);
        if (logClassName == null) {             logClassName = (String) getAttribute(LOG_PROPERTY_OLD);
        }
        if (logClassName == null) {
            try {
                logClassName = System.getProperty(LOG_PROPERTY);
            } catch (SecurityException e) {
                ;
            }
        }
        if (logClassName == null) {             try {
                logClassName = System.getProperty(LOG_PROPERTY_OLD);
            } catch (SecurityException e) {
                ;
            }
        }
        
        if ((logClassName == null) && isJdk14Available()) {
            logClassName = "org.act.soap.util.logging.impl.Jdk14Logger";
        }
        if (logClassName == null) {
            logClassName = "org.act.soap.util.logging.impl.SimpleLog";
        }
        return (logClassName);
    }
    
    protected Constructor getLogConstructor()
        throws LogConfigurationException {
                if (logConstructor != null) {
            return logConstructor;
        }
        String logClassName = getLogClassName();
                Class logClass = null;
        try {
            logClass = loadClass(logClassName);
            if (logClass == null) {
                throw new LogConfigurationException
                    ("No suitable Log implementation for " + logClassName);
            }
            if (!Log.class.isAssignableFrom(logClass)) {
                throw new LogConfigurationException
                    ("Class " + logClassName + " does not implement Log");
            }
        } catch (Throwable t) {
            throw new LogConfigurationException(t);
        }
                try {
            logMethod = logClass.getMethod("setLogFactory",
                                           logMethodSignature);
        } catch (Throwable t) {
            logMethod = null;
        }
                try {
            logConstructor = logClass.getConstructor(logConstructorSignature);
            return (logConstructor);
        } catch (Throwable t) {
            throw new LogConfigurationException
                ("No suitable Log constructor " +
                 logConstructorSignature+ " for " + logClassName, t);
        }
    }
    
    private static Class loadClass( final String name )
        throws ClassNotFoundException
    {
        Object result = AccessController.doPrivileged(
            new PrivilegedAction() {
                public Object run() {
                    ClassLoader threadCL = getContextClassLoader();
                    if (threadCL != null) {
                        try {
                            return threadCL.loadClass(name);
                        } catch( ClassNotFoundException ex ) {
                                                    }
                    }
                    try {
                        return Class.forName( name );
                    } catch (ClassNotFoundException e) {
                        return e;
                    }
                }
            });
        if (result instanceof Class)
            return (Class)result;
        throw (ClassNotFoundException)result;
    }
    
    protected boolean isJdk14Available() {
        try {
            loadClass("java.util.logging.Logger");
            loadClass("org.act.soap.util.logging.impl.Jdk14Logger");
            return (true);
        } catch (Throwable t) {
            return (false);
        }
    }
    
    protected boolean isLog4JAvailable() {
        try {
            loadClass("org.apache.log4j.Logger");
            loadClass("org.act.soap.util.logging.impl.Log4JLogger");
            return (true);
        } catch (Throwable t) {
            return (false);
        }
    }
    
    protected Log newInstance(String name) throws LogConfigurationException {
        Log instance = null;
        try {
            Object params[] = new Object[1];
            params[0] = name;
            instance = (Log) getLogConstructor().newInstance(params);
            if (logMethod != null) {
                params[0] = this;
                logMethod.invoke(instance, params);
            }
            return (instance);
        } catch (Throwable t) {
            throw new LogConfigurationException(t);
        }
    }
}
