package org.act.soap.util.wsar;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.*;
import java.net.URL;
import java.net.JarURLConnection;
import java.util.Iterator;
import java.util.Map;
import java.util.zip.*;
import javax.swing.UIManager;
import javax.wsdl.Definition;
import javax.wsdl.factory.*;
import javax.wsdl.xml.WSDLWriter;
import javax.xml.namespace.QName;
import org.w3c.dom.*;
import org.act.soap.util.*;
import org.act.soap.util.i18n.Message;
import org.act.soap.wsdd.*;
import org.act.soap.wsdl.java2wsdl.*;

public class Java2WSAR {
    
    private String javaClassName = null;
    
    private String wsarPath = null;
    
    private String serviceName = null;
    
    private String targetNamespace;
    
    private String locationURI;
    
    private Definition wsdlDefinition = null;
    
    private Deployment wsddDeployment = null;
    
    private Types types = null;
    
    public Java2WSAR() {
    }
    
    public static void main(String[] args) {
        if ( args != null && args.length == 2 ) {
            Java2WSAR java2WSAR = new Java2WSAR();
            try {
                java2WSAR.run( args );
            }
            catch ( Exception ex ) {
                System.err.println( ex.getMessage() );
                usage();
            }
        }
        else {
                        try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                Java2WSARFrame frame = new Java2WSARFrame();
                frame.validate();
                frame.setSize( 400, 400 );
                                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                Dimension frameSize = frame.getSize();
                if (frameSize.height > screenSize.height) {
                    frameSize.height = screenSize.height;
                }
                if (frameSize.width > screenSize.width) {
                    frameSize.width = screenSize.width;
                }
                frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
                frame.setVisible(true);
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    public void run (String[] args) throws Exception {
                parseArgs( args );
                generateWSDL();
                generateWSDD();
                generateWSAR();
    }
    
    private void parseArgs ( String[] args ) throws Exception {
        if ( args == null || args.length != 2 ) {
            throw new java.lang.IllegalArgumentException( Message.getMessage( "JAVA_TO_WSAR_IllegalArgument" ) );
        }
        this.javaClassName = args[0];
        File fileWSAR = new File( args[1] );
        this.wsarPath = fileWSAR.getParent();
        String name = fileWSAR.getName();
        int pos = name.lastIndexOf('.');
        if ( pos > 0 ) {
            this.serviceName = name.substring( 0, pos );
        }
        else {
            this.serviceName = name;
        }
        if ( serviceName == null || serviceName.length() == 0 ) {
            throw new java.lang.IllegalArgumentException( Message.getMessage( "JAVA_TO_WSAR_NO_SERVICE_NAME" ) );
        }
        this.targetNamespace = createNamespaceURI( this.javaClassName );
        this.locationURI = "http://localhost:8080/ACTSoap/" + this.serviceName;
        System.out.println( "=========================================" );
        System.out.println( "" );
        System.out.println( "javaClassName=" + this.javaClassName );
        System.out.println( "wsarPath=" + this.wsarPath );
        System.out.println( "serviceName=" + this.serviceName );
        System.out.println( "targetNamespace=" + this.targetNamespace );
        System.out.println( "locationURI=" + this.locationURI );
        System.out.println( "=========================================" );
    }
    
    private void generateWSDL () throws Exception {
        Emitter emitter = new Emitter();
        emitter.setStyle( "rpc" );
        emitter.setServiceClassName( this.javaClassName );
        emitter.setTargetNamespace( this.targetNamespace );
        emitter.setLocationURI( this.locationURI );
        this.wsdlDefinition = emitter.emit();
        types = emitter.getTypes();
    }
    
    private void generateWSDD () throws Exception {
        this.wsddDeployment = new Deployment();
                this.wsddDeployment.setServiceName( this.serviceName );
        this.wsddDeployment.setWSDLLocation( this.serviceName + ".wsdl" );
                Transport transport = new Transport();
        transport.setType( "http" );
        transport.addParameter( "URLPattern", "/" + this.serviceName );
        Transports transports = new Transports();
        transports.addTransport( transport );
        this.wsddDeployment.setTransports( transports );
                Provider provider = new Provider();
        provider.setType( "java" );
        provider.addParameter( "ClassName", this.javaClassName );
        this.wsddDeployment.setProvider( provider );
                if ( this.types != null ) {
            Map schemaElements = types.getAllSchemaElements();
            org.act.soap.encoding.TypeMappingImpl typemapping =
                (org.act.soap.encoding.TypeMappingImpl)types.getTypeMapping();
            TypeMappings tms = new TypeMappings();
            for ( Iterator i = schemaElements.keySet().iterator(); i.hasNext(); ) {
                QName xmlType = (QName) i.next();
                                if ( ! xmlType.getNamespaceURI().equals( TMConstants.URI_ACT_SOAP ) ) {
                    Class javaType = typemapping.getJavaType( xmlType );
                    if ( javaType != null ) {
                        Class serClass = typemapping.getSerializer( javaType, xmlType ).getClass();
                        Class dserClass = typemapping.getDeserializer( javaType, xmlType ).getClass();
                        TypeMapping tm = new TypeMapping();
                        tm.setSerializer( serClass.getName() );
                        tm.setDeserializer( dserClass.getName() );
                        tm.setName( javaType.getName() );
                        tm.setQName( xmlType );
                        tm.setEncodingStyle( TMConstants.DEFAULT_ENCODINGSTYLE );
                        tm.setLanguageSpecificType( javaType.getName() );
                        tms.addTypeMapping( tm );
                    }
                }
            }
            if ( tms.getAllTypeMappings().size() > 0 ) {
                this.wsddDeployment.setTypeMappings( tms );
            }
        }
    }
    
    private String createNamespaceURI (String className) {
                                if( null == className) {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        sb.append("http://");
        int pos = className.lastIndexOf('.');
        if ( pos > 0 ) {
                        int pos1 = className.lastIndexOf('.', pos - 1);
            sb.append( className.substring(pos1 + 1, pos) );
            pos = pos1;
        }
        while ( pos > 0 ) {
            int pos1 = className.lastIndexOf('.', pos - 1);
            sb.append( "." );
            sb.append( className.substring(pos1 + 1, pos) );
            pos = pos1;
        }
        return sb.toString();
    }
    
    private void generateWSAR () throws Exception {
                File wsarFile = new File( this.wsarPath + "/" + this.serviceName + ".wsar" );
        FileOutputStream fout = new FileOutputStream(wsarFile);
        ZipOutputStream zos = new ZipOutputStream(fout);
                copyClasses( zos );
        ZipEntry zeMetaInf = new ZipEntry("META-INF/");
        zos.putNextEntry(zeMetaInf);
                ZipEntry zeServiceXml = new ZipEntry("META-INF/service.xml");
        zos.putNextEntry( zeServiceXml );
        Document wsddDoc = XMLUtils.createDocument();
        Element elmWSDD = this.wsddDeployment.toElement( wsddDoc );
        wsddDoc.appendChild( elmWSDD );
        String wsddStr = XMLUtils.nodeToPrettyString( wsddDoc );
        zos.write( wsddStr.getBytes() );
                ZipEntry zeWSDL = new ZipEntry("META-INF/" + serviceName + ".wsdl");
        zos.putNextEntry( zeWSDL );
        WSDLFactory factory = WSDLFactory.newInstance();
        WSDLWriter writer = factory.newWSDLWriter();
        writer.writeWSDL( this.wsdlDefinition, zos );
                zos.close();
    }
    
    private void copyClasses (ZipOutputStream zos) throws Exception {
        String classLocation = lookupClass( this.javaClassName );
        if( classLocation != null && classLocation.length() > 0 ) {
            File fileToBeCopied = new File( classLocation );
            if ( fileToBeCopied.isDirectory() ) {
                                
                String classFileName = this.javaClassName.replace( '.', '/' ) + ".class" ;
                fileToBeCopied = new File( fileToBeCopied, classFileName);
                ZipEntry zeClasses = new ZipEntry("classes/");
                zos.putNextEntry( zeClasses );
                                int pos = classFileName.indexOf( '/' );
                while ( pos > 0 ) {
                    String classPath = classFileName.substring( 0, pos + 1 );
                    ZipEntry zeClassPath = new ZipEntry("classes/" + classPath );
                    zos.putNextEntry( zeClassPath );
                    pos = classFileName.indexOf( '/', pos + 1 );
                }
                ZipEntry zeClassFile = new ZipEntry("classes/" + classFileName );
                zos.putNextEntry( zeClassFile );
            }
            else {
                String libFileName = fileToBeCopied.getName();
                ZipEntry zeLib = new ZipEntry("lib/" );
                zos.putNextEntry( zeLib );
                ZipEntry zeLibFile = new ZipEntry("lib/" + libFileName );
                zos.putNextEntry( zeLibFile );
            }
            System.out.println( "Copying " + fileToBeCopied.getPath() );
            FileInputStream fis = new FileInputStream( fileToBeCopied );
            FileUtil.copy( fis, zos, true, false );
        }
    }
    
    private String lookupClass(String className) throws IOException {
                Class clazz = null;
        try {
            ClassLoader cl = Thread.currentThread().getContextClassLoader();
            clazz = cl.loadClass(className);
            if (clazz == null)
                return null;
        }
        catch (ClassNotFoundException e) {
            return null;
        }
                URL url = null;
        try {
            url = clazz.getProtectionDomain().getCodeSource().getLocation();
            if (url == null)
                return "";
        }
        catch(Throwable t) {
            return "";
        }
                String location = getLocation(url);
        if (location == null)
            return "";
        else
            return location;
    }
    
    private String getLocation(URL url) {
        try {
            String location = url.toString();
            if ( location.startsWith( "jar:file:/" ) ) {
                File file = new File( url.getFile() );
                return file.getPath().substring( 6 );
            }
            else if ( location.startsWith( "jar" ) ) {
                url = ( ( JarURLConnection ) url.openConnection() ).getJarFileURL();
                return url.toString();
            }
            else if ( location.startsWith( "file" ) ) {
                File file = new File( url.getFile() );
                return file.getAbsolutePath();
            }
            else {
                return url.toString();
            }
        }
        catch ( Throwable t ) {
            return null;
        }
    }
    
    private static void usage () {
        System.out.println( Message.getMessage( "JAVA_TO_WSAR_USAGE" ) );
    }
}
