package org.act.soap.wsdd;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.act.soap.util.i18n.Message;

public class Deployment
    extends WSDDElement {
    
    public static final String TAG_NAME = "Deployment";
    
    public static final String ATTR_SERVICENAME = "ServiceName";
    
    public static final String ATTR_WSDLLOCATION = "WSDLLocation";
    
    private Transports transports = null;
    
    private Handlers handlers = null;
    
    private RBAC rbac = null;
    
    private Provider provider = null;
    
    private TypeMappings typeMappings = null;
    
    private String serviceName = null;
    
    private String wsdlLocation = null;
    
    String name;
    
    public Deployment () {
    }
    
    public Deployment (Element eDeployment) throws WSDDException {
        
        if (!isValidElement(eDeployment, TAG_NAME)) {
            throw new WSDDException(Message.getMessage("wrongWSDDElement",
            TAG_NAME));
        }
        
        this.serviceName = eDeployment.getAttribute(ATTR_SERVICENAME);
        this.wsdlLocation = eDeployment.getAttribute(ATTR_WSDLLOCATION);
        Node childNode = null;
        NodeList nodeList = eDeployment.getChildNodes();
        final int sum = nodeList.getLength();
        for (int i = 0; i < sum; i++) {
            childNode = nodeList.item(i);
            if (isValidElement(childNode, Handlers.TAG_NAME)) {
                this.handlers = new Handlers( (Element) childNode);
                continue;
            }
            if (isValidElement(childNode, RBAC.TAG_NAME)) {
                this.rbac = new RBAC( (Element) childNode);
                continue;
            }
            if (isValidElement(childNode, Transports.TAG_NAME)) {
                this.transports = new Transports( (Element) childNode);
                continue;
            }
            if (isValidElement(childNode, TypeMappings.TAG_NAME)) {
                this.typeMappings = new TypeMappings( (Element) childNode);
                continue;
            }
            if (isValidElement(childNode, Provider.TAG_NAME)) {
                this.provider = new Provider( (Element) childNode);
                continue;
            }
            if (isValidElement(childNode, Documentation.TAG_NAME)) {
                documentation = new Documentation( (Element) childNode);
                continue;
            }
        }
        return;
    }
    
    public void setServiceName (String newServiceName) {
        this.serviceName = newServiceName;
    }
    
    public String getServiceName () {
        return this.serviceName;
    }
    
    public String getName() {
        return this.name;
    }
    
    public void setName(String Name) {
        this.name = Name;
    }
    
    public void setWSDLLocation (String newWSDLLocation) {
        this.wsdlLocation = newWSDLLocation;
    }
    
    public String getWSDLLocation () {
        return this.wsdlLocation;
    }
    
    public void setHandlers (Handlers newHandlers) {
        this.handlers = newHandlers;
    }
    
    public Handlers getHandlers () {
        return this.handlers;
    }
    
    public void setTransports (Transports newTransports) {
        this.transports = newTransports;
    }
    
    public Transports getTransports () {
        return this.transports;
    }
    
    public void setRBAC (RBAC newRBAC) {
        this.rbac = newRBAC;
    }
    
    public void setProvider (Provider newProvider) {
        this.provider = newProvider;
    }
    
    public Provider getProvider () {
        return this.provider;
    }
    
    public RBAC getRBAC () {
        return this.rbac;
    }
    
    public void setTypeMappings (TypeMappings newTypeMappings) {
        this.typeMappings = newTypeMappings;
    }
    
    public TypeMappings getTypeMappings () {
        return this.typeMappings;
    }
    
    public Element toElement (Document doc) {
        Element result = null;
        if (null != doc) {
            result = doc.createElementNS(WSDD_URI, TAG_NAME);
            
            if (null != this.documentation) {
                result.appendChild(this.documentation.toElement(doc));
            }
            
            if (null != this.transports) {
                result.appendChild(this.transports.toElement(doc));
            }
            if (null != this.handlers) {
                result.appendChild(this.handlers.toElement(doc));
            }
            if (null != this.rbac) {
                result.appendChild(this.rbac.toElement(doc));
            }
            if (null != this.provider) {
                result.appendChild(this.provider.toElement(doc));
            }
            if (null != this.typeMappings) {
                result.appendChild(this.typeMappings.toElement(doc));
            }
            
            if (null != this.serviceName) {
                result.setAttributeNS(doc.getNamespaceURI(),
                                ATTR_SERVICENAME, serviceName);
            }
            if (null != this.wsdlLocation) {
                result.setAttributeNS(doc.getNamespaceURI(),
                                ATTR_WSDLLOCATION, wsdlLocation);
            }
        }
        result.setAttribute("xmlns", WSDD_URI);
        return result;
    }
}
