
package org.act.soap.wsdd;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.act.soap.util.i18n.Message;

public class Provider
    extends WSDDElement {
    
    public static final String TAG_NAME = "Provider";
    
    public static final String ATTR_TYPE = "Type";
    
    private Vector parametersVector = new Vector();
    
    private String type = null;
    
    public Provider () {
    }
    
    public Provider (Element eProvider) throws WSDDException {
        if (!isValidElement(eProvider, TAG_NAME)) {
            throw new WSDDException(Message.getMessage("invalidWSDDElement"));
        }
        type = eProvider.getAttribute(ATTR_TYPE);        NodeList nodeList = eProvider.getChildNodes();
        final int sum = nodeList.getLength();
        this.parametersVector = new Vector();
        for (int i = 0; i < sum; i++) {
            Node childNode = nodeList.item(i);
            if (isValidElement(childNode, Parameter.TAG_NAME)) {
                              parametersVector.add(new Parameter( (Element) childNode));
                continue;
            }
            if (isValidElement(childNode, Documentation.TAG_NAME)) {
                documentation = new Documentation( (Element) childNode);
                continue;
            }
        }
    }
    
    public void setType (String newType) {
        this.type = newType;
    }
    
    public String getType () {
        return this.type;
    }
    
    public void addParameter (String newName, String newValue) throws
        WSDDException {
        
        if (null == newName) {
            throw new WSDDException(Message.getMessage("invalidWSDDElement"));
        }
        
        Parameter newParameter = new Parameter();
        
        newParameter.setName(newName);
        newParameter.setValue(newValue);
        
        parametersVector.add(newParameter);
    }
    
    public Parameter getParameter (String tagName) {
        Parameter newParameter = null;
        
        final int sum = parametersVector.size();
        for (int i = 0; i < sum; i++) {
            if (((Parameter) parametersVector.get(i)).getName().equals(tagName)){
                newParameter = (Parameter) parametersVector.get(i);
                break;
            }
        }
        
        return newParameter;
    }
    
    public void removeParameter (String tagName) {
        if (null == this.parametersVector) {
            return;
        }
        Parameter newParameter = new Parameter();
        for (int i = (parametersVector.size() - 1); 0 <= i; i--) {
            newParameter = (Parameter) parametersVector.get(i);
            if (newParameter.getName().equals(tagName)) {
                parametersVector.remove(i);
            }
        }
    }
    
    public Element toElement (Document doc) {
        Element result = null;
        if (null != doc) {
            result = doc.createElementNS(WSDD_URI, TAG_NAME);
            if (null != this.documentation) {
                result.appendChild(this.documentation.toElement(doc));
            }
            if (null != parametersVector) {
                for (int i = 0; i < parametersVector.size(); i++) {
                    Element eParameter =
                                ( (Parameter) this.parametersVector.
                                    get(i)).toElement(doc);
                    result.appendChild(eParameter);
                }
            }
            result.setAttributeNS(WSDD_URI, ATTR_TYPE, this.type);
        }
        return result;
    }
}
