
package org.act.soap.wsdd;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.act.soap.util.i18n.Message;

public class Transports
    extends WSDDElement {
    
    public static final String TAG_NAME = "Transports";
    
    private Vector transportsVector = new Vector();
    
    public Transports () {
    }
    
    public Transports (Element eTransports) throws WSDDException {
        
        if (!isValidElement(eTransports, TAG_NAME)) {
            throw new WSDDException(Message.getMessage("invalidWSDDElement"));
        }
        
        NodeList nodeList = eTransports.getChildNodes();
        final int sum = nodeList.getLength();
        
        transportsVector = new Vector();
        Transport transport = null;
        
        for (int i = 0; i < sum; i++) {
            Node childNode = nodeList.item(i);
            
            if (isValidElement(childNode, Transport.TAG_NAME)) {
                transport = new Transport( (Element) childNode);
                transportsVector.add(transport);
                continue;
            }
            
            if (isValidElement(childNode, Documentation.TAG_NAME)) {
                documentation = new Documentation( (Element) childNode);
                continue;
            }
        }
    }
    
    public void addTransport (Transport newTransport) {
        if (null == this.transportsVector) {
            this.transportsVector = new Vector();
        }
        if (null != newTransport) {
            this.transportsVector.add(newTransport);
        }
    }
    
    public void removeTransport (String type) {
        if (null == this.transportsVector) {
            return;
        }
        Transport transport = new Transport();
        for (int i = (this.transportsVector.size() - 1); 0 <= i; i--) {
            transport = (Transport) transportsVector.get(i);
            if (transport.getType().equals(type)) {
                transportsVector.remove(i);
            }
        }
    }
    
    public Vector getAllTransport () {
        return this.transportsVector;
    }
    
    public Element toElement (Document doc) {
        Element result = null;
        if (null != doc) {
            result = doc.createElementNS(WSDD_URI, TAG_NAME);
            
            if (null != this.documentation) {
                result.appendChild(this.documentation.toElement(doc));
            }
            
            if (null != transportsVector) {
                for (int i = 0; i < transportsVector.size(); i++) {
                    Transport tp = (Transport) transportsVector.get(i);
                    Element eTransport = tp.toElement(doc);
                    result.appendChild(eTransport);
                }
            }
        }
        return result;
    }
}
