
package org.act.soap.wsdd;
import javax.xml.namespace.QName;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.act.soap.util.i18n.Message;

public class TypeMapping
    extends WSDDElement {
    
    public static final String TAG_NAME = "TypeMapping";
    
    public static final String ATTR_NAME = "Name";
    
    public static final String ATTR_ENCODING_STYLE = "EncodingStyle";
    
    public static final String ATTR_QNAME = "QName";
    
    public static final String ATTR_LANGUAGE_SPECIFIC_TYPE =
        "LanguageSpecificType";
    
    public static final String ATTR_SERIALIZER = "Serializer";
    
    public static final String ATTR_DESERIALIZER = "Deserializer";
    
    public static final String QNAME_NAMESPACEURI =
        "http://www.w3.org/2000/xmlns/";
    
    public static final String QNAME_NS_PREFIX = "xmlns";
    
    private String name = null;
    
    private String encodingStyle = null;
    
    private QName qName = null;
    
    private String languageSpecificType = null;
    
    private String serializer = null;
    
    private String deserializer = null;
    
    public TypeMapping () {
    }
    
    public TypeMapping (Element eTypeMapping) throws WSDDException {
        if (!isValidElement(eTypeMapping, TAG_NAME)) {
            throw new WSDDException(Message.getMessage("invalidWSDDElement"));
        }
        this.name = eTypeMapping.getAttribute(ATTR_NAME);
        this.languageSpecificType = eTypeMapping.getAttribute(
            ATTR_LANGUAGE_SPECIFIC_TYPE);
        this.serializer = eTypeMapping.getAttribute(ATTR_SERIALIZER);
        this.deserializer = eTypeMapping.getAttribute(ATTR_DESERIALIZER);
        this.encodingStyle = eTypeMapping.getAttribute(ATTR_ENCODING_STYLE);
        
        String qNameTag = eTypeMapping.getAttribute(ATTR_QNAME);
        String qNamePreFix = null;
        String qNameLocalPart = null;
        String qNameURI = null;
        
        if ( -1 != qNameTag.indexOf(':')) {
            qNamePreFix = qNameTag.substring(0, qNameTag.indexOf(':'));
            qNameLocalPart = qNameTag.substring(qNameTag.indexOf(':') + 1);
            String qNameURITag = QNAME_NS_PREFIX + ":" + qNamePreFix;
            qNameURI = eTypeMapping.getAttribute(qNameURITag);
        }
        else {
            qNamePreFix = "";
            qNameLocalPart = qNameTag;
            String qNameURITag = QNAME_NS_PREFIX;
            qNameURI = eTypeMapping.getAttribute(qNameURITag);
        }
        this.qName = new QName(qNameURI, qNameLocalPart, qNamePreFix);
        
        NodeList nodeList = eTypeMapping.getChildNodes();
        final int sum = nodeList.getLength();
        for (int i = 0; i < sum; i++) {
            Node childNode = nodeList.item(i);
            if (isValidElement(childNode, Documentation.TAG_NAME)) {
                documentation = new Documentation( (Element) childNode);
            }
        }
    }
    
    public void setName (String newName) {
        this.name = newName;
    }
    
    public String getName () {
        return this.name;
    }
    
    public void setEncodingStyle (String newEncodingStyle) {
        this.encodingStyle = newEncodingStyle;
    }
    
    public String getEncodingStyle () {
        return this.encodingStyle;
    }
    
    public void setLanguageSpecificType (String newLanguageSpecificType) {
        this.languageSpecificType = newLanguageSpecificType;
    }
    
    public String getLanguageSpecificType () {
        return this.languageSpecificType;
    }
    
    public void setSerializer (String newSerializer) {
        this.serializer = newSerializer;
    }
    
    public String getSerializer () {
        return this.serializer;
    }
    
    public void setDeserializer (String newDeserializer) {
        this.deserializer = newDeserializer;
    }
    
    public String getDeserializer () {
        return this.deserializer;
    }
    
    public void setQName (QName newQName) {
        this.qName = newQName;
    }
    
    public QName getQName () {
        return this.qName;
    }
    
    public Element toElement (Document doc) {
        Element result = null;
        if (null != doc) {
            result = doc.createElementNS(WSDD_URI, TAG_NAME);
            
            result.setAttributeNS(WSDD_URI, ATTR_DESERIALIZER,
                                    this.deserializer);
            
            result.setAttributeNS(WSDD_URI, ATTR_ENCODING_STYLE,
                                    this.encodingStyle);
            
            result.setAttributeNS(WSDD_URI,
                                    ATTR_LANGUAGE_SPECIFIC_TYPE,
                                    this.languageSpecificType);
            
            result.setAttributeNS(WSDD_URI, ATTR_NAME, this.name);
            
            result.setAttributeNS(WSDD_URI, ATTR_SERIALIZER,
                                    this.serializer);
            
            if (null != this.qName) {
                if (qName.getPrefix().equals("")) {
                    
                    result.setAttributeNS(WSDD_URI,
                                        ATTR_QNAME,
                                        this.qName.getPrefix() +
                                        "" +
                                        this.qName.
                                        getLocalPart());
                    result.setAttributeNS(QNAME_NAMESPACEURI,
                                        QNAME_NS_PREFIX +
                                        this.qName.
                                        getPrefix(),
                                        this.qName.
                                        getNamespaceURI());
                }
                else {
                    
                    result.setAttributeNS(WSDD_URI, ATTR_QNAME,
                                        this.qName.getPrefix() +
                                        ":" +
                                        this.qName.
                                        getLocalPart());
                    result.setAttributeNS(QNAME_NAMESPACEURI,
                                        QNAME_NS_PREFIX +
                                        ":" +
                                        this.qName.
                                        getPrefix(),
                                        this.qName.getNamespaceURI());
                }
            }
        }
        return result;
    }
}
