
package org.act.soap.wsdd;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.act.soap.util.i18n.Message;

public class TypeMappings
    extends WSDDElement {
    
    public static final String TAG_NAME = "TypeMappings";
    
    private Vector typeMappingsVector = new Vector();
    
    public TypeMappings () {
    }
    
    public TypeMappings (Element eTypeMappings) throws WSDDException {
        
        if (!isValidElement(eTypeMappings, TAG_NAME)) {
            throw new WSDDException(Message.getMessage("invalidWSDDElement"));
        }
        typeMappingsVector = new Vector();
        NodeList nodeList = eTypeMappings.getChildNodes();
        final int sum = nodeList.getLength();
        
        for (int i = 0; i < sum; i++) {
            Node childNode = nodeList.item(i);
            
            if (isValidElement(childNode, TypeMapping.TAG_NAME)) {
                typeMappingsVector.add(new TypeMapping( (Element) childNode));
                continue;
            }
            
            if (isValidElement(childNode, Documentation.TAG_NAME)) {
                documentation = new Documentation( (Element) childNode);
                continue;
            }
        }
    }
    public void addTypeMapping (TypeMapping newTypeMapping) {
        if (null == this.typeMappingsVector) {
            this.typeMappingsVector = new Vector();
        }
        if (null != newTypeMapping) {
            this.typeMappingsVector.add(newTypeMapping);
        }
    }
    
    public void removeTypeMapping (String name) {
        if (null == this.typeMappingsVector) {
            return;
        }
        TypeMapping typeMapping = new TypeMapping();
        
        for (int i = (typeMappingsVector.size() - 1); 0 <= i; i--) {
            typeMapping = (TypeMapping) typeMappingsVector.get(i);
            if (typeMapping.getName().equals(name)) {
                typeMappingsVector.remove(i);
            }
        }
    }
    
    public Vector getAllTypeMappings () {
        return this.typeMappingsVector;
    }
    
    public void removeAllTypeMappings () {
        if (null == this.typeMappingsVector) {
            return;
        }
        this.typeMappingsVector.clear();
    }
    
    public Element toElement (Document doc) {
        Element result = null;
        if (null != doc) {
            result = doc.createElementNS(WSDD_URI, TAG_NAME);
            
            if (null != this.documentation) {
                result.appendChild(this.documentation.toElement(doc));
            }
            
            if (null != typeMappingsVector) {
                for (int i = 0; i < typeMappingsVector.size(); i++) {
                    Element eTypeMapping = ( (TypeMapping) this.
                                            typeMappingsVector.
                                            get(i)).toElement(doc);
                    result.appendChild(eTypeMapping);
                }
            }
        }
        return result;
    }
}
