package org.act.soap.wsdl.java2wsdl;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.wsdl.Binding;
import javax.wsdl.BindingFault;
import javax.wsdl.BindingInput;
import javax.wsdl.BindingOperation;
import javax.wsdl.BindingOutput;
import javax.wsdl.Definition;
import javax.wsdl.Fault;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.Output;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.UnknownExtensibilityElement;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.extensions.soap.SOAPBinding;
import javax.wsdl.extensions.soap.SOAPBody;
import javax.wsdl.extensions.soap.SOAPFault;
import javax.wsdl.extensions.soap.SOAPOperation;
import javax.wsdl.factory.WSDLFactory;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.TypeMapping;
import com.ibm.wsdl.extensions.soap.SOAPAddressImpl;
import com.ibm.wsdl.extensions.soap.SOAPBindingImpl;
import com.ibm.wsdl.extensions.soap.SOAPBodyImpl;
import com.ibm.wsdl.extensions.soap.SOAPFaultImpl;
import com.ibm.wsdl.extensions.soap.SOAPOperationImpl;
import org.act.soap.encoding.TypeMappingImpl;
import org.act.soap.util.Constants;
import org.act.soap.util.TMConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.act.soap.util.JavaUtils;
import java.util.Iterator;

public class Emitter {
    
    private String mServiceName = "";
    
    private String mServiceClassName = "";
    
    private String mTargetNamespace = "";
    
    private String mLocationURI = "";
    
    private String mStyle = Constants.PROCESS_STYLE_RPC;
    
    private Class mServiceClass = null;
    
    private List mMethods = null;
    
    private Definition mDefinition = null;
    
    private Binding mBinding = null;
    
    private PortType mPortType = null;
    
    private TypeMapping mTypeMapping = null;
    
    private Types mTypes = null;
    
    private Map faultMap = null;
    
    public Emitter() {
        mTypeMapping = new TypeMappingImpl();
        mTypes = new Types(mTypeMapping);
        faultMap = new HashMap(5);
    }
    
    public void setServiceClassName(String serviceClassName) {
        mServiceClassName = serviceClassName;
        mServiceName = JavaUtils.extractShortClassName(serviceClassName);
    }
    
    public void setTargetNamespace(String targetNamespace) {
        mTargetNamespace = targetNamespace;
    }
    
    public void setLocationURI(String locationURI) {
        mLocationURI = locationURI;
    }
    
    public void setStyle(String soapBindingStyle) {
        mStyle = "rpc".equals(soapBindingStyle) ? "rpc" : "document";
    }
    
    public Definition emit() throws EmitterException, TypesException {
        if (null == mServiceClassName) {
            throw new EmitterException();
        }
        try {
            ClassLoader classLoader =
                Thread.currentThread().getContextClassLoader();
            mServiceClass = classLoader.loadClass(mServiceClassName);
                                } catch (ClassNotFoundException ex) {
            throw new EmitterException(
                org.act.soap.util.i18n.Message.getMessage(
                "javaClassNotFoundException", ex.getMessage()), ex);
        }
        try {
            mDefinition = WSDLFactory.newInstance().newDefinition();
        } catch (WSDLException ex) {
            throw new EmitterException(
                org.act.soap.util.i18n.Message.getMessage(
                "javaWSDLException", ex.getMessage()), ex);
        }
        parseClass();
        addNamespaces();
        createBinding();
        createPortType();
        for (int i = 0; i < mMethods.size(); i++) {
            createMethodRelated( (Method) mMethods.get(i));
        }
        createService();
        createTypes();
        return mDefinition;
    }
    
    private void parseClass() {
        Method[] methods = mServiceClass.getDeclaredMethods();
        mMethods = new ArrayList(methods.length);
        for (int i = 0; i < methods.length; i++) {
            if (Modifier.isPublic(methods[i].getModifiers())) {
                mMethods.add(methods[i]);
            }
        }
    }
    
    private void addNamespaces() {
        if ( (null == mTargetNamespace) || "".equals(mTargetNamespace)) {
            mTargetNamespace = mServiceClass.getPackage().getName();
        }
        mDefinition.setTargetNamespace(mTargetNamespace);
        mDefinition.addNamespace("tns", mTargetNamespace);
        mDefinition.addNamespace(
            Constants.NS_PREFIX_WSDL_SOAP, Constants.NS_URI_WSDL_SOAP);
        mDefinition.addNamespace(
            Constants.NS_PREFIX_WSDL, Constants.NS_URI_WSDL);
        mDefinition.addNamespace(
            Constants.NS_PREFIX_SOAP_ENC, Constants.NS_URI_SOAP_ENC);
        mDefinition.addNamespace(
            Constants.NS_PREFIX_SCHEMA_XSD, Constants.NS_URI_SCHEMA_XSD);
        mTypes.setTargetNamespace(mTargetNamespace);
    }
    
    private void createBinding() {
        mBinding = mDefinition.createBinding();
        mBinding.setUndefined(false);
        mBinding.setQName(
            new QName(mTargetNamespace, mServiceName + "SOAPBinding"));
                SOAPBinding soapBinding = new SOAPBindingImpl();
        soapBinding.setStyle(mStyle);
        soapBinding.setTransportURI(Constants.URI_SOAP11_HTTP);
        mBinding.addExtensibilityElement(soapBinding);
        mDefinition.addBinding(mBinding);
    }
    
    private void createPortType() {
        mPortType = mDefinition.createPortType();
        mPortType.setUndefined(false);
        mPortType.setQName(new QName(mTargetNamespace, mServiceName));
                mDefinition.addPortType(mPortType);
        mBinding.setPortType(mPortType);
    }
    
    private void createMethodRelated(Method method) throws TypesException {
                if (JavaUtils.isMethodOfDocumentStyle(method)) {
            Operation operation = mDefinition.createOperation();
            operation.setName(method.getName());
            operation.setUndefined(false);
            mPortType.addOperation(operation);
            BindingOperation bindingOperation =
                mDefinition.createBindingOperation();
            bindingOperation.setName(method.getName());
            SOAPOperation soapOperation = new SOAPOperationImpl();
            soapOperation.setSoapActionURI("");
            soapOperation.setStyle("document");
            bindingOperation.addExtensibilityElement(soapOperation);
            BindingInput bindingInput = mDefinition.createBindingInput();
            bindingInput.setName(method.getName() + "Request");
            SOAPBody inputSOAPBody = new SOAPBodyImpl();
            inputSOAPBody.setUse("literal");
            bindingInput.addExtensibilityElement(inputSOAPBody);
            bindingOperation.setBindingInput(bindingInput);
            BindingOutput bindingOutput = mDefinition.createBindingOutput();
            bindingOutput.setName(method.getName() + "Response");
            SOAPBody outputSOAPBody = new SOAPBodyImpl();
            outputSOAPBody.setUse("literal");
            bindingOutput.addExtensibilityElement(outputSOAPBody);
            bindingOperation.setBindingOutput(bindingOutput);
            bindingOperation.setOperation(operation);
            mBinding.addBindingOperation(bindingOperation);
            return;
        }
                Message inputMessage = createInputMessage(method);
        Message outputMessage = createOutputMessage(method);
        Class[] exceptions = method.getExceptionTypes();
        for (int i = 0; i < exceptions.length; i++) {
            if ( (exceptions[i] != java.rmi.RemoteException.class) &&
                !exceptions[i].getName().startsWith("java.") &&
                !exceptions[i].getName().startsWith("javax.")) {
                createFaultMessage(exceptions[i]);
            }
        }
        Message[] faultMessages =
            (Message[]) faultMap.values().toArray(new Message[faultMap.size()]);
        Operation operation = createOperation(method,
                                              inputMessage,
                                              outputMessage,
                                              faultMessages);
        mPortType.addOperation(operation);
        BindingOperation bindingOperation =
            createBindingOperation(method,
                                   inputMessage,
                                   outputMessage,
                                   faultMessages);
        bindingOperation.setOperation(operation);
        mBinding.addBindingOperation(bindingOperation);
    }
    
    private Operation createOperation(Method method, Message inputMessage,
                                      Message outputMessage,
                                      Message[] faultMessages) {
        Operation operation = mDefinition.createOperation();
        operation.setName(method.getName());
        operation.setUndefined(false);
        Class[] parameterTypes = method.getParameterTypes();
        String[] parameterNames = new String[parameterTypes.length];
        for (int i = 0; i < parameterTypes.length; i++) {
            parameterNames[i] = "param" + String.valueOf(i);
        }
        operation.setParameterOrdering(Arrays.asList(parameterNames));
        Input input = mDefinition.createInput();
        input.setMessage(inputMessage);
        input.setName(inputMessage.getQName().getLocalPart());
        operation.setInput(input);
        Output output = mDefinition.createOutput();
        output.setMessage(outputMessage);
        output.setName(outputMessage.getQName().getLocalPart());
        operation.setOutput(output);
        for (int i = 0; i < faultMessages.length; i++) {
            Fault fault = mDefinition.createFault();
            fault.setMessage(faultMessages[i]);
            fault.setName(faultMessages[i].getQName().getLocalPart());
            operation.addFault(fault);
        }
        return operation;
    }
    
    private BindingOperation createBindingOperation(Method method,
        Message inputMessage, Message outputMessage, Message[] faultMessages) {
        BindingOperation bindingOperation =
            mDefinition.createBindingOperation();
        bindingOperation.setName(method.getName());
        SOAPOperation soapOperation = new SOAPOperationImpl();
        soapOperation.setSoapActionURI("");
        bindingOperation.addExtensibilityElement(soapOperation);
        BindingInput bindingInput = mDefinition.createBindingInput();
        bindingInput.setName(inputMessage.getQName().getLocalPart());
        SOAPBody inputSOAPBody = createSOAPBody(inputMessage);
        bindingInput.addExtensibilityElement(inputSOAPBody);
        bindingOperation.setBindingInput(bindingInput);
        BindingOutput bindingOutput = mDefinition.createBindingOutput();
        bindingOutput.setName(outputMessage.getQName().getLocalPart());
        SOAPBody outputSOAPBody = createSOAPBody(outputMessage);
        bindingOutput.addExtensibilityElement(outputSOAPBody);
        bindingOperation.setBindingOutput(bindingOutput);
        for (int i = 0; i < faultMessages.length; i++) {
            BindingFault bindingFault = mDefinition.createBindingFault();
            bindingFault.setName(faultMessages[i].getQName().getLocalPart());
            SOAPFault soapFault = createSOAPFault(faultMessages[i]);
            bindingFault.addExtensibilityElement(soapFault);
            bindingOperation.addBindingFault(bindingFault);
        }
        return bindingOperation;
    }
    
    private SOAPBody createSOAPBody(Message message) {
        SOAPBody soapBody = new SOAPBodyImpl();
        if ("rpc".equals(mStyle)) {
            soapBody.setUse("encoded");
            List encodingStyle = new ArrayList(1);
            encodingStyle.add(Constants.NS_URI_SOAP_ENC);
            soapBody.setEncodingStyles(encodingStyle);
        } else {
            soapBody.setUse("literal");
        }
        soapBody.setNamespaceURI(message.getQName().getNamespaceURI());
        return soapBody;
    }
    
    private SOAPFault createSOAPFault(Message faultMessage) {
        SOAPFault soapFault = new SOAPFaultImpl();
        if ("rpc".equals(mStyle)) {
            soapFault.setUse("encoded");
            List encodingStyle = new ArrayList(1);
            encodingStyle.add(Constants.NS_URI_SOAP_ENC);
            soapFault.setEncodingStyles(encodingStyle);
            soapFault.setNamespaceURI(
                faultMessage.getQName().getNamespaceURI());
        } else {
            soapFault.setUse("literal");
        }
        return soapFault;
    }
    
    private Message createInputMessage(Method method) throws TypesException {
        Message message = mDefinition.createMessage();
        message.setQName(createMessageQName(method.getName() + "Request"));
        message.setUndefined(false);
        Class[] parameterTypes = method.getParameterTypes();
        for (int i = 0; i < parameterTypes.length; i++) {
            Part part =
                createMessagePart(
                "param" + i, parameterTypes[i]);
            if (null != part) {
                message.addPart(part);
            }
        }
        
        if ("rpc".equals(mStyle)) {
        } else {
        }
        mDefinition.addMessage(message);
        return message;
    }
    
    private Message createOutputMessage(Method method) throws TypesException {
        Message message = mDefinition.createMessage();
        message.setQName(createMessageQName(method.getName() + "Response"));
        message.setUndefined(false);
        
        Part part =
            createMessagePart(
            method.getName() + "Return", method.getReturnType());
        if (null != part) {
            message.addPart(part);
        }
        mDefinition.addMessage(message);
        return message;
    }
    
    private Message createFaultMessage(Class exceptionClass) throws
        TypesException {
        String className = exceptionClass.getName();
        Message message = (Message) faultMap.get(className);
        if (null == message) {
            message = mDefinition.createMessage();
            message.setQName(
                createMessageQName(JavaUtils.extractShortClassName(className)));
            message.setUndefined(false);
            
            mDefinition.addMessage(message);
            faultMap.put(className, message);
        }
        return message;
    }
    
    private QName createMessageQName(String messageName) {
        QName messageQName = new QName(mTargetNamespace, messageName);
                int i = 1;
        while (null != mDefinition.getMessage(messageQName)) {
            messageQName = new QName(mTargetNamespace, messageName + i++);
                    }
        return messageQName;
    }
    
    private Part createMessagePart(String parameterName, Class parameterType) throws
        TypesException {
        if (parameterType == java.lang.Void.TYPE) {
            return null;
        }
        Part part = mDefinition.createPart();
        part.setName(parameterName);
        
        QName xmlType = mTypes.getTypeQName(parameterType);
        part.setTypeName(xmlType);
        return part;
    }
    
    private void createService() {
        Service service = mDefinition.createService();
        service.setQName(
            new QName(mTargetNamespace, mServiceName + "Service"));
        mDefinition.addService(service);
        Port port = mDefinition.createPort();
        port.setBinding(mBinding);
        port.setName(mServiceName);
        SOAPAddress address = new SOAPAddressImpl();
        address.setLocationURI(mLocationURI);
        port.addExtensibilityElement(address);
        service.addPort(port);
    }
    
    private void createTypes() throws TypesException {
        if (0 < mTypes.getAllSchemaElements().size()) {
            mTypes.writeSchemas(mDefinition);
        }

    }
    public Types getTypes () {
        return mTypes;
    }
}
