package org.act.soap.wsdl.java2wsdl;
import java.util.*;
import javax.xml.namespace.*;
import javax.xml.rpc.encoding.*;
import org.w3c.dom.*;
import org.act.soap.encoding.*;
import org.act.soap.encoding.ser.*;
import org.act.soap.util.*;
import org.act.soap.util.i18n.*;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.Definition;
import javax.wsdl.extensions.UnknownExtensibilityElement;
import org.w3c.dom.Element;
import javax.wsdl.WSDLException;

public class Types {
    
    private Map mSchemaElements;
    
    private Map mNamespaces;
    
    private TypeMapping mTypeMapping;
    
    private String mTargetNamespace = null;
    
    public Types () {
        mSchemaElements = new HashMap();
        mNamespaces = new HashMap();
    }
    
    public Types (TypeMapping typeMapping) {
        mTypeMapping = typeMapping;
        mSchemaElements = new HashMap();
        mNamespaces = new HashMap();
        this.putNamespace(TMConstants.URI_DEFAULT_SCHEMA_XSD_PREFIX,
                          TMConstants.URI_DEFAULT_SCHEMA_XSD);
        this.putNamespace(TMConstants.URI_DEFAULT_SOAP_ENC_PREFIX,
                          TMConstants.URI_DEFAULT_SOAP_ENC);
    }
    
    public void setTypeMapping (TypeMapping typeMapping) {
        mTypeMapping = typeMapping;
    }
    
    public TypeMapping getTypeMapping () {
        return mTypeMapping;
    }
    
    public void putNamespace (String prefix, String uri) {
        mNamespaces.put(prefix, uri);
    }
    
    public String putNamespace (String uri) {
        String prefix = "tns" + mNamespaces.size();
        mNamespaces.put(prefix, uri);
        return prefix;
    }
    
    public String getNamespaceURI (String prefix) {
        return (String) mNamespaces.get(prefix);
    }
    
        public String getNamespacePrefix (String uri) {
        for (Iterator iterator = mNamespaces.entrySet().iterator();
            iterator.hasNext(); ) {
            Map.Entry entry = (Map.Entry) iterator.next();
            if (((String) entry.getValue()).equals(uri)) {
                return (String) entry.getKey();
            }
        }
        return null;
    }
    
    public String createNamespaceURI (String className) {
                                if( null == className) {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        sb.append("http://");
        int pos = className.lastIndexOf('.');
        if ( pos > 0 ) {
                        int pos1 = className.lastIndexOf('.', pos - 1);
            sb.append( className.substring(pos1 + 1, pos) );
            pos = pos1;
        }
        while ( pos > 0 ) {
            int pos1 = className.lastIndexOf('.', pos - 1);
            sb.append( "." );
            sb.append( className.substring(pos1 + 1, pos) );
            pos = pos1;
        }
        return sb.toString();
    }
    
    public void putSchemaElement (QName schemaName, Element schemaElement) {
        mSchemaElements.put(schemaName, schemaElement);
    }
    
    public Element getSchemaElement (QName schemaName) {
        return (Element) mSchemaElements.get(schemaName);
    }
    
    public Map getAllSchemaElements () {
        return mSchemaElements;
    }
    
    public void setTargetNamespace (String targetNamespace) {
        mTargetNamespace = targetNamespace;
        putNamespace("tns", mTargetNamespace);
    }
    
    public String getTargetNamespace () {
        return mTargetNamespace;
    }
    
    public Map getAllNamespaces(){
        return this.mNamespaces;
    }
    
        public String getAllSchemaString () {
        StringBuffer strSchema = new StringBuffer(1000);
        for (Iterator iterator = mSchemaElements.values().iterator();
            iterator.hasNext(); ) {
            strSchema.append(
                XMLUtils.nodeToPrettyString((Element) iterator.next()));
            strSchema.append("\n");
        }
        return strSchema.toString();
    }
    
    public QName getTypeQName(Class javaType) throws TypesException {
        if (javaType == null ) {
            return null;
        }
                QName xmlType = ( (TypeMappingImpl) mTypeMapping).getXmlType(javaType);
                        if (javaType.isArray() && (null != xmlType) &&
            isQNameEqual(TMConstants.SOAP_ARRAY, xmlType)) {
            Class componentType = javaType.getComponentType();
            QName cpName = getTypeQName(componentType);
            String tnsPrefix = getNamespacePrefix(mTargetNamespace);
            if (mTargetNamespace.equals(cpName.getNamespaceURI())) {
                xmlType = new QName( mTargetNamespace,
                                     "ArrayOf" + cpName.getLocalPart(),
                                     tnsPrefix);
            }
            else {
                String prefix = getNamespacePrefix(cpName.getNamespaceURI());
                xmlType = new QName( mTargetNamespace,
                                     "ArrayOf_" + prefix + "_" + cpName.getLocalPart(),
                                     tnsPrefix);
            }
        }
                if (null == xmlType) {
            String localPart
                = JavaUtils.extractShortClassName(javaType.getName());
            String ns = createNamespaceURI(javaType.getName());
            String prefix = getNamespacePrefix(ns);
            if (null == prefix) {
                                prefix = putNamespace(ns);
            }
            localPart = localPart.replace('$', '_');
            xmlType = new QName(ns, localPart, prefix);
        }
                createTypeSchema(javaType, xmlType);
                this.putNamespace( xmlType.getPrefix(), xmlType.getNamespaceURI() );
        return xmlType;
    }
    protected void createTypeSchema( Class javaType, QName xmlType )
        throws TypesException {
        SerializerFactory serializerFactory =
            mTypeMapping.getSerializer(javaType, xmlType);
        DeserializerFactory deserializerFactory =
            mTypeMapping.getDeserializer(javaType, xmlType);
        if (null == serializerFactory) {
                        if (javaType.isArray() && (null != xmlType)) {
                serializerFactory = new ArraySerializerFactory(javaType, xmlType);
            }
            else {
                if (JavaUtils.isEnumClass(javaType)) {
                    serializerFactory =
                        new EnumSerializerFactory(javaType, xmlType);
                    deserializerFactory =
                        new EnumDeserializerFactory(javaType, xmlType);
                } else if (JavaUtils.isBeanClass(javaType)) {
                    serializerFactory =
                        new BeanSerializerFactory(javaType, xmlType);
                    deserializerFactory =
                        new BeanDeserializerFactory(javaType, xmlType);
                } else {
                    throw new TypesException( Message.getMessage("unsupportJavaTypeException",
                                                                 javaType.getName()));
                }
            }
                        mTypeMapping.register(javaType, xmlType, serializerFactory, deserializerFactory);
        }
        if ( this.mSchemaElements.get(xmlType) == null ) {
            Serializer serializer =
                serializerFactory.getSerializerAs(
                TMConstants.ENCODING_MECHANISMTYPE);
            if (null == serializer) {
                throw new TypesException(
                     Message.getMessage(
                     "unsupportJavaTypeException", javaType.getName()));
            }
            Element typeSchema = null;
            try {
                typeSchema =
                    ((SerializerImpl) serializer).writeSchema(javaType,
                                                              xmlType,
                                                              this);
            } catch (TypeMappingException ex) {
                throw new TypesException(
                            Message.getMessage(
                                  "typeMappingException", ex.getMessage()), ex);
            }
            if (null != typeSchema) {
                                this.mSchemaElements.put(xmlType, typeSchema);
            }
        }
    }
    
        public static boolean isQNameEqual(QName first, QName second) {
        if ( (null == first) || (null == second)) {
            return false;
        }
        if (first.equals(second)) {
            return true;
        }
        if (!first.getLocalPart().equals(second.getLocalPart())) {
            return false;
        }
        String namespaceURI = first.getNamespaceURI();
        String[] search = null;
        if (namespaceURI.equals(TMConstants.URI_DEFAULT_SOAP_ENC)) {
            search = TMConstants.URIS_SOAP_ENC;
        }
        else if (namespaceURI.equals(TMConstants.URI_DEFAULT_SCHEMA_XSD)) {
            search = TMConstants.URIS_SCHEMA_XSD;
        }
        else {
            search = new String[] {
                namespaceURI};
        }
        for (int i = 0; i < search.length; i++) {
            if (search[i].equals(second.getNamespaceURI())) {
                return true;
            }
        }
        return false;
    }
    
        public static boolean isSchemaXSD(String s) {
        for (int i = 0; i < TMConstants.URIS_SCHEMA_XSD.length; i++) {
            if (TMConstants.URIS_SCHEMA_XSD[i].equals(s)) {
                return true;
            }
        }
        return false;
    }
    
        public String getQNameString(QName qname) {
        String prefix = this.getNamespacePrefix(qname.getNamespaceURI());
        return prefix + ':' + qname.getLocalPart();
    }
    
        public static boolean isNullable(Class type) {
        if (type.isPrimitive() ||
            (type.isArray() && (type.getComponentType() == byte.class))) {
            return false;
        }
        else {
            return true;
        }
    }
    
    public void writeSchemas ( Definition mDefinition ) throws MissingResourceException, TypesException {
        try {
            for (Iterator i = this.mNamespaces.entrySet().iterator();
                 i.hasNext();) {
                Map.Entry entry = (Map.Entry)i.next();
                String prefix = (String)entry.getKey();
                String uri = (String)entry.getValue();
                mDefinition.addNamespace( prefix, uri );
            }
            javax.wsdl.Types types = mDefinition.createTypes();
            mDefinition.setTypes(types);
            WSDLFactory wsdlFactory = WSDLFactory.newInstance();
            Document document =
                wsdlFactory.newWSDLWriter().getDocument(mDefinition);
                        HashMap schemas = new HashMap();
            for( Iterator i = mSchemaElements.entrySet().iterator(); i.hasNext(); ) {
                Map.Entry entry = (Map.Entry)i.next();
                QName xmlType = (QName)entry.getKey();
                String targetNamespace = xmlType.getNamespaceURI();
                Element schemaElement = (Element) schemas.get( targetNamespace );
                if( null == schemaElement ) {
                    schemaElement = document.createElement("schema");
                    schemaElement.setAttribute( "xmlns", TMConstants.URI_DEFAULT_SCHEMA_XSD);
                    schemaElement.setAttribute("targetNamespace", targetNamespace);
                    UnknownExtensibilityElement schemaExtensibilityElement =
                        new UnknownExtensibilityElement();
                    schemaExtensibilityElement.setElement(schemaElement);
                    types.addExtensibilityElement(schemaExtensibilityElement);
                    schemas.put(targetNamespace, schemaElement);
                }
                Element typeElement = (Element)entry.getValue();
                org.w3c.dom.Node node = document.importNode( typeElement, true );
                schemaElement.appendChild(node);
            }
        } catch (WSDLException ex) {
            throw new TypesException(
                      org.act.soap.util.i18n.Message.getMessage(
                      "javaWSDLException", ex.getMessage()), ex);
        }
    }
}
