package org.act.xdsec.core.dsig;
import java.security.*;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.core.exception.DigestException;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.extension.algorithm.transform.*;
import org.act.xdsec.extension.resource.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class Reference {
    
    private Reference () {
    }
    
    public static void processReference ( Element reference
                                          , String baseURI
                                          , ResourceResolver uriResolver
                                          , AlgorithmFactory algFactory
                                          , Element sig ) throws
        DataObjectNotFoundException,
                        TransformationException,
        DigestException,         NoSuchAlgorithmException     {
        Log.setMsg( "get into function processReference" );
        final byte[] value = URIToDigestValue( reference, baseURI, uriResolver,
                                               algFactory, sig );
                String base64Value = JavaUtils.Base64Encode( value );
                Document doc = reference.getOwnerDocument();
                        Element digestValue = XMLUtils.getElementInXMLSigNS( reference,
            Constant.TAG_DIGESTVALUE_NAME );
        digestValue.appendChild( doc.createTextNode( base64Value ) );
        Log.setMsg( "exit function processReference" );
    }
    
    public static Validity verifyReference ( Element reference
                                             , String baseURI
                                             , ResourceResolver uriResolver
                                             , AlgorithmFactory algFactory
                                             , Element sig ) throws
        DataObjectNotFoundException,
                        TransformationException,
        NoSuchAlgorithmException,         DigestException {                 Log.setMsg( "get into function VerifyReference." );
                String newValue = JavaUtils.Base64Encode(
            URIToDigestValue( reference, baseURI, uriResolver, algFactory, sig ) );
        Log.setMsg( "Caculate Digest Value" );
                                Element el = XMLUtils.getElementInXMLSigNS( reference,
            Constant.TAG_DIGESTVALUE_NAME );
        String oldValue = XMLUtils.getTextValueOfElement( el );
        return newValue.equals( oldValue ) ? new Validity( true )
            : new Validity( false );
    }
    
    private static final byte[] URIToDigestValue ( Element reference
        , String baseURI
        , ResourceResolver uriResolver
        , AlgorithmFactory algFactory
        , Element sig ) throws
        DataObjectNotFoundException,
                        TransformationException,
        DigestException,         NoSuchAlgorithmException {                 processDecryptionTransforms( reference, algFactory );
                                Log.setStartTime( "processURI" );
        DataObject data = processURI( reference, baseURI, uriResolver );
                Log.setEndTime( "processURI" );
        
                        Log.setStartTime( "processTransforms" );
        final byte[] input = processTransforms( reference, data, algFactory,
                                                sig );
        Log.setEndTime( "processTransforms" );
                Log.setMsg( "----------------- after transform -----------------" );
        Log.setMsg( new String( input ) );
                Log.setMsg( "---------------------------------------------------" );
                                        Element digest = XMLUtils.getElementInXMLSigNS( reference,
            Constant.TAG_DIGESTMETHOD_NAME );
        if ( null == digest ) {
                        Log.setMsg( "digestmethod element is null" );
            throw new DigestException( "digestmethod element is null" );                     }
        String algUri = digest.getAttribute( Constant.ATT_ALGORITHM );
        if ( null == algUri ) {
                        Log.setMsg(
                "shit, attribute algorithm not found in element digest." );
            throw new DigestException( "attribute algorithm not found" );                     }
        if ( null == algFactory ) {
                        Log.setMsg( "alg factory is null" );
            return new byte[0];
        }
        DigestAlgorithm algDigest = algFactory.createDigestMethod( algUri );
        if ( null == algDigest ) {
            throw new NoSuchAlgorithmException( "no such digest algorithm" );         }
        algDigest.setParameter( digest );
        Log.setMsg( "to digest...length of input is " + input.length );
        Log.setStartTime( "digest" );
        final byte[] rtn = algDigest.digest( input );                 Log.setMsg(
            "get digest value, get out of function reference.uritodigestvalue" );
        Log.setEndTime( "digest" );
        return rtn;
    }
    
    public static DataObject processURI ( Element reference
                                          , String baseURI
                                          , ResourceResolver uriResolver ) throws
        DataObjectNotFoundException {
                DataObject data = null;
                Log.setMsg( "get into function reference.uritodigestvalue" );
        Attr uri = reference.getAttributeNode( Constant.ATT_URI );
        data = uriResolver.processURI( uri, baseURI );
        if ( null == data ) {
                        Log.setMsg( "failue to get data object" );
            throw new DataObjectNotFoundException();                     }
                Log.setMsg( "process uri reference ,get data object." );
        return data;
    }
    
    public static byte[] processTransforms ( Element reference,
                                             DataObject data,
                                             AlgorithmFactory algFactory,
                                             Element sig ) throws                 TransformationException,
        NoSuchAlgorithmException {
                                NodeList trans = reference.getElementsByTagNameNS( Constant.
            NS_XMLDSIG_NAME, Constant.TAG_TRANSFORM_NAME );
        if ( null == trans || 0 >= trans.getLength() ) {
                        trans = reference.getElementsByTagName( Constant.TAG_TRANSFORM_NAME );
        }
        try {
            final int length = trans.getLength();
            for ( int i = 0; i < length; i++ ) {
                Element tran = ( Element ) trans.item( i );                 TransformAlgorithm alg = algFactory.createTransformMethod( tran.
                    getAttribute( Constant.ATT_ALGORITHM ) );
                if ( null == alg ) {
                    throw new NoSuchAlgorithmException(
                        "no such transformation algorithm" );                 }
                Log.setStartTime( alg.getURI() );
                                Log.setMsg( "Processing Transform " + String.valueOf( i )
                            + "  " +
                            alg.getURI() );
                                alg.setParameter( tran );
                data = alg.transform( data, sig );
                Log.setEndTime( alg.getURI() );
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            Log.setMsg( e.getMessage() );
            throw new TransformationException();         }
                Log.setMsg( "Transform data" );
                byte[] input = null;
        Log.setStartTime( "canonicalizeNodeList" );
        try {
            if ( data.getType().equals( Constant.DAT_TYPE_NODELIST ) ) {
                input = XMLUtils.canonicalizeNodeList( data.getNodeList(), false );
            }
            else if ( data.getType().equals( Constant.DAT_TYPE_NODE ) ) {
                input = XMLUtils.canonicalizeNode( data.getDataNode(), false );
            }
            else if ( data.getType().equals( Constant.DAT_TYPE_STREAM ) ) {
                                input = JavaUtils.getBytesFromStream( data.getDataStream() );
                            }
            else if ( data.getType().equals( Constant.DAT_TYPE_BYTE ) ) {
                input = data.getBytes();
            }
            else {
                                Log.setMsg( "unknown data type in data object" );
                return new byte[0];
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
            throw new TransformationException();         }
        Log.setEndTime( "canonicalizeNodeList" );
        if ( null == input ) {
                        Log.setMsg( "failue to get data bytes." );
            return new byte[0];
        }
        else {
                        Log.setMsg( "get data bytes." );
        }
        return input;
    }
    
    public static void processDecryptionTransforms ( Element reference,
        AlgorithmFactory algFactory ) throws TransformationException,
        NoSuchAlgorithmException {
        NodeList trans = reference.getElementsByTagNameNS( Constant.
            NS_XMLDSIG_NAME, Constant.TAG_TRANSFORM_NAME );
        if ( null == trans || 0 >= trans.getLength() ) {
                        trans = reference.getElementsByTagName( Constant.TAG_TRANSFORM_NAME );
        }
        final int length = trans.getLength();
        for ( int i = 0; i < length; i++ ) {
            Element tran = ( Element ) trans.item( i );                         if ( tran.getAttribute( Constant.ATT_ALGORITHM ).equalsIgnoreCase(
                Constant.ALG_DECTRANSFORM_URI ) ) {
                DecryptionTransform alg = ( DecryptionTransform ) algFactory.
                    createTransformMethod( Constant.ALG_DECTRANSFORM_URI );
                if ( null == alg ) {
                    throw new NoSuchAlgorithmException(
                        "no such transformation algorithm" );                 }
                Log.setMsg( "Processing Transform " + String.valueOf( i )
                            + "  " +
                            alg.getURI() );
                alg.setParameter( tran );
                alg.transform();
            }
        }
    }
}
