package org.act.xdsec.core.dsig;
import java.security.*;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.core.exception.DigestException;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.extension.keyinfo.*;
import org.act.xdsec.extension.resource.*;
import org.act.xdsec.syntax.dsig.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class Signature {
    
    private AlgorithmFactory algFactory = null;
    
    private ResourceResolver uriResolver = null;
    
    private KeyInfoResolver keyinfoResolver = null;
    
    public Signature () {
        this( null );
    }
    
    public Signature ( String provider ) {
        Log.enterFunc( "Signature.Signature" );
        this.algFactory = new AlgorithmFactory( provider );
        this.uriResolver = new ResourceResolver();
        this.keyinfoResolver = new KeyInfoResolver();
        Log.exitFunc();
    }
    
    public final AlgorithmFactory getAlgFactory () {
        return this.algFactory;
    }
    
    public final void setAlgFactory ( AlgorithmFactory iAlgFactory ) {
        this.algFactory = iAlgFactory;
    }
    
    public final ResourceResolver getUriResolver () {
        return this.uriResolver;
    }
    
    public final void setUriResolver ( ResourceResolver iUriResolver ) {
        this.uriResolver = iUriResolver;
    }
    
    public final KeyInfoResolver getKeyinfoResolver () {
        return this.keyinfoResolver;
    }
    
    public final void setKeyinfoResolver ( KeyInfoResolver iKeyinfoResolver ) {
        this.keyinfoResolver = iKeyinfoResolver;
    }
                
    public void sign ( XSignature xsig, String baseURI, Key key,
                       String signKeyName ) throws
        DataObjectNotFoundException,
                CanonicalizationException,
        TransformationException,
        NoSuchAlgorithmException,         InvalidKeyException,         SignatureException,         DigestException {         Log.enterFunc( "Signature.sign" );
        this.sign( xsig.toElement(), baseURI, key, signKeyName );
        Log.exitFunc();
    }
    
    public void sign ( Element sig, String baseURI, Key key, String signKeyName ) throws
        DataObjectNotFoundException,
                CanonicalizationException,
        TransformationException,
        NoSuchAlgorithmException,         InvalidKeyException,         SignatureException,         DigestException {         NodeList nodelist = null;
        Node node = null;
        Element el = null;
                Log.enterFunc( "Signature.sign" );
        Log.setMsg( "get into function sign" );
                if ( null == key ) {
            throw new InvalidKeyException();
        }
        
                Element signedinfo = XMLUtils.getElementInXMLSigNS( sig,
            Constant.TAG_SIGNEDINFO_NAME );
                                nodelist = signedinfo.getElementsByTagNameNS( Constant.NS_XMLDSIG_NAME,
            Constant.TAG_REFERENCE_NAME );
                
                        el = XMLUtils.getElementInXMLSigNS( signedinfo,
                                            Constant.
                                            TAG_CANONICALIZATIONMETHOD_NAME );
                if ( null == el ) {
            Log.setMsg( "miss canonical method" );
            throw new CanonicalizationException();         }
        else {
            Log.setMsg( "found canonical method " +
                        el.getAttribute( Constant.ATT_ALGORITHM ) );
        }
        CanonicalizationAlgorithm alg = algFactory.createCanonicalizationMethod(
            el.getAttribute( Constant.ATT_ALGORITHM ) );
        if ( null == alg ) {
            Log.setMsg( "fail to create canonical algorithm" );
            throw new NoSuchAlgorithmException(
                "no such canonicalization algorithm" );         }
        else {
            Log.setMsg( "succeed to create canonical algorithm " );
        }
        alg.setParameter( el );
        Log.setMsg( "create canonicalization method" );
                Log.setStartTime( "Reference" );
                if ( null == nodelist || 0 >= nodelist.getLength() ) {
            nodelist = signedinfo.getElementsByTagName( Constant.
                TAG_REFERENCE_NAME );
        }
                final int length = nodelist.getLength();
        for ( int i = 0; i < length; i++ ) {
            node = nodelist.item( i );
            Reference.processReference( ( Element ) ( node )
                                        , baseURI, uriResolver, algFactory, sig );
        }
        Log.setMsg( "process reference,number = " + nodelist.getLength() );
        Log.setEndTime( "Reference" );
        byte[] data = alg.canonicalizeNode( signedinfo );
        Log.setMsg( "canonicalize element signedinfo" );
        Log.setMsg( "The Canonical XML Element Is :" );
        Log.setMsg( new String( data ) );
                SignatureAlgorithm algSign = null;
        
                el = XMLUtils.getElementInXMLSigNS( signedinfo,
                                            Constant.TAG_SIGNATUREMETHOD_NAME );
                algSign = algFactory.createSignatureMethod( el.getAttribute( Constant.
            ATT_ALGORITHM ) );
        if ( null == algSign ) {
            throw new NoSuchAlgorithmException( "no such signature algorithm" );         }
                        algSign.setParameter( el );
                        try {
                        data = algSign.sign( data, key );
        }
        catch ( Exception e ) {             throw new SignatureException( e.getMessage() );         }
        Log.setMsg( "caculate signature value of canonicalized signedinfo" );
                        el = XMLUtils.getElementInXMLSigNS( sig,
                                            Constant.TAG_SIGNATUREVALUE_NAME );
                Document doc = sig.getOwnerDocument();
        el.appendChild( doc.createTextNode( JavaUtils.Base64Encode( data ) ) );
        Log.setMsg( " signature value insert into element signaturevalue" );
                        el = XMLUtils.getElementInXMLSigNS( sig, Constant.TAG_KEYINFO_NAME );
        Element keyInfo = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_KEYINFO_NAME );
        Element keyName = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_KEYNAME_NAME );
        keyName.appendChild( doc.createTextNode( signKeyName ) );
        keyInfo.appendChild( keyName );
        sig.replaceChild( keyInfo, el );
                Log.exitFunc();
            }
    
    public Validity verify ( Element sig, String baseURI ) throws
        DataObjectNotFoundException,
                CanonicalizationException,
        TransformationException,
        DigestException,         NoSuchAlgorithmException,         KeyResolverException,         SignatureException {         return verify( sig, baseURI, null );
    }
    
    public Validity verify ( Element sig, String baseURI, Key key ) throws
        DataObjectNotFoundException,
                CanonicalizationException,
        TransformationException,
        DigestException,         NoSuchAlgorithmException,         KeyResolverException,         SignatureException {         Log.enterFunc( "Signature.verify" );
        NodeList nodelist = null;
        Node node = null;
        Element signedinfo = null;
        Element el = null;
                Log.setMsg( "Signature.verify,Get into verify..." );
                        signedinfo = XMLUtils.getElementInXMLSigNS( sig,
            Constant.TAG_SIGNEDINFO_NAME );
        Log.setMsg( "Signature.verify,get signedinfo element" );
                Log.setStartTime( "Canonicalization" );
        CanonicalizationAlgorithm alg = null;
                        el = XMLUtils.getElementInXMLSigNS( signedinfo,
                                            Constant.
                                            TAG_CANONICALIZATIONMETHOD_NAME );
        if ( null == el ) {
            Log.setMsg( "miss canonical method" );
            throw new CanonicalizationException();         }
        alg = algFactory.createCanonicalizationMethod( el.getAttribute(
            Constant.
            ATT_ALGORITHM ) );
        if ( null == alg ) {
            throw new NoSuchAlgorithmException(
                "no such canonicalization algorithm" );         }
        alg.setParameter( el );
        final byte[] data = alg.canonicalizeNode( signedinfo );         Log.setMsg( "Signature.verify,canonicalize signedinfo" );
        Log.setMsg( "Canonical Form of SingedInfo." );
        Log.setMsg( new String( data ) );
        Log.setEndTime( "Canonicalization" );
                Log.setStartTime( "Reference" );
                        nodelist = signedinfo.getElementsByTagNameNS( Constant.NS_XMLDSIG_NAME,
            Constant.TAG_REFERENCE_NAME );
                if ( null == nodelist || 0 >= nodelist.getLength() ) {
            nodelist = signedinfo.getElementsByTagName( Constant.
                TAG_REFERENCE_NAME );
        }
                Validity valid = null;
        final int length = nodelist.getLength();
        for ( int i = 0; i < length; i++ ) {
            node = nodelist.item( i );
            valid = Reference.verifyReference( ( Element ) node, baseURI,
                                               uriResolver,
                                               algFactory, sig );
                        if ( !valid.getValidity() ) {
                Log.setMsg( "fail to verify reference" );
                Log.exitFunc();
                return valid;
            }
        }
        Log.setMsg( "Signature.verify, verify reference" );
        Log.setEndTime( "Reference" );
                if ( null == key ) {
                                    Element keyinfo = XMLUtils.getElementInXMLSigNS( sig,
                Constant.TAG_KEYINFO_NAME );
            if ( null == keyinfo ) {
                                throw new KeyResolverException();             }             key = this.keyinfoResolver.processKeyInfo( keyinfo, true );
        }
        if ( null == key ) {
            Log.setMsg( "failed to get a key " );
            throw new KeyResolverException();                     }
        Log.setMsg( "Signature.verify,get public key" );
        Log.setStartTime( "SignatureAlgorithm" );
                        SignatureAlgorithm algSign = null;
                        el = XMLUtils.getElementInXMLSigNS( signedinfo,
                                            Constant.TAG_SIGNATUREMETHOD_NAME );
        algSign = algFactory.createSignatureMethod( el.getAttribute( Constant.
            ATT_ALGORITHM ) );
                if ( null == algSign ) {
            throw new NoSuchAlgorithmException( "no such signature algorithm" );         }
                        algSign.setParameter( el );
                Log.setMsg( "Signature.verify,create signature algorithm" );
                                el = XMLUtils.getElementInXMLSigNS( sig,
                                            Constant.TAG_SIGNATUREVALUE_NAME );
        Log.setMsg( "get signature value in element SignatureValue" );
        Log.setEndTime( "SignatureAlgorithm" );
                Log.setStartTime( "only_Verify" );
        valid = new Validity();
        try {
                                    String sigvalue = XMLUtils.getTextValueOfElement( el );
            final boolean result = algSign.verify( JavaUtils.Base64Decode(
                sigvalue ), data, key );
            valid.setValidity( result );
            Log.setMsg( "Signature.verify,verify signature value" );
        }
        catch ( Exception e ) {
            throw new SignatureException( "verify signature exception" +
                                          e.getMessage() );         }
        Log.setEndTime( "only_Verify" );
        Log.exitFunc();
        return valid;
    }
    
    public Validity processManifest ( XManifest manifest ) {
        Log.enterFunc( "Signature.processManifest" );
                Log.exitFunc();
        return null;
    }
    
    public Validity verifyManifest ( XManifest manifest ) {
        Log.enterFunc( "Signature.verifyManifest" );
                Log.exitFunc();
        return null;
    }
                            
    public void addSignatureMethod ( SignatureAlgorithm alg ) {
        Log.enterFunc( "Signature.addSignatureMethod" );
        this.algFactory.addSignatureMethod( alg );
        Log.exitFunc();
    }
    
    public void addCanonicalizationMethod ( CanonicalizationAlgorithm alg ) {
        Log.enterFunc( "Signature.addCanonicalizationMethod" );
        this.algFactory.addCanonicalizationMethod( alg );
        Log.exitFunc();
    }
    
    public void addTransform ( TransformAlgorithm alg ) {
        Log.enterFunc( "Signature.addTransform" );
        this.algFactory.addTransform( alg );
        Log.exitFunc();
    }
    
    public void addDigestMethod ( DigestAlgorithm alg ) {
        Log.enterFunc( "Signature.addDigestMethod" );
        this.algFactory.addDigestMethod( alg );
        Log.exitFunc();
    }
                            public void addKeyProcessor ( KeyProcessor processor ) {
        Log.enterFunc( "Signature.addKeyProcessor" );
        this.keyinfoResolver.addKeyProcessor( processor );
        Log.exitFunc();
    }
    
    public void setX509DataProcessor ( X509DataProcessor processor ) {
        Log.enterFunc( "Signature.setX509DataProcessor" );
        this.keyinfoResolver.setX509DataProcessor( processor );
        Log.exitFunc();
    }
    
    public void setKeyNameProcessor ( KeyNameProcessor processor ) {
        Log.enterFunc( "Signature.setKeyNameProcessor" );
        this.keyinfoResolver.setKeyNameProcessor( processor );
        Log.exitFunc();
    }
    
    public void setPGPDataProcessor ( PGPDataProcessor processor ) {
        Log.enterFunc( "Signature.setPGPDataProcessor" );
        this.keyinfoResolver.setPGPDataProcessor( processor );
        Log.exitFunc();
    }
    
    public void setSPKIDataProcessor ( SPKIDataProcessor processor ) {
        Log.enterFunc( "Signature.setSPKIDataProcessor" );
        this.keyinfoResolver.setSPKIProcessor( processor );
        Log.exitFunc();
    }
    
    public void setMGMTDataProcessor ( MGMTDataProcessor processor ) {
        Log.enterFunc( "Signature.setMGMTDataProcessor" );
        this.keyinfoResolver.setMGMTDataProcessor( processor );
        Log.exitFunc();
    }
    
    public void setRetrievalMethodProcessor ( RetrievalMethodProcessor
                                              processor ) {
        Log.enterFunc( "Signature.setRetrievalMethodProcessor" );
        processor.setKeyInfoResolver( this.keyinfoResolver );
        this.keyinfoResolver.setRetrievalMethodProcessor( processor );
        Log.exitFunc();
    }
                                    public void setIDProcessor ( FragmentProcessor processor ) {
        Log.enterFunc( "Signature.setIDProcessor" );
        this.uriResolver.setIDProcessor( processor );
        Log.exitFunc();
    }
    
    public void setNullURIProcessor ( NULLRefProcessor processor ) {
        Log.enterFunc( "Signature.setNullURIProcessor" );
        this.uriResolver.setNullURIProcessor( processor );
        Log.exitFunc();
    }
    
    public void setURIProcessor ( URIProcessor processor ) {
        Log.enterFunc( "Signature.setURIProcessor" );
        this.uriResolver.setURIProcessor( processor );
        Log.exitFunc();
    }
}
