package org.act.xdsec.core.enc;


import java.security.*;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.dsig.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.extension.keyinfo.*;
import org.act.xdsec.extension.resource.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public abstract class EncryptedItem {
    
    public static int ENCRYPTEDKEY = 1;
    
    public static int ENCRYPTEDDATA = 2;
    
    protected byte[] plaintext = null;
    
    private int mOpType;
    
    protected Element mencItem;
    
    private AlgorithmFactory algFactory = new AlgorithmFactory( null );
    
    private KeyInfoResolver keyinfoResolver = new KeyInfoResolver();
    
    private ResourceResolver uriResolver = new ResourceResolver();
    
    private String EncAlgName = Constant.ALG_DES_URI;     
    public EncryptedItem ( int optype ) {
        this.mOpType = optype;
        this.mencItem = null;
    }
    
    public final String getEncAlgName () {
        return EncAlgName;
    }
    
    public final void setEncAlgName ( String algname ) {
        this.EncAlgName = algname;
    }
    
    protected final void setUriResolver ( ResourceResolver resolver ) {
        this.uriResolver = resolver;
    }
    
    public final void setAlgorithmFactory ( AlgorithmFactory factory ) {
        this.algFactory = factory;
    }
    
    public final void setKeyInfoResolver ( KeyInfoResolver resolver ) {
        this.keyinfoResolver = resolver;
    }
    
    public void setKeyNameProcessor ( KeyNameProcessor processor ) {
        this.keyinfoResolver.setKeyNameProcessor( processor );
    }
    
    public void setEncryptedKeyProcessor ( EncryptedKeyProcessor processor ) {
                                                processor.setKeyInfoResolver( this.keyinfoResolver );
        this.keyinfoResolver.addKeyProcessor( processor );
    }
    
    public void setAgreementMethodProcessor ( AgreementMethodProcessor
                                              processor ) {
        this.keyinfoResolver.addKeyProcessor( processor );
    }
    
    public void addAnyKeyProcessor ( KeyProcessor processor ) {
        this.keyinfoResolver.addKeyProcessor( processor );
    }
    
    protected byte[] innerEncrypt ( byte[] data, Key key, String algName ) throws
        NoSuchAlgorithmException,
        EncryptException {         Element encData = this.mencItem;
                        EncryptionAlgorithm algMet = createEncAlg( encData, algName );
        if ( null == algMet ) {
            throw new NoSuchAlgorithmException( "no such encryption algorithm" );         }
                                final byte[] cipher = algMet.encrypt( data, key );
                                                
        Element eleCipherData = XMLUtils.getFirstChildElementNamed( encData,
            Constant.ENC_CIPHERDATA_TAG );
        Element eleCipherValue = XMLUtils.getFirstChildElementNamed(
            eleCipherData,
            Constant.ENC_CIPHERVALUE_TAG );
        if ( null != eleCipherValue ) {
            XMLUtils.setTextValue( eleCipherValue,
                                   JavaUtils.Base64Encode( cipher ) );
        }
        return cipher;
    }
    
    protected byte[] innerDecrypt ( Key key, byte[] keyData, String baseURI,
                                    String algName ) throws
        NoSuchAlgorithmException,
        CipherDataNotFoundException,
        DataObjectNotFoundException,
                        TransformationException,
        KeyResolverException,         DecryptException {         Element encData = this.mencItem;
                EncryptionAlgorithm algMet = createEncAlg( encData, algName );
        if ( null == algMet ) {
            throw new NoSuchAlgorithmException( "no such encryption algorithm" );         }
                Key key2 = key;         if ( null == key && null != keyData ) {             key2 = algMet.generateKey( keyData );
        }
        if ( null == key2 ) {             key2 = getKey();
        }
        if ( null == key2 ) {
                        throw new KeyResolverException();         }
                final byte[] cipher = getCipher( encData, baseURI );
        return algMet.decrypt( cipher, key2 );
    }
    
    private Key getKey ()
    
    {
        Key key = null;
                Element xpathElement = null;
        try {
            Element encData = this.mencItem;
            Document doc = encData.getOwnerDocument();
            xpathElement = ( Element ) encData.getElementsByTagNameNS(
                Constant.NS_XMLDSIG_NAME,
                Constant.TAG_KEYINFO_NAME ).item( 0 );
            if ( null == xpathElement ) {
                xpathElement = ( Element ) encData.getElementsByTagName(
                    Constant.
                    TAG_KEYINFO_NAME ).item( 0 );
            }
        }
        catch ( Exception ex ) {
                        return key;
        }
        if ( null != xpathElement ) {
            key = this.keyinfoResolver.processKeyInfo( xpathElement, false );
        }
        return key;
    }
    
    private final EncryptionAlgorithm createEncAlg ( Element encData,
        String defaultAlgName ) throws
        NoSuchAlgorithmException {
        EncryptionAlgorithm algMet = null;
                Element algElem = null;
        try {
            Document doc = encData.getOwnerDocument();
            algElem = ( Element ) encData.getElementsByTagNameNS(
                Constant.NS_XMLENC_NAME,
                Constant.ENC_ENCRYPTIONMETHOD_TAG ).item( 0 );
            if ( null == algElem ) {
                algElem = ( Element ) encData.getElementsByTagName( Constant.
                    ENC_ENCRYPTIONMETHOD_TAG ).item( 0 );
            }
        }
        catch ( Exception ex ) {
                        return null;
        }
        if ( null != algElem ) {
            String algName = algElem.getAttribute( Constant.ATT_ALGORITHM );
            algMet = algFactory.createEncryptionMethod( algName );
            if ( null == algMet ) {
                throw new NoSuchAlgorithmException();
            }
            algMet.setParameter( algElem );
        }
        else {
            if ( ( null == defaultAlgName ) || ( 1 > defaultAlgName.length() ) ) {
                algMet = algFactory.createEncryptionMethod( EncAlgName );
            }
            else {
                algMet = algFactory.createEncryptionMethod( defaultAlgName );
            }
            if ( null == algMet ) {
                throw new NoSuchAlgorithmException(
                    "no such encryption algorithm" );             }
        }
        return algMet;
    }
    
    private final byte[] getCipher ( Element encData, String baseURI ) throws
        CipherDataNotFoundException,
        DataObjectNotFoundException,
                        TransformationException,
        NoSuchAlgorithmException {                                 
        Element cipherData = XMLUtils.getFirstChildElementNamed( encData,
            Constant.ENC_CIPHERDATA_TAG );
        if ( null == cipherData ) {
            throw new CipherDataNotFoundException();
        }
                NodeList nodes = cipherData.getChildNodes();
        if ( null == cipherData ) {
            throw new CipherDataNotFoundException();
        }
        int length = nodes.getLength();
        for ( int i = 0; i < length; i++ ) {
            Node node = nodes.item( i );
            if ( node.getNodeType() != Node.ELEMENT_NODE ) {
                continue;
            }
                        String name = node.getLocalName();
            if ( name.equals( Constant.ENC_CIPHERVALUE_TAG ) ) {
                return processCipherValue( ( Element ) node );
            }
            if ( name.equals( Constant.ENC_CIPHERREFERENCE_TAG ) ) {
                return processCipherReference( ( Element ) node, baseURI );
            }
        }
                return new byte[0];
    }
    
    private final byte[] processCipherValue ( Element cipherValue ) throws
        CipherDataNotFoundException {
        String value = XMLUtils.getTextValueOfElement( cipherValue );
        if ( null == value ) {
            throw new CipherDataNotFoundException();
        }
        return JavaUtils.Base64Decode( value );
    }
    
    private final byte[] processCipherReference ( Element cipherReference,
                                                  String baseURI ) throws
        CipherDataNotFoundException,
        DataObjectNotFoundException,
                        TransformationException,
        NoSuchAlgorithmException {         DataObject data = Reference.processURI( cipherReference, baseURI,
                                                this.uriResolver );
                final byte[] cipher = Reference.processTransforms( cipherReference,
            data,
            algFactory, null );
        if ( null == cipher ) {
            throw new CipherDataNotFoundException();
        }
        return cipher;
    }
}
