
package org.act.xdsec.extension.algorithm;
import java.util.*;
import org.act.xdsec.util.*;

public class AlgorithmFactory {
    protected String provider = null;
    protected Hashtable signatureTable = null;
    protected Hashtable encryptionTable = null;     protected Hashtable digestTable = null;
    protected Hashtable transformTable = null;
    protected Hashtable canonicalizerTable = null;
    
    public AlgorithmFactory () {
    }
    
    public AlgorithmFactory ( String iProvider ) {
        this.provider = iProvider;
                                        this.signatureTable = new Hashtable();
        this.signatureTable.put( Constant.ALG_RSAWITHSHA1_URI,
                                 Constant.ALG_RSAWITHSHA1_IMPL );
        this.signatureTable.put( Constant.ALG_DSAWITHSHA1_URI,
                                 Constant.ALG_DSAWITHSHA1_IMPL );
        this.signatureTable.put( Constant.ALG_HMAC_URI, Constant.ALG_HMAC_IMPL );
                        this.encryptionTable = new Hashtable();
        this.encryptionTable.put( Constant.ALG_DES_URI, Constant.ALG_DES_IMPL );
        this.encryptionTable.put( Constant.ALG_AES128_URI,
                                  Constant.ALG_AES128_IMPL );
        this.encryptionTable.put( Constant.ALG_AES192_URI,
                                  Constant.ALG_AES192_IMPL );
        this.encryptionTable.put( Constant.ALG_AES256_URI,
                                  Constant.ALG_AES256_IMPL );
        this.encryptionTable.put( Constant.ALG_KW3DES_URI,
                                  Constant.ALG_KW3DES_IMPL );
        this.encryptionTable.put( Constant.ALG_KWAES128_URI,
                                  Constant.ALG_KWAES128_IMPL );
        this.encryptionTable.put( Constant.ALG_KWAES192_URI,
                                  Constant.ALG_KWAES192_IMPL );
        this.encryptionTable.put( Constant.ALG_KWAES256_URI,
                                  Constant.ALG_KWAES256_IMPL );
        this.encryptionTable.put( Constant.ALG_RSA15_URI,
                                  Constant.ALG_RSA15_IMPL );
        this.encryptionTable.put( Constant.ALG_RSAOAEP_URI,
                                  Constant.ALG_RSAOAEP_IMPL );
                        this.digestTable = new Hashtable();
        this.digestTable.put( Constant.ALG_SHA1_URI, Constant.ALG_SHA1_IMPL );
                this.canonicalizerTable = new Hashtable();
        this.canonicalizerTable.put( Constant.ALG_CANONICALIZATION_URI,
                                     Constant.ALG_CANONICALIZATION_IMPL );
        this.canonicalizerTable.put( Constant.ALG_CANONICALIZATIONWC_URI,
                                     Constant.ALG_CANONICALIZATIONWC_IMPL );
        this.transformTable = new Hashtable();
        this.transformTable.put( Constant.ALG_C14N_TRAN_URI,
                                 Constant.ALG_C14N_TRAN_IMPL );
        this.transformTable.put( Constant.ALG_C14NWC_TRAN_URI,
                                 Constant.ALG_C14NWC_TRAN_IMPL );
        this.transformTable.put( Constant.ALG_BASE64_URI,
                                 Constant.ALG_BASE64_IMPL );
        this.transformTable.put( Constant.ALG_ENVELOPEDSIG_URI,
                                 Constant.ALG_ENVELOPEDSIG_IMPL );
        this.transformTable.put( Constant.ALG_XSLT_URI, Constant.ALG_XSLT_IMPL );
        this.transformTable.put( Constant.ALG_XPATH_URI,
                                 Constant.ALG_XPATH_IMPL );
                this.transformTable.put( Constant.ALG_XPATHFILTER2_URI,
                                 Constant.ALG_XPATHFILTER2_IMPL );
        this.transformTable.put( Constant.ALG_DECTRANSFORM_URI,
                                 Constant.ALG_DECTRANSFORM_IMPL );
    }
    
    public void addSignatureMethod ( SignatureAlgorithm sigAlg ) {
        this.signatureTable.put( sigAlg, sigAlg.getURI() );
    }
    
    public void addEncryptionMethod ( EncryptionAlgorithm encAlg ) {
        this.encryptionTable.put( encAlg, encAlg.getURI() );
    }
    
    public void addCanonicalizationMethod ( CanonicalizationAlgorithm
                                            canonicalizationAlg ) {
        this.canonicalizerTable.put( canonicalizationAlg,
                                     canonicalizationAlg.getURI() );
    }
    
    public void addTransform ( TransformAlgorithm transAlg ) {
        this.transformTable.put( transAlg, transAlg.getURI() );
    }
    
    public void addDigestMethod ( DigestAlgorithm digestAlg ) {
        this.digestTable.put( digestAlg, digestAlg.getURI() );
    }
    
    public void registerSignatureMethod ( String uri, String classPath ) {
        this.signatureTable.put( uri, classPath );
    }
    
    public void registerEncryptionMethod ( String uri, String classPath ) {
        this.encryptionTable.put( uri, classPath );
    }
    
    public void registerCanonicalizationMethod ( String uri, String classPath ) {
        this.canonicalizerTable.put( uri, classPath );
    }
    
    public void registerTransform ( String uri, String classPath ) {
        this.transformTable.put( uri, classPath );
    }
    
    public void registerDigestMethod ( String uri, String classPath ) {
        this.digestTable.put( uri, classPath );
    }
    
    public SignatureAlgorithm createSignatureMethod ( String uri ) {
                return ( SignatureAlgorithm ) createAlgorithm( this.signatureTable, uri );
    }
    
    public EncryptionAlgorithm createEncryptionMethod ( String uri ) {
                return ( EncryptionAlgorithm ) createAlgorithm( this.encryptionTable,
            uri );
    }
    
    public CanonicalizationAlgorithm createCanonicalizationMethod ( String uri ) {
                return ( CanonicalizationAlgorithm ) createAlgorithm( this.
            canonicalizerTable, uri );
    }
    
    public DigestAlgorithm createDigestMethod ( String uri ) {
                return ( DigestAlgorithm ) createAlgorithm( this.digestTable, uri );
    }
    
    public TransformAlgorithm createTransformMethod ( String uri ) {
        return ( TransformAlgorithm ) createAlgorithm( this.transformTable, uri );
    }
    
    private static final Algorithm createAlgorithm ( Hashtable table,
        String uri ) {
        Log.enterFunc( "AlgorithmFactory", "createAlgorithm" );
        Log.setMsg( "AlrotithmFactory.createAlgorithm,in..." );
        Object obj = JavaUtils.searchHashtable( table, uri );
        if ( null == obj ) {
                        return null;
        }
        Log.setMsg( "AlrotithmFactory.createAlgorithm,get algorithm impl name" );
        Algorithm alg = null;
        if ( obj instanceof String ) {
            try {
                Log.setMsg(
                    "AlrotithmFactory.createAlgorithm,Algorithm Impl : " +
                    ( String ) obj );
                alg = ( Algorithm ) Class.forName( ( String ) obj ).newInstance();
                alg.setURI( uri );
            }
            catch ( Exception e ) {
                Log.setMsg( "AlrotithmFactory.createAlgorithm,catch exception" );
            }
                                            }
        else {
            alg = ( Algorithm ) obj;
        }
        Log.setMsg( "AlrotithmFactory.createAlgorithm, algorithm created." );
        Log.exitFunc();
        return alg;
    }
}
