package org.act.xdsec.extension.algorithm;
import java.io.*;
import java.security.*;
import java.util.*;
import javax.crypto.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import org.act.xdsec.util.*;
import org.apache.xpath.*;
import org.w3c.dom.*;


public abstract class JCEAlgorithm
    extends Algorithm {
    
    public static String config_KeyOfSysProperty = "org.act.XDSec.config";
    
    public static String config_DefaultName = Constant.JCE_CONFIG_FILE;
    
    private String malgType;     private String mprovider;     private String malgName;     public JCEAlgorithm ( String algType ) {
        super();
        malgType = algType;         JCEAlgorithm.init();
    }
    
    public String getAlgType () {
        return this.malgType;
    }
    
    public String getProvider () {
                                if ( this.mprovider == null ) {
            JCEAlgorithm.parseURItoJCEID( this, this.getURI() );
        }
        return this.mprovider;
    }
    
    public String getAlgId () {
                                                if ( this.malgName == null ) {
            if ( this.mprovider == null ) {
                JCEAlgorithm.parseURItoJCEID( this, this.getURI() );
            }
            else {
                JCEAlgorithm.parseURItoJCEID( this, this.getURI(),
                                              this.getProvider() );
            }
        }
        return this.malgName;
    }
    
    protected void setAlgType ( String algType ) {
        malgType = algType;
    }
    
    protected void setProvider ( String pd ) {
        if ( pd == null || pd.equals( "" ) ) {
            this.mprovider = pd;
        }
        else if ( this.malgName == null || this.malgName.equals( "" ) ) {
            this.mprovider = pd;
        }
        else if ( JCEAlgorithm.isAlgAvailable( pd, this.malgType, this.malgName ) ) {
            JCEAlgorithm.addProvider( pd );             this.mprovider = pd;
        }
        else {
                                }
    }
    
    protected void setAlgId ( String name ) {
        if ( name == null || name.equals( "" ) ) {
            this.malgName = name;
        }
        else if ( this.mprovider == null || this.mprovider.equals( "" ) ) {
            this.malgName = name;
        }
        else if ( JCEAlgorithm.isAlgAvailable( this.mprovider, this.malgType,
                                               name ) ) {
                        this.malgName = name;
        }
        else {
                                }
    }
    
    protected Cipher getCipherInstance () {
        Cipher cipher = null;
                try {
            if ( this.getProvider() == null ) {
                cipher = Cipher.getInstance( this.getAlgId() );
            }
            else {
                cipher = Cipher.getInstance( this.getAlgId(), this.getProvider() );
            }
        }
        catch ( NoSuchPaddingException ex ) {
        }
        catch ( NoSuchAlgorithmException e ) {
        }
        catch ( NoSuchProviderException ex ) {
        }
        return cipher;
    }
    
    private static Element _providerList = null;
    
    private static Element _nscontext = null;
    
    private static boolean _alreadyInitialized = false;
    
    public synchronized static void init () {
        if ( !_alreadyInitialized ) {
            _alreadyInitialized = true;
            try {
                
                DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                dbf.setNamespaceAware( true );
                dbf.setValidating( false );
                DocumentBuilder db = dbf.newDocumentBuilder();
                                                InputStream is = JCEAlgorithm.readConfigFile();
                                Document doc = db.parse( is );
                Element context = XMLUtils.createXSecCtx(
                    doc, "x", Constant.NS_XDSEC_CONFIG );
                
                {
                                        
                    JCEAlgorithm._providerList =
                        ( Element ) doc.getElementsByTagNameNS(
                            Constant.NS_XDSEC_CONFIG, "JCEAlgorithmMappings" ).
                        item( 0 );
                                        JCEAlgorithm._nscontext = XMLUtils.createXSecCtx( doc, "x",
                        Constant.NS_XDSEC_CONFIG );
                }
            }
            catch ( Exception e ) {
                e.printStackTrace();
            }
        }     }
    
    protected static InputStream readConfigFile () {
        String sRtn = null;
        String str = null;
        File oFile;
        InputStream is = null;
        Log.enterFunc( "JCEAlgorithm", "readConfigFile" );
                        if ( sRtn == null ) {
            str = java.lang.System.getProperty( JCEAlgorithm.
                                                config_KeyOfSysProperty );
            if ( str != null ) {
                oFile = new File( str );
                if ( oFile.isFile() ) {
                    Log.setMsg( "Get the file from System Property" );
                    sRtn = str;
                    try {
                        is = new FileInputStream( sRtn );
                    }
                    catch ( FileNotFoundException ex ) {
                        ex.printStackTrace();
                    }
                }
            }
        }
                        if ( sRtn == null ) {
            
            try {
                str = Class.forName( Constant.JCE_MAP_CLASS ).
                    getProtectionDomain().
                    getCodeSource().getLocation().getPath();
            }
            catch ( ClassNotFoundException ex ) {
                str = null;
            }
            str = str + JCEAlgorithm.config_DefaultName;
            oFile = new File( str );
            if ( oFile.isFile() ) {
                Log.setMsg( "Get the file from the path of jar file" );
                sRtn = str;
                try {
                    is = new FileInputStream( sRtn );
                }
                catch ( FileNotFoundException ex ) {
                    ex.printStackTrace();
                }
            }
        }
        if ( sRtn == null ) {
            
            try {
                is = Class.forName( Constant.JCE_MAP_CLASS ).
                    getResourceAsStream(
                        JCEAlgorithm.config_DefaultName );
            }
            catch ( ClassNotFoundException ex ) {
                ex.printStackTrace();
            }
        }
        Log.exitFunc();
        return is;
    }
    
    protected static void addProvider ( String Id ) {
        try {
            if ( Security.getProvider( Id ) == null ) {
                Element providerElem = ( Element ) XPathAPI.selectSingleNode(
                    JCEAlgorithm._providerList,
                    "./x:Providers/x:Provider[@Id='" + Id + "']",
                    JCEAlgorithm._nscontext );
                String providerClass = providerElem.getAttribute( "Class" );
                java.security.Provider prov =
                    ( java.security.Provider ) Class.forName( providerClass )
                    .newInstance();
                java.security.Security.addProvider( prov );
            }
        }
        catch ( TransformerException ex ) {}
        catch ( ClassNotFoundException ex ) {}
        catch ( IllegalAccessException ex ) {}
        catch ( InstantiationException ex ) {}
    }
    
    protected static boolean isAlgAvailable ( String providerId, String algType,
                                              String algId ) {
                if ( providerId == null || providerId.equals( "" ) ) {
            return true;
        }
        try {
                        addProvider( providerId );
            Provider prov = Security.getProvider( providerId );
            if ( prov == null ) {                 return false;
            }
                                    if ( algId == null ) {
                return true;
            }
                        boolean available = false;
                        if ( prov.containsKey( algType + "." + algId )
                 || prov.containsKey( "Alg.Alias." + algType + "." + algId ) ) {
                available = true;
            }
                                                if ( ( available == false ) && algType.equals( "Cipher" ) ) {
                Cipher cipher = Cipher.getInstance( algId, providerId );
                if ( cipher != null ) {
                    available = true;
                }
            }
            return available;         }
        catch ( java.security.GeneralSecurityException ex ) {
            Log.setMsg( ex.getMessage() );
        }
        return false;
    }
    
    protected static void parseURItoJCEID ( JCEAlgorithm alg,
                                            String algorithmURI ) {
        try {
            
            Log.setStartTime( "getProviderNodeList" );
                        
            NodeList elt_Algorithms = JCEAlgorithm._providerList.
                getElementsByTagNameNS(
                    Constant.NS_XDSEC_CONFIG,
                    "Algorithms" );
            Vector providers = new Vector();
            boolean find = false;
            for ( int i = 0; i < elt_Algorithms.getLength(); i++ ) {
                                NodeList elt_Algorithm = ( ( Element ) elt_Algorithms.item( i ) ).
                    getElementsByTagNameNS(
                        Constant.NS_XDSEC_CONFIG,
                        "Algorithm" );
                for ( int j = 0; j < elt_Algorithm.getLength(); j++ ) {
                    Element focusElt = ( Element ) elt_Algorithm.item( j );
                    String strURI = focusElt.getAttribute( "URI" );
                    if ( strURI != null
                         && strURI.equalsIgnoreCase( algorithmURI ) ) {
                                                NodeList providerNodes = focusElt.
                            getElementsByTagNameNS(
                                Constant.NS_XDSEC_CONFIG,
                                "ProviderAlgo" );
                        if ( providerNodes != null &&
                             providerNodes.getLength() != 0 ) {
                            for ( int m = 0; m < providerNodes.getLength(); m++ ) {
                                providers.addElement( providerNodes.item( m ) );
                            }
                            find = true;
                            break;
                        }
                    }                 }                 if ( find ) {
                    break;
                }
            }
            Log.setEndTime( "getProviderNodeList" );
                        Log.setStartTime( "find the first available jce privider" );
                                    for ( int i = 0; i < providers.size(); i++ ) {
                Element pro = ( Element ) providers.elementAt( i );
                                String jcename = pro.getAttribute( "JCEName" );
                String providerid = pro.getAttribute( "ProviderId" );
                                if ( JCEAlgorithm.isAlgAvailable( providerid, alg.getAlgType(),
                                                  jcename ) ) {
                    alg.mprovider = providerid;
                    JCEAlgorithm.addProvider( providerid );
                    alg.malgName = jcename;
                    break;
                }
            }
            Log.setEndTime( "find the first available jce privider" );
        }
        catch ( Exception ex ) {
        }
                    }
    
    protected static void parseURItoJCEID ( JCEAlgorithm alg,
                                            String AlgorithmURI,
                                            String requestedProviderId ) {
        try {
                        
            Element pro = null;
            NodeList elt_Algorithms = JCEAlgorithm._providerList.
                getElementsByTagNameNS(
                    Constant.NS_XDSEC_CONFIG,
                    "Algorithms" );
            boolean find = false;
            for ( int i = 0; i < elt_Algorithms.getLength(); i++ ) {
                                NodeList elt_Algorithm = ( ( Element ) elt_Algorithms.item( i ) ).
                    getElementsByTagNameNS(
                        Constant.NS_XDSEC_CONFIG,
                        "Algorithm" );
                for ( int j = 0; j < elt_Algorithm.getLength(); j++ ) {
                    Element focusElt = ( Element ) elt_Algorithm.item( j );
                    String strURI = focusElt.getAttribute( "URI" );
                    if ( strURI != null
                         && strURI.equalsIgnoreCase( AlgorithmURI ) ) {
                                                NodeList providerNodes = focusElt.
                            getElementsByTagNameNS(
                                Constant.NS_XDSEC_CONFIG,
                                "ProviderAlgo" );
                        if ( providerNodes != null &&
                             providerNodes.getLength() != 0 ) {
                            for ( int m = 0; m < providerNodes.getLength(); m++ ) {
                                String providerId = ( ( Element ) providerNodes.
                                    item( m ) )
                                    .getAttribute( "ProviderId" );
                                if ( providerId != null &&
                                     providerId.equalsIgnoreCase(
                                         requestedProviderId ) ) {
                                    pro = ( Element ) providerNodes.item( m );
                                    find = true;
                                    break;
                                }
                            }
                            if ( find ) {
                                break;
                            }
                        }
                    }                 }                 if ( find ) {
                    break;
                }
            }
            if ( pro != null ) {
                String jcename = pro.getAttribute( "JCEName" );
                                if ( JCEAlgorithm.isAlgAvailable( requestedProviderId,
                                                  alg.getAlgType(),
                                                  jcename ) ) {
                    alg.mprovider = requestedProviderId;
                    JCEAlgorithm.addProvider( requestedProviderId );
                    alg.malgName = jcename;
                }
            }
                    }
        catch ( Exception ex ) {
        }
    }
}
