package org.act.xdsec.extension.algorithm;
import java.security.*;
import javax.crypto.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class KeyTransportAlgorithmImpl
    extends KeyTransportAlgorithm {
    public byte[] encrypt ( byte[] keyData, Key key ) throws EncryptException {         try {
                        Cipher cipher = getCipherInstance();
            cipher.init( Cipher.ENCRYPT_MODE, key );
                        int emLen = cipher.getBlockSize();
            byte[] block = null;
            if ( this.getURI().equals( Constant.ALG_RSA15_URI ) ) {
                                block = this.encryptPKCS1( keyData, emLen );
            }
            else if ( this.getURI().equals( Constant.ALG_RSAOAEP_URI ) ) {
                                block = this.encryptOAEP( keyData, emLen );
            }
                        return cipher.doFinal( block, 0, block.length );
        }
        catch ( BadPaddingException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new EncryptException( "bad padding exception" );                     }
        catch ( IllegalBlockSizeException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new EncryptException( "illegal block size exception" );                     }
        catch ( InvalidKeyException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new EncryptException( "invalid key exception" );                     }
            }
    public byte[] decrypt ( byte[] keyData, Key key ) throws DecryptException {         byte[] block = null;
        try {
                        Cipher cipher = getCipherInstance();
            cipher.init( Cipher.DECRYPT_MODE, key );
                        block = cipher.doFinal( keyData, 0, keyData.length );
        }
        catch ( BadPaddingException ex ) {
            throw new DecryptException( "bad padding exception" );                     }
        catch ( IllegalBlockSizeException ex ) {
            throw new DecryptException( "illegale block size exception" );                     }
        catch ( InvalidKeyException ex ) {
            throw new DecryptException( "invalid key exception" );                     }
        if ( this.getURI().equals( Constant.ALG_RSAOAEP_URI ) ) {
            return this.decryptOAEP( block );
        }
        else if ( this.getURI().equals( Constant.ALG_RSA15_URI ) ) {
            return this.decryptPKCS1( block );
        }
        else {
            return null;
        }
    }
    
    public byte[] encrypt ( Key keyData, Key key ) throws EncryptException {         return this.encrypt( keyData.getEncoded(), key );
    }
    public Key generateKey () {
                return null;
    }
            
    public Key generateKey ( byte[] keyData ) {
                return null;
    }
    
    public byte[] encryptPKCS1 ( byte[] keyData, int emLen ) throws
        EncryptException {         return pkcsEncode( keyData, emLen );
    }
    public byte[] decryptPKCS1 ( byte[] keyData ) throws DecryptException {         return PKCS1Decode( keyData );
    }
    private byte[] pkcsEncode ( byte[] msg, int emLen ) throws EncryptException {         if ( msg.length > emLen - 10 ) {
            throw new EncryptException( "data too long exception" );                     }
        SecureRandom random = PRNG.getInstance().getSecureRandom();
                byte[] block = new byte[emLen];
        random.nextBytes( block );         block[0] = 0x02;                 for ( int i = 1; i < block.length - msg.length - 1; i++ ) {
            while ( block[i] == 0 ) {
                block[i] = ( byte ) random.nextInt();
            }
        }
        block[block.length - msg.length - 1] = 0x00;         System.arraycopy( msg, 0, block, block.length - msg.length, msg.length );
        return block;
    }
    private byte[] PKCS1Decode ( byte[] em ) throws DecryptException {         if ( em.length < 10 ) {
            throw new DecryptException( "invalid key exception" );                     }
                if ( em[0] != 0x02 ) {
            throw new DecryptException( "invalid key exception" );                     }
        int start;
        for ( start = 1; start != em.length; start++ ) {
            if ( em[start] == 0 ) {
                break;
            }
        }
        start++;                 if ( ( start >= em.length ) || ( start < 10 ) ) {
            throw new DecryptException( "invalid key exception" );                     }
        byte[] result = new byte[em.length - start];
        System.arraycopy( em, start, result, 0, result.length );
        return result;
    }
    
    
    public byte[] encryptOAEP ( byte[] keyData, int emLen ) throws
        EncryptException {                 String algHash = null;
        String str = null;
        try {
            Element eEncMethod = this.getParameter();                         
            Element digestMethodElt = ( Element ) eEncMethod.
                getElementsByTagNameNS(
                    Constant.NS_XMLENC_NAME,
                    "DigestMethod" ).item( 0 );
            Attr attrDigest = ( Attr ) digestMethodElt.getAttributeNode(
                "Algorithm" );
            algHash = attrDigest.getNodeValue();
            Element eOAEP = ( Element ) eEncMethod.getElementsByTagNameNS(
                Constant.NS_XMLENC_NAME,
                "OAEPparams" ).item( 0 );
                        str = XMLUtils.getTextValueOfElement( eOAEP );
            str = XMLUtils.stripWhitespace( str );         }
        catch ( Exception e ) {
            throw new EncryptException( "xml document access exception" );                     }
        return OAEPEncode( keyData, str.getBytes(), emLen, algHash );
    }
    public byte[] decryptOAEP ( byte[] keyData ) throws DecryptException {         String algHash = null;
        String str = null;
        try {
            Element eEncMethod = this.getParameter();                         
            Element digestMethodElt = ( Element ) eEncMethod.
                getElementsByTagNameNS(
                    Constant.NS_XMLENC_NAME,
                    "DigestMethod" ).item( 0 );
            Attr attrDigest = ( Attr ) digestMethodElt.getAttributeNode(
                "Algorithm" );
            algHash = attrDigest.getNodeValue();
            Element eOAEP = ( Element ) eEncMethod.getElementsByTagNameNS(
                Constant.NS_XMLENC_NAME,
                "OAEPparams" ).item( 0 );
                        str = XMLUtils.getTextValueOfElement( eOAEP );
            str = XMLUtils.stripWhitespace( str );         }
        catch ( Exception e ) {
            throw new DecryptException( "xml documnet access exception" );                     }
        return OAEPDecode( keyData, str.getBytes(), algHash );
    }
    
    private byte[] OAEPEncode ( byte[] msg, byte[] p, int emLen, String algHash ) throws
        EncryptException {                 DigestAlgorithmImpl digest = new DigestAlgorithmImpl();
        digest.setURI( algHash );
        int hLen = digest.getOutputLength();
        if ( msg.length > ( emLen - 2 * hLen - 1 ) ) {
            throw new EncryptException( "data too long exception" );
                    }
        byte[] ps = new byte[ ( emLen - msg.length - 2 * hLen - 1 )];
        for ( int i = 0; i < ps.length; i++ ) {
            ps[i] = 0x0;
        }
        byte[] pHash = digest.digest( p );
        byte[] DB = new byte[pHash.length + ps.length + 1 + msg.length];
        System.arraycopy( pHash, 0, DB, 0, pHash.length );
        System.arraycopy( ps, 0, DB, pHash.length, ps.length );
        DB[pHash.length + ps.length] = 0x01;
        System.arraycopy( msg, 0, DB, pHash.length + ps.length + 1, msg.length );
        SecureRandom random = PRNG.getInstance().getSecureRandom();
        byte[] seed = new byte[hLen];
        random.nextBytes( seed );
        byte[] dbMask = maskGeneratorFunc1( seed, emLen - hLen, digest );
        byte[] maskedDB = new byte[DB.length];
        for ( int i = 0; i < DB.length; i++ ) {
            maskedDB[i] = ( byte ) ( DB[i] ^ dbMask[i] );
        }
        byte[] seedMask = maskGeneratorFunc1( maskedDB, hLen, digest );
        byte[] maskedSeed = new byte[seed.length];
        for ( int i = 0; i < seed.length; i++ ) {
            maskedSeed[i] = ( byte ) ( seed[i] ^ seedMask[i] );
        }
        byte[] em = new byte[maskedSeed.length + maskedDB.length];
        System.arraycopy( maskedSeed, 0, em, 0, maskedSeed.length );
        System.arraycopy( maskedDB, 0, em, maskedSeed.length, maskedDB.length );
        return em;
    }
    
    private byte[] OAEPDecode ( byte[] em, byte[] p, String algHash ) throws
        DecryptException {                 DigestAlgorithmImpl digest = new DigestAlgorithmImpl();
        digest.setURI( algHash );
        int hLen = digest.getOutputLength();
        if ( em.length < ( 2 * hLen + 1 ) ) {
            throw new DecryptException( "oaep decode exception" );                     }
        byte[] maskedSeed = new byte[hLen];
        byte[] maskedDB = new byte[em.length - hLen];
        System.arraycopy( em, 0, maskedSeed, 0, hLen );
        System.arraycopy( em, hLen, maskedDB, 0, em.length - hLen );
        byte[] seedMask = maskGeneratorFunc1( maskedDB, hLen, digest );
        byte[] seed = new byte[maskedSeed.length];
        for ( int i = 0; i < seed.length; i++ ) {
            seed[i] = ( byte ) ( maskedSeed[i] ^ seedMask[i] );
        }
        byte[] dbMask = maskGeneratorFunc1( seed, em.length - hLen, digest );
        byte[] DB = new byte[maskedDB.length];
        for ( int i = 0; i < DB.length; i++ ) {
            DB[i] = ( byte ) ( maskedDB[i] ^ dbMask[i] );
        }
        byte[] pHash = digest.digest( p );
        byte[] pHash2 = new byte[hLen];
        int index = 0;
        while ( index < DB.length ) {
            if ( DB[index] == 0x01 ) {
                break;
            }
            index++;
        }
        index++;
        if ( index == DB.length ) {             throw new DecryptException( "oaep decode exception" );                     }
        byte[] msg = new byte[DB.length - index];
        System.arraycopy( DB, index, msg, 0, DB.length - index );
        for ( int i = 0; i < pHash.length; i++ ) {
            if ( pHash[i] != pHash2[i] ) {
                throw new DecryptException( "oaep decode exception" );                             }
        }
        return msg;
    }
    
    static void ItoOSP ( int i, byte[] sp ) {
        sp[0] = ( byte ) ( i >>> 24 );
        sp[1] = ( byte ) ( i >>> 16 );
        sp[2] = ( byte ) ( i >>> 8 );
        sp[3] = ( byte ) ( i >>> 0 );
    }
    
    
    static byte[] maskGeneratorFunc1 (
        byte[] Z, int length, DigestAlgorithm Hash ) {
        int hLen = Hash.getOutputLength();
        byte[] mask = new byte[length];         byte[] C = new byte[4];
        int counter = 0;
        byte[] buf = new byte[Z.length + 4];
                do {
            ItoOSP( counter, C );
            System.arraycopy( Z, 0, buf, 0, Z.length );
            System.arraycopy( C, 0, buf, Z.length, 4 );
            byte hashBuf[] = Hash.digest( buf );
            System.arraycopy( hashBuf, 0, mask, counter * hLen, hLen );
        }
        while ( ++counter < ( length / hLen ) );
        if ( ( counter * hLen ) < length ) {
            ItoOSP( counter, C );
            System.arraycopy( Z, 0, buf, 0, Z.length );
            System.arraycopy( C, 0, buf, Z.length, 4 );
            byte hashBuf[] = Hash.digest( buf );
            System.arraycopy( hashBuf, 0, mask, counter * hLen,
                              mask.length - ( counter * hLen ) );
        }
        return mask;
    }
}
