package org.act.xdsec.extension.algorithm;
import java.security.*;
import javax.crypto.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

class SignatureAlgorithmImpl
    extends SignatureAlgorithm {
    
    public byte[] sign ( byte[] input, Key key ) throws InvalidKeyException,
        SignatureException {
        byte[] result = null;
        if ( getURI().equals( Constant.ALG_DSAWITHSHA1_URI ) ) {
            result = signWithRSAorDSA( input, key );
        }
        else if ( getURI().equals( Constant.ALG_RSAWITHSHA1_URI ) ) {
            result = signWithRSAorDSA( input, key );
        }
        else if ( getURI().equals( Constant.ALG_HMAC_URI ) ) {
            result = signWithHMac( input, key );
        }
        return result;
    }
    
    public boolean verify ( byte[] sigvalue, byte[] input, Key key ) throws
        InvalidKeyException, SignatureException {
        boolean result = false;
        if ( getURI().equals( Constant.ALG_DSAWITHSHA1_URI ) ) {
            result = verifyWithRSAorDSA( sigvalue, input, key );
        }
        else if ( getURI().equals( Constant.ALG_RSAWITHSHA1_URI ) ) {
            result = verifyWithRSAorDSA( sigvalue, input, key );
        }
        else if ( getURI().equals( Constant.ALG_HMAC_URI ) ) {
            result = verifyWithHMac( sigvalue, input, key );
        }
        return result;
    }
                
    protected byte[] signWithRSAorDSA ( byte[] input, Key key ) throws
        InvalidKeyException, SignatureException {
                Log.setStartTime( "signWithRSAorDSA_getInstance" );
        Signature sig = getSignatureInstance();
        Log.setEndTime( "signWithRSAorDSA_getInstance" );
                        sig.initSign( ( PrivateKey ) key );
                                sig.update( input );
                if ( getURI().equals( Constant.ALG_DSAWITHSHA1_URI ) ) {
                                    byte[] data = decodeRSfromASN1( sig.sign() );
                        return data;
        }
        else if ( getURI().equals( Constant.ALG_RSAWITHSHA1_URI ) ) {
            Log.setStartTime( "signWithRSAorDSA_sign" );
            byte[] data = sig.sign();
            Log.setEndTime( "signWithRSAorDSA_sign" );
            return data;
        }
        else {
            return null;
        }
    }
    
    public boolean verifyWithRSAorDSA ( byte[] sigvalue, byte[] input, Key key ) throws
        InvalidKeyException, SignatureException {
        Log.setStartTime( "verifyWithRSAorDSA_Instance" );
        Signature sig = getSignatureInstance();
        Log.setEndTime( "verifyWithRSAorDSA_Instance" );
        Log.setStartTime( "verifyWithRSAorDSA_init" );
        sig.initVerify( ( PublicKey ) key );
        Log.setEndTime( "verifyWithRSAorDSA_init" );
        Log.setStartTime( "verifyWithRSAorDSA_update" );
        sig.update( input );         Log.setEndTime( "verifyWithRSAorDSA_update" );
        if ( getURI().equals( Constant.ALG_DSAWITHSHA1_URI ) ) {
                        Log.setStartTime( "verifyWithRSAorDSA_verify" );
            boolean validate = sig.verify( encodeRSinASN1( sigvalue ) );
            Log.setEndTime( "verifyWithRSAorDSA_verify" );
            return validate;
        }
        else if ( getURI().equals( Constant.ALG_RSAWITHSHA1_URI ) ) {
            Log.setStartTime( "verifyWithRSAorDSA_verify" );
            boolean validate = sig.verify( sigvalue );
            Log.setEndTime( "verifyWithRSAorDSA_verify" );
            return validate;
        }
        else {                         return false;
        }
    }
    
    private Signature getSignatureInstance () {
        Signature sig = null;
                try {
            if ( this.getProvider() == null ) {
                sig = Signature.getInstance( this.getAlgId() );
            }
            else {
                sig = Signature.getInstance( this.getAlgId(), this.getProvider() );
            }
        }
        catch ( NoSuchAlgorithmException e ) {
        }
        catch ( NoSuchProviderException ex ) {
        }
        return sig;
    }
    
    static private byte[] encodeRSinASN1 ( byte[] rs ) {
        
                if ( rs.length != 40 ) {
            System.err.println( "encodeRSinASN1 : DEBUG: r and s: " + rs.length +
                                " Byte" );
            return null;
        }
        int rlength = 20;
        int slength = 20;
        if ( ( ( int ) rs[0] ) < 0 ) {
            rlength++;
        }
        if ( ( ( int ) rs[20] ) < 0 ) {
            slength++;
        }
        byte[] result = new byte[rlength + slength + 6];
        result[0] = 0x30;         result[1] = ( byte ) ( rlength + slength + 4 );         result[2] = 0x02;         result[3] = ( byte ) rlength;         int offset = 4 + rlength - 20;
                        System.arraycopy( rs, 0, result, offset, 20 );
        offset += 20;
        result[offset++] = 0x02;         result[offset++] = ( byte ) slength;                         offset += slength - 20;
        System.arraycopy( rs, 20, result, offset, 20 );
        return result;
            }
    
    static private byte[] decodeRSfromASN1 ( byte[] asn ) {
        int rlength = asn[3];
        int slength = asn[4 + rlength + 1];
        byte[] result = new byte[40];
                                                                                                                                
                int srcoffset = 4, destoffset = 0;
        if ( rlength > 20 ) {
            srcoffset += rlength - 20;         }
        else if ( rlength < 20 ) {
            destoffset += 20 - rlength;         }
        System.arraycopy( asn, srcoffset, result, destoffset,
                          rlength < 20 ? rlength : 20 );
        srcoffset = rlength + 6;
        destoffset = 20;
        if ( slength > 20 ) {
            srcoffset += slength - 20;         }
        else if ( slength < 20 ) {
            destoffset += 20 - slength;         }
        System.arraycopy( asn, srcoffset, result, destoffset,
                          slength < 20 ? slength : 20 );
                return result;
    }
    
    protected boolean verifyWithHMac ( byte[] sigValue, byte[] input, Key key ) throws
        InvalidKeyException, SignatureException {
        Log.setStartTime( "verifyWithHMac_verify" );
        byte[] veriValue = signWithHMac( input, key );
        Log.setEndTime( "verifyWithHMac_verify" );
        return MessageDigest.isEqual( veriValue, sigValue );
    }
    
    protected byte[] signWithHMac ( byte[] input, Key key ) throws
        InvalidKeyException, SignatureException {
        this.setAlgType( "Mac" );         Mac mac = null;
        byte[] result = null;
        try {
            Log.setStartTime( "signWithHMac_getInstance" );
            if ( this.getProvider() == null ) {
                mac = Mac.getInstance( this.getAlgId() );
            }
            else {
                mac = Mac.getInstance( this.getAlgId(), this.getProvider() );
            }
            Log.setEndTime( "signWithHMac_getInstance" );
            Log.setStartTime( "signWithHMac_init" );
            mac.init( key );
            Log.setEndTime( "signWithHMac_init" );
            Log.setStartTime( "signWithHMac_update" );
            mac.update( input );
            Log.setEndTime( "signWithHMac_update" );
            Log.setStartTime( "signWithHMac_doFinal" );
            result = mac.doFinal();
            Log.setEndTime( "signWithHMac_doFinal" );
            int iOutputLength = getHMACOutputLength();
            if ( ( iOutputLength > 0 ) && ( iOutputLength < 160 ) ) {
                result = reduceBitLength( result, iOutputLength );
            }
        }
        catch ( NoSuchAlgorithmException e ) {
                    }
        catch ( NoSuchProviderException ex ) {
                    }
        return result;
    }
    
    private int getHMACOutputLength () {
        int length = 0;
        if ( ( this.para != null ) && ( this.para.getChildNodes() != null )
             && ( this.para.getChildNodes().getLength() > 0 ) ) {
            try {
                                
                Element hmaclengthElt = ( Element )this.para.
                    getElementsByTagNameNS(
                        Constant.NS_XMLDSIG_NAME,
                        Constant.TAG_HMACOUTPUTLENGTH_NAME ).item( 0 );
                Text hmaclength = ( Text ) hmaclengthElt.getChildNodes().item(
                    0 );
                                if ( hmaclength != null ) {
                    length = Integer.parseInt( hmaclength.getData() );
                }
            }
            catch ( Exception ex ) {
                            }
        }
        return length;
    }
    
    private static byte[] reduceBitLength ( byte completeResult[], int length ) {
        int bytes = length / 8;
        int abits = length % 8;
        byte[] strippedResult = new byte[bytes + ( abits == 0 ? 0 : 1 )];
        System.arraycopy( completeResult, 0, strippedResult, 0, bytes );
        if ( abits > 0 ) {
            byte[] MASK = {
                ( byte ) 0x00, ( byte ) 0x80, ( byte ) 0xC0, ( byte ) 0xE0,
                ( byte ) 0xF0, ( byte ) 0xF8, ( byte ) 0xFC, ( byte ) 0xFE};
            strippedResult[bytes] = ( byte ) ( completeResult[bytes]
                                               & MASK[abits] );
        }
        return strippedResult;
    }
}



