package org.act.xdsec.extension.algorithm;
import java.io.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.util.*;


public class SymmetricKeyWrapAlgorithmImpl
    extends SymmetricKeyWrapAlgorithm {
    private static byte[] mIV = {
        ( byte ) 0x4a, ( byte ) 0xdd, ( byte ) 0xa2, ( byte ) 0x2c,
        ( byte ) 0x79, ( byte ) 0xe8, ( byte ) 0x21, ( byte ) 0x05};
    private static byte[] mAesIv = {
        ( byte ) 0xA6, ( byte ) 0xA6, ( byte ) 0xA6, ( byte ) 0xA6,
        ( byte ) 0xA6, ( byte ) 0xA6, ( byte ) 0xA6, ( byte ) 0xA6};
    
    public byte[] encrypt ( byte[] data, Key kek ) throws EncryptException {         if ( this.uri.equals( Constant.ALG_KW3DES_URI ) ) {             return this.kw3DES( data, kek );
        }
        else if ( this.uri.equals( Constant.ALG_KWAES128_URI ) ) {
                        if ( kek.getEncoded().length != 16 ) {
                throw new EncryptException( "invalid key, should be 128 bit" );                             }
            return kwAES( data, kek, Constant.ALG_AES128_URI );
        }
        else if ( this.uri.equals( Constant.ALG_KWAES192_URI ) ) {
                        if ( kek.getEncoded().length != 24 ) {
                throw new EncryptException( "invalid key, should be 192 bit" );                             }
            return kwAES( data, kek, Constant.ALG_AES192_URI );
        }
        else if ( this.uri.equals( Constant.ALG_KWAES256_URI ) ) {
                        if ( kek.getEncoded().length != 32 ) {
                throw new EncryptException( "invalid key, should be 256 bit" );                             }
            return kwAES( data, kek, Constant.ALG_AES256_URI );
        }
        else {
            return null;
        }
    }
    public byte[] decrypt ( byte[] data, Key kek ) throws DecryptException {         if ( this.uri.equals( Constant.ALG_KW3DES_URI ) ) {             return this.unkw3DES( data, kek );
        }
        else if ( this.uri.equals( Constant.ALG_KWAES128_URI ) ) {
                        if ( kek.getEncoded().length != 16 ) {
                throw new DecryptException( "invalid key, should be 128 bit" );                             }
            return unkwAES( data, kek, Constant.ALG_AES128_URI );
        }
        else if ( this.uri.equals( Constant.ALG_KWAES192_URI ) ) {
                        if ( kek.getEncoded().length != 24 ) {
                throw new DecryptException( "invalid key, should be 192 bit" );                             }
            return unkwAES( data, kek, Constant.ALG_AES192_URI );
        }
        else if ( this.uri.equals( Constant.ALG_KWAES256_URI ) ) {
                        if ( kek.getEncoded().length != 32 ) {
                throw new DecryptException( "invalid key, should be 256 bit" );                             }
            return unkwAES( data, kek, Constant.ALG_AES256_URI );
        }
        else {
            return null;
        }
    }
    private byte[] kwAES ( byte[] keyData, Key kek, String aesAlg ) throws
        EncryptException {                 if ( keyData.length <= 0 || keyData.length % 8 != 0 ) {
            throw new EncryptException( "invalid keydata length" );                     }
                try {
            int inLen = keyData.length;
            int n = inLen / 8;
            byte[] bufP = new byte[keyData.length];
            byte[] bufR = new byte[keyData.length];
            byte[] bufC = new byte[keyData.length + 8];
            byte[] bufB = new byte[16];
            byte[] bufA = new byte[8];
            Log.setStartTime( "kwAES_getInstance" );
                        Cipher aesCipher = getCipherInstance();
            Log.setEndTime( "kwAES_getInstance" );
                        Log.setStartTime( "kwAES_init" );
            aesCipher.init( Cipher.ENCRYPT_MODE, kek );
            Log.setEndTime( "kwAES_init" );
            System.arraycopy( keyData, 0, bufP, 0, keyData.length );
                                                            if ( n == 1 ) {
                Log.setStartTime( "kwAES_Wrap" );
                byte[] buf = new byte[16];
                System.arraycopy( mAesIv, 0, buf, 0, mAesIv.length );
                System.arraycopy( bufP, 0, buf, mAesIv.length,
                                  bufP.length );
                byte[] result = aesCipher.doFinal( buf );
                Log.setEndTime( "kwAES_Wrap" );
                return result;
            }
                        if ( n > 1 ) {
                                                                                System.arraycopy( mAesIv, 0, bufA, 0, bufA.length );
                System.arraycopy( bufP, 0, bufR, 0, bufP.length );
                                                                                                                                Log.setStartTime( "kwAES_Wrap" );
                int t = 0;
                for ( int j = 0; j <= 5; j++ ) {
                    for ( int i = 1; i <= n; i++ ) {
                        t = i + j * n;
                        System.arraycopy( bufA, 0, bufB, 0, bufA.length );
                        System.arraycopy( bufR, 8 * ( i - 1 ), bufB,
                                          bufA.length,
                                          8 );
                        byte buf[] = aesCipher.doFinal( bufB );
                        System.arraycopy( buf, 0, bufB, 0, bufB.length );
                        for ( int k = 7; k >= 0; k-- ) {
                            byte v = ( byte ) t;
                            bufA[k] = ( byte ) ( v ^ bufB[k] );
                            t >>>= 8;
                        }
                        System.arraycopy( bufB, 8, bufR, 8 * ( i - 1 ), 8 );
                    }
                }
                                                                                System.arraycopy( bufA, 0, bufC, 0, 8 );
                System.arraycopy( bufR, 0, bufC, 8, bufR.length );
                Log.setEndTime( "kwAES_Wrap" );
                return bufC;
            }
        }
        catch ( InvalidKeyException ex ) {
            throw new EncryptException( "invalid key exception" );                     }
        catch ( javax.crypto.BadPaddingException ex ) {
            throw new EncryptException( "bad padding exception" );                     }
        catch ( IllegalBlockSizeException ex ) {
            throw new EncryptException( "illegal block size exception" );                     }
        return null;
    }
    private byte[] kw3DES ( byte[] data, Key kek ) throws EncryptException {         if ( !this.uri.equals( Constant.ALG_KW3DES_URI ) ) {             return null;
        }
        try {
                                                            DigestAlgorithmImpl algHash = new DigestAlgorithmImpl();
            algHash.setURI( Constant.ALG_SHA1_URI );
            byte[] cks = JavaUtils.getBytes( algHash.digest( data ), 0, 8 );
                        byte[] wkcs = JavaUtils.combine2Bytes( data, cks );
                                                            Cipher cipher = getCipherInstance();
                        cipher.init( Cipher.ENCRYPT_MODE, kek );
                        byte[] iv = cipher.getIV();
            byte[] temp1 = cipher.doFinal( wkcs );
                        byte[] temp2 = JavaUtils.combine2Bytes( iv, temp1 );
                        byte[] temp3 = JavaUtils.reverseBytes( temp2 );
                                                AlgorithmParameters algp = this.getCipherParameters( mIV );
            cipher.init( Cipher.ENCRYPT_MODE, kek, algp );
            byte[] result = cipher.doFinal( temp3 );
            return result;
        }
        catch ( InvalidAlgorithmParameterException ex ) {
            throw new EncryptException( "invalid algorithm parameter exception" );         }
        catch ( BadPaddingException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new EncryptException( "bad padding exception" );         }
        catch ( IllegalBlockSizeException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new EncryptException( "illegal block size exception" );         }
        catch ( InvalidKeyException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new EncryptException( "invalid key exception" );         }
            }
    
    private byte[] unkwAES ( byte[] keyData, Key kek, String aesAlg ) throws
        DecryptException {                 if ( keyData.length <= 8 || keyData.length % 8 != 0 ) {
            throw new DecryptException( "invalid keydata length" );                     }
                try {
            int inLen = keyData.length;
            int n = inLen / 8 - 1;             byte[] bufP = new byte[keyData.length - 8];
            byte[] bufR = new byte[keyData.length - 8];
            byte[] bufC = new byte[keyData.length];
            byte[] bufB = new byte[16];
            byte[] bufA = new byte[8];
                        Log.setStartTime( "unkwAES_getInstance" );
            Cipher aesCipher = getCipherInstance();
            Log.setEndTime( "unkwAES_getInstance" );
                        Log.setStartTime( "unkwAES_init" );
            aesCipher.init( Cipher.DECRYPT_MODE, kek );
            Log.setEndTime( "unkwAES_init" );
            System.arraycopy( keyData, 0, bufC, 0, keyData.length );
                                                if ( n == 1 ) {
                Log.setStartTime( "unkwAES_UnWrap" );
                byte[] buf = aesCipher.doFinal( bufC );
                System.arraycopy( aesCipher.doFinal( bufC ), 0, bufB, 0,
                                  bufB.length );
                System.arraycopy( bufB, 8, bufP, 0, bufP.length );
                                                int ii = 0;
                for ( ii = 0; ii < 8; ii++ ) {
                    if ( bufB[ii] != mAesIv[ii] ) {
                        break;
                    }
                }
                if ( ii < 8 ) {
                    throw new DecryptException( "decrypt data check fail" );                                     }
                Log.setEndTime( "unkwAES_UnWrap" );
                return bufP;
            }
                        if ( n > 1 ) {
                                                                                Log.setStartTime( "unkwAES_UnWrap" );
                System.arraycopy( bufC, 0, bufA, 0, bufA.length );
                System.arraycopy( bufC, 8, bufR, 0, bufR.length );
                                                                                                                                for ( int j = 5; j >= 0; j-- ) {
                    for ( int i = n; i >= 1; i-- ) {
                        int t = i + j * n;
                        for ( int k = 7; k >= 0; k-- ) {
                            byte v = ( byte ) t;
                            bufA[k] ^= v;
                            t >>>= 8;
                        }
                        System.arraycopy( bufA, 0, bufB, 0, 8 );
                        System.arraycopy( bufR, 8 * ( i - 1 ), bufB, 8, 8 );
                        System.arraycopy( aesCipher.doFinal( bufB ), 0, bufB, 0,
                                          bufB.length );
                        System.arraycopy( bufB, 0, bufA, 0, 8 );
                        System.arraycopy( bufB, 8, bufR, 8 * ( i - 1 ), 8 );
                    }
                }
                                                                System.arraycopy( bufR, 0, bufP, 0, bufR.length );
                                int index = 0;
                for ( index = 0; index < 8; index++ ) {
                    if ( bufA[index] != mAesIv[index] ) {
                        break;
                    }
                }
                if ( index < 8 ) {
                    throw new DecryptException( "decrypt data check fail" );                                     }
                Log.setEndTime( "unkwAES_UnWrap" );
                return bufP;
            }
        }
        catch ( InvalidKeyException ex ) {
            throw new DecryptException( "invalid key" );                     }
        catch ( javax.crypto.BadPaddingException ex ) {
            throw new DecryptException( "bad padding exception" );                     }
        catch ( IllegalBlockSizeException ex ) {
            throw new DecryptException( "illegal block size exception" );                     }
        return null;
    }
    private byte[] unkw3DES ( byte[] data, Key kek ) throws DecryptException {         if ( !this.uri.equals( Constant.ALG_KW3DES_URI ) ) {             return null;
        }
                                                if ( data.length != 40 ) {
                        return null;
        }
        try {
                                                Cipher cipher = getCipherInstance();
            AlgorithmParameters algp = this.getCipherParameters( mIV );
            cipher.init( Cipher.DECRYPT_MODE, kek, algp );
            byte[] temp3 = cipher.doFinal( data );
                        byte[] temp2 = JavaUtils.reverseBytes( temp3 );
                                    byte[] iv = new byte[8];
            byte[] temp1 = new byte[temp2.length - 8];
            JavaUtils.decompose2Bytes( temp2, iv, temp1, 8 );
                                    algp = this.getCipherParameters( iv );
            cipher.init( Cipher.DECRYPT_MODE, kek, algp );
            byte[] wkcs = cipher.doFinal( temp1 );
                                    byte[] cks = new byte[8];
            byte[] wk = new byte[wkcs.length - 8];
            JavaUtils.decompose2Bytes( wkcs, wk, cks, wk.length );
                                                DigestAlgorithmImpl algHash = new DigestAlgorithmImpl();
            algHash.setURI( Constant.ALG_SHA1_URI );
            byte[] cks2 = JavaUtils.getBytes( algHash.digest( wk ), 0, 8 );
            if ( JavaUtils.compare2Bytes( cks, cks2 ) != 0 ) {
                                return null;
            }
                        return wk;
        }
        catch ( InvalidAlgorithmParameterException ex ) {
            throw new DecryptException( "invalid algorithm parameter exception" );         }
        catch ( BadPaddingException ex ) {
            throw new DecryptException( "bad padding exception" );         }
        catch ( IllegalBlockSizeException ex ) {
            throw new DecryptException( "illegal block size excepiton" );         }
        catch ( InvalidKeyException ex ) {
            throw new DecryptException( "invalid key exception" );         }
            }
    
    public Key generateKey () {
        String salgname = this.getAlgId();
        int index = salgname.indexOf( "/" );
        if ( index > 0 ) {
            salgname = salgname.substring( 0, index );
        }
        Key key = null;
        try {
            KeyGenerator keyg = KeyGenerator.getInstance( salgname,
                this.getProvider() );
            key = keyg.generateKey();
                    }
        catch ( NoSuchProviderException ex ) {
        }
        catch ( NoSuchAlgorithmException ex ) {
        }
        return key;
    }
            
    public Key generateKey ( byte[] keyData ) {
        Key key = null;
        key = new SecretKeySpec( keyData, this.getAlgName() );
                return key;
    }
        private int getParamLength () {
        int length;
        if ( this.uri.equals( Constant.ALG_KW3DES_URI ) ) {
            length = 8;
        }
        else {
            length = -1;
        }
        return length;
    }
    private AlgorithmParameters getCipherParameters ( byte[] data ) {
                        int length = getParamLength();
        if ( length < 0 || length > data.length ) {
                        return null;
        }
        byte[] params = JavaUtils.getBytes( data, 0, length );
        AlgorithmParameters algParam = null;
        try {
            if ( this.getProvider() == null ) {
                algParam = AlgorithmParameters.getInstance( this.getAlgId() );
            }
            else {
                String salgname2 = this.getAlgId();
                int index = salgname2.indexOf( "/" );
                if ( index > 0 ) {
                    salgname2 = salgname2.substring( 0, index );
                }
                algParam = AlgorithmParameters.getInstance( salgname2,
                    this.getProvider() );
            }
            algParam.init( params );
        }
        catch ( NoSuchAlgorithmException e ) {
        }
        catch ( NoSuchProviderException ex ) {
        }
        catch ( IOException ex ) {
        }
        return algParam;
    }
}
