package org.act.xdsec.extension.algorithm.transform;
import java.io.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.util.*;
import org.apache.xpath.*;
import org.apache.xpath.compiler.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class TransformEnvelopedSignature
    extends TransformAlgorithm {
    
    static {
        FunctionTable.installFunction( "here", new FuncHere() );
    }
    
    public DataObject transform ( DataObject input, Element sig ) throws
        IOException, CanonicalizationException,
        TransformationException {
        try {
            
                        NodeList inputNodes = null;
            if ( input.getType().equals( Constant.DAT_TYPE_STREAM ) ) {
                inputNodes = XMLUtils.buildNodeSetFromStream( input.
                    getDataStream(), Constant.XPATH_C14N_WITH_COMMENTS );
            }
            else if ( input.getType().equals( Constant.DAT_TYPE_NODELIST ) ) {
                inputNodes = input.getNodeList();
            }
            if ( inputNodes.getLength() == 0 ) {
                Object exArgs[] = {
                    "input node set contains no nodes"};
                throw new TransformationException( "generic.EmptyMessage",
                    exArgs );
            }
                                                            HelperNodeList resultNodes = new HelperNodeList();
            
                        Document doc = getParameter().getOwnerDocument();
            Element nscontext = XMLUtils.createXSecCtx( doc, "ds",
                Constant.NS_XMLDSIG_NAME );
                        
                                    Log.setStartTime( "TransformEnveloped_selectNodeList" );
            NodeList sigNodes = XPathAPI.selectNodeList( sig,
                "(.//. | .//@*)" + "[not(self::comment())]" );
            Log.setEndTime( "TransformEnveloped_selectNodeList" );
            Log.setStartTime( "TransformEnveloped_transform" );
            for ( int i = 0; i < inputNodes.getLength(); i++ ) {
                Node currentContextNode = inputNodes.item( i );
                boolean issignode = false;
                for ( int j = 0; j < sigNodes.getLength(); j++ ) {
                    if ( currentContextNode.equals( sigNodes.item( j ) ) ) {
                        issignode = true;
                        break;
                    }
                }
                if ( !issignode ) {
                    resultNodes.appendChild( currentContextNode );
                }
            }
            Log.setEndTime( "TransformEnveloped_transform" );
            DataObject result = new DataObject( resultNodes );
            return result;
        }
        catch ( IOException ex ) {
            throw new TransformationException( "empty", ex );
        }
        catch ( SAXException ex ) {
            throw new TransformationException( "empty", ex );
        }
        catch ( TransformerException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new TransformationException( "empty", ex );
        }
        catch ( ParserConfigurationException ex ) {
            Log.setMsg( ex.getMessage() );
            throw new TransformationException( "empty", ex );
        }
    }
}
