package org.act.xdsec.extension.algorithm.transform;
import java.io.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.extension.algorithm.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class TransformXSLT
    extends TransformAlgorithm {
    
    public DataObject transform ( DataObject input, Element sig ) throws
        IOException, CanonicalizationException,
        TransformationException {
        try {
                        
            TransformerFactory tFactory = TransformerFactory.newInstance();
                        byte[] data = null;
            if ( input.getType().equals( Constant.DAT_TYPE_STREAM ) ) {
                                input.getDataStream().read( data );
            }
            else if ( input.getType().equals( Constant.DAT_TYPE_NODELIST ) ) {
                data = DOMWriter.nodeToString( input.getNodeList().item( 0 ) ).
                    getBytes();
            }
                        Source xmlSource =
                new StreamSource( new ByteArrayInputStream( data ) );
                        
            
            
            Source stylesheet;
            Element transformElement = getParameter();             Document doc = transformElement.getOwnerDocument();
            Element nscontext = XMLUtils.createXSecCtx( doc, "xslt",
                Constant.NS_XSLT_NSNAME );
            
                                    
            NodeList xsltNodeList = transformElement.getElementsByTagNameNS(
                Constant.NS_XSLT_NSNAME,
                Constant.TAG_STYLESHEET_NAME );
            Element xsltElement = null;
            if ( xsltNodeList == null || xsltNodeList.getLength() == 0 ) {
                Object exArgs[] = {
                    "xslt:stylesheet", "Transform"};
                throw new TransformationException( "xml.WrongContent", exArgs );
            }
            else {
                xsltElement = ( Element ) xsltNodeList.item( 0 );
            }
                        
            {
                ByteArrayOutputStream os = new ByteArrayOutputStream();
                Transformer transformer = tFactory.newTransformer();
                DOMSource source = new DOMSource( ( Node ) xsltElement );
                StreamResult result = new StreamResult( os );
                transformer.transform( source, result );
                stylesheet =
                    new StreamSource( new ByteArrayInputStream( os.toByteArray() ) );
            }
                        Transformer transformer = tFactory.newTransformer( stylesheet );
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            StreamResult outputTarget = new StreamResult( baos );
                        transformer.transform( xmlSource, outputTarget );
            return new DataObject( baos.toByteArray() );
        }
        catch ( TransformerConfigurationException ex ) {
            Object exArgs[] = {
                ex.getMessage()};
            throw new TransformationException( "generic.EmptyMessage", exArgs,
                                               ex );
        }
        catch ( TransformerException ex ) {
            ex.printStackTrace();
            Object exArgs[] = {
                ex.getMessage()};
            throw new TransformationException( "generic.EmptyMessage", exArgs,
                                               ex );
        }
    }
}
