package org.act.xdsec.extension.keyinfo;
import java.security.*;
import java.util.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;


public class KeyInfoResolver {
    
    private X509DataProcessor x509Processor;
    private KeyNameProcessor keynameProcessor;
    private KeyValueProcessor keyvalueProcessor;
    private PGPDataProcessor pgpProcessor;
    private MGMTDataProcessor mgmtProcessor;
    private SPKIDataProcessor spkiProcessor;
    private RetrievalMethodProcessor retProcessor;
    private Vector keyProcessors;
    
    public Key processKeyInfo ( Element keyInfo, boolean ispublic ) {
        NodeList nodeKeys;
        Node nodeKey;
        Key key = null;
        Log.setMsg( "KeyInfo.processKeyInfo, in ..." );
        try {             nodeKeys = keyInfo.getChildNodes();
            for ( int i = 0; i < nodeKeys.getLength(); i++ ) {
                nodeKey = nodeKeys.item( i );
                if ( nodeKey instanceof Element ) {
                                        String tagName = ( ( Element ) nodeKey ).getLocalName();
                    if ( tagName.equals( Constant.TAG_KEYNAME_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, keyname" );
                        key = processKeyName( nodeKey, ispublic );
                    }
                    else if ( tagName.equals( Constant.TAG_KEYVALUE_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, key value" );
                        key = processKeyValue( nodeKey, ispublic );
                    }
                    else if ( tagName.equals( Constant.TAG_MGMTDATA_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, Mgmt data" );
                        key = processMgmt( nodeKey, ispublic );
                    }
                    else if ( tagName.equals( Constant.TAG_X509DATA_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, X509Data" );
                        key = processX509( nodeKey, ispublic );
                    }
                    else if ( tagName.equals( Constant.TAG_SPKIDATA_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, SPKIData" );
                        key = processSpki( nodeKey, ispublic );
                    }
                    else if ( tagName.equals( Constant.TAG_PGPDATA_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, PGPData" );
                        key = processPgp( nodeKey, ispublic );
                    }
                    else if ( tagName.equals( Constant.TAG_RETIREVALMETHOD_NAME ) ) {
                        Log.setMsg( "KeyInfo.processKeyInfo, Retrieval Method" );
                        key = processRet( nodeKey, ispublic );
                    }
                    else {
                        Log.setMsg( "KeyInfo.processKeyInfo, AnyKey" );
                        key = processAny( nodeKey, ispublic );
                    }
                }
                else if ( nodeKey instanceof Text ) {
                                        Log.setMsg( "KeyInfo.processKeyInfo, Text representing key" );
                    key = processAny( nodeKey, ispublic );
                }
                if ( key != null ) {
                    Log.setMsg( "KeyInfo.processKeyInfo, get key" );
                    break;
                }
                            }
            Log.setMsg( "KeyInfo.processKeyInfo, exit ..." );
            return key;
        }
        catch ( Exception ex ) {
            return null;         }
    }
    
    public void addKeyProcessor ( KeyProcessor processor ) {
        if ( this.keyProcessors == null ) {
            keyProcessors = new Vector();
        }
        this.keyProcessors.add( processor );
    }
    
    public void setX509DataProcessor ( X509DataProcessor processor ) {
        this.x509Processor = processor;
    }
    
    public void setKeyNameProcessor ( KeyNameProcessor processor ) {
        this.keynameProcessor = processor;
    }
    
    public void setPGPDataProcessor ( PGPDataProcessor processor ) {
        this.pgpProcessor = processor;
    }
    
    public void setSPKIProcessor ( SPKIDataProcessor processor ) {
        this.spkiProcessor = processor;
    }
    
    public void setMGMTDataProcessor ( MGMTDataProcessor processor ) {
        this.mgmtProcessor = processor;
    }
    public void setRetrievalMethodProcessor ( RetrievalMethodProcessor
                                              processor ) {
        this.retProcessor = processor;
    }
    private Key processKeyName ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        Log.setMsg( "KeyInfo.processKeyKeyName, in ..." );
        if ( this.keynameProcessor == null ) {
            try {
                this.keynameProcessor = ( KeyNameProcessor ) Class.forName(
                    Constant.DFT_KEY_KEYNAMEPROCESSOR ).newInstance();
                Log.setMsg(
                    "KeyInfo.processKeyKeyName, create default keyname processor" );
            }
            catch ( Exception e ) {
                                Log.setMsg( "KeyInfo.processKeyKeyName, catch Exception" );
                return null;
            }
        }
        Key key = this.keynameProcessor.getKey( nodeKey, ispublic );
        if ( key == null ) {
            Log.setMsg( "KeyInfo.processKeyKeyName, key is null" );
        }
        else {
            Log.setMsg( "KeyInfo.processKeyKeyName, get valid key" );
        }
        Log.setMsg( "KeyInfo.processKeyKeyName, exit ..." );
        return key;
    }
    private Key processKeyValue ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        Log.setMsg( "KeyInfo.processKeyKeyValue, in ..." );
        if ( this.keyvalueProcessor == null ) {
            try {
                this.keyvalueProcessor = ( KeyValueProcessor ) Class.forName(
                    Constant.DFT_KEY_KEYVALUEPROCESSOR ).newInstance();
                Log.setMsg(
                    "KeyInfo.processKeyKeyValue, create default keyvalue processor" );
            }
            catch ( Exception e ) {
                                Log.setMsg( "KeyInfo.processKeyKeyValue, catch Exception" );
                return null;
            }
        }
        Key key = this.keyvalueProcessor.getKey( nodeKey, ispublic );
        if ( key == null ) {
            Log.setMsg( "KeyInfo.processKeyKeyValue, key is null" );
        }
        else {
            Log.setMsg( "KeyInfo.processKeyKeyValue, get valid key" );
        }
        Log.setMsg( "KeyInfo.processKeyKeyValue, exit ..." );
        return key;
    }
    private Key processMgmt ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        return this.mgmtProcessor.getKey( nodeKey, ispublic );
    }
    private Key processX509 ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        return this.x509Processor.getKey( nodeKey, ispublic );
    }
    private Key processSpki ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        return this.spkiProcessor.getKey( nodeKey, ispublic );
    }
    private Key processPgp ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        return this.keynameProcessor.getKey( nodeKey, ispublic );
    }
    private Key processRet ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        return this.retProcessor.getKey( nodeKey, ispublic );
    }
        private Key processAny ( Node nodeKey, boolean ispublic ) throws
        InvalidKeyException, NoSuchAlgorithmException {
        if ( this.keyProcessors != null ) {
            for ( int i = 0; i < keyProcessors.size(); i++ ) {
                KeyProcessor processor = ( KeyProcessor ) keyProcessors.
                    elementAt(
                        i );
                Key rtn = processor.getKey( nodeKey, ispublic );
                if ( rtn != null ) {
                    return rtn;
                }
            }
        }
        return null;
    }
}
