package org.act.xdsec.extension.keyinfo;
import java.math.*;
import java.security.*;
import java.security.spec.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class KeyValueProcessorImpl
    implements KeyValueProcessor {
    public Key getKey ( Node node, boolean ispublic ) throws
        InvalidKeyException,
        NoSuchAlgorithmException {
        Key key = null;
        Log.setMsg( "DefaultKeyValueProcessor.getKey, in ..." );
        if ( node == null || ! ( node instanceof Element ) ) {
            return null;
        }
        NodeList children = node.getChildNodes();
        for ( int i = 0; i < children.getLength(); i++ ) {
            Node child = children.item( i );
            if ( child instanceof Element ) {
                                String type = child.getLocalName();
                if ( type.equals( Constant.TAG_DSAKEY_NAME ) ) {
                    try {
                        Log.setMsg(
                            "DefaultKeyValueProcessor.getKey, process DSA Key" );
                        key = getDSAKey( ( Element ) child );
                    }
                    catch ( InvalidKeySpecException ex ) {
                        throw new InvalidKeyException( ex.getMessage() );
                    }
                    Log.setMsg( "DefaultKeyValueProcessor.getKey, get DSA Key" );
                    break;
                }
                if ( type.equals( Constant.TAG_RSAKEY_NAME ) ) {
                    try {
                        Log.setMsg(
                            "DefaultKeyValueProcessor.getKey, process rsakey" );
                        key = getRSAKey( ( Element ) child );
                    }
                    catch ( InvalidKeySpecException ex ) {
                        throw new InvalidKeyException( ex.getMessage() );
                    }
                    Log.setMsg( "DefaultKeyValueProcessor.getKey, get rsakey" );
                    break;
                }
            }
        }
        Log.setMsg( "DefaultKeyValueProcessor.getKey, exit ..." );
        return key;
    }
    Key getRSAKey ( Element rsakv ) throws InvalidKeySpecException,
        NoSuchAlgorithmException {
        BigInteger modulus = null, exponent = null;
                        Element el = XMLUtils.getElementInXMLSigNS( rsakv,
            Constant.TAG_RSAKEY_MODULUS );
        if ( el == null ) {
            throw new InvalidKeySpecException(
                "No Modulus element in the RSAKeyValue element." );
        }
        if ( !XMLUtils.isXmldsig( el.getNamespaceURI() ) ) {
            throw new InvalidKeySpecException( "Unknown element: " +
                                               el.getNodeName() );
        }
        modulus = new BigInteger( 1,
                                  JavaUtils.Base64Decode( XMLUtils.
            getTextValueOfElement( el ) ) );
                        el = XMLUtils.getElementInXMLSigNS( rsakv, Constant.TAG_RSAKEY_EXPONENT );
        if ( el == null ) {
            throw new InvalidKeySpecException(
                "No Modulus element in the RSAKeyValue element." );
        }
        if ( !XMLUtils.isXmldsig( el.getNamespaceURI() ) ) {
            throw new InvalidKeySpecException( "Unknown element: " +
                                               el.getNodeName() );
        }
        exponent = new BigInteger( 1,
                                   JavaUtils.Base64Decode( XMLUtils.
            getTextValueOfElement( el ) ) );
        KeyFactory factory = KeyFactory.getInstance( "RSA" );
        return factory.generatePublic( new RSAPublicKeySpec( modulus, exponent ) );
    }
    
    Key getDSAKey ( Element dsakv ) throws InvalidKeySpecException,
        NoSuchAlgorithmException {
        BigInteger p = null, q = null, g = null, y = null;
        for ( Node node = XMLUtils.getFirstChild2( dsakv );
              node != null;
              node = XMLUtils.getNextSibling2( node ) ) {
            if ( node.getNodeType() != Node.ELEMENT_NODE ) {
                continue;
            }
            Element el = ( Element ) node;
            if ( !XMLUtils.isXmldsig( XMLUtils.getNamespace( el ) ) ) {
                throw new InvalidKeySpecException( "Unknown element: " +
                    el.getNodeName() );
            }
            String local = XMLUtils.getLocalPart( el );
            if ( local.equals( "P" ) ) {
                p = new BigInteger( 1,
                                    JavaUtils.Base64Decode( XMLUtils.
                    getTextValueOfElement( el ) ) );
            }
            else if ( local.equals( "Q" ) ) {
                q = new BigInteger( 1,
                                    JavaUtils.Base64Decode( XMLUtils.
                    getTextValueOfElement( el ) ) );
            }
            else if ( local.equals( "G" ) ) {
                g = new BigInteger( 1,
                                    JavaUtils.Base64Decode( XMLUtils.
                    getTextValueOfElement( el ) ) );
            }
            else if ( local.equals( "Y" ) ) {
                y = new BigInteger( 1,
                                    JavaUtils.Base64Decode( XMLUtils.
                    getTextValueOfElement( el ) ) );
            }
            else if ( local.equals( "J" ) ) {             }
            else if ( local.equals( "Seed" ) ) {             }
            else if ( local.equals( "PgenCounter" ) ) {             }
            else {
                throw new InvalidKeySpecException( "Unknown element: " +
                    el.getNodeName() );
            }
        }
        if ( p == null ) {
            throw new InvalidKeySpecException(
                "No P element in the DSAKeyValue element." );
        }
        if ( q == null ) {
            throw new InvalidKeySpecException(
                "No Q element in the DSAKeyValue element." );
        }
        if ( g == null ) {
            throw new InvalidKeySpecException(
                "No G element in the DSAKeyValue element." );
        }
        if ( y == null ) {
            throw new InvalidKeySpecException(
                "No Y element in the DSAKeyValue element." );
        }
        KeyFactory factory = KeyFactory.getInstance( "DSA" );
        return factory.generatePublic( new DSAPublicKeySpec( y, p, q, g ) );
    }
}
