package org.act.xdsec.extension.resource;
import java.io.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.util.*;
import org.apache.xpath.*;
import org.w3c.dom.*;


public class FragmentProcessorImpl
    implements FragmentProcessor {
    
    public DataObject process ( DataObject data, String fragment ) throws
        ResourceResolverException {
                        Document doc = null;
                if ( data.getType().equals( Constant.DAT_TYPE_BYTE )
             || data.getType().equals( Constant.DAT_TYPE_STREAM ) ) {
            try {
                                DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                dbf.setNamespaceAware( true );
                dbf.setValidating( true );
                DocumentBuilder db = dbf.newDocumentBuilder();
                if ( data.getType().equals( Constant.DAT_TYPE_STREAM ) ) {
                    doc = db.parse( new ByteArrayInputStream(
                        JavaUtils.getBytesFromStream( data.getDataStream() ) ) );
                }
                else {
                    doc = db.parse( new ByteArrayInputStream( data.getBytes() ) );
                }
            }
            catch ( Exception ex ) {
                throw new ResourceResolverException( "Unkown data type" );
            }
                    }
        else if ( data.getType().equals( Constant.DAT_TYPE_NODE ) ) {
            Node node = data.getDataNode();
            if ( node instanceof Document ) {
                doc = ( Document ) node;
            }
            else {
                throw new ResourceResolverException( "Unkown data type" );
            }
        }
        NodeList resultNodes = null;
                if ( fragment.equals( "" ) ) {             resultNodes = processNullId( doc );
        }
        else if ( fragment.startsWith( "#xpointer(" ) ) {             String idExpr = fragment.substring( 1 );             resultNodes = processXPointer( doc, idExpr );
        }
        else if ( fragment.startsWith( "#" ) ) {             String idExpr = fragment.substring( 1 );             resultNodes = processBarenameId( doc, idExpr );
        }
        DataObject result = new DataObject( resultNodes );
        return result;
    }
    protected NodeList processNullId ( Document doc ) throws
        ResourceResolverException {
        NodeList resultNodes = null;
        
        Log.setStartTime( "processNullId" );
        try {
            resultNodes = XPathAPI.selectNodeList( doc
                ,
                Constant.
                XPATH_C14N_OMIT_COMMENTS );
        }
        catch ( javax.xml.transform.TransformerException ex ) {
            throw new ResourceResolverException( "generic.EmptyMessage" );
        }
        Log.setEndTime( "processNullId" );
        return resultNodes;
    }
    
    protected NodeList processBarenameId ( Document doc, String idExpr ) throws
        ResourceResolverException {
        
                Element selectedElem = getElementById( doc, idExpr );
        NodeList resultNodes = null;
        if ( selectedElem != null ) {
            try {
                resultNodes = XPathAPI.selectNodeList( selectedElem
                    ,
                    Constant.XPATH_C14N_OMIT_COMMENTS_SINGLE_NODE );
            }
            catch ( javax.xml.transform.TransformerException ex ) {
                throw new ResourceResolverException( "generic.EmptyMessage" );
            }
        }
        return resultNodes;
    }
    public NodeList processXPointer ( Document doc, String xpointer ) throws
        ResourceResolverException {
        NodeList resultNodes = null;
        if ( isXPointerSlash( xpointer ) ) {
            try {
                resultNodes =
                    XPathAPI.selectNodeList( doc,
                                             Constant.XPATH_C14N_WITH_COMMENTS );
            }
            catch ( javax.xml.transform.TransformerException ex ) {
                throw new ResourceResolverException( "generic.EmptyMessage" );
            }
        }
        else if ( isXPointerId( xpointer ) ) {
            String id = getXPointerId( xpointer );
                        Element selectedElem = FragmentProcessorImpl.getElementById( doc,
                id );
            try {
                resultNodes = XPathAPI.selectNodeList( selectedElem
                    ,
                    Constant.XPATH_C14N_WITH_COMMENTS_SINGLE_NODE );
            }
            catch ( javax.xml.transform.TransformerException ex ) {
                throw new ResourceResolverException( "generic.EmptyMessage" );
            }
        }
        return resultNodes;
    }
    
    private boolean isXPointerSlash ( String idExpr ) {
        if ( idExpr.equals( "xpointer(/)" ) ) {
            return true;
        }
        return false;
    }
    
    private boolean isXPointerId ( String idExpr ) {
        if ( idExpr.startsWith( "xpointer(id(" )
             && idExpr.endsWith( "))" ) ) {
            String idPlusDelim = idExpr.substring( "xpointer(id(".length(),
                idExpr.length()
                - "))".length() );
            if ( ( ( idPlusDelim.charAt( 0 ) == '"' ) && ( idPlusDelim
                .charAt( idPlusDelim.length() -
                         1 ) == '"' ) ) || ( ( idPlusDelim
                                               .charAt( 0 ) == '\'' )
                                             && ( idPlusDelim
                                                  .charAt( idPlusDelim.length()
                - 1 ) == '\'' ) ) ) {
                return true;
            }
        }
        return false;
    }
    
    private String getXPointerId ( String idExpr ) {
        if ( idExpr.startsWith( "xpointer(id(" )
             && idExpr.endsWith( "))" ) ) {
            String idPlusDelim = idExpr.substring( "xpointer(id(".length(),
                idExpr.length()
                - "))".length() );
            if ( ( ( idPlusDelim.charAt( 0 ) == '"' ) && ( idPlusDelim
                .charAt( idPlusDelim.length() -
                         1 ) == '"' ) ) || ( ( idPlusDelim
                                               .charAt( 0 ) == '\'' )
                                             && ( idPlusDelim
                                                  .charAt( idPlusDelim.length()
                - 1 ) == '\'' ) ) ) {
                return idPlusDelim.substring( 1, idPlusDelim.length() - 1 );
            }
        }
        return null;
    }
    
    public static Element getElementById ( Document doc, String id ) {
        Element result = null;
        result = FragmentProcessorImpl.getElementByIdType( doc, id );
        if ( result != null ) {
            return result;
        }
        result = FragmentProcessorImpl.getElementByIdInDSNamespace( doc, id );
        if ( result != null ) {
            return result;
        }
        result = FragmentProcessorImpl.getElementByIdInSOAPSignatureNamespace(
            doc,
            id );
        if ( result != null ) {
            return result;
        }
        result = FragmentProcessorImpl.getElementByIdInWSUNamespace( doc, id );
        if ( result != null ) {
            return result;
        }
        result = FragmentProcessorImpl.getElementByIdUnsafeMatchByIdName( doc,
            id );
        if ( result != null ) {
            return result;
        }
        return null;
    }
    
    private static Element getElementByIdType ( Document doc, String id ) {
        return doc.getElementById( id );
    }
    
    private static Element getElementByIdInDSNamespace ( Document doc,
        String id ) {
        try {
            Element nscontext = XMLUtils.createDSctx( doc, "ds" );
            Element element = ( Element ) XPathAPI.selectSingleNode( doc,
                "//ds:*[@Id='" + id + "']", nscontext );
            return element;
        }
        catch ( TransformerException ex ) {
        }
        return null;
    }
    
    private static Element getElementByIdInSOAPSignatureNamespace ( Document
        doc,
        String id ) {
        try {
            Element nscontext = XMLUtils.createXSecCtx(
                doc, "SOAP-SEC",
                "http://schemas.xmlsoap.org/soap/security/2000-12" );
            Element element = ( Element ) XPathAPI.selectSingleNode( doc,
                "//*[@SOAP-SEC:id='" + id + "']", nscontext );
            return element;
        }
        catch ( TransformerException ex ) {
                    }
        return null;
    }
    
    private static Element getElementByIdInWSUNamespace ( Document doc,
        String id ) {
        try {
            Element nscontext = XMLUtils.createXSecCtx(
                doc, "wsu",
                "http://schemas.xmlsoap.org/ws/2002/07/utility" );
            Element element = ( Element ) XPathAPI.selectSingleNode( doc,
                "//*[@wsu:Id='" + id + "']", nscontext );
            return element;
        }
        catch ( TransformerException ex ) {
                    }
        return null;
    }
    private static Element getElementByIdUnsafeMatchByIdName ( Document doc,
        String id ) {
        try {
            Element element_Id = ( Element ) XPathAPI.selectSingleNode( doc,
                "//*[@Id='" + id + "']" );
            if ( element_Id != null ) {
                return element_Id;
            }
            Element element_ID = ( Element ) XPathAPI.selectSingleNode( doc,
                "//*[@ID='" + id + "']" );
            if ( element_ID != null ) {
                return element_ID;
            }
            Element element_id = ( Element ) XPathAPI.selectSingleNode( doc,
                "//*[@id='" + id + "']" );
            if ( element_id != null ) {
                return element_id;
            }
            Element element_iD = ( Element ) XPathAPI.selectSingleNode( doc,
                "//*[@iD='" + id + "']" );
            if ( element_iD != null ) {
                return element_iD;
            }
        }
        catch ( TransformerException ex ) {
        }
        return null;
    }
}
