
package org.act.xdsec.extension.resource;
import org.act.xdsec.core.com.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class ResourceResolver {
    private URIProcessor uriProcessor;
    private FragmentProcessor idProcessor;
    private NULLRefProcessor nullUriProcessor;
    public ResourceResolver () {
        this( null, null );
    }
    public ResourceResolver ( URIProcessor uriProcessor,
                              FragmentProcessor idProcessor ) {
        this( uriProcessor, idProcessor, null );
    }
    public ResourceResolver ( URIProcessor uriProcessor,
                              FragmentProcessor idProcessor,
                              NULLRefProcessor nullUriProcessor ) {
        if ( uriProcessor == null ) {
            this.uriProcessor = new URIProcessorImpl();
        }
        else {
            this.uriProcessor = uriProcessor;
        }
        if ( idProcessor == null ) {
            this.idProcessor = new FragmentProcessorImpl();
        }
        else {
            this.idProcessor = idProcessor;
        }
        this.nullUriProcessor = nullUriProcessor;
    }
    
    public DataObject processURI ( Attr uriAttr, String baseURI ) throws
        DataObjectNotFoundException {
        DataObject data;
        try {
                        if ( uriAttr == null ) {
                if ( this.nullUriProcessor != null ) {
                    return nullUriProcessor.getData( baseURI );
                }
                else {
                    throw new DataObjectNotFoundException();
                }
            }
                                    String uriRef = uriAttr.getNodeValue();
            if ( uriRef == null ) {
                uriRef = "";
            }
            String uri = uriRef;
                                    String fragment = null;
            int indexOfSharp = uriRef.indexOf( "#" );
            if ( indexOfSharp >= 0 ) {
                uri = uriRef.substring( 0, indexOfSharp );
                fragment = uriRef.substring( indexOfSharp );
            }
            else {
                if ( uri.equalsIgnoreCase( "" ) ) {
                    fragment = "";
                }
            }
            
            data = new DataObject();
            if ( uri.length() > 0 ) {
                                if ( this.uriProcessor == null ) {
                    this.uriProcessor = new URIProcessorImpl();
                }
                data.setDataStream( this.uriProcessor.process( uri, baseURI ) );
            }
            else {
                                data.setDataNode( uriAttr.getOwnerElement().getOwnerDocument() );
            }
                                                                                                Log.setStartTime( "idProcessor" );
            if ( fragment != null ) {
                if ( this.idProcessor == null ) {
                    this.idProcessor = new FragmentProcessorImpl();
                }
                data = idProcessor.process( data, fragment );
            }
            Log.setEndTime( "idProcessor" );
        }
        catch ( ResourceResolverException ex ) {
            throw new DataObjectNotFoundException();
        }
        if ( data == null ) {
            throw
                new DataObjectNotFoundException();
        }
        return data;
    }
    
    public void setIDProcessor ( FragmentProcessor idProcessor ) {
        this.idProcessor = idProcessor;
    }
    
    public void setNullURIProcessor ( NULLRefProcessor nullUriProcessor ) {
        this.nullUriProcessor = nullUriProcessor;
    }
    
    public void setURIProcessor ( URIProcessor processor ) {
        this.uriProcessor = processor;
    }
}
