package org.act.xdsec.extension.resource;
import java.io.*;
import java.net.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.util.*;
import org.apache.xml.utils.URI;

public class URIProcessorImpl
    implements URIProcessor {
    public InputStream process ( String uri, String baseURI ) throws
        ResourceResolverException {
                if ( isLocalFile( uri, baseURI ) ) {
            return processLocalFile( uri, baseURI );
        }
        else if ( isHTTP( uri, baseURI ) ) {
            return processHTTP( uri, baseURI );
        }
        return null;
    }
    private String proxyHost;
    private String proxyPort;
    private String proxyUser;
    private String proxyPass;
    public void setHttpProxy ( String host, String port ) {
        this.setHttpProxy( host, port, null, null );
    }
    public void setHttpProxy ( String host, String port, String username,
                               String pwd ) {
        this.proxyHost = host;
        this.proxyPort = port;
        this.proxyUser = username;
        this.proxyPass = pwd;
    }
    private boolean isHTTP ( String uri, String baseURI ) {
        try {
            URI uriNew = null;
            if ( baseURI == null || baseURI == "" ) {
                uriNew = new URI( uri );
            }
            else {
                uriNew = new URI( new URI( baseURI ), uri );
            }
                        if ( uriNew.getScheme().equals( "http" ) ) {
                return true;
            }
        }
        catch ( URI.MalformedURIException ex ) {}
        return false;
    }
    
    protected InputStream processHTTP ( String uri, String BaseURI ) throws
        ResourceResolverException {
        try {
            boolean useProxy = false;
            if ( ( proxyHost != null ) && ( proxyPort != null ) ) {
                useProxy = true;
            }
            String oldProxySet =
                ( String ) System.getProperties().get( "http.proxySet" );
                        String oldProxyHost =
                ( String ) System.getProperties().get( "http.proxyHost" );
                        String oldProxyPort =
                ( String ) System.getProperties().get( "http.proxyPort" );
            boolean switchBackProxy = ( ( oldProxySet != null )
                                        && ( oldProxyHost != null )
                                        && ( oldProxyPort != null ) );
                        if ( useProxy ) {
                System.getProperties().put( "http.proxySet", "true" );
                                System.getProperties().put( "http.proxyHost", proxyHost );
                                System.getProperties().put( "http.proxyPort", proxyPort );
            }
                        URI uriNew;
            if ( BaseURI == null || BaseURI == "" ) {
                uriNew = new URI( uri );
            }
            else {
                uriNew = new URI( new URI( BaseURI ), uri );
            }
                        URI uriNewNoFrag = new URI( uriNew );
            uriNewNoFrag.setFragment( null );
            URL url = new URL( uriNewNoFrag.toString() );
            HttpURLConnection a;
            {
                if ( ( proxyUser != null ) && ( proxyPass != null ) ) {
                    URLConnection connection = url.openConnection();
                    String password = proxyUser + ":" + proxyPass;
                    String encodedPassword = JavaUtils.Base64Encode( password.
                        getBytes() );
                                        connection.setRequestProperty( "Proxy-Authorization",
                        encodedPassword );
                }
            }
            URLConnection urlConnection = url.openConnection();
            String mimeType = urlConnection.getHeaderField( "Content-Type" );
            String contentLength = urlConnection.getHeaderField(
                "Content-Length" );
            InputStream result = urlConnection.getInputStream();
                        if ( switchBackProxy ) {
                System.getProperties().put( "http.proxySet", oldProxySet );
                                System.getProperties().put( "http.proxyHost", oldProxyHost );
                                System.getProperties().put( "http.proxyPort", oldProxyPort );
            }
            
            return result;
        }
        catch ( MalformedURLException ex ) {
            throw new ResourceResolverException( "generic.EmptyMessage" );
        }
        catch ( IOException ex ) {
            throw new ResourceResolverException( "generic.EmptyMessage" );
        }
    }
    private boolean isLocalFile ( String uri, String baseURI ) {
        try {
            URI uriNew = new URI( new URI( baseURI ), uri );
            if ( uriNew.getScheme().equals( "file" ) ) {
                return true;
            }
        }
        catch ( Exception e ) {}
        return false;
    }
    
    protected InputStream processLocalFile ( String uri, String BaseURI ) throws
        ResourceResolverException {
        try {
            URI uriNew = new URI( new URI( BaseURI ), uri );
                        URI uriNewNoFrag = new URI( uriNew );
            uriNewNoFrag.setFragment( null );
            String fileName =
                URIProcessorImpl.translateUriToFilename( uriNewNoFrag.toString() );
            FileInputStream result = new FileInputStream( fileName );
            return result;
        }
        catch ( Exception e ) {
            throw new ResourceResolverException( "" );
                    }
    }
    
    private static String translateUriToFilename ( String uri ) {
        String subStr = uri.substring( "file:/".length() );
        subStr.replace( '/', java.io.File.separatorChar );
        return subStr;
    }
}
