package org.act.xdsec.syntax.dsig;
import java.util.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class XAlgorithm {
    public String algType;
    public String uriReference;
    public Vector params;
        int keySize = -1;
    XAlgorithm xdigest;
    Node oaep;
    int hmacoutputlength = -1;
    public XAlgorithm () {
        this( null, null, null );
    }
    public XAlgorithm ( String algType ) {
        this( algType, null, null );
    }
    public XAlgorithm ( String algType, String uri ) {
        this( algType, uri, null );
    }
    public XAlgorithm ( String algType, String uri, NodeList params ) {
        setAlgorithmType( algType );
        setAlgorithmUri( uri );
        if ( params != null ) {
            for ( int i = 0; i < params.getLength(); i++ ) {
                this.params.add( params.item( i ) );
            }
        }
    }
    public void setAlgorithmType ( String algType ) {
        this.algType = algType;
    }
    public void setAlgorithmUri ( String uri ) {
        this.uriReference = uri;
    }
        public void setEncryptionMethod ( String uri, int keySize,
                                      XAlgorithm digest,
                                      Node oaepparams ) {
        this.setAlgorithmType( Constant.ENC_ENCRYPTIONMETHOD_TAG );
        this.setAlgorithmUri( uri );
        this.keySize = keySize;
        xdigest = digest;
        oaep = oaepparams;
    }
    public void processEncContent ( Document doc, Element parent ) {
        if ( this.keySize > 0 ) {
            parent.appendChild( XMLUtils.createTextElementInSig( doc,
                Constant.ENC_KEYSIZE_TAG, Integer.toString( keySize ) ) );
            parent.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        if ( this.xdigest != null ) {
            parent.appendChild( xdigest.toElement( doc ) );
            parent.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        if ( this.oaep != null ) {
            parent.appendChild( oaep );
            parent.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
    }
    public void addParam ( Node param ) {
                if ( this.params == null ) {
            params = new Vector();
        }
        params.add( param );
    }
    public Element toElement ( Document doc ) {
        Element alg;
        if ( this.hmacoutputlength > 0 ) {
            Element outputlength = XMLUtils.createElementInSignatureSpace( doc,
                Constant.TAG_HMACOUTPUTLENGTH_NAME );
            outputlength.appendChild( doc.createTextNode(
                String.valueOf( this.hmacoutputlength ) ) );
            this.addParam( outputlength );
        }
                        alg = XMLUtils.createElementInSignatureSpace( doc, this.algType );
        alg.setAttribute( Constant.ATT_ALGORITHM, this.uriReference );
        processEncContent( doc, alg );
        if ( params != null ) {
            alg.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            for ( int i = 0; i < params.size(); i++ ) {
                alg.appendChild( ( Node ) params.elementAt( i ) );
                alg.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            }
        }
        return alg;
    }
    public void setHMACOutputLength ( int length ) {
        if ( this.uriReference.equalsIgnoreCase( Constant.ALG_HMAC_URI ) ) {
            this.hmacoutputlength = length;
        }
    }
}
