
package org.act.xdsec.syntax.dsig;
import java.util.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class XReference {
        String id;
    String uri;
    String type;
    Vector xtrans;
    XAlgorithm digest;
    
    public XReference () {
        this( null, null, null );
    }
    
    public XReference ( String id ) {
        this( id, null, null );
    }
    
    public XReference ( String id, String uri ) {
        this( id, uri, null );
    }
    
    public XReference ( String id, String uri, String type ) {
                this.setId( id );
        this.setURI( uri );
        this.setType( type );
    }
                
    public void setId ( java.lang.String id ) {
        this.id = id;
    }
    
    public String getId () {
        return this.id;
    }
    
    public String getType () {
        return this.type;
    }
    
    public void setType ( String type ) {
        this.type = type;
    }
    
    public String getURI () {
        return this.uri;
    }
    
    public void setURI ( String uri ) {
        this.uri = uri;
    }
    
    public XTransform createTransform () {
        return new XTransform();
    }
    
    public int getTransformCount () {
        if ( this.xtrans == null ) {
            return 0;
        }
        else {
            return this.xtrans.size();
        }
    }
    
    public XTransform[] getTransforms () {
        return ( XTransform[] )this.xtrans.toArray();
    }
        
    public void addTransform ( XTransform transform ) {
        this.insertTransformAt( transform, -1 );
    }
    
    public void addXpathTransform ( String xpathExpr ) {
        XTransform transform;
        Element para;
        transform = new XTransform();
        transform.setXPath( xpathExpr );
        this.addTransform( transform );
    }
    
    public void addTransform ( String uri, NodeList params ) {
        XTransform transform;
        Element para;
        transform = new XTransform();
        transform.setURI( uri );
        if ( params != null ) {
            for ( int i = 0; i < params.getLength(); i++ ) {
                transform.addParam( params.item( i ) );
            }
        }
        this.addTransform( transform );
    }
    
    public void insertTransformAt ( XTransform transform, int position ) {
        if ( xtrans == null ) {
            xtrans = new Vector();
        }
        if ( position > xtrans.size() || position <= 0 ) {
            xtrans.add( transform );
        }
        else {
            this.xtrans.add( position, transform );
        }
    }
    
    public void removeTransformAt ( int position ) {
        if ( xtrans != null ) {
            xtrans.remove( position );
        }
    }
    
    public void setDigestMethod ( String uri ) {
        this.setDigestMethod( uri, null );
    }
    
    public void setDigestMethod ( String uri, NodeList params ) {
        this.digest = new XAlgorithm( Constant.TAG_DIGESTMETHOD_NAME, uri,
                                      params );
    }
    public Element toElement ( Document doc ) {
        if ( this.digest == null ) {
            this.setDigestMethod( Constant.DFT_ALG_DIGEST );
        }
        Element ref;
                        ref = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_REFERENCE_NAME );
        ref.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        if ( this.id != null ) {
            ref.setAttribute( Constant.ATT_ID, this.id );
        }
        if ( this.type != null ) {
            ref.setAttribute( Constant.ATT_TYPE, this.type );
        }
        if ( this.uri != null ) {
            ref.setAttribute( Constant.ATT_URI, this.uri );
        }
        if ( this.xtrans != null ) {
                                    Element el = XMLUtils.createElementInSignatureSpace( doc,
                Constant.TAG_TRANSFORMS_NAME );
            el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            for ( int i = 0; i < this.xtrans.size(); i++ ) {
                el.appendChild( ( ( XTransform ) xtrans.elementAt( i ) ).
                                toElement(
                                    doc ) );
                el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            }
            ref.appendChild( el );
            ref.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        if ( this.digest != null ) {
            ref.appendChild( digest.toElement( doc ) );
            ref.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
                        ref.appendChild( XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_DIGESTVALUE_NAME ) );
        ref.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        return ref;
    }
}
