

package org.act.xdsec.syntax.dsig;
import java.util.*;
import org.act.xdsec.syntax.keyinfo.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;


public class XSignature {
    Document doc;     String id;     String signedinfoid;
    XAlgorithm algSignature;
    XAlgorithm algCanolization;
    Vector xreferences;
    XKeyInfo xkeyinfo;
    Vector xobjects;
    
    public XSignature ( Document doc ) {
        this( doc, null );
    }
    
    public XSignature ( Document doc, String id ) {
        this.doc = doc;
        this.id = id;
            }
    
    public Element toElement () {
        Log.setMsg( "XSignature.toElement(), in ..." );
        if ( this.algSignature == null ) {
            this.setSignatureMethod( Constant.DFT_ALG_SIGNATURE );
        }
        if ( this.algCanolization == null ) {
            this.setCanonicalizationMethod( Constant.DFT_ALG_CANONICALIZATION );
        }
        Element signature, signedinfo, ele;
                signature = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_SIGNATURE_NAME );
        signature.setAttribute(
            Constant.ATT_XMLNS + ":" + Constant.PREFIX_SIGNATURE,
            Constant.NS_XMLDSIG_NAME );
        signature.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        if ( this.id != null ) {
            signature.setAttribute( Constant.ATT_ID, this.id );
        }
        Log.setMsg( "Xsignature.toElement,create Signature,set attributes" );
                signedinfo = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_SIGNEDINFO_NAME );
        signedinfo.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        if ( this.signedinfoid != null ) {
            signedinfo.setAttribute( Constant.ATT_ID, this.signedinfoid );
        }
        Log.setMsg( "Xsignature.toElement,create signedinfo element" );
        if ( this.algSignature != null ) {
            signedinfo.appendChild( this.algSignature.toElement( doc ) );
            signedinfo.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        Log.setMsg( "Xsignature.toElement,set Signature Algorithm" );
        if ( this.algCanolization != null ) {
            signedinfo.appendChild( this.algCanolization.toElement( doc ) );
            signedinfo.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        Log.setMsg( "Xsignature.toElement,set canonicalization algorithm" );
        if ( xreferences != null ) {
            for ( int i = 0; i < xreferences.size(); i++ ) {
                signedinfo.appendChild( ( ( XReference ) ( this.xreferences.
                    elementAt( i ) ) ).toElement( doc ) );
                signedinfo.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            }
        }
        Log.setMsg( "Xsignature.toElement, set reference" );
        signature.appendChild( signedinfo );
        signature.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
                ele = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_SIGNATUREVALUE_NAME );
        signature.appendChild( ele );
        signature.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        if ( xkeyinfo != null ) {
            Element keyinfoelt = xkeyinfo.toElement( this.doc );
                        if ( keyinfoelt.getNamespaceURI().equalsIgnoreCase( signature.
                getNamespaceURI() ) ) {
                keyinfoelt.removeAttribute( Constant.NS_XMLNS + ":" +
                                            Constant.PREFIX_SIGNATURE );
            }
            signature.appendChild( keyinfoelt );
            signature.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        Log.setMsg( "Xsignature.toElement,set Keyinfo" );
        if ( this.xobjects != null ) {
            for ( int i = 0; i < xobjects.size(); i++ ) {
                signature.appendChild( ( ( XObject ) ( xobjects.elementAt( i ) ) ).
                                       toElement() );
            }
            signature.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        Log.setMsg( "Xsignature.toElement,set Objects" );
        Log.setMsg( "Xsignature.toElement,exit ..." );
        return signature;
    }
    
    public String getId () {
        return this.id;
    }
    
    public String getSignInfoId () {
        return this.signedinfoid;
    }
    
    public void setId ( String id ) {
        this.id = id;
    }
    
    public void setSignInfoId ( String id ) {
        this.signedinfoid = id;
    }
    
    public void setDocument ( Document doc ) {
        this.doc = doc;
            }
    
    public Document getDocument () {
        return this.doc;
    }
    
    public void setSignatureMethod ( String uri ) {
        this.setSignatureMethod( uri, null );
    }
    
    public void setSignatureMethod ( String uri, NodeList params ) {
        this.algSignature = new XAlgorithm( Constant.TAG_SIGNATUREMETHOD_NAME,
                                            uri, params );
    }
        
    public void setHMACOutputLength ( int length ) {
        if ( this.algSignature != null ) {
            this.algSignature.setHMACOutputLength( length );
        }
    }
    
    public void setCanonicalizationMethod ( String uriReference ) {
        this.setCanonicalizationMethod( uriReference, null );
    }
    
    public void setCanonicalizationMethod ( String uri, NodeList params ) {
        this.algCanolization = new XAlgorithm( Constant.
                                               TAG_CANONICALIZATIONMETHOD_NAME,
                                               uri, params );
    }
    
    public XObject[] getObjects () {
        return ( XObject[] )this.xobjects.toArray();
    }
    
    public XObject createObject ( DocumentFragment content ) {
        return null;
    }
    
    public XObject createObject ( Element content ) {
        return null;
    }
    
    public XObject createObject ( XManifest manifest ) {
        return null;
    }
    
    public XObject createObject ( XSignatureProperties signatureProperties ) {
        return null;
    }
    
    public XObject createObject ( String content ) {
        return null;
    }
    
    public void addObject ( XObject object ) {
        if ( this.xobjects == null ) {
            this.xobjects = new Vector();
        }
        xobjects.add( object );
    }
    
    public void removeObject ( XObject object ) {
        if ( this.xobjects == null ) {
                                }
                xobjects.remove( object );
    }
    
    public void addKeyInfo ( XKeyInfo keyinfo ) {
        this.xkeyinfo = keyinfo;
    }
    
    public XKeyInfo createKeyInfo () {
        return new XKeyInfo();
    }
    
    public XKeyInfo getKeyInfo () {
        return this.xkeyinfo;
    }
    
    public void removeKeyInfo () {
        this.xkeyinfo = null;
    }
    
    public void addReference ( XReference reference ) {
        if ( this.xreferences == null ) {
            this.xreferences = new Vector();
        }
        xreferences.add( reference );
    }
    
    public XReference createReference () {
        return new XReference();
    }
    
    public XReference[] getReferences () {
        return ( XReference[] )this.xreferences.toArray();
    }
    
    public void removeReference ( XReference reference ) {
        if ( this.xreferences == null ) {
            return;
        }
                xreferences.remove( reference );
    }
}
