package org.act.xdsec.syntax.keyinfo;

import java.math.*;
import java.security.cert.*;
import java.util.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.syntax.keyinfo.x509.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class X509Data
    implements XKeyType {
    
    Vector cert = new Vector();
    XKeyType crl = null;
    public Element toElement ( Document doc ) {
        Element e1 = null;
                e1 = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_X509DATA_NAME );
        e1.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        if ( !this.cert.isEmpty() ) {
            for ( int i = 0; i < this.cert.size(); i++ ) {
                Object obj = this.cert.elementAt( i );
                if ( obj instanceof Element ) {
                    e1.appendChild( ( Element ) ( obj ) );
                }
                else if ( obj instanceof XKeyType ) {
                    e1.appendChild( ( ( XKeyType ) ( obj ) ).toElement( doc ) );
                }
                e1.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
            }
        }
        if ( this.crl != null ) {
            e1.appendChild( this.crl.toElement( doc ) );
            e1.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        }
        return e1;
    }
    public void addIssuerSerial ( String X509IssuerName,
                                  BigInteger X509SerialNumber ) {
        this.cert.add( new XMLX509IssuerSerial( X509IssuerName,
                                                X509SerialNumber ) );
        this.crl = null;
    }
    public void addIssuerSerial ( String X509IssuerName,
                                  String X509SerialNumber ) {
        this.cert.add( new XMLX509IssuerSerial( X509IssuerName,
                                                X509SerialNumber ) );
        this.crl = null;
    }
    public void addIssuerSerial ( String X509IssuerName, int X509SerialNumber ) {
        this.cert.add( new XMLX509IssuerSerial( X509IssuerName,
                                                X509SerialNumber ) );
        this.crl = null;
    }
    public void add ( XMLX509IssuerSerial xmlX509IssuerSerial ) {
        this.cert.add( xmlX509IssuerSerial );
        this.crl = null;
    }
    public void addSKI ( byte[] skiBytes ) {
        this.cert.add( new XMLX509SKI( skiBytes ) );
        this.crl = null;
    }
    public void addSKI ( X509Certificate x509certificate ) throws
        XMLSecurityException {
        this.cert.add( new XMLX509SKI( x509certificate ) );
        this.crl = null;
    }
    public void add ( XMLX509SKI xmlX509SKI ) {
        this.cert.add( xmlX509SKI );
        this.crl = null;
    }
    public void addSubjectName ( String subjectName ) {
        this.cert.add( new XMLX509SubjectName( subjectName ) );
        this.crl = null;
    }
    public void addSubjectName ( X509Certificate x509certificate ) {
        this.cert.add( new XMLX509SubjectName( x509certificate ) );
        this.crl = null;
    }
    public void add ( XMLX509SubjectName xmlX509SubjectName ) {
        this.cert.add( xmlX509SubjectName );
        this.crl = null;
    }
    public void addCertificate ( X509Certificate x509certificate ) throws
        XMLSecurityException {
        this.cert.add( new XMLX509Certificate( x509certificate ) );
        this.crl = null;
    }
    public void addCertificate ( byte[] x509certificateBytes ) {
        this.cert.add( new XMLX509Certificate( x509certificateBytes ) );
        this.crl = null;
    }
    public void add ( XMLX509Certificate xmlX509Certificate ) {
        this.cert.add( xmlX509Certificate );
        this.crl = null;
    }
    public void addUnknownElement ( Element element ) {
        this.cert.add( element );
        this.crl = null;
    }
        public void addCRL ( byte[] crlBytes ) {
        this.crl = new XMLX509CRL( crlBytes );
        this.cert.clear();
    }
    public void add ( XMLX509CRL xmlX509CRL ) {
        this.crl = xmlX509CRL;
        this.cert.clear();
    }
    
}

