package org.act.xdsec.syntax.keyinfo;

import java.security.*;
import java.security.interfaces.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class XKeyValue
    implements XKeyType {
    
    RSAPublicKey rsakey;
    
    DSAPublicKey dsakey;
    
    Node anyKey;
    
    public XKeyValue ( Key key ) {
        Log.setMsg( "XKeyValue.XKeyValue, in..." );
        if ( key instanceof RSAPublicKey ) {
            Log.setMsg( "  XKeyValue.XKeyValue, RSAKey" );
            setRSAKey( ( RSAPublicKey ) key );
        }
        else if ( key instanceof DSAPublicKey ) {
            Log.setMsg( "  XKeyValue.XKeyValue, DSAKey" );
            setDSAKey( ( DSAPublicKey ) key );
        }
        else {
            Log.setMsg( "  XKeyValue.XKeyValue, any key" );
                    }
        Log.setMsg( "XKeyValue.XKeyValue, exit..." );
    }
    
    public XKeyValue ( Node xKey ) {
        this.setAnyKey( xKey );
    }
    
    public void setRSAKey ( RSAPublicKey rsakey ) {
        this.rsakey = rsakey;
        this.dsakey = null;
        this.anyKey = null;
    }
    
    public void setDSAKey ( DSAPublicKey dsakey ) {
        this.dsakey = dsakey;
        this.rsakey = null;
        this.anyKey = null;
    }
    
    public void setAnyKey ( Node anyKeyValue ) {
        this.anyKey = anyKeyValue;
    }
    
    public Element toElement ( Document doc ) {
        Log.setMsg( "XKeyValue.toElement,in ..." );
        Element key;
        key = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_KEYVALUE_NAME );
        key.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        if ( this.rsakey != null ) {
            key.appendChild( buildRSAKey( doc ) );
        }
        else if ( this.dsakey != null ) {
            key.appendChild( buildDSAKey( doc ) );
        }
        else if ( this.anyKey != null ) {
            key.appendChild( this.anyKey );
        }
        key.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        Log.setMsg( "XKeyValue.toElement, exit ..." );
        return key;
    }
    
    private Element buildRSAKey ( Document doc ) {
        Log.setMsg( "XKeyValue.buildRSAKey,in ..." );
        Element elResult, elTemp;
        String text;
        elResult = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_RSAKEY_NAME );
                        text = JavaUtils.Base64BigIntegerToString( rsakey.getModulus() );
        elTemp = XMLUtils.createTextElementInSig( doc,
                                                  Constant.TAG_RSAKEY_MODULUS,
                                                  text );
        elResult.appendChild( elTemp );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
                text = JavaUtils.Base64BigIntegerToString( rsakey.getPublicExponent() );
        elTemp = XMLUtils.createTextElementInSig( doc,
                                                  Constant.TAG_RSAKEY_EXPONENT,
                                                  text );
        elResult.appendChild( elTemp );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        Log.setMsg( "XKeyValue.buildRSAKey,exit ..." );
        return elResult;
    }
    
    private Element buildDSAKey ( Document doc ) {
        Log.setMsg( "XKeyValue.buildDSAKey,in..." );
        Element elResult;
        elResult = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_DSAKEY_NAME );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        String str;
                str = JavaUtils.Base64BigIntegerToString( this.dsakey.getParams().getP() );
        elResult.appendChild( XMLUtils.createTextElementInSig( doc,
            Constant.TAG_DSAKEY_P, str ) );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
                str = JavaUtils.Base64BigIntegerToString( this.dsakey.getParams().getQ() );
        elResult.appendChild( XMLUtils.createTextElementInSig( doc,
            Constant.TAG_DSAKEY_Q, str ) );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
                str = JavaUtils.Base64BigIntegerToString( this.dsakey.getParams().getG() );
        elResult.appendChild( XMLUtils.createTextElementInSig( doc,
            Constant.TAG_DSAKEY_G, str ) );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
                str = JavaUtils.Base64BigIntegerToString( this.dsakey.getY() );
        elResult.appendChild( XMLUtils.createTextElementInSig( doc,
            Constant.TAG_DSAKEY_Y, str ) );
        elResult.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
                Log.setMsg( "XKeyValue.buildDSAKey,exit ..." );
        return elResult;
    }
}
