package org.act.xdsec.syntax.keyinfo.x509;

import java.math.*;
import java.security.cert.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.syntax.keyinfo.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;

public class XMLX509IssuerSerial
    implements XKeyType {
    String issuerName;
    BigInteger serialNumber;
    public XMLX509IssuerSerial ( String issuerName,
                                 BigInteger serialNumber ) {
        this.issuerName = issuerName;
        this.serialNumber = serialNumber;
    }
    public XMLX509IssuerSerial ( String X509IssuerName,
                                 String X509SerialNumber ) {
        this( X509IssuerName, new BigInteger( X509SerialNumber ) );
    }
    public XMLX509IssuerSerial ( String X509IssuerName,
                                 int X509SerialNumber ) {
        this( X509IssuerName,
              new BigInteger( Integer.toString( X509SerialNumber ) ) );
    }
    public XMLX509IssuerSerial ( X509Certificate x509certificate ) {
        this( RFC2253Parser.normalize( x509certificate.getIssuerDN().getName() ),
              x509certificate.getSerialNumber() );
    }
    public Element toElement ( Document doc ) {
        Element el, e2;
                el = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_X509ISSUERSERIAL_NAME );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        e2 = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_X509ISSUERNAME_NAME );
        e2.appendChild( doc.createTextNode( this.issuerName ) );
        el.appendChild( e2 );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        e2 = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_X509SERIALNUMBER_NAME );
        e2.appendChild( doc.createTextNode( this.serialNumber.toString() ) );
        el.appendChild( e2 );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        return el;
    }
    public BigInteger getSerialNumber () throws XMLSecurityException {
        return this.serialNumber;
    }
    public int getSerialNumberInteger () throws XMLSecurityException {
        return this.getSerialNumber().intValue();
    }
    public String getIssuerName () throws XMLSecurityException {
        return RFC2253Parser.normalize( this.issuerName );
    }
    public boolean equals ( Object obj ) {
        if ( !obj.getClass().getName().equals( this.getClass().getName() ) ) {
            return false;
        }
        XMLX509IssuerSerial other = ( XMLX509IssuerSerial ) obj;
        try {
            if ( other.getSerialNumber().equals( this.getSerialNumber() )
                 && other.getIssuerName().equals( this.getIssuerName() ) ) {
                return true;
            }
            return false;
        }
        catch ( XMLSecurityException ex ) {
            return false;
        }
    }
}
