package org.act.xdsec.syntax.keyinfo.x509;

import java.io.*;
import java.security.cert.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.syntax.keyinfo.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;
import sun.security.util.*;

public class XMLX509SKI
    implements XKeyType {
    
    public static final String SKI_OID = "2.5.29.14";
    private byte[] ski = null;
    public XMLX509SKI ( byte[] skiBytes ) {
        this.ski = skiBytes;
    }
    public XMLX509SKI ( X509Certificate x509certificate ) throws
        XMLSecurityException {
        this.ski = XMLX509SKI.getSKIBytesFromCert( x509certificate );
    }
    public Element toElement ( Document doc ) {
        Element el;
                el = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_X509SKI_NAME );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        el.appendChild( doc.createTextNode( JavaUtils.Base64Encode( this.ski ) ) );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        return el;
    }
    
    public byte[] getSKIBytes () throws XMLSecurityException {
        return this.ski;
    }
    
    public static byte[] getSKIBytesFromCert ( X509Certificate cert ) throws
        XMLSecurityException {
        try {
            
            byte[] derEncodedValue = cert.getExtensionValue( XMLX509SKI.SKI_OID );
            if ( cert.getVersion() < 3 ) {
                Object exArgs[] = {
                    new Integer( cert.getVersion() )};
                throw new XMLSecurityException( "certificate.noSki.lowVersion",
                                                exArgs );
            }
            DerValue dervalue = new DerValue( derEncodedValue );
            if ( dervalue == null ) {
                throw new XMLSecurityException( "certificate.noSki.null" );
            }
            if ( dervalue.tag != DerValue.tag_OctetString ) {
                throw new XMLSecurityException(
                    "certificate.noSki.notOctetString" );
            }
            byte[] extensionValue = dervalue.getOctetString();
            
            byte abyte0[] = new byte[extensionValue.length - 2];
            System.arraycopy( extensionValue, 2, abyte0, 0, abyte0.length );
            return abyte0;
        }
        catch ( IOException ex ) {
            throw new XMLSecurityException( "generic.EmptyMessage", ex );
        }
    }
    
    public boolean equals ( Object obj ) {
        if ( !obj.getClass().getName().equals( this.getClass().getName() ) ) {
            return false;
        }
        XMLX509SKI other = ( XMLX509SKI ) obj;
        try {
            return JavaUtils.binaryCompare( other.getSKIBytes(),
                                            this.getSKIBytes() );
        }
        catch ( XMLSecurityException ex ) {
            return false;
        }
    }
}
