package org.act.xdsec.syntax.keyinfo.x509;

import java.io.*;
import java.security.cert.*;
import org.act.xdsec.core.exception.*;
import org.act.xdsec.syntax.keyinfo.*;
import org.act.xdsec.util.*;
import org.w3c.dom.*;
import sun.security.x509.*;

public class XMLX509SubjectName
    implements XKeyType {
    String subjectName;
        public XMLX509SubjectName ( X509Certificate x509certificate ) {
        this( RFC2253Parser.normalize( x509certificate.getSubjectDN().getName() ) );
    }
    public XMLX509SubjectName ( String X509SubjectNameString ) {
        this.subjectName = X509SubjectNameString;
    }
    public Element toElement ( Document doc ) {
        Element el;
                el = XMLUtils.createElementInSignatureSpace( doc,
            Constant.TAG_X509SUBJECTNAME_NAME );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        el.appendChild( doc.createTextNode( this.subjectName ) );
        el.appendChild( doc.createTextNode( Constant.X_CTRLF ) );
        return el;
    }
    public String getSubjectName () throws XMLSecurityException {
        return RFC2253Parser.normalize( this.subjectName );
    }
    public static X500Name createX500Name (
        String common, String orgUnit, String org, String country ) throws
        IOException {
        return new X500Name( common, orgUnit, org, country );
    }
    
    public static X500Name createX500Name (
        String common, String orgUnit, String org, String locality,
        String state, String country ) throws IOException {
        return new X500Name( common, orgUnit, org, locality, state, country );
    }
    
    public boolean equals ( Object obj ) {
        if ( !obj.getClass().getName().equals( this.getClass().getName() ) ) {
            return false;
        }
        try {
            XMLX509SubjectName other = ( XMLX509SubjectName ) obj;
            String otherSubject = other.getSubjectName();
            String thisSubject = this.getSubjectName();
            if ( otherSubject.equals( thisSubject ) ) {
                return true;
            }
            return false;
        }
        catch ( XMLSecurityException ex ) {
            return false;
        }
    }
}
