package org.act.xdsec.util;
import java.text.*;
import java.util.*;

public class I18n {
    
    public static String defaultLanguageCode;     
    public static String defaultCountryCode;     
    protected static ResourceBundle resourceBundle;
    
    protected static boolean alreadyInitialized = false;
    
    protected static String _languageCode = null;
    
    protected static String _countryCode = null;
    
    public static String translate ( String message, Object[] args ) {
        return getExceptionMessage( message, args );
    }
    
    public static String translate ( String message ) {
        return getExceptionMessage( message );
    }
    
    public static String getExceptionMessage ( String msgID ) {
        try {
            String s = resourceBundle.getString( msgID );
            return s;
        }
        catch ( Throwable t ) {
            String s = "No message with ID \"" + msgID
                + "\" found in resource bundle \""
                + Constant.ERR_MSGRERSOURCEBUNDLE_BASE + "\"";
            return s;
        }
    }
    
    public static String getExceptionMessage ( String msgID,
                                               Exception originalException ) {
        try {
            Object exArgs[] = {
                originalException.getMessage()};
            String s = MessageFormat.format( resourceBundle.getString( msgID ),
                                             exArgs );
            return s;
        }
        catch ( Throwable t ) {
            String s = "No message with ID \"" + msgID
                + "\" found in resource bundle \""
                + Constant.ERR_MSGRERSOURCEBUNDLE_BASE
                + "\". Original Exception was a "
                + originalException.getClass().getName() + " and message "
                + originalException.getMessage();
            return s;
        }
    }
    
    public static String getExceptionMessage ( String msgID, Object exArgs[] ) {
        try {
            String s = MessageFormat.format( resourceBundle.getString( msgID ),
                                             exArgs );
            return s;
        }
        catch ( Throwable t ) {
            String s = "No message with ID \"" + msgID
                + "\" found in resource bundle \""
                + Constant.ERR_MSGRERSOURCEBUNDLE_BASE + "\"";
            return s;
        }
    }
    
    public static void init ( String _defaultLanguageCode,
                              String _defaultCountryCode ) {
        I18n.defaultLanguageCode = _defaultLanguageCode;
        if ( I18n.defaultLanguageCode == null ) {
            I18n.defaultLanguageCode = Locale.getDefault().getLanguage();
        }
        I18n.defaultCountryCode = _defaultCountryCode;
        if ( I18n.defaultCountryCode == null ) {
            I18n.defaultCountryCode = Locale.getDefault().getCountry();
        }
        initLocale( I18n.defaultLanguageCode, I18n.defaultCountryCode );
    }
    
    public static void initLocale ( String languageCode, String countryCode ) {
        if ( alreadyInitialized && languageCode.equals( _languageCode )
             && countryCode.equals( _countryCode ) ) {
            return;
        }
        if ( ( languageCode != null ) && ( countryCode != null )
             && ( languageCode.length() > 0 ) && ( countryCode.length() > 0 ) ) {
            _languageCode = languageCode;
            _countryCode = countryCode;
        }
        else {
            _countryCode = I18n.defaultCountryCode;
            _languageCode = I18n.defaultLanguageCode;
        }
        I18n.resourceBundle =
            ResourceBundle.getBundle( Constant.ERR_MSGRERSOURCEBUNDLE_BASE,
                                      new Locale( _languageCode, _countryCode ) );
    }
}
