package org.act.xdsec.util;
import java.io.*;
import java.math.*;
import java.util.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import org.act.xdsec.core.exception.*;
import org.apache.xpath.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class XMLUtils {
    
    private static String[] nodeTypeString = new String[] {
        "", "ELEMENT",
        "ATTRIBUTE",
        "TEXT_NODE",
        "CDATA_SECTION",
        "ENTITY_REFERENCE",
        "ENTITY",
        "PROCESSING_INSTRUCTION",
        "COMMENT",
        "DOCUMENT",
        "DOCUMENT_TYPE",
        "DOCUMENT_FRAGMENT",
        "NOTATION"};
    
    public static String getNodeTypeString ( short nodeType ) {
        if ( ( nodeType > 0 ) && ( nodeType < 13 ) ) {
            return nodeTypeString[nodeType];
        }
        else {
            return "";
        }
    }
    
    public static String getNodeTypeString ( Node n ) {
        return getNodeTypeString( n.getNodeType() );
    }
    
    public static Vector getAncestorElements ( Node ctxNode ) {
        if ( ctxNode.getNodeType() != Node.ELEMENT_NODE ) {
            return null;
        }
        Vector ancestorVector = new Vector();
        Node parent = ctxNode;
        while ( ( parent = parent.getParentNode() ) != null
                && ( parent.getNodeType() == Node.ELEMENT_NODE ) ) {
            ancestorVector.add( parent );
        }
        ancestorVector.trimToSize();
        return ancestorVector;
    }
    
    public static Attr createAttr ( Document doc, String QName, String Value,
                                    String NamespaceURI ) {
        Attr attr = null;
        if ( ( NamespaceURI != null ) && ( NamespaceURI.length() > 0 ) ) {
            attr = doc.createAttributeNS( NamespaceURI, QName );
        }
        else {
            attr = doc.createAttribute( QName );
        }
        attr.appendChild( doc.createTextNode( Value ) );
        return attr;
    }
    
    public static void setAttr ( Element elem, String QName, String Value ) {
        Document doc = elem.getOwnerDocument();
        Attr attr = doc.createAttributeNS( Constant.NS_XMLDSIG_NAME, QName );
        attr.appendChild( doc.createTextNode( Value ) );
        elem.setAttributeNodeNS( attr );
    }
    
    public static Element createElementFromBigint (
        Document doc, String elementName, BigInteger bigInteger ) throws
        XMLSignatureException {
        Element element = doc.createElementNS( Constant.NS_XMLDSIG_NAME,
                                               Constant.NS_XMLDSIG_NAME                                                + ":" + elementName );
        
        if ( bigInteger.signum() != 1 ) {
            throw new XMLSignatureException( "signature.Util.BignumNonPositive" );
        }
        byte byteRepresentation[] = bigInteger.toByteArray();
        while ( byteRepresentation[0] == 0 ) {
            byte oldByteRepresentation[] = byteRepresentation;
            byteRepresentation = new byte[oldByteRepresentation.length - 1];
            System.arraycopy( oldByteRepresentation, 1, byteRepresentation, 0,
                              oldByteRepresentation.length - 1 );
        }
        Text text =
            doc.createTextNode( JavaUtils.Base64Encode( byteRepresentation ) );
        element.appendChild( text );
        return element;
    }
    
    public static BigInteger getBigintFromElement ( Element element ) throws
        XMLSignatureException {
        if ( element.getChildNodes().getLength() != 1 ) {
            throw new XMLSignatureException( "signature.Util.TooManyChilds" );
        }
        Node child = element.getFirstChild();
        if ( ( child == null ) || ( child.getNodeType() != Node.TEXT_NODE ) ) {
            throw new XMLSignatureException( "signature.Util.NonTextNode" );
        }
        Text text = ( Text ) child;
        String textData = text.getData();
        byte magnitude[] = JavaUtils.Base64Decode( textData );
        int signum = 1;
        BigInteger bigInteger = new BigInteger( signum, magnitude );
        return bigInteger;
    }
    
    public static byte[] getBytesFromElement ( Element element ) throws
        XMLSignatureException {
        if ( element.getChildNodes().getLength() != 1 ) {
            throw new XMLSignatureException( "signature.Util.TooManyChilds" );
        }
        Node child = element.getFirstChild();
        if ( ( child == null ) || ( child.getNodeType() != Node.TEXT_NODE ) ) {
            throw new XMLSignatureException( "signature.Util.NonTextNode" );
        }
        Text text = ( Text ) child;
        String textData = text.getData();
        byte bytes[] = JavaUtils.Base64Decode( textData );
        return bytes;
    }
    
    public static Element createElementInSignatureSpace ( Document doc,
        String elementName ) {
        if ( doc == null ) {
            throw new RuntimeException( "Document is null" );
        }
                String ds = Constant.PREFIX_SIGNATURE;
        if ( ( ds == null ) || ( ds.length() == 0 ) ) {
            Element element = doc.createElementNS( Constant.NS_XMLDSIG_NAME,
                elementName );
            element.setAttribute( "xmlns", Constant.NS_XMLDSIG_NAME );
            return element;
        }
        else {
            Element element = doc.createElementNS( Constant.NS_XMLDSIG_NAME,
                ds + ":" + elementName );
                        return element;
        }
    }
    
    public static Element createElementInEncryptionSpace ( Document doc,
        String elementName ) {
        if ( doc == null ) {
            throw new RuntimeException( "Document is null" );
        }
                String xenc = Constant.PREFIX_ENCRYPTION;
        if ( ( xenc == null ) || ( xenc.length() == 0 ) ) {
            Element element =
                doc.createElementNS( Constant.NS_XMLENC_NAME,
                                     elementName );
            element.setAttribute( "xmlns", Constant.NS_XMLDSIG_NAME );
            return element;
        }
        else {
            Element element =
                doc.createElementNS( Constant.NS_XMLENC_NAME,
                                     xenc + ":" + elementName );
                        return element;
        }
    }
    
    public static boolean elementIsInSignatureSpace ( Element element,
        String localName ) {
        if ( element == null ) {
            return false;
        }
        if ( element.getNamespaceURI() == null ) {
            return false;
        }
        if ( !element.getNamespaceURI().equals( Constant.NS_XMLDSIG_NAME ) ) {
            return false;
        }
        if ( !element.getLocalName().equals( localName ) ) {
            return false;
        }
        return true;
    }
    
    public static boolean elementIsInEncryptionSpace ( Element element,
        String localName ) {
        if ( element == null ) {
            return false;
        }
        if ( element.getNamespaceURI() == null ) {
            return false;
        }
        if ( !element.getNamespaceURI().equals( Constant.NS_XMLENC_NAME ) ) {
            return false;
        }
        if ( !element.getLocalName().equals( localName ) ) {
            return false;
        }
        return true;
    }
    
    public static void guaranteeThatElementInSignatureSpace (
        Element element, String localName ) throws XMLSignatureException {
        
        if ( element == null ) {
            Object exArgs[] = {
                localName, null};
            throw new XMLSignatureException( "xml.WrongElement", exArgs );
        }
        if ( ( localName == null ) || localName.equals( "" )
             || !elementIsInSignatureSpace( element, localName ) ) {
            Object exArgs[] = {
                localName, element.getLocalName()};
            throw new XMLSignatureException( "xml.WrongElement", exArgs );
        }
    }
    
    public static void guaranteeThatElementInEncryptionSpace (
        Element element, String localName ) throws XMLSecurityException {
        if ( element == null ) {
            Object exArgs[] = {
                localName, null};
            throw new XMLSecurityException( "xml.WrongElement", exArgs );
        }
        if ( ( localName == null ) || localName.equals( "" )
             || !elementIsInEncryptionSpace( element, localName ) ) {
            Object exArgs[] = {
                localName, element.getLocalName()};
            throw new XMLSecurityException( "xml.WrongElement", exArgs );
        }
    }
    
    public static Document getOwnerDocument ( Node node ) {
        if ( node.getNodeType() == Node.DOCUMENT_NODE ) {
            return ( Document ) node;
        }
        else {
            return node.getOwnerDocument();
        }
    }
    
    public static Element createXSecCtx ( Document doc, String prefix,
                                          String namespace ) {
        Element ctx = doc.createElement( "namespaceContext" );
        ctx.setAttribute( "xmlns:" + prefix.trim(), namespace );
        return ctx;
    }
    
    public static Element createDSctx ( Document doc, String prefix ) {
        return XMLUtils.createXSecCtx( doc, prefix, Constant.NS_XMLDSIG_NAME );
    }
    public static Element createDEctx ( Document doc, String prefix ) {
        return XMLUtils.createXSecCtx( doc, prefix, Constant.NS_XMLENC_NAME );
    }
                    public static Element createTextElementInSig ( Document doc, String name,
        String text ) {
        Element el;
        String ds = Constant.PREFIX_SIGNATURE;
        el = doc.createElementNS( Constant.NS_XMLDSIG_NAME,
                                  ds + ":" + name );
        el.appendChild( doc.createTextNode( text ) );
        return el;
    }
    public static Element createTextElementInEnc ( Document doc, String name,
        String text ) {
        Element el;
        String ds = Constant.PREFIX_ENCRYPTION;
        el = doc.createElementNS( Constant.NS_XMLENC_NAME,
                                  ds + ":" + name );
        el.appendChild( doc.createTextNode( text ) );
        return el;
    }
    
    public static void setTextValue ( Element parent, String txt ) {
                Node p = parent.getFirstChild();
        Node q = null;
        while ( p != null ) {
            q = p;
            p = p.getNextSibling();
            if ( q.getNodeType() == Node.TEXT_NODE ) {
                parent.removeChild( q );
            }
        }
                Document doc = parent.getOwnerDocument();
        parent.appendChild( doc.createTextNode( txt ) );
    }
    public static Element getFirstChildElementByName ( Element parent,
        String childName ) {
        NodeList nodes = parent.getElementsByTagName( childName );
        if ( nodes == null ) {
            return null;
        }
        return ( Element ) nodes.item( 0 );
    }
    public static String getTextValueOfElement ( Element el ) {
                        NodeList nl = el.getChildNodes();
        Node node = null;
        for ( int i = 0; i < nl.getLength(); i++ ) {
            node = nl.item( i );
            if ( ( node instanceof Text ) && ( node.getNodeValue().length() > 0 ) ) {
                return node.getNodeValue();
            }
        }
        return null;
        
    }
        
    public static String getNamespaceForPrefix ( String prefix, Node current ) {
        if ( prefix.equals( "xml" ) ) {
            return Constant.NS_XMLNS_NAME;
        }
        if ( prefix.equals( Constant.NS_XMLNS ) ) {
            return prefix;         }
        String attributeName = prefix.length() == 0 ? Constant.NS_XMLNS :
            ( "xmlns:" + prefix );
        Node element = current;
        do {
                        Node attr = element.getAttributes().getLength() == 0
                ? null
                : ( ( Element ) element ).getAttributeNode( attributeName );
            if ( attr != null ) {
                return attr.getNodeValue();
            }
            while ( true ) {
                element = element.getParentNode();
                if ( element == null ) {
                    return attributeName == Constant.NS_XMLNS ? "" : null;
                }
                int type = element.getNodeType();
                if ( type == Node.ENTITY_REFERENCE_NODE ) {
                }
                else if ( type == Node.ELEMENT_NODE ) {
                    break;
                }
                else {
                    return attributeName == Constant.NS_XMLNS ? "" : null;
                }
            }
        }
        while ( element != null );         return null;     }
    
    public static String getNamespace ( Element el ) {
        String name = el.getNodeName();
        int index = name.indexOf( ':' );
        String prefix = index <= 0 ? "" : name.substring( 0, index );
        String uri = XMLUtils.getNamespaceForPrefix( prefix, el );
        if ( uri == null || uri.length() == 0 ) {
            uri = null;
        }
        return uri;
    }
    
    public static String getLocalPart ( Element el ) {
        String name = el.getNodeName();
        int index = name.indexOf( ':' );
        return index < 0 ? name : name.substring( index + 1 ).intern();
    }
        
    public static Node getPreviousSibling2 ( Node node ) {
        Node ret = node.getPreviousSibling();
        if ( ret == null ) {
            Node parent = node.getParentNode();
            if ( parent == null ) {
                return null;
            }
            if ( parent.getNodeType() == Node.ENTITY_REFERENCE_NODE ) {
                ret = getPreviousSibling2( parent );
            }
        }
        else {             while ( ret.getNodeType() == Node.ENTITY_REFERENCE_NODE ) {
                Node last = ret.getLastChild();
                if ( last == null ) {
                    ret = getPreviousSibling2( ret );
                    break;
                }
                else {
                    ret = last;
                }
            }
        }
        return ret;
    }
    
    public static Node getNextSibling2 ( Node node ) {
        Node ret = node.getNextSibling();
        if ( ret == null ) {
            Node parent = node.getParentNode();
            if ( parent == null ) {
                return null;
            }
            if ( parent.getNodeType() == Node.ENTITY_REFERENCE_NODE ) {
                ret = getNextSibling2( parent );
            }
        }
        else {             while ( ret.getNodeType() == Node.ENTITY_REFERENCE_NODE ) {
                Node first = ret.getFirstChild();
                if ( first == null ) {
                    ret = getNextSibling2( ret );
                    break;
                }
                else {
                    ret = first;
                }
            }
        }
        return ret;
    }
    
    public static Node getFirstChild2 ( Node node ) {
        Node ret = node.getFirstChild();
        while ( ret != null && ret.getNodeType() == Node.ENTITY_REFERENCE_NODE ) {
            Node first = ret.getFirstChild();
            if ( first == null ) {
                ret = getNextSibling2( ret );
                break;
            }
            ret = first;
        }
        return ret;
    }
    
    public static Node getLastChild2 ( Node node ) {
        Node ret = node.getLastChild();
        while ( ret != null && ret.getNodeType() == Node.ENTITY_REFERENCE_NODE ) {
            Node last = ret.getLastChild();
            if ( last == null ) {
                ret = getPreviousSibling2( ret );
                break;
            }
            ret = last;
        }
        return ret;
    }
    
    public static Element getNextElement ( Node element ) {
        Node next = element;
        while ( ( next = getNextSibling2( next ) ) != null ) {
            if ( next.getNodeType() == Node.ELEMENT_NODE ) {
                break;
            }
        }
        return ( Element ) next;
    }
    
    public static Element getFirstChildElement ( Node parent ) {
        Node child = getFirstChild2( parent );
        for ( ; child != null; child = getNextSibling2( child ) ) {
            if ( child.getNodeType() == Node.ELEMENT_NODE ) {
                break;
            }
        }
        return ( Element ) child;
    }
    
    public static Element getFirstChildElementNamed ( Node parent, String ns,
        String local ) {
        Node child = parent.getFirstChild();
        for ( ; child != null; child = getNextSibling2( child ) ) {
            if ( child.getNodeType() == Node.ELEMENT_NODE ) {
                Element el = ( Element ) child;
                String elns = getNamespace( el );
                if ( ( local.equals( "*" ) || getLocalPart( el ).equals( local ) )
                     && ( ns == null && elns == null
                          || ns != null && ns.equals( "*" )
                          || ns != null && ns.equals( elns ) ) ) {
                    break;
                }
            }
        }
        return ( Element ) child;
    }
    
    public static Element getFirstChildElementNamed ( Node parent, String qname ) {
        Node child = parent.getFirstChild();
        for ( ; child != null; child = getNextSibling2( child ) ) {
            if ( child.getNodeType() == Node.ELEMENT_NODE
                                  && child.getLocalName().equals( qname ) ) {
                break;
            }
        }
        return ( Element ) child;
    }
    
    public static Element getFirstChildElementNamed ( Element parent,
        String childname ) {
        NodeList children = parent.getChildNodes();
        Node child = null;
        for ( int i = 0; i < children.getLength(); i++ ) {
            child = children.item( i );
            if ( child.getNodeType() == Node.ELEMENT_NODE
                 && child.getLocalName().equals( childname ) ) {
                                return ( Element ) child;
            }
        }
        return null;
    }
    public static final boolean isXmldsig ( String nsuri ) {
        if ( nsuri == null ) {
            return false;
        }
        if ( nsuri == Constant.NS_XMLDSIG_NAME ) {
            return true;
        }
        return nsuri.equals( Constant.NS_XMLDSIG_NAME );
    }
    
    public static final void removeChildNodes ( Element dataObject ) {
        NodeList childNodes = dataObject.getChildNodes();
        int length = childNodes.getLength();
        for ( int i = length - 1; i >= 0; i-- ) {
            Node nodex = childNodes.item( i );
            dataObject.removeChild( nodex );
        }
    }
            
    public static NodeList buildNodeSetFromStream ( InputStream is,
        String xpathString ) throws
        ParserConfigurationException, IOException, SAXException {
                DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
        dfactory.setValidating( false );
        dfactory.setNamespaceAware( true );
        DocumentBuilder db = dfactory.newDocumentBuilder();
        try {
            Document document = db.parse( is );
            return XPathAPI.selectNodeList( document, xpathString );
        }
        catch ( TransformerException ex ) {
                                    return null;
                    }
    }
    public static Node buildNodeFromStream ( InputStream is ) throws
        ParserConfigurationException, IOException, SAXException {
                DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
        dfactory.setValidating( false );
        dfactory.setNamespaceAware( true );
        DocumentBuilder db = dfactory.newDocumentBuilder();
        Document document = db.parse( is );
        return document.getDocumentElement();
    }
    
    public static Element getElementInXMLSigNS ( Element parent,
                                                 String sElementLocalName ) {
        NodeList nl = parent.getElementsByTagNameNS( Constant.NS_XMLDSIG_NAME,
            sElementLocalName );
        if ( nl == null || nl.getLength() <= 0 ) {
                        nl = parent.getElementsByTagName( sElementLocalName );
        }
        if ( nl != null && nl.getLength() > 0 ) {
            return ( Element ) nl.item( 0 );
        }
        else {
            return null;
        }
    }
    
    public static byte[] canonicalizeAll ( Document doc, boolean withComments ) {
        
                return XPathCanonicalizer.serializeAll( doc, withComments );
            }
    
    public static void canonicalizeAll ( Document doc, boolean withComments,
                                         Writer wr ) throws IOException {
        
                XPathCanonicalizer.serializeAll( doc, withComments, wr );
            }
    
    public static byte[] canonicalizeNode ( Node node, boolean withComments ) {
        return XPathCanonicalizer.serializeSubset( node, withComments );
    }
    
    public static void canonicalizeNode ( Node node, boolean withComments,
                                          Writer wr ) throws IOException {
        XPathCanonicalizer.serializeSubset( node, withComments, wr );
    }
    
    public static byte[] canonicalizeNodeList ( NodeList nodeList,
                                                boolean withComments ) {
        return XPathCanonicalizer.serializeSubset( nodeList, withComments );
    }
    public static void canonicalizeNodeList ( NodeList nodeList,
                                              boolean withComments, Writer wr ) throws
        IOException {
        XPathCanonicalizer.serializeSubset( nodeList, withComments, wr );
    }
    
    
    public static NodeList toNodeset ( Node node, Node exceptedNode,
                                       boolean withComments ) {
        return XPathCanonicalizer.toNodeset( node, exceptedNode, withComments );
    }
    
    public static String stripWhitespace ( String str ) {
                return str;
    }
}



