package org.act.xdsec.util;
import java.io.*;
import java.util.*;
import javax.xml.transform.*;
import org.apache.xpath.*;
import org.w3c.dom.*;

public class XPathCanonicalizer {
    
    public static byte[] serializeAll ( Document doc, boolean withComments ) {
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            Writer wr = new OutputStreamWriter( baos, "UTF-8" );
                        serializeAll( doc, withComments, wr );
                        wr.close();
            return baos.toByteArray();
        }
        catch ( IOException ioe ) {
            ioe.printStackTrace();
            throw new RuntimeException( "Internal Error: " + ioe );
        }
    }
    
    public static void serializeAll ( Document doc, boolean withComments,
                                      Writer wr ) throws IOException {
        try {
            NodeList nl = XPathAPI.selectNodeList( doc,
                                Constant.
                XPATH_C14N_WITH_COMMENTS_SINGLE_NODE );
                        XMLUtils.canonicalizeNodeList( nl, true, wr );
        }
        catch ( TransformerException ex ) {
            ex.printStackTrace();
        }
    }
    
    public static byte[] serializeSubset ( Node node, boolean withComments ) {
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            Writer wr = new OutputStreamWriter( baos, "UTF-8" );
            serializeNode( node, node, null, withComments, true, wr );
            wr.close();
            return baos.toByteArray();
        }
        catch ( IOException ioe ) {
            ioe.printStackTrace();
            throw new RuntimeException( "Internal Error: " + ioe );
        }
    }
    
    public static void serializeSubset ( Node node, boolean withComments,
                                         Writer wr ) throws IOException {
        serializeNode( node, node, null, withComments, true, wr );
    }
    
    public static byte[] serializeSubset ( NodeList nodeList,
                                           boolean withComments ) {
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            Writer wr = new OutputStreamWriter( baos, "UTF-8" );
            serializeSubset( nodeList, withComments, wr );
            wr.close();
            return baos.toByteArray();
        }
        catch ( IOException ioe ) {
            ioe.printStackTrace();
            throw new RuntimeException( "Internal Error: " + ioe );
        }
    }
    
    public static void serializeSubset ( NodeList nodeList,
                                         boolean withComments,
                                         Writer wr ) throws IOException {
        Vector nodeVector = new Vector( nodeList.getLength() );
        Document doc = null;
        for ( int i = 0; i < nodeList.getLength(); i++ ) {
            Node node = nodeList.item( i );
            if ( node.getNodeType() != Node.ATTRIBUTE_NODE ) {
                nodeVector.addElement( node );
                                                if ( node.getNodeType() == Node.ELEMENT_NODE ) {
                    doc = node.getOwnerDocument();
                    Hashtable attributes = null;
                                                                                                    if ( attributes == null ) {
                        attributes = new Hashtable();
                    }
                    Node nodeCurrent = node;
                    while ( ( nodeCurrent = nodeCurrent.getParentNode() ) != null
                            && nodeCurrent.getNodeType() == Node.ELEMENT_NODE ) {
                        NamedNodeMap attributeList = nodeCurrent.getAttributes();
                        if ( attributeList != null ) {
                            int nAttrs = attributeList.getLength();
                            for ( int j = 0; j < nAttrs; j++ ) {
                                Attr attrNS = ( Attr ) attributeList.item( j );
                                                                String attrName = attrNS.getNodeName();
                                if ( attrName.equals( "xmlns" ) ||
                                     attrName.startsWith( "xmlns:" ) ) {
                                    if ( !attributes.containsKey( attrName ) ) {
                                        attributes.put( attrName, attrNS );
                                    }
                                }
                                                                String attrNamespaceURI = attrNS.
                                    getNamespaceURI();
                                if ( attrNamespaceURI == Constant.NS_XMLNS_NAME ) {
                                    if ( !attributes.containsKey( attrName ) ) {
                                        attributes.put( attrName, attrNS );
                                    }
                                }
                            }
                        }
                    }                                         Node attr;
                                                            if ( i + 1 < nodeList.getLength()
                         &&
                         ( ( attr = nodeList.item( i + 1 ) ) ).getNodeType() ==
                         Node.ATTRIBUTE_NODE
                         && ( ( Attr ) attr ).getOwnerElement() == node ) {
                                                if ( attributes == null ) {
                            attributes = new Hashtable();
                        }
                        for ( i++; i < nodeList.getLength(); i++ ) {
                            Node nodeNext = nodeList.item( i );
                            if ( nodeNext.getNodeType() != Node.ATTRIBUTE_NODE ) {
                                break;
                            }
                            if ( node != ( ( Attr ) nodeNext ).getOwnerElement() ) {                                 break;
                            }
                            attributes.put( nodeNext.getNodeName(), nodeNext );
                        }
                        i--;
                        nodeVector.addElement( new Attributes( node, attributes ) );
                    }
                    else if ( attributes != null ) {
                        nodeVector.addElement( new Attributes( node, attributes ) );
                    }
                    else {
                                                                    }
                                                            if ( attributes != null
                         && attributes.containsKey( "xmlns" )
                         &&
                         ( ( Attr ) attributes.get( "xmlns" ) ).getNodeValue().
                         length() ==
                         0 ) {
                        attributes.remove( "xmlns" );
                    }
                                    }             }
            else {                             }
        }
                                                                Stack stack = new Stack();
        for ( Node currentChild = doc.getDocumentElement();
              currentChild != null;
              currentChild = currentChild.getNextSibling() ) {
            serializeSubset( stack, nodeVector, currentChild, withComments, wr );
        }
        
    }
    
    public static NodeList toNodeset ( Node node, Node exceptedNode,
                                       boolean withComments ) {
        NodeListImpl nodeList = new NodeListImpl();
        toNodeset_addNode( nodeList, node, exceptedNode, withComments );
        return nodeList;
    }
    
    private static void toNodeset_addNode ( NodeListImpl nodeList, Node node,
                                            Node exceptedNode,
                                            boolean withComments ) {
        if ( node == exceptedNode ) {
            return;
        }
        switch ( node.getNodeType() ) {
            case Node.ELEMENT_NODE:
                                nodeList.add( node );
                                Hashtable namespaces = collectNamespaceNodesInAncestors( node );
                for ( Enumeration enum1 = namespaces.elements();
                      enum1.hasMoreElements(); ) {
                    Attr attr = ( Attr )enum1.nextElement();
                                        if ( attr.getNodeName().equals( "xmlns" ) &&
                         attr.getNodeValue().length() == 0 ) {
                        continue;
                    }
                    nodeList.add( attr );
                }
                NamedNodeMap map = node.getAttributes();
                for ( int i = 0; i < map.getLength(); i++ ) {
                    Node attr = map.item( i );
                    if ( !attr.getNodeName().equals( "xmlns" )
                         && !attr.getNodeName().startsWith( "xmlns:" ) ) {
                        nodeList.add( attr );
                    }
                }
                                for ( Node child = node.getFirstChild(); child != null;
                      child = child.getNextSibling() ) {
                    toNodeset_addNode( nodeList, child, exceptedNode,
                                       withComments );
                }
                break;
            case Node.ENTITY_REFERENCE_NODE:
                for ( Node child = node.getFirstChild(); child != null;
                      child = child.getNextSibling() ) {
                    toNodeset_addNode( nodeList, child, exceptedNode,
                                       withComments );
                }
                break;
            case Node.COMMENT_NODE:
                if ( withComments ) {
                    nodeList.add( node );
                }
                break;
            case Node.ATTRIBUTE_NODE:
            case Node.TEXT_NODE:
            case Node.CDATA_SECTION_NODE:
            case Node.PROCESSING_INSTRUCTION_NODE:
                nodeList.add( node );
                break;
            case Node.DOCUMENT_NODE:
                for ( Node child = node.getFirstChild(); child != null;
                      child = child.getNextSibling() ) {
                    toNodeset_addNode( nodeList, child, exceptedNode,
                                       withComments );
                }
                break;
            case Node.DOCUMENT_TYPE_NODE:
                break;
            case Node.ENTITY_NODE:
            case Node.DOCUMENT_FRAGMENT_NODE:
            case Node.NOTATION_NODE:
                throw new RuntimeException(
                    "Internal Error: Invalid node type: " + node.getNodeType() );
        }
    }
            static class Attributes {
        Node parent;
        Hashtable attributes;
        Attributes ( Node node, Hashtable attrs ) {
            this.parent = node;
            this.attributes = attrs;
        }
        void serialize ( Attributes parent, Writer wr ) throws IOException {
                                    int attrLength = this.attributes.size();
            String[] as = new String[attrLength];             int[] indexMap = new int[attrLength];
            Attr[] attrArray = new Attr[attrLength];
            Enumeration enum1 = this.attributes.elements();
            for ( int j = 0; j < attrLength; j++ ) {
                Attr attr = ( Attr )enum1.nextElement();
                indexMap[j] = j;
                attrArray[j] = attr;
                as[j] = createSortedString( attr, this.parent );
            }
            JavaUtils.heapSort( indexMap, as, attrLength );
                        for ( int j = 0; j < attrLength; j++ ) {
                Attr attr = attrArray[indexMap[j]];
                String attrName = attr.getNodeName();
                                                if ( attrName.equals( "xmlns:xml" ) &&
                     attr.getNodeValue() == Constant.NS_XML_NAME ) {
                    continue;
                }
                                                if ( ( attrName.equals( "xmlns" )
                       || attrName.startsWith( "xmlns:" ) )
                     && parent != null
                     && parent.attributes.containsKey( attrName )
                     &&
                     ( ( Attr ) parent.attributes.get( attrName ) ).
                     getNodeValue().
                     equals( attr.getNodeValue() ) ) {
                    continue;
                }
                XPathCanonicalizer.serializeAttribute( attr, wr );
            }
        }
        
        public String toString () {
            return "Attributes[#=" + this.attributes.size() + "]";
        }
    }
                    private static void serializeSubset ( Stack stack, Vector nodeVector,
                                          Node currentChild,
                                          boolean withComments, Writer wr ) throws
        IOException {
        int i = nodeVector.lastIndexOf( currentChild );
                if ( i < 0 ) {
            for ( Node focusnode = currentChild.getFirstChild();
                  focusnode != null;
                  focusnode = focusnode.getNextSibling() ) {
                serializeSubset( stack, nodeVector, focusnode, withComments, wr );
            }
            return;
        }
        Object obj = nodeVector.elementAt( i );
        if ( obj == null ) {
            return;
        }
                if ( obj instanceof Attributes ) {
            nodeVector.setElementAt( null, i );
            ( ( Attributes ) obj ).serialize( null, wr );
            return;
        }
        Node node = ( Node ) obj;
        int type = node.getNodeType();
        if ( type == Node.DOCUMENT_NODE ) {
            return;
        }
                if ( type != Node.ELEMENT_NODE ) {
            serializeSimpleNode( null, null, node, null, withComments, true, wr );
                                    return;
        }
                        wr.write( "<" );
        wr.write( node.getNodeName() );
        Attributes ancestorAttrs = stack.empty() ? null :
            ( Attributes ) stack.peek();
        Attributes attrs = null;
        int next = i + 1;
        if ( next >= nodeVector.size() ||
             nodeVector.elementAt( next ) instanceof Node ) {
                        Node parent = node.getParentNode();
                        if ( parent.getNodeType() != Node.DOCUMENT_NODE
                 && ancestorAttrs != null &&
                 ancestorAttrs.attributes.containsKey( "xmlns" ) ) {
                wr.write( " xmlns=\"\"" );
            }
        }
        else {
            attrs = ( Attributes ) nodeVector.elementAt( next );
                        if ( !attrs.attributes.containsKey( "xmlns" ) ) {
                                                Node parent = node.getParentNode();
                                if ( parent.getNodeType() != Node.DOCUMENT_NODE
                     && ancestorAttrs != null &&
                     ancestorAttrs.attributes.containsKey( "xmlns" ) ) {
                    wr.write( " xmlns=\"\"" );
                }
            }
            attrs.serialize( ancestorAttrs, wr );
            nodeVector.setElementAt( null, next );
        }
        wr.write( ">" );         stack.push( attrs );
        
        for ( Node focusnode = currentChild.getFirstChild();
              focusnode != null;
              focusnode = focusnode.getNextSibling() ) {
            serializeSubset( stack, nodeVector, focusnode, withComments, wr );
        }
        stack.pop();
        wr.write( "</" );
        wr.write( node.getNodeName() );
        wr.write( ">" );
    }
    
    private static boolean isAncestor ( Object me, Node anc ) {
        Node parent;
        if ( me instanceof Attributes ) {
            parent = ( ( Attributes ) me ).parent;
        }
        else {
            parent = ( Node ) me;
        }
        while ( parent != anc ) {
            if ( parent.getNodeType() == Node.ATTRIBUTE_NODE ) {
                parent = ( ( Attr ) parent ).getOwnerElement();
            }
            else {
                parent = parent.getParentNode();
            }
            if ( parent == null ) {
                return false;
            }
        }
        return true;
    }
    
    private static void serializeNode ( Node topNode, Node node,
                                        Node exceptedNode,
                                        boolean withComments,
                                        boolean xmlAttributes, Writer wr ) throws
        IOException {
        serializeNode( topNode, null, node, exceptedNode, withComments,
                       xmlAttributes, wr );
    }
                    private static void serializeNode ( Node topNode, Stack stack, Node node,
                                        Node exceptedNode,
                                        boolean withComments,
                                        boolean xmlAttributes, Writer wr ) throws
        IOException {
        if ( exceptedNode == node ) {
            return;
        }
        if ( node.getNodeType() == Node.ELEMENT_NODE ) {
            if ( stack == null ) {
                stack = new Stack();
            }
            wr.write( "<" );
            wr.write( node.getNodeName() );
            serializeAttributes( topNode, stack, ( Element ) node,
                                 xmlAttributes,
                                 wr );             wr.write( ">" );
            for ( Node child = node.getFirstChild(); child != null;
                  child = child.getNextSibling() ) {
                serializeNode( topNode, stack, child, exceptedNode,
                               withComments, xmlAttributes, wr );
            }
            stack.pop();
            wr.write( "</" );
            wr.write( node.getNodeName() );
            wr.write( ">" );
        }
        else if ( node.getNodeType() == Node.ATTRIBUTE_NODE ) {
            serializeAttribute( ( Attr ) node, wr );
        }
        else {
            serializeSimpleNode( topNode, stack, node, exceptedNode,
                                 withComments, xmlAttributes, wr );
        }
    }
        private static void serializeSimpleNode ( Node topNode, Stack stack,
                                              Node node, Node exceptedNode,
                                              boolean withComments,
                                              boolean xmlAttributes, Writer wr ) throws
        IOException {
        boolean lesser = false;
        boolean greater = false;
        switch ( node.getNodeType() ) {
            case Node.TEXT_NODE:
            case Node.CDATA_SECTION_NODE:
                String text = node.getNodeValue();
                for ( int i = 0; i < text.length(); i++ ) {
                    char ch = text.charAt( i );
                    if ( ch == '<' ) {
                        wr.write( "&lt;" );
                    }
                    else if ( ch == '>' ) {
                        wr.write( "&gt;" );
                    }
                    else if ( ch == '&' ) {
                        wr.write( "&amp;" );
                    }
                    else if ( ch == '\r' ) {
                        wr.write( "&#xD;" );
                    }
                    else {
                        wr.write( ch );
                    }
                }
                break;
            case Node.DOCUMENT_NODE:
            case Node.ENTITY_REFERENCE_NODE:
                for ( Node child = node.getFirstChild(); child != null;
                      child = child.getNextSibling() ) {
                    serializeNode( topNode, stack, child, exceptedNode,
                                   withComments, xmlAttributes, wr );
                }
                break;
            case Node.PROCESSING_INSTRUCTION_NODE:
                if ( node.getParentNode().getNodeType() == Node.DOCUMENT_NODE ) {
                    Node prev = node;
                    do {
                        prev = prev.getPreviousSibling();
                    }
                    while ( prev != null &&
                            prev.getNodeType() != Node.ELEMENT_NODE );
                                        if ( prev == null ) {
                        lesser = true;
                    }
                    else {
                        greater = true;
                    }
                }
                ProcessingInstruction pi = ( ProcessingInstruction ) node;
                if ( greater ) {
                    wr.write( "\n" );
                }
                wr.write( "<?" );
                wr.write( pi.getTarget() );
                if ( pi.getData() != null && pi.getData().length() > 0 ) {
                    wr.write( " " );
                                                            String data = pi.getData();
                    int start = 0;
                    for ( int index = 0; index < data.length(); index++ ) {
                        if ( data.charAt( index ) == '\r' ) {
                            if ( index > start ) {
                                wr.write( data, start, index - start );
                            }
                            wr.write( "&#xD;" );
                            start = index + 1;
                        }
                    }
                    if ( start < data.length() ) {
                        wr.write( data, start, data.length() - start );
                    }
                }
                wr.write( "?>" );
                if ( lesser ) {
                    wr.write( "\n" );
                }
                break;
            case Node.COMMENT_NODE:
                if ( !withComments ) {
                    break;
                }
                if ( node.getParentNode().getNodeType() == Node.DOCUMENT_NODE ) {
                    Node prev = node;
                    do {
                        prev = prev.getPreviousSibling();
                    }
                    while ( prev != null &&
                            prev.getNodeType() != Node.ELEMENT_NODE );
                                        if ( prev == null ) {
                        lesser = true;
                    }
                    else {
                        greater = true;
                    }
                }
                if ( greater ) {
                    wr.write( "\n" );
                }
                wr.write( "<!--" );
                wr.write( node.getNodeValue() );
                wr.write( "-->" );
                if ( lesser ) {
                    wr.write( "\n" );
                }
                break;
            case Node.DOCUMENT_TYPE_NODE:
                                break;
            case Node.ENTITY_NODE:
            case Node.DOCUMENT_FRAGMENT_NODE:
            case Node.NOTATION_NODE:
                throw new RuntimeException(
                    "Internal Error: Invalid Node Type: " + node.getNodeType() );
        }
    }
    private static boolean isTopElement ( Node topNode, Element element ) {
        if ( topNode == element ) {
            return true;
        }
        Node parent = element.getParentNode();
        while ( parent != null ) {
            switch ( parent.getNodeType() ) {
                case Node.DOCUMENT_NODE:
                    return true;
                case Node.ELEMENT_NODE:
                    return false;
                case Node.ENTITY_REFERENCE_NODE:
                    parent = parent.getParentNode();
                default:
                    throw new RuntimeException(
                        "Internal Error: Unexpected node type: "
                        + parent.getNodeType() );
            }
        }
        return true;     }
    
    private static void serializeAttributes ( Node topNode, Stack stack,
                                              Element element,
                                              boolean xmlAttributes, Writer wr ) throws
        IOException {
        Hashtable nsattrs = collectNamespaceNodesInAncestors( element );
        NamedNodeMap nnm = element.getAttributes();
        int atlen = nnm.getLength();
        for ( int i = 0; i < atlen; i++ ) {
            nsattrs.put( nnm.item( i ).getNodeName(), nnm.item( i ) );
        }
        if ( xmlAttributes ) {
            Hashtable xmlattrs = collectXMLPrefixAttributesInAncestors( element );
            if ( xmlattrs != null ) {
                for ( Enumeration enum1 = xmlattrs.elements();
                      enum1.hasMoreElements(); ) {
                    Attr attr = ( Attr )enum1.nextElement();
                    nsattrs.put( attr.getNodeName(), attr );
                }
            }
        }
        Hashtable parentAttrs = stack.isEmpty() ? null
            : ( Hashtable ) stack.peek();
        stack.push( nsattrs );
                atlen = nsattrs.size();
        String[] as = new String[atlen];         int[] indexMap = new int[atlen];
        Attr[] attrs = new Attr[atlen];
        int i = 0;
        Enumeration en = nsattrs.elements();
        while ( en.hasMoreElements() ) {
            Attr attr = ( Attr ) en.nextElement();
            indexMap[i] = i;
            attrs[i] = attr;
            as[i] = createSortedString( attr, element );
            i++;
        }
        JavaUtils.heapSort( indexMap, as, atlen );
                        for ( int j = 0; j < atlen; j++ ) {
            Attr a = attrs[indexMap[j]];
            String attrName = a.getNodeName();
                        if ( attrName.equals( "xmlns:xml" ) &&
                 a.getNodeValue() == Constant.NS_XMLNS_NAME ) {
                continue;
            }
                        if ( attrName.equals( "xmlns" ) || attrName.startsWith( "xmlns:" ) ) {
                                                if ( true ) {
                    if ( parentAttrs != null ) {
                        Attr nsa = ( Attr ) parentAttrs.get( attrName );
                        if ( nsa != null &&
                             nsa.getNodeValue().equals( a.getNodeValue() ) ) {
                            continue;
                        }
                    }
                }
                else {
                    boolean found = false;
                    String value = a.getNodeValue();
                    Node parent = element;
                    while ( parent != topNode ) {
                        parent = parent.getParentNode();
                        if ( parent == null ) {
                            break;
                        }
                        if ( parent.getNodeType() == Node.ELEMENT_NODE ) {
                            Element parentElement = ( Element ) parent;
                            if ( parentElement.hasAttribute( attrName ) ) {                                 if ( parentElement.getAttribute( attrName ).
                                     equals(
                                         value ) ) {
                                    found = true;
                                }
                                break;
                            }
                        }
                    }
                    if ( found ) {
                        continue;
                    }
                }
            }
            serializeAttribute( a, wr );
        }
    }
    
    private static void serializeAttribute ( Attr attr, Writer wr ) throws
        IOException {
        String name = attr.getNodeName();
        boolean checkURI = name.equals( "xmlns" ) || name.startsWith( "xmlns:" );
        wr.write( " " );
        wr.write( name );
        wr.write( "=\"" );
        String value = attr.getNodeValue();
        boolean hasColon = false;
        for ( int j = 0; j < value.length(); j++ ) {
            char ch = value.charAt( j );
            if ( ch == '&' ) {
                wr.write( "&amp;" );
            }
            else if ( ch == '<' ) {
                wr.write( "&lt;" );
            }
            else if ( ch == '"' ) {
                wr.write( "&quot;" );
            }
            else if ( ch == '\t' ) {
                wr.write( "&#x9;" );
            }
            else if ( ch == '\n' ) {
                wr.write( "&#xA;" );
            }
            else if ( ch == '\r' ) {
                wr.write( "&#xD;" );
            }
            else {
                if ( checkURI ) {
                    if ( ch == ':' ) {
                        hasColon = true;
                    }
                    else if ( ch == '/' && !hasColon ) {
                        throw new RuntimeException( "Found a relative URI: " +
                            value );
                    }
                }
                wr.write( ch );
            }
        }
        
                if ( checkURI && value.length() > 0 && !hasColon ) {
            throw new RuntimeException( "Found a relative URI: " + value );
        }
                wr.write( "\"" );
    }
    
    private static class AttrProxy
        implements Attr {
        Attr real;
        Element owner;
        public AttrProxy ( Node owner, Document factory, String name,
                           String value ) {
            this.real = factory.createAttribute( name );
            this.real.setValue( value );
            this.owner = ( Element ) owner;
        }
        public Element getOwnerElement () {
            return this.owner;
        }
        public String getName () {
            return this.real.getName();
        }
        public boolean getSpecified () {
            return false;
        }
        public String getValue () {
            return this.real.getValue();
        }
        public void setValue ( String value ) {
            this.real.setValue( value );
        }
        public Node appendChild ( Node newChild ) {
            return this.real.appendChild( newChild );
        }
        public Node cloneNode ( boolean deep ) {
            return this.real.cloneNode( deep );
        }
        public NamedNodeMap getAttributes () {
            return this.real.getAttributes();
        }
        public NodeList getChildNodes () {
            return this.real.getChildNodes();
        }
        public Node getFirstChild () {
            return this.real.getFirstChild();
        }
        public Node getLastChild () {
            return this.real.getLastChild();
        }
        public String getLocalName () {
            return this.real.getLocalName();
        }
        public String getNamespaceURI () {
            return this.real.getNamespaceURI();
        }
        public Node getNextSibling () {
            return this.real.getNextSibling();
        }
        public String getNodeName () {
            return this.real.getNodeName();
        }
        public short getNodeType () {
            return this.real.getNodeType();
        }
        public String getNodeValue () {
            return this.real.getNodeValue();
        }
        public Document getOwnerDocument () {
            return this.real.getOwnerDocument();
        }
        public Node getParentNode () {
            return this.real.getParentNode();
        }
        public String getPrefix () {
            return this.real.getPrefix();
        }
        public Node getPreviousSibling () {
            return this.real.getPreviousSibling();
        }
        public boolean hasAttributes () {
            return false;
        }
        public boolean hasChildNodes () {
            return this.real.hasChildNodes();
        }
        public Node insertBefore ( Node newChild, Node refChild ) {
            return this.real.insertBefore( newChild, refChild );
        }
        public void normalize () {
            this.real.normalize();
        }
        public Node removeChild ( Node oldChild ) {
            return this.real.removeChild( oldChild );
        }
        public Node replaceChild ( Node newChild, Node oldChild ) {
            return this.real.replaceChild( newChild, oldChild );
        }
        public void setNodeValue ( String nodeValue ) {
            this.real.setNodeValue( nodeValue );
        }
        public void setPrefix ( String prefix ) {
            this.real.setPrefix( prefix );
        }
        public boolean isSupported ( String feature, String version ) {
            return this.real.isSupported( feature, version );
        }
    }
    
    
    public static Hashtable collectNamespaceNodesInAncestors ( Node startNode ) {
        Node node = startNode;
        Hashtable nsnodes = new Hashtable();
        Document factory = startNode.getOwnerDocument();         do {
            NamedNodeMap attributeList = node.getAttributes();
            if ( attributeList != null ) {
                int nAttrs = attributeList.getLength();
                for ( int j = 0; j < nAttrs; j++ ) {
                    Attr attr = ( Attr ) attributeList.item( j );
                    String attrName = attr.getNodeName();
                    if ( attrName.equals( "xmlns" ) ||
                         attrName.startsWith( "xmlns:" ) ) {
                        if ( !nsnodes.containsKey( attrName ) ) {
                            if ( node == startNode ) {
                                nsnodes.put( attrName, attr );
                            }
                            else {
                                Attr newAttr = new AttrProxy( startNode,
                                    factory,
                                    attrName, attr.getNodeValue() );
                                nsnodes.put( attrName, newAttr );
                            }
                        }
                    }
                }
            }
        }
        while ( ( node = node.getParentNode() ) != null );
                        if ( !nsnodes.containsKey( "xmlns:xml" ) ) {
            nsnodes.put( "xmlns:xml",
                         new AttrProxy( startNode, factory, "xmlns:xml",
                                        Constant.NS_XMLNS_NAME ) );
        }
                return nsnodes;
    }
    public static void copyNamespaceNodesInAncestors ( Element startNode ) {
        Node node = startNode;
        while ( ( node = node.getParentNode() ) != null ) {
            NamedNodeMap attributeList = node.getAttributes();
            if ( attributeList != null ) {
                int nAttrs = attributeList.getLength();
                for ( int j = 0; j < nAttrs; j++ ) {
                    Attr attr = ( Attr ) attributeList.item( j );
                    String attrName = attr.getNodeName();
                    if ( attrName.equals( "xmlns" ) ||
                         attrName.startsWith( "xmlns:" ) ) {
                        if ( !startNode.hasAttribute( attrName ) ) {
                            startNode.setAttribute(
                                attrName, attr.getNodeValue() );
                        }
                    }
                }
            }
        }
    }
    public static void copyNamespaceNodesInAncestorsRecursively ( Node
        startNode ) {
        if ( startNode.getNodeType() == Node.ELEMENT_NODE ) {
            copyNamespaceNodesInAncestors( ( Element ) startNode );
        }
        for ( Node child = startNode.getFirstChild(); child != null;
              child = child.getNextSibling() ) {
            switch ( child.getNodeType() ) {
                case Node.ELEMENT_NODE:
                case Node.ENTITY_REFERENCE_NODE:
                    copyNamespaceNodesInAncestorsRecursively( child );
                    break;
            }
        }
    }
    
    public static Hashtable collectXMLPrefixAttributesInAncestors ( Node
        startNode ) {
        Node node = startNode.getParentNode();
        Hashtable attrs = null;
        do {
            NamedNodeMap attributeList = node.getAttributes();
            if ( attributeList != null ) {
                int nAttrs = attributeList.getLength();
                for ( int j = 0; j < nAttrs; j++ ) {
                    Attr attr = ( Attr ) attributeList.item( j );
                    String attrName = attr.getNodeName();
                    if ( attrName.startsWith( "xml:" ) ) {
                        if ( attrs == null || !attrs.containsKey( attrName ) ) {
                            if ( attrs == null ) {
                                attrs = new Hashtable();
                            }
                            attrs.put( attrName, attr );
                        }
                    }
                }
            }
        }
        while ( ( node = node.getParentNode() ) != null );
        if ( attrs != null ) {
            NamedNodeMap attributeList = startNode.getAttributes();
            if ( attributeList != null ) {
                int nAttrs = attributeList.getLength();
                for ( int j = 0; j < nAttrs; j++ ) {
                    Attr attr = ( Attr ) attributeList.item( j );
                    String attrName = attr.getNodeName();
                    if ( attrName.startsWith( "xml:" ) ) {
                        if ( attrs.containsKey( attrName ) ) {
                            attrs.remove( attrName );
                        }
                    }
                }
            }
        }
        return attrs;
    }
    
    private static String createSortedString ( Attr node, Node element ) {
        String ret;
        if ( node.getNodeType() == Node.ATTRIBUTE_NODE ) {
            String name = node.getNodeName();
            if ( name.equals( "xmlns" ) ) {
                return "\u0000";
            }
            int index = name.indexOf( ':' );
            if ( index <= 0 ) {
                return "\u0001" + name;
            }
            String prefix = name.substring( 0, index ).intern();
            if ( prefix.equals( "xmlns" ) ) {
                return "\u0000" + name.substring( index + 1 );
            }
            String uri = XMLUtils.getNamespaceForPrefix( prefix, element );
            if ( uri == null || uri.length() == 0 ) {
                ret = "\u0001" + name;
            }
            else {
                ret = uri + "\u0001" + name.substring( index + 1 );
            }
        }
        else {
            throw new IllegalArgumentException( "Requires an Attr node." );
        }
        return ret;
    }
    public static class NodeListImpl
        implements NodeList {
        Vector vector = null;
        public NodeListImpl () {
        }
        public NodeListImpl ( int size ) {
            this.vector = new Vector( size );
        }
        public void add ( Node node ) {
            if ( this.vector == null ) {
                this.vector = new Vector();
            }
            this.vector.addElement( node );
        }
        public Node item ( int index ) {
            return this.vector == null || index < 0 ||
                this.vector.size() <= index
                ? null : ( Node )this.vector.elementAt( index );
        }
        public int getLength () {
            return this.vector == null ? 0 : this.vector.size();
        }
    }
    public static void main ( String args[] ) throws Exception {
        FileInputStream fis = new FileInputStream( "d:/test.xml" );
        byte[] f = new byte[fis.available()];
        fis.read( f );
        Document doc = DOMWriter.StringToDocument( new String( f ), true );
        byte[] result = serializeAll( doc, true );
        System.out.println( new String( result ) );
        FileOutputStream fos = new FileOutputStream( "d:/result.xml" );
        fos.write( result );
        fos.close();
    }
}
